/* ============================================================================
 * TFTPProtocolCommsManagement(TFTPProtocolDataChannelManagement).java
 * ============================================================================
 * THIS SOFTWARE IS RELEASED UNDER THE FOLLOWING BINDING LEGAL AGREEMENT
 * (BSD-LICENSE).
 * ============================================================================
 *
 * Copyright (C) 1999-2001 by Paregos Mediadesign, Erik Eklund,
                 Johan "Spocke" Sorlin, http://sourceforge.net/projects/danoftp
 * Copyright (C) 2009
 * by J.H.H.C. van der Meijs (tcpconns@jhhcvandermeijs.nl)
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    * Neither the name of tcpconns nor the names of its copyright holders or
 *      its contributors may be used to endorse or promote products derived from
 *      this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 * ============================================================================
 * IF YOU DO NOT AGREE TO THE ABOVE TERMS AND CONDITIONS THEN DO *NOT* USE THIS
 * SOFTWARE AND *DO* DESTROY ALL COPIES OF THIS SOFTWARE THAT YOU HAVE.
 * ============================================================================
 *
 * TCPCONNS’ CURRENT CODE IS NOT KNOWN TO VIOLATE ANY COPYRIGHTS OR INFRINGE ON
 * ANY PATENTS. MOREOVER, IT ADHERES TO GENERAL PROGRAMMING RULES AND PRACTICES
 * AND IS OF SUCH A SIMPLISTIC NATURE THAT A SIMILAR PACKAGE COULD BE
 * CONSTRUCTED BY ANYONE. IMPORTANT(!): IF YOU FEEL THAT A PIECE OF CODE IN
 * TCPCONNS VIOLATES A COPYRIGHT OR INFRINGES ON A PATENT, PLEASE DO INFORM ONE
 * OF THE AUTHORS IMMEDIATELY. DO ALWAYS STATE EXACTLY WHAT CODE IS NOT ALLOWED
 * UNDER EXACTLY WHAT RULING, COPYRIGHT, OR PATENT; AND PLEASE DO ELABORATE ON
 * EXACTLY HOW THE CODE VIOLATES THE SPECIFIC COPYRIGHT OR INFRINGES ON
 * THE SPECIFIC PATENT.
 *
 * ============================================================================
 *
 * This software is Open Source Initiative approved Open Source Software.
 * Open Source Initiative Approved is a trademark of the Open Source Initiative.
 * http://www.opensource.org/
 *
 * ============================================================================
 */
package nl.jhhcvandermeijs.ftp.server;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import nl.jhhcvandermeijs.tcpconns.client.TTCPConnection;
import nl.jhhcvandermeijs.tcpconns.server.TTCPServer;

/**
 * TFTPProtocolCommsManagement deals with data transfer threads
 */
public class TFTPProtocolCommsManagement extends TFTPProtocolDataChannelManagement {

    /**
     * Constructor. Do not make objects of this class directly. Instances are created
     * by the TFTPServer class when a client connects through the method newSeverSideProtocol.
     *
     * @param aTTCPConnection handle to the client; the client can be accessed through this.conn()
     * @param aTTCPServer handle to the server
     */
    public TFTPProtocolCommsManagement(TTCPConnection aTTCPConnection, TTCPServer aTTCPServer) {super(aTTCPConnection, aTTCPServer);}

    protected static class FTPDataChannelThread extends Thread {
        private TFTPProtocolCommsManagement fOwner = null;
        public synchronized TFTPProtocolCommsManagement getOwner() {return this.fOwner;}
        public synchronized void setOwner(TFTPProtocolCommsManagement client) {this.fOwner = client;}

    }

    /**
     * A class to send a file list (directory contents) over the data channel
     * (separate thread)
     */
    protected static class SendFileListThread extends FTPDataChannelThread {
        /**
         * a list containing the directory contents to be send over
         */
        public String[] aFileList = null;

        /**
         * the method that contains the code to be executed by this thread
         */
        @Override
        public void run() {
            this.getOwner().setDataChannelState(DATA_CHANNEL_STATE_BUSY);
            
            PrintWriter out = null;
            try {
                
                out = new PrintWriter(new OutputStreamWriter(this.getOwner().getDataChannel().mySocket().getOutputStream(), "UTF-8")); //ASCII or BINARY MODE has no consequences here

                int i = 1-1; int n = aFileList.length;
                while ((i <= (n-1)) && (!(Thread.currentThread().isInterrupted()))) {
                    out.print(aFileList[i]+"\r\n");                   
                    //out.flush();
                    i++;
                }
                out.flush();
                
                this.getOwner().sendResponse(FTP_CLOSING_DATA_TRANSFER_PORT_OK, "Listing of \"" + this.getOwner().getCurrentDir() + "\" complete");
            } catch (Exception e) {
                this.getOwner().sendResponse(FTP_SYNTAX_ERROR, "Error listing \"" + this.getOwner().getCurrentDir() + "\"  [" + e + "]");
            }
            this.getOwner().closeDataChannel();
        }
    }

    /**
     * A class to send a file to the client over the data channel (separate thread)
     */
    public static class SendFileThread extends FTPDataChannelThread {
        /**
         * forms a handle to the file that is to be send
         */
        public String fileName = null;

        /**
         * the method that contains the code to be executed by this thread
         */
        @Override
        public void run() {
            this.getOwner().setDataChannelState(DATA_CHANNEL_STATE_BUSY);

            if (this.getOwner().getDataChannelDataMode() == TFTPProtocol.DATA_CHANNEL_DATA_MODE_ASCII) {
                BufferedWriter outStream = null;
                BufferedReader inStream = null;
                try {                    
                    String line = null;

                    outStream = new BufferedWriter(new OutputStreamWriter(this.getOwner().getDataChannel().mySocket().getOutputStream(), "UTF-8"));
                    inStream = new BufferedReader(new InputStreamReader(new FileInputStream(new File(fileName)), "UTF-8"));

                    line = "";
                    while (line != null) {
                        line = inStream.readLine();
                        if (line != null) {
                            outStream.write(line + "\r\n");
                        }
                    }

                    outStream.flush();
                    this.getOwner().sendResponse(FTP_CLOSING_DATA_TRANSFER_PORT_OK, "Retrieval of \"" + this.fileName + "\" complete");
                } catch (Exception e) {
                    this.getOwner().sendResponse(FTP_SYNTAX_ERROR, "Could not retrieve file \"" + this.fileName + "\" due to an error [" + e + "]");
                } finally {
                    try {inStream.close();} catch (Exception e2) {}
                }
            } else {
                BufferedOutputStream outStream = null;
                FileInputStream inStream = null;
                try {                                       
                    byte[] buffer = new byte[4096];
                    int bytes = 1;

            		outStream = new BufferedOutputStream(this.getOwner().getDataChannel().mySocket().getOutputStream());
                    inStream = new FileInputStream(new File(fileName));

                    while ((bytes != -1) && (!(Thread.currentThread().isInterrupted()))) {
                        bytes = inStream.read(buffer);
                        if (bytes != -1) {
                            outStream.write(buffer, 0, bytes);
                        }
                    }
                    outStream.flush();
                    this.getOwner().sendResponse(FTP_CLOSING_DATA_TRANSFER_PORT_OK, "Retrieval of \"" + this.fileName + "\" complete");
                } catch (Exception e) {
                    this.getOwner().sendResponse(FTP_SYNTAX_ERROR, "Could not retrieve file \"" + this.fileName + "\" due to an error [" + e + "]");
                } finally {
                    try{inStream.close();} catch (Exception e2) {}
                }
            }
            this.getOwner().closeDataChannel();
        }
    }

    /**
     * A class to receive a file from the client over the data channel (separate thread)
     */
    public static class StoreFileThread extends FTPDataChannelThread {
        /**
         * forms a handle to the file that is to be stored/appended
         */
        public String fileName = null;
        /**
         * append == false when STOR was requested and append == true when APPE was requested
         */
        public boolean append = false;

        /**
         * the method that contains the code to be executed by this thread
         */
        @Override
        public void run() {            
            this.getOwner().setDataChannelState(DATA_CHANNEL_STATE_BUSY);

            if (this.getOwner().getDataChannelDataMode() == TFTPProtocol.DATA_CHANNEL_DATA_MODE_ASCII) {
                BufferedWriter outStream = null;
                BufferedReader inStream = null;
                try {
                    String line = null;

                    inStream = new BufferedReader(new InputStreamReader(this.getOwner().getDataChannel().mySocket().getInputStream(), "UTF-8"));
                    outStream = new BufferedWriter(new OutputStreamWriter((new FileOutputStream(new File(fileName), this.append)), "UTF-8"));

                    line = "";
                    while (line != null) {
                        line = inStream.readLine();
                        if (line != null) {
                            outStream.write(line + "\r\n");
                        }
                    }

                    outStream.flush();
                    this.getOwner().sendResponse(FTP_CLOSING_DATA_TRANSFER_PORT_OK, "Storage of \"" + this.fileName + "\" complete");
                } catch (Exception e) {
                    this.getOwner().sendResponse(FTP_SYNTAX_ERROR, "Could not store \"" + this.fileName + "\" due to an error [" + e + "]");
                } finally {
                    try {outStream.close();} catch (Exception e2) {}
                }
            } else {
                FileOutputStream outStream = null;
                BufferedInputStream inStream = null;
                try {
                    byte[] buffer = new byte[4096];
                    int bytes = 1;

                    inStream = new BufferedInputStream(this.getOwner().getDataChannel().mySocket().getInputStream());
            		outStream = new FileOutputStream(new File(fileName), this.append);

                    while ((bytes != -1) && (!(Thread.currentThread().isInterrupted()))) {
                           bytes = inStream.read(buffer);
                           if (bytes != -1) {
                               outStream.write(buffer, 0, bytes);
                           }
                    }
                    outStream.flush();

                    this.getOwner().sendResponse(FTP_CLOSING_DATA_TRANSFER_PORT_OK, "Storage of \"" + this.fileName + "\" complete");
                } catch (Exception e) {
                    this.getOwner().sendResponse(FTP_SYNTAX_ERROR, "Could not store \"" + this.fileName + "\" due to an error [" + e + "]");
                } finally {
                    try{outStream.flush();} catch (Exception e2) {}
                    try{outStream.close();} catch (Exception e2) {}
                }
            }
            this.getOwner().closeDataChannel();
        }
    }

    /**
     * Intermediary method that creates the thread that will send the file list
     * to the client.
     * @param aFileList a list that contains the directory contents
     */
    protected void sendFileList(String[] aFileList) {
        if (this.getDataChannelState() != DATA_CHANNEL_STATE_IDLE) {this.sendResponse(FTP_ACCESS_DENIED, "WARNING! DATA CHANNEL NOT IDLE");}

               SendFileListThread aSendFileListThread = new SendFileListThread();
                                  aSendFileListThread.setOwner(this);
                                  aSendFileListThread.aFileList = aFileList;
        
        this.setDataChannelThread(aSendFileListThread);

        this.getDataChannelThread().start();
    }

    /**
     * Intermediary method that creates the thread that will send a file
     * to the client.
     * @param aFileName the file that is to be send
     */
    protected void sendFile(String aFileName) {
        if (this.getDataChannelState() != DATA_CHANNEL_STATE_IDLE) {this.sendResponse(FTP_ACCESS_DENIED, "WARNING! DATA CHANNEL NOT IDLE");}

                   SendFileThread aSendFileThread = new SendFileThread();
                                  aSendFileThread.setOwner(this);
                                  aSendFileThread.fileName = aFileName;

        this.setDataChannelThread(aSendFileThread);

        this.getDataChannelThread().start();
    }

    /**
     * Intermediary method that creates the thread that will receive a file
     * from the client.
     * @param aFileName a file that is to be (re)created
     * @param append true if aFileName needs to be appended (STOR => append == false;
     * APPE => append == true)
     */
    protected void storeFile(String aFileName, boolean append) {
        if (this.getDataChannelState() != DATA_CHANNEL_STATE_IDLE) {this.sendResponse(FTP_ACCESS_DENIED, "WARNING! DATA CHANNEL NOT IDLE");}

                  StoreFileThread aStoreFileThread = new StoreFileThread();
                                  aStoreFileThread.setOwner(this);
                                  aStoreFileThread.fileName = aFileName;
                                  aStoreFileThread.append = append;
        
        this.setDataChannelThread(aStoreFileThread);

        this.getDataChannelThread().start();
    }

	/**
	 * Sends a reply to the ftp client. Both a code and a message need
	 * to be sent to the client, since the code tells the client itself
	 * what is going on, and the message tells the user what is happening.
	 * This method should not be used to send a multiple line response to
	 * the client.
	 *
	 * @param aReturnCode the server reply code
	 * @param aMessage a string containing the message to the client
	 */
	public synchronized void sendResponse(int aReturnCode, String aMessage) {
        try {
            this.log(aMessage);
            this.out().print(aReturnCode + " " + aMessage +"\r\n");
            this.out().flush();
        } catch (Exception e) {
            try {this.conn().disconnect();} catch (Exception e2) {}
        }
	}

    /**
     * A method to log events. Invokes ((TFTPServer)(this.fTTCPServer)).log(aMessage);
     * @param aMessage an event/a message
     */
    public synchronized void log(String aMessage) {

        String security = null;
        if (this.isDataChannelOpen()) {
            if (this.getDataChannel().isSecure()) {
                security = this.getDataChannel().getProtocol() + ":" + this.getDataChannel().getCipherSuite() + " (CHANNEL)";
            } else {
                security = "UNSECURE CHANNEL";
            }
        } else {
            if (this.conn().isSecure()) {
                security = this.conn().getProtocol() + ":" + this.conn().getCipherSuite() + " (CONNECTION)";
            } else {
                security = "UNSECURE CONNECTION";
            }
        }

        aMessage =  "" + (new java.text.SimpleDateFormat("yyyyMMdd'T'HHmm' 'ss'.'SSS")).format(System.currentTimeMillis()) + ";"
                       + this.conn().mySocket().getInetAddress().getHostAddress() + ":" + this.conn().mySocket().getPort() + ";"
                       + security + ";" + "" + this.fTTCPServer.getClientCount() + ";"
                       + this.getUserName() + ";"

                       + aMessage + ";"

                       + this.fRequest;

        this.getServer().log(aMessage);
    }

}
