/* ============================================================================
 * TFTPProtocolSettingsAndState(TFTPProtocolStatics).java
 * ============================================================================
 * THIS SOFTWARE IS RELEASED UNDER THE FOLLOWING BINDING LEGAL AGREEMENT
 * (BSD-LICENSE).
 * ============================================================================
 *
 * Copyright (C) 1999-2001 by Paregos Mediadesign, Erik Eklund,
                 Johan "Spocke" Sorlin, http://sourceforge.net/projects/danoftp
 * Copyright (C) 2009
 * by J.H.H.C. van der Meijs (tcpconns@jhhcvandermeijs.nl)
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    * Neither the name of tcpconns nor the names of its copyright holders or
 *      its contributors may be used to endorse or promote products derived from
 *      this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 * ============================================================================
 * IF YOU DO NOT AGREE TO THE ABOVE TERMS AND CONDITIONS THEN DO *NOT* USE THIS
 * SOFTWARE AND *DO* DESTROY ALL COPIES OF THIS SOFTWARE THAT YOU HAVE.
 * ============================================================================
 *
 * TCPCONNS’ CURRENT CODE IS NOT KNOWN TO VIOLATE ANY COPYRIGHTS OR INFRINGE ON
 * ANY PATENTS. MOREOVER, IT ADHERES TO GENERAL PROGRAMMING RULES AND PRACTICES
 * AND IS OF SUCH A SIMPLISTIC NATURE THAT A SIMILAR PACKAGE COULD BE
 * CONSTRUCTED BY ANYONE. IMPORTANT(!): IF YOU FEEL THAT A PIECE OF CODE IN
 * TCPCONNS VIOLATES A COPYRIGHT OR INFRINGES ON A PATENT, PLEASE DO INFORM ONE
 * OF THE AUTHORS IMMEDIATELY. DO ALWAYS STATE EXACTLY WHAT CODE IS NOT ALLOWED
 * UNDER EXACTLY WHAT RULING, COPYRIGHT, OR PATENT; AND PLEASE DO ELABORATE ON
 * EXACTLY HOW THE CODE VIOLATES THE SPECIFIC COPYRIGHT OR INFRINGES ON
 * THE SPECIFIC PATENT.
 *
 * ============================================================================
 *
 * This software is Open Source Initiative approved Open Source Software.
 * Open Source Initiative Approved is a trademark of the Open Source Initiative.
 * http://www.opensource.org/
 *
 * ============================================================================
 */
package nl.jhhcvandermeijs.ftp.server;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.security.cert.X509Certificate;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import javax.net.ssl.SSLSocketFactory;
import nl.jhhcvandermeijs.tcpconns.client.TTCPConnection;
import nl.jhhcvandermeijs.tcpconns.keys.KeyStoreHelper;
import nl.jhhcvandermeijs.tcpconns.server.TTCPServer;
import nl.jhhcvandermeijs.tcpconns.tls.TTLSProvider;
import nl.jhhcvandermeijs.tcpconns.utils.Utils;
import nl.jhhcvandermeijs.tcpconns.utils.XMLFileStore;

/**
 * TFTPProtocolParentToBase implements some basic features of TFTPProtocol
 */
public class TFTPProtocolSettingsAndState extends TFTPProtocolStatics {
    /**
     * Constructor. Do not make objects of this class directly. Instances are created
     * by the TFTPServer class when a client connects through the method newSeverSideProtocol.
     *
     * @param aTTCPConnection handle to the client; the client can be accessed through this.conn()
     * @param aTTCPServer handle to the server
     */
    public TFTPProtocolSettingsAndState(TTCPConnection aTTCPConnection, TTCPServer aTTCPServer) {super(aTTCPConnection, aTTCPServer);}

    public static class FTPProtocolData {
        private int fMaxClients = 256;
        public synchronized int getMaxClients() {return this.fMaxClients;}
        public synchronized void setMaxClients(int maxClients) {this.fMaxClients = maxClients;}

        private int fServerPort = 2100;
        public synchronized int getServerPort() {return this.fServerPort;}
        public synchronized void setServerPort(int serverPort) {this.fServerPort = serverPort;}

        private String fBindIPAddress = "0.0.0.0";
        public synchronized String getBindIPAddress() {return this.fBindIPAddress;}
        public synchronized void setBindIPAddress(String bindIPAddress) {this.fBindIPAddress = bindIPAddress;}

        private String fExternalIPOfThisServer = "0.0.0.0";
        public synchronized String getExternalIPOfThisServer() {return this.fExternalIPOfThisServer;}
        public synchronized void setExternalIPOfThisServer(String externalIPOfThisServer) {this.fExternalIPOfThisServer = externalIPOfThisServer;}

        private boolean fTrackChanges = false;
        public synchronized boolean getTrackChanges() {return this.fTrackChanges;}
        public synchronized void setTrackChanges(boolean trackChanges) {this.fTrackChanges = trackChanges;}

        private String fLogDirectory = "";
        public synchronized String getLogDirectory() {return this.fLogDirectory;}
        public synchronized void setLogDirectory(String logDirectory) {this.fLogDirectory = logDirectory;}

        private String fMainLogFileName = "";
        public synchronized String getMainLogFileName() {return this.fMainLogFileName;}
        public synchronized void setMainLogFileName(String mainLogFileName) {this.fMainLogFileName = mainLogFileName;}

        private boolean fLogEvents = false;
        public synchronized boolean getLogEvents() {return this.fLogEvents;}
        public synchronized void setLogEvents(boolean setLogEvents) {this.fLogEvents = setLogEvents;}

        private boolean fLogAppend = false;
        public synchronized boolean getLogAppend() {return this.fLogAppend;}
        public synchronized void setLogAppend(boolean logAppend) {this.fLogAppend = logAppend;}

        private boolean fAllowTLS = false;
        public synchronized boolean getAllowTLS() {return this.fAllowTLS;}
        public synchronized void setAllowTLS(boolean allowTLS) {this.fAllowTLS = allowTLS;}

        private boolean fImplicitTLS = false;
        public synchronized boolean getImplicitTLS() {return this.fImplicitTLS;}
        public synchronized void setImplicitTLS(boolean implicitTLS) {this.fImplicitTLS = implicitTLS;}

        private boolean fEnforceExplicitTLSWhenNotImplicit = false;
        public synchronized boolean getEnforceExplicitTLSWhenNotImplicit() {return this.fEnforceExplicitTLSWhenNotImplicit;}
        public synchronized void setEnforceExplicitTLSWhenNotImplicit(boolean enforceExplicitTLSWhenNotImplicit) {this.fEnforceExplicitTLSWhenNotImplicit = enforceExplicitTLSWhenNotImplicit;}

        private boolean fEnforcePROTPWhenExplicit = false;
        public synchronized boolean getEnforcePROTPWhenExplicit() {return this.fEnforcePROTPWhenExplicit;}
        public synchronized void setEnforcePROTPWhenExplicit(boolean enforcePROTPWhenExplicit) {this.fEnforcePROTPWhenExplicit = enforcePROTPWhenExplicit;}

        private boolean fRequireClientAuthentication = false;
        public synchronized boolean getRequireClientAuthentication() {return this.fRequireClientAuthentication;}
        public synchronized void setRequireClientAuthentication(boolean requireClientAuthentication) {this.fRequireClientAuthentication = requireClientAuthentication;}

        private String fPEMEncodedServerCertAndKeyFileName = "";
        public synchronized String getPEMEncodedServerCertAndKeyFileName() {return this.fPEMEncodedServerCertAndKeyFileName;}
        public synchronized void setPEMEncodedServerCertAndKeyFileName(String pemEncodedServerCertAndKeyFileName) {this.fPEMEncodedServerCertAndKeyFileName = pemEncodedServerCertAndKeyFileName;}

        private String fKeyPassphrase = "";
        public synchronized String getKeyPassphrase() {return this.fKeyPassphrase;}
        public synchronized void setKeyPassphrase(String keyPassphrase) {this.fKeyPassphrase = keyPassphrase;}

        private String fTrustedCACertsDirectory = "";
        public synchronized String getTrustedCACertsDirectory() {return this.fTrustedCACertsDirectory;}
        public synchronized void setTrustedCACertsDirectory(String trustedCACertsDirectory) {this.fTrustedCACertsDirectory = trustedCACertsDirectory;}

        private List<User>  fUsersList  = Collections.synchronizedList(new LinkedList<User>());
        public synchronized List<User> getUsersList() {return this.fUsersList;}
        public synchronized void setUser(List<User> synchronizedUsersList) {this.fUsersList = synchronizedUsersList;}

        private List<Group> fGroupsList = Collections.synchronizedList(new LinkedList<Group>());
        public synchronized List<Group> getGroupsList() {return this.fGroupsList;}
        public synchronized void setGroupsList(List<Group> synchronizedGroupsList) {this.fGroupsList = synchronizedGroupsList;}

        public static class User {
            private String fUserName = "";
            public synchronized String getUserName() {return this.fUserName;}
            public synchronized void setUserName(String userName) {this.fUserName = userName;}

            private String fPassphrase = "";
            public synchronized String getPassphrase() {return this.fPassphrase;}
            public synchronized void setPassphrase(String passphrase) {this.fPassphrase = passphrase;}

            private boolean fRequirePassphraseToBeCorrect = true;
            public synchronized boolean getRequirePassphraseToBeCorrect() {return this.fRequirePassphraseToBeCorrect;}
            public synchronized void setRequirePassphraseToBeCorrect(boolean requirePassphraseToBeCorrect) {this.fRequirePassphraseToBeCorrect = requirePassphraseToBeCorrect;}

            private boolean fAccountIsActive = true;
            public synchronized boolean getAccountIsActive() {return this.fAccountIsActive;}
            public synchronized void setAccountIsActive(boolean accountIsActive) {this.fAccountIsActive = accountIsActive;}

            String fGroupName = "";
            public synchronized String getGroupName() {return this.fGroupName;}
            public synchronized void setGroupName(String groupName) {this.fGroupName = groupName;}
        }

        public static class Group {
            private String fGroupName = "";
            public synchronized String getGroupName() {return this.fGroupName;}
            public synchronized void setGroupName(String groupName) {this.fGroupName = groupName;}

            private String fOpeningDirectory = "";
            public synchronized String getOpeningDirectory() {return this.fOpeningDirectory;}
            public synchronized void setOpeningDirectory(String openingDirectory) {this.fOpeningDirectory = openingDirectory;}

            private List<Folder> fFoldersList = Collections.synchronizedList(new LinkedList<Folder>());
            public synchronized List<Folder> getFoldersList() {return this.fFoldersList;}
            public synchronized void setFoldersList(List<Folder> synchronizedFoldersList) {this.fFoldersList = synchronizedFoldersList;}
        }

        public static class Folder {
            private String fFolderName = "";
            public synchronized String getFolderName() {return this.fFolderName;}
            public synchronized void setFolderName(String folderName) {this.fFolderName = folderName;}

            private boolean fIsAllowedToSeeFiles = false;
            public synchronized boolean getIsAllowedToSeeFiles() {return this.fIsAllowedToSeeFiles;}
            public synchronized void setIsAllowedToSeeFiles(boolean isAllowedToSeeFiles) {this.fIsAllowedToSeeFiles = isAllowedToSeeFiles;}

            private boolean fIsAllowedToReadFiles = false;
            public synchronized boolean getIsAllowedToReadFiles() {return this.fIsAllowedToReadFiles;}
            public synchronized void setIsAllowedToReadFiles(boolean isAllowedToReadFiles) {this.fIsAllowedToReadFiles = isAllowedToReadFiles;}

            private boolean fIsAllowedToRenameFiles = false;
            public synchronized boolean getIsAllowedToRenameFiles() {return this.fIsAllowedToRenameFiles;}
            public synchronized void setIsAllowedToRenameFiles(boolean isAllowedToRenameFiles) {this.fIsAllowedToRenameFiles = isAllowedToRenameFiles;}

            private boolean fIsAllowedToAppendToFiles = false;
            public synchronized boolean getIsAllowedToAppendToFiles() {return this.fIsAllowedToAppendToFiles;}
            public synchronized void setIsAllowedToAppendToFiles(boolean isAllowedToAppendToFiles) {this.fIsAllowedToAppendToFiles = isAllowedToAppendToFiles;}

            private boolean fIsAllowedToCreateNewFiles = false;
            public synchronized boolean getIsAllowedToCreateNewFiles() {return this.fIsAllowedToCreateNewFiles;}
            public synchronized void setIsAllowedToCreateNewFiles(boolean isAllowedToCreateNewFiles) {this.fIsAllowedToCreateNewFiles = isAllowedToCreateNewFiles;}

            private boolean fIsAllowedToOverwriteFiles = false;
            public synchronized boolean getIsAllowedToOverwriteFiles() {return this.fIsAllowedToOverwriteFiles;}
            public synchronized void setIsAllowedToOverwriteFiles(boolean isAllowedToOverwriteFiles) {this.fIsAllowedToOverwriteFiles = isAllowedToOverwriteFiles;}

            private boolean fIsAllowedToDeleteFiles = false;
            public synchronized boolean getIsAllowedToDeleteFiles() {return this.fIsAllowedToDeleteFiles;}
            public synchronized void setIsAllowedToDeleteFiles(boolean isAllowedToDeleteFiles) {this.fIsAllowedToDeleteFiles = isAllowedToDeleteFiles;}

            private boolean fIsAllowedToSeeSubDirectories = false;
            public synchronized boolean getIsAllowedToSeeSubDirectories() {return this.fIsAllowedToSeeSubDirectories;}
            public synchronized void setIsAllowedToSeeSubDirectories(boolean isAllowedToSeeSubDirectories) {this.fIsAllowedToSeeSubDirectories = isAllowedToSeeSubDirectories;}

            private boolean fIsAllowedToTraverseSubDirectories = false;
            public synchronized boolean getIsAllowedToTraverseSubDirectories() {return this.fIsAllowedToTraverseSubDirectories;}
            public synchronized void setIsAllowedToTraverseSubDirectories(boolean isAllowedToTraverseSubDirectories) {this.fIsAllowedToTraverseSubDirectories = isAllowedToTraverseSubDirectories;}

            private boolean fIsAllowedToRenameSubDirectories = false;
            public synchronized boolean getIsAllowedToRenameSubDirectories() {return this.fIsAllowedToRenameSubDirectories;}
            public synchronized void setIsAllowedToRenameSubDirectories(boolean isAllowedToRenameSubDirectories) {this.fIsAllowedToRenameSubDirectories = isAllowedToRenameSubDirectories;}

            private boolean fIsAllowedToMakeSubDirectories = false;
            public synchronized boolean getIsAllowedToMakeSubDirectories() {return this.fIsAllowedToMakeSubDirectories;}
            public synchronized void setIsAllowedToMakeSubDirectories(boolean isAllowedToMakeSubDirectories) {this.fIsAllowedToMakeSubDirectories = isAllowedToMakeSubDirectories;}

            private boolean fIsAllowedToRemoveSubDirectories = false;
            public synchronized boolean getIsAllowedToRemoveSubDirectories() {return this.fIsAllowedToRemoveSubDirectories;}
            public synchronized void setIsAllowedToRemoveSubDirectories(boolean isAllowedToRemoveSubDirectories) {this.fIsAllowedToRemoveSubDirectories = isAllowedToRemoveSubDirectories;}

            private boolean fIsAllowedToSeeHiddenFiles = false;
            public synchronized boolean getIsAllowedToSeeHiddenFiles() {return this.fIsAllowedToSeeHiddenFiles;}
            public synchronized void setIsAllowedToSeeHiddenFiles(boolean isAllowedToSeeHiddenFiles) {this.fIsAllowedToSeeHiddenFiles = isAllowedToSeeHiddenFiles;}

            private boolean fIsAllowedToReadHiddenFiles = false;
            public synchronized boolean getIsAllowedToReadHiddenFiles() {return this.fIsAllowedToReadHiddenFiles;}
            public synchronized void setIsAllowedToReadHiddenFiles(boolean isAllowedToReadHiddenFiles) {this.fIsAllowedToReadHiddenFiles = isAllowedToReadHiddenFiles;}

            private boolean fIsAllowedToRenameHiddenFiles = false;
            public synchronized boolean getIsAllowedToRenameHiddenFiles() {return this.fIsAllowedToRenameHiddenFiles;}
            public synchronized void setIsAllowedToRenameHiddenFiles(boolean isAllowedToRenameHiddenFiles) {this.fIsAllowedToRenameHiddenFiles = isAllowedToRenameHiddenFiles;}

            private boolean fIsAllowedToAppendToHiddenFiles = false;
            public synchronized boolean getIsAllowedToAppendToHiddenFiles() {return this.fIsAllowedToAppendToHiddenFiles;}
            public synchronized void setIsAllowedToAppendToHiddenFiles(boolean isAllowedToAppendToHiddenFiles) {this.fIsAllowedToAppendToHiddenFiles = isAllowedToAppendToHiddenFiles;}

            private boolean fIsAllowedToCreateNewHiddenFiles = false;
            public synchronized boolean getIsAllowedToCreateNewHiddenFiles() {return this.fIsAllowedToCreateNewHiddenFiles;}
            public synchronized void setIsAllowedToCreateNewHiddenFiles(boolean isAllowedToCreateNewHiddenFiles) {this.fIsAllowedToCreateNewHiddenFiles = isAllowedToCreateNewHiddenFiles;}

            private boolean fIsAllowedToOverwriteHiddenFiles = false;
            public synchronized boolean getIsAllowedToOverwriteHiddenFiles() {return this.fIsAllowedToOverwriteHiddenFiles;}
            public synchronized void setIsAllowedToOverwriteHiddenFiles(boolean isAllowedToOverwriteHiddenFiles) {this.fIsAllowedToOverwriteHiddenFiles = isAllowedToOverwriteHiddenFiles;}

            private boolean fIsAllowedToDeleteHiddenFiles = false;
            public synchronized boolean getIsAllowedToDeleteHiddenFiles() {return this.fIsAllowedToDeleteHiddenFiles;}
            public synchronized void setIsAllowedToDeleteHiddenFiles(boolean isAllowedToDeleteHiddenFiles) {this.fIsAllowedToDeleteHiddenFiles = isAllowedToDeleteHiddenFiles;}

            private boolean fIsAllowedToSeeHiddenSubDirectories = false;
            public synchronized boolean getIsAllowedToSeeHiddenSubDirectories() {return this.fIsAllowedToSeeHiddenSubDirectories;}
            public synchronized void setIsAllowedToSeeHiddenSubDirectories(boolean isAllowedToSeeHiddenSubDirectories) {this.fIsAllowedToSeeHiddenSubDirectories = isAllowedToSeeHiddenSubDirectories;}

            private boolean fIsAllowedToTraverseHiddenSubDirectories = false;
            public synchronized boolean getIsAllowedToTraverseHiddenSubDirectories() {return this.fIsAllowedToTraverseHiddenSubDirectories;}
            public synchronized void setIsAllowedToTraverseHiddenSubDirectories(boolean isAllowedToTraverseHiddenSubDirectories) {this.fIsAllowedToTraverseHiddenSubDirectories = isAllowedToTraverseHiddenSubDirectories;}

            private boolean fIsAllowedToRenameHiddenSubDirectories = false;
            public synchronized boolean getIsAllowedToRenameHiddenSubDirectories() {return this.fIsAllowedToRenameHiddenSubDirectories;}
            public synchronized void setIsAllowedToRenameHiddenSubDirectories(boolean isAllowedToRenameHiddenSubDirectories) {this.fIsAllowedToRenameHiddenSubDirectories = isAllowedToRenameHiddenSubDirectories;}

            private boolean fIsAllowedToMakeHiddenSubDirectories = false;
            public synchronized boolean getIsAllowedToMakeHiddenSubDirectories() {return this.fIsAllowedToMakeHiddenSubDirectories;}
            public synchronized void setIsAllowedToMakeHiddenSubDirectories(boolean isAllowedToMakeHiddenSubDirectories) {this.fIsAllowedToMakeHiddenSubDirectories = isAllowedToMakeHiddenSubDirectories;}

            private boolean fIsAllowedToRemoveHiddenSubDirectories = false;
            public synchronized boolean getIsAllowedToRemoveHiddenSubDirectories() {return this.fIsAllowedToRemoveHiddenSubDirectories;}
            public synchronized void setIsAllowedToRemoveHiddenSubDirectories(boolean isAllowedToRemoveHiddenSubDirectories) {this.fIsAllowedToRemoveHiddenSubDirectories = isAllowedToRemoveHiddenSubDirectories;}

        }

        public synchronized User getUser(String userName) {
            if (this.getUsersList().size() > 0) {
                int i = 1-1; int n = this.getUsersList().size();
                while (i <= (n-1)) {
                    if (userName.equals(this.getUsersList().get(i).getUserName())) {
                        return this.getUsersList().get(i);
                    }
                    i++;
                }
            }
            return null;
        }

        public synchronized boolean userExists(String userName) {
            if ((this.getUser(userName)) == null) {
                return false;
            }
            return true;
        }

        public synchronized Group getGroup(String groupName) {
            if (this.getGroupsList().size() > 0) {
                int i = 1-1; int n = this.getGroupsList().size();
                while (i <= (n-1)) {
                    if (groupName.equals(this.getGroupsList().get(i).getGroupName())) {
                        return this.getGroupsList().get(i);
                    }
                    i++;
                }
            }
            return null;
        }

        public synchronized boolean groupExists(String groupName) {
            if ((this.getGroup(groupName)) == null) {
                return false;
            }
            return true;
        }

        public synchronized Folder getFolder(String groupName, String folderName) {
            Group group = this.getGroup(groupName);
            if (group != null) {
                if (group.getFoldersList().size() > 0) {
                    int i = 1-1; int n = group.getFoldersList().size();
                    while (i <= (n-1)) {
                        if (folderName.equals(group.getFoldersList().get(i).getFolderName())) {
                            return group.getFoldersList().get(i);
                        }
                        i++;
                    }
                }
            }
            return null;
        }

        public synchronized User[] usersOfGroup(String groupName) {
            User[] usersOfGroup = new User[0];
            if ((this.getUsersList().size() > 0) && (this.groupExists(groupName))) {
                List<User> users = new LinkedList<User>();
                int i = 1-1; int n = this.getUsersList().size();
                while (i <= (n-1)) {
                    if (groupName.equals(this.getUsersList().get(i).getGroupName())) {
                        users.add(this.getUsersList().get(i));
                    }
                    i++;
                }
            }
            return usersOfGroup;
        }

        public synchronized boolean folderExists(String groupName, String folderName) {
            if ((this.getFolder(groupName, folderName)) == null) {
                return false;
            }
            return true;
        }

        public synchronized void loadFromXML(String xmlFileName) {
            XMLFileStore xml = new XMLFileStore();
            xml.loadFromFile(xmlFileName);

            this.setMaxClients(xml.getSection(SERVER_SETTINGS).getElement(MAX_CLIENTS).getValueAsInteger());
            this.setBindIPAddress(xml.getSection(SERVER_SETTINGS).getElement(BIND_IP_ADDRESS).getValue());
            this.setServerPort(xml.getSection(SERVER_SETTINGS).getElement(SERVER_PORT).getValueAsInteger());
            this.setExternalIPOfThisServer(xml.getSection(SERVER_SETTINGS).getElement(EXTERNAL_IP_OF_THIS_SERVER).getValue());
            this.setTrackChanges(xml.getSection(SERVER_SETTINGS).getElement(TRACK_CHANGES).getValueAsBoolean());

            this.setLogDirectory(xml.getSection(SERVER_SETTINGS).getSection(LOG_SETTINGS).getElement(LOG_DIRECTORY).getValue());
            this.setMainLogFileName(xml.getSection(SERVER_SETTINGS).getSection(LOG_SETTINGS).getElement(MAIN_LOG_FILE_NAME).getValue());
            this.setLogEvents(xml.getSection(SERVER_SETTINGS).getSection(LOG_SETTINGS).getElement(LOG_EVENTS).getValueAsBoolean());
            this.setLogAppend(xml.getSection(SERVER_SETTINGS).getSection(LOG_SETTINGS).getElement(LOG_APPEND).getValueAsBoolean());

            this.setAllowTLS(xml.getSection(SERVER_SETTINGS).getSection(TLS_SETTINGS).getElement(ALLOW_TLS).getValueAsBoolean());
            this.setImplicitTLS(xml.getSection(SERVER_SETTINGS).getSection(TLS_SETTINGS).getElement(IMPLICIT_TLS).getValueAsBoolean());
            this.setEnforceExplicitTLSWhenNotImplicit(xml.getSection(SERVER_SETTINGS).getSection(TLS_SETTINGS).getElement(ENFORCE_EXPLICIT_TLS_WHEN_NOT_IMPLICIT).getValueAsBoolean());
            this.setEnforcePROTPWhenExplicit(xml.getSection(SERVER_SETTINGS).getSection(TLS_SETTINGS).getElement(ENFORCE_PROTP_WHEN_EXPLICIT).getValueAsBoolean());
            this.setRequireClientAuthentication(xml.getSection(SERVER_SETTINGS).getSection(TLS_SETTINGS).getElement(REQUIRE_CLIENT_AUTHENTICATION).getValueAsBoolean());
            this.setPEMEncodedServerCertAndKeyFileName(xml.getSection(SERVER_SETTINGS).getSection(TLS_SETTINGS).getElement(PEM_ENCODED_SERVER_CERT_AND_KEY_FILE_NAME).getValue());
            this.setKeyPassphrase(xml.getSection(SERVER_SETTINGS).getSection(TLS_SETTINGS).getElement(PEM_ENCODED_SERVER_CERT_AND_KEY_FILE_NAME).getAttribute(KEY_PASSPHRASE_ATTRIBUTE).getValue());
            this.setTrustedCACertsDirectory(xml.getSection(SERVER_SETTINGS).getSection(TLS_SETTINGS).getElement(TRUSTEDCA_CERTS_DIRECTORY).getValue());

            //GET GROUPS
            this.getGroupsList().clear();
            XMLFileStore.Element[] groups = xml.getSection(GROUPS).getElements(GROUP);
            XMLFileStore.Element   group  = null;
            if (groups != null) {
                int i = 1-1; int n = groups.length;
                while (i <= (n-1)) {
                    FTPProtocolData.Group aGroup = new FTPProtocolData.Group();
                    group = groups[i];
                    aGroup.setGroupName(group.getAttribute(GROUP_NAME_ATTRIBUTE).getValue());
                    aGroup.setOpeningDirectory(group.getAttribute(GROUP_OPENINGDIR_ATTRIBUTE).getValue());

                    //GET FOLDERS PER GROUP
                    XMLFileStore.Element[] folderElements = group.getElement(FOLDERS).getElements(FOLDER);
                    if (folderElements != null) {
                        int j = 1-1; int m = folderElements.length;
                        while (j <= (m-1)) {
                            XMLFileStore.Element folderElement = folderElements[j];
                            FTPProtocolData.Folder aFolder = new FTPProtocolData.Folder();
                            aFolder.setFolderName(folderElement.getAttribute(FOLDER_NAME_ATTRIBUTE).getValue());

                            aFolder.setIsAllowedToSeeFiles(folderElement.getElement(IS_ALLOWED_TO_SEE_FILES).getValueAsBoolean());
                            aFolder.setIsAllowedToReadFiles(folderElement.getElement(IS_ALLOWED_TO_READ_FILES).getValueAsBoolean());
                            aFolder.setIsAllowedToRenameFiles(folderElement.getElement(IS_ALLOWED_TO_RENAME_FILES).getValueAsBoolean());
                            aFolder.setIsAllowedToAppendToFiles(folderElement.getElement(IS_ALLOWED_TO_APPEND_TO_FILES).getValueAsBoolean());
                            aFolder.setIsAllowedToCreateNewFiles(folderElement.getElement(IS_ALLOWED_TO_CREATE_NEW_FILES).getValueAsBoolean());
                            aFolder.setIsAllowedToOverwriteFiles(folderElement.getElement(IS_ALLOWED_TO_OVERWRITE_FILES).getValueAsBoolean());
                            aFolder.setIsAllowedToDeleteFiles(folderElement.getElement(IS_ALLOWED_TO_DELETE_FILES).getValueAsBoolean());

                            aFolder.setIsAllowedToSeeSubDirectories(folderElement.getElement(IS_ALLOWED_TO_SEE_SUB_DIRECTORIES).getValueAsBoolean());
                            aFolder.setIsAllowedToTraverseSubDirectories(folderElement.getElement(IS_ALLOWED_TO_TRAVERSE_SUB_DIRECTORIES).getValueAsBoolean());
                            aFolder.setIsAllowedToRenameSubDirectories(folderElement.getElement(IS_ALLOWED_TO_RENAME_SUB_DIRECTORIES).getValueAsBoolean());
                            aFolder.setIsAllowedToMakeSubDirectories(folderElement.getElement(IS_ALLOWED_TO_MAKE_SUB_DIRECTORIES).getValueAsBoolean());
                            aFolder.setIsAllowedToRemoveSubDirectories(folderElement.getElement(IS_ALLOWED_TO_REMOVE_SUB_DIRECTORIES).getValueAsBoolean());

                            aFolder.setIsAllowedToSeeHiddenFiles(folderElement.getElement(IS_ALLOWED_TO_SEE_HIDDEN_FILES).getValueAsBoolean());
                            aFolder.setIsAllowedToReadHiddenFiles(folderElement.getElement(IS_ALLOWED_TO_READ_HIDDEN_FILES).getValueAsBoolean());
                            aFolder.setIsAllowedToRenameHiddenFiles(folderElement.getElement(IS_ALLOWED_TO_RENAME_HIDDEN_FILES).getValueAsBoolean());
                            aFolder.setIsAllowedToAppendToHiddenFiles(folderElement.getElement(IS_ALLOWED_TO_APPEND_TO_HIDDEN_FILES).getValueAsBoolean());
                            aFolder.setIsAllowedToCreateNewHiddenFiles(folderElement.getElement(IS_ALLOWED_TO_CREATE_NEW_HIDDEN_FILES).getValueAsBoolean());
                            aFolder.setIsAllowedToOverwriteHiddenFiles(folderElement.getElement(IS_ALLOWED_TO_OVERWRITE_HIDDEN_FILES).getValueAsBoolean());
                            aFolder.setIsAllowedToDeleteHiddenFiles(folderElement.getElement(IS_ALLOWED_TO_DELETE_HIDDEN_FILES).getValueAsBoolean());

                            aFolder.setIsAllowedToSeeHiddenSubDirectories(folderElement.getElement(IS_ALLOWED_TO_SEE_HIDDEN_SUB_DIRECTORIES).getValueAsBoolean());
                            aFolder.setIsAllowedToTraverseHiddenSubDirectories(folderElement.getElement(IS_ALLOWED_TO_TRAVERSE_HIDDEN_SUB_DIRECTORIES).getValueAsBoolean());
                            aFolder.setIsAllowedToRenameHiddenSubDirectories(folderElement.getElement(IS_ALLOWED_TO_RENAME_HIDDEN_SUB_DIRECTORIES).getValueAsBoolean());
                            aFolder.setIsAllowedToMakeHiddenSubDirectories(folderElement.getElement(IS_ALLOWED_TO_MAKE_HIDDEN_SUB_DIRECTORIES).getValueAsBoolean());
                            aFolder.setIsAllowedToRemoveHiddenSubDirectories(folderElement.getElement(IS_ALLOWED_TO_REMOVE_HIDDEN_SUB_DIRECTORIES).getValueAsBoolean());

                            aGroup.getFoldersList().add(aFolder);
                            j++;
                        }
                    }
                    this.getGroupsList().add(aGroup);
                    i++;
                }
            }

            //GET USERS
            this.getUsersList().clear();
            XMLFileStore.Element[] users  = xml.getSection(USERS).getElements(USER);            
            if (users != null) {
                int i = 1-1; int n = users.length;
                while (i <= (n-1)) {
                    FTPProtocolData.User aUser = new FTPProtocolData.User();
                    XMLFileStore.Element userElement = users[i];
                    aUser.setUserName(userElement.getAttribute(USER_NAME_ATTRIBUTE).getValue());
                    aUser.setPassphrase(userElement.getAttribute(USER_PASSPHRASE_ATTRIBUTE).getValue());
                    aUser.setRequirePassphraseToBeCorrect(userElement.getElement(USER_REQUIRE_PASSPHRASE_TO_BE_CORRECT).getValueAsBoolean());
                    aUser.setGroupName(userElement.getElement(USER_GROUP_MEMBERSHIP).getValue());
                    aUser.setAccountIsActive(userElement.getElement(USER_ACCOUNT_IS_ACTIVE).getValueAsBoolean());
                    this.getUsersList().add(aUser);
                    i++;
                }
            }
            xml.clear();
            xml = null;
        }


        public synchronized void saveToXML(String xmlFileName) {
            XMLFileStore xml = new XMLFileStore(ROOT_ELEMENT_NAME, ROOT_COMMENT);

            //ADD VARIOUS SERVER SETTINGS
            xml.addSection(SERVER_SETTINGS).addElement(MAX_CLIENTS).setValueAsInteger(this.getMaxClients());
            xml.getSection(SERVER_SETTINGS).addElement(SERVER_PORT).setValueAsInteger(this.getServerPort());
            xml.getSection(SERVER_SETTINGS).addElement(BIND_IP_ADDRESS).setValue(this.getBindIPAddress());
            xml.getSection(SERVER_SETTINGS).addElement(EXTERNAL_IP_OF_THIS_SERVER).setValue(this.getExternalIPOfThisServer());
            xml.getSection(SERVER_SETTINGS).addElement(TRACK_CHANGES).setValueAsBoolean(this.getTrackChanges());

            //ADD LOG SETTINGS
            xml.getSection(SERVER_SETTINGS).addSection(LOG_SETTINGS).addElement(LOG_DIRECTORY).setValue(this.getLogDirectory());
            xml.getSection(SERVER_SETTINGS).getSection(LOG_SETTINGS).addElement(MAIN_LOG_FILE_NAME).setValue(this.getMainLogFileName());
            xml.getSection(SERVER_SETTINGS).getSection(LOG_SETTINGS).addElement(LOG_EVENTS).setValueAsBoolean(this.getLogEvents());
            xml.getSection(SERVER_SETTINGS).getSection(LOG_SETTINGS).addElement(LOG_APPEND).setValueAsBoolean(this.getLogAppend());

            //ADD TLS SETTINGS
            xml.getSection(SERVER_SETTINGS).addSection(TLS_SETTINGS).addElement(ALLOW_TLS).setValueAsBoolean(this.getAllowTLS());
            xml.getSection(SERVER_SETTINGS).getSection(TLS_SETTINGS).addElement(IMPLICIT_TLS).setValueAsBoolean(this.getImplicitTLS());
            xml.getSection(SERVER_SETTINGS).getSection(TLS_SETTINGS).addElement(ENFORCE_EXPLICIT_TLS_WHEN_NOT_IMPLICIT).setValueAsBoolean(this.getEnforceExplicitTLSWhenNotImplicit());
            xml.getSection(SERVER_SETTINGS).getSection(TLS_SETTINGS).addElement(ENFORCE_PROTP_WHEN_EXPLICIT).setValueAsBoolean(this.getEnforcePROTPWhenExplicit());
            xml.getSection(SERVER_SETTINGS).getSection(TLS_SETTINGS).addElement(REQUIRE_CLIENT_AUTHENTICATION).setValueAsBoolean(this.getRequireClientAuthentication());
            xml.getSection(SERVER_SETTINGS).getSection(TLS_SETTINGS).addElement(PEM_ENCODED_SERVER_CERT_AND_KEY_FILE_NAME).setValue(this.getPEMEncodedServerCertAndKeyFileName());
            xml.getSection(SERVER_SETTINGS).getSection(TLS_SETTINGS).getElement(PEM_ENCODED_SERVER_CERT_AND_KEY_FILE_NAME).addAttribute(KEY_PASSPHRASE_ATTRIBUTE, this.getKeyPassphrase());
            xml.getSection(SERVER_SETTINGS).getSection(TLS_SETTINGS).addElement(TRUSTEDCA_CERTS_DIRECTORY).setValue(Utils.validateDirectory(this.getTrustedCACertsDirectory()));

            //ADD GROUPS
            xml.addSection(GROUPS);
            if (this.getGroupsList().size() > 0) {
                int i = 1-1; int n = this.getGroupsList().size();
                while (i <= (n-1)) {
                     XMLFileStore.Element group = xml.getSection(GROUPS).addSection(GROUP);
                     group.addAttribute(GROUP_NAME_ATTRIBUTE, this.getGroupsList().get(i).getGroupName());
                     group.addAttribute(GROUP_OPENINGDIR_ATTRIBUTE, (Utils.validateDirectory(this.getGroupsList().get(i).getOpeningDirectory())));
                     if (this.getGroupsList().get(i).getFoldersList().size() > 0) {
                         group.addSection(FOLDERS);
                         int j = 1-1; int m = this.getGroupsList().get(i).getFoldersList().size();
                         while (j <= (m-1)) {
                            XMLFileStore.Element folders = group.getSection(FOLDERS);
                            XMLFileStore.Element folder = folders.addSection(FOLDER);
                                 folder.addAttribute(FOLDER_NAME_ATTRIBUTE, Utils.validateDirectory(this.getGroupsList().get(i).getFoldersList().get(j).getFolderName()));

                                 folder.addElement(IS_ALLOWED_TO_SEE_FILES).setValueAsBoolean(this.getGroupsList().get(i).getFoldersList().get(j).getIsAllowedToSeeFiles());
                                 folder.addElement(IS_ALLOWED_TO_READ_FILES).setValueAsBoolean(this.getGroupsList().get(i).getFoldersList().get(j).getIsAllowedToReadFiles());
                                 folder.addElement(IS_ALLOWED_TO_RENAME_FILES).setValueAsBoolean(this.getGroupsList().get(i).getFoldersList().get(j).getIsAllowedToRenameFiles());
                                 folder.addElement(IS_ALLOWED_TO_APPEND_TO_FILES).setValueAsBoolean(this.getGroupsList().get(i).getFoldersList().get(j).getIsAllowedToAppendToFiles());
                                 folder.addElement(IS_ALLOWED_TO_CREATE_NEW_FILES).setValueAsBoolean(this.getGroupsList().get(i).getFoldersList().get(j).getIsAllowedToCreateNewFiles());
                                 folder.addElement(IS_ALLOWED_TO_OVERWRITE_FILES).setValueAsBoolean(this.getGroupsList().get(i).getFoldersList().get(j).getIsAllowedToOverwriteFiles());
                                 folder.addElement(IS_ALLOWED_TO_DELETE_FILES).setValueAsBoolean(this.getGroupsList().get(i).getFoldersList().get(j).getIsAllowedToDeleteFiles());

                                 folder.addElement(IS_ALLOWED_TO_SEE_SUB_DIRECTORIES).setValueAsBoolean(this.getGroupsList().get(i).getFoldersList().get(j).getIsAllowedToSeeSubDirectories());
                                 folder.addElement(IS_ALLOWED_TO_TRAVERSE_SUB_DIRECTORIES).setValueAsBoolean(this.getGroupsList().get(i).getFoldersList().get(j).getIsAllowedToTraverseSubDirectories());
                                 folder.addElement(IS_ALLOWED_TO_RENAME_SUB_DIRECTORIES).setValueAsBoolean(this.getGroupsList().get(i).getFoldersList().get(j).getIsAllowedToRenameSubDirectories());
                                 folder.addElement(IS_ALLOWED_TO_MAKE_SUB_DIRECTORIES).setValueAsBoolean(this.getGroupsList().get(i).getFoldersList().get(j).getIsAllowedToMakeSubDirectories());
                                 folder.addElement(IS_ALLOWED_TO_REMOVE_SUB_DIRECTORIES).setValueAsBoolean(this.getGroupsList().get(i).getFoldersList().get(j).getIsAllowedToRemoveSubDirectories());

                                 folder.addElement(IS_ALLOWED_TO_SEE_HIDDEN_FILES).setValueAsBoolean(this.getGroupsList().get(i).getFoldersList().get(j).getIsAllowedToSeeHiddenFiles());
                                 folder.addElement(IS_ALLOWED_TO_READ_HIDDEN_FILES).setValueAsBoolean(this.getGroupsList().get(i).getFoldersList().get(j).getIsAllowedToReadHiddenFiles());
                                 folder.addElement(IS_ALLOWED_TO_RENAME_HIDDEN_FILES).setValueAsBoolean(this.getGroupsList().get(i).getFoldersList().get(j).getIsAllowedToRenameHiddenFiles());
                                 folder.addElement(IS_ALLOWED_TO_APPEND_TO_HIDDEN_FILES).setValueAsBoolean(this.getGroupsList().get(i).getFoldersList().get(j).getIsAllowedToAppendToHiddenFiles());
                                 folder.addElement(IS_ALLOWED_TO_CREATE_NEW_HIDDEN_FILES).setValueAsBoolean(this.getGroupsList().get(i).getFoldersList().get(j).getIsAllowedToCreateNewHiddenFiles());
                                 folder.addElement(IS_ALLOWED_TO_OVERWRITE_HIDDEN_FILES).setValueAsBoolean(this.getGroupsList().get(i).getFoldersList().get(j).getIsAllowedToOverwriteHiddenFiles());
                                 folder.addElement(IS_ALLOWED_TO_DELETE_HIDDEN_FILES).setValueAsBoolean(this.getGroupsList().get(i).getFoldersList().get(j).getIsAllowedToDeleteHiddenFiles());

                                 folder.addElement(IS_ALLOWED_TO_SEE_HIDDEN_SUB_DIRECTORIES).setValueAsBoolean(this.getGroupsList().get(i).getFoldersList().get(j).getIsAllowedToSeeHiddenSubDirectories());
                                 folder.addElement(IS_ALLOWED_TO_TRAVERSE_HIDDEN_SUB_DIRECTORIES).setValueAsBoolean(this.getGroupsList().get(i).getFoldersList().get(j).getIsAllowedToTraverseHiddenSubDirectories());
                                 folder.addElement(IS_ALLOWED_TO_RENAME_HIDDEN_SUB_DIRECTORIES).setValueAsBoolean(this.getGroupsList().get(i).getFoldersList().get(j).getIsAllowedToRenameHiddenSubDirectories());
                                 folder.addElement(IS_ALLOWED_TO_MAKE_HIDDEN_SUB_DIRECTORIES).setValueAsBoolean(this.getGroupsList().get(i).getFoldersList().get(j).getIsAllowedToMakeHiddenSubDirectories());
                                 folder.addElement(IS_ALLOWED_TO_REMOVE_HIDDEN_SUB_DIRECTORIES).setValueAsBoolean(this.getGroupsList().get(i).getFoldersList().get(j).getIsAllowedToRemoveHiddenSubDirectories());

                             j++;
                         }
                     }
                    i++;
                }
            }

            //ADD USERS
            xml.addSection(USERS);
            if (this.getUsersList().size() > 0) {
                int i = 1-1; int n = this.getUsersList().size();
                while (i <= (n-1)) {
                    XMLFileStore.Element user = xml.getSection(USERS).addSection(USER);
                    user.addAttribute(USER_NAME_ATTRIBUTE, this.getUsersList().get(i).getUserName());
                    user.addAttribute(USER_PASSPHRASE_ATTRIBUTE, this.getUsersList().get(i).getPassphrase());
                    user.addElement(USER_REQUIRE_PASSPHRASE_TO_BE_CORRECT).setValueAsBoolean(this.getUsersList().get(i).getRequirePassphraseToBeCorrect()); //normally should be TRUE!!!
                    user.addElement(USER_ACCOUNT_IS_ACTIVE).setValueAsBoolean(this.getUsersList().get(i).getAccountIsActive());
                    user.addElement(USER_GROUP_MEMBERSHIP).setValue(this.getUsersList().get(i).getGroupName());
                    i++;
                }
            }

            //SAVE XML
            xml.saveToFile(xmlFileName);
            xml.clear();
            xml = null;
        }
    }

    private int fMaxClients = 256;
    public synchronized int getMaxClients() {return this.fMaxClients;}
    public synchronized void setMaxClients(int maxClients) {this.fMaxClients = maxClients;}

    private int fServerPort = 2100;
    public synchronized int getServerPort() {return this.fServerPort;}
    public synchronized void setServerPort(int serverPort) {this.fServerPort = serverPort;}

    private String fBindIPAddress = "0.0.0.0";
    public synchronized String getBindIPAddress() {return this.fBindIPAddress;}
    public synchronized void setBindIPAddress(String bindIPAddress) {this.fBindIPAddress = bindIPAddress;}

    private String fExternalIPOfThisServer = "0.0.0.0";
    public synchronized String getExternalIPOfThisServer() {return this.fExternalIPOfThisServer;}
    public synchronized void setExternalIPOfThisServer(String externalIPOfThisServer) {this.fExternalIPOfThisServer = externalIPOfThisServer;}

    private boolean fTrackChanges = false;
    public synchronized boolean getTrackChanges() {return this.fTrackChanges;}
    public synchronized void setTrackChanges(boolean trackChanges) {this.fTrackChanges = trackChanges;}

    private String fLogDirectory = "";
    public synchronized String getLogDirectory() {return this.fLogDirectory;}
    public synchronized void setLogDirectory(String logDirectory) {this.fLogDirectory = logDirectory;}

    private String fMainLogFileName = "";
    public synchronized String getMainLogFileName() {return this.fMainLogFileName;}
    public synchronized void setMainLogFileName(String mainLogFileName) {this.fMainLogFileName = mainLogFileName;}

    private boolean fLogEvents = false;
    public synchronized boolean getLogEvents() {return this.fLogEvents;}
    public synchronized void setLogEvents(boolean setLogEvents) {this.fLogEvents = setLogEvents;}

    private boolean fLogAppend = false;
    public synchronized boolean getLogAppend() {return this.fLogAppend;}
    public synchronized void setLogAppend(boolean logAppend) {this.fLogAppend = logAppend;}

    private boolean fAllowTLS = false;
    public synchronized boolean getAllowTLS() {return this.fAllowTLS;}
    public synchronized void setAllowTLS(boolean allowTLS) {this.fAllowTLS = allowTLS;}

    private boolean fImplicitTLS = false;
    public synchronized boolean getImplicitTLS() {return this.fImplicitTLS;}
    public synchronized void setImplicitTLS(boolean implicitTLS) {this.fImplicitTLS = implicitTLS;}

    private boolean fEnforceExplicitTLSWhenNotImplicit = false;
    public synchronized boolean getEnforceExplicitTLSWhenNotImplicit() {return this.fEnforceExplicitTLSWhenNotImplicit;}
    public synchronized void setEnforceExplicitTLSWhenNotImplicit(boolean enforceExplicitTLSWhenNotImplicit) {this.fEnforceExplicitTLSWhenNotImplicit = enforceExplicitTLSWhenNotImplicit;}

    private boolean fEnforcePROTPWhenExplicit = false;
    public synchronized boolean getEnforcePROTPWhenExplicit() {return this.fEnforcePROTPWhenExplicit;}
    public synchronized void setEnforcePROTPWhenExplicit(boolean enforcePROTPWhenExplicit) {this.fEnforcePROTPWhenExplicit = enforcePROTPWhenExplicit;}

    private boolean fRequireClientAuthentication = false;
    public synchronized boolean getRequireClientAuthentication() {return this.fRequireClientAuthentication;}
    public synchronized void setRequireClientAuthentication(boolean requireClientAuthentication) {this.fRequireClientAuthentication = requireClientAuthentication;}

    private String fPEMEncodedServerCertAndKeyFileName = "";
    public synchronized String getPEMEncodedServerCertAndKeyFileName() {return this.fPEMEncodedServerCertAndKeyFileName;}
    public synchronized void setPEMEncodedServerCertAndKeyFileName(String pemEncodedServerCertAndKeyFileName) {this.fPEMEncodedServerCertAndKeyFileName = pemEncodedServerCertAndKeyFileName;}

    private String fKeyPassphrase = "";
    public synchronized String getKeyPassphrase() {return this.fKeyPassphrase;}
    public synchronized void setKeyPassphrase(String keyPassphrase) {this.fKeyPassphrase = keyPassphrase;}

    private String fTrustedCACertsDirectory = "";
    public synchronized String getTrustedCACertsDirectory() {return this.fTrustedCACertsDirectory;}
    public synchronized void setTrustedCACertsDirectory(String trustedCACertsDirectory) {this.fTrustedCACertsDirectory = trustedCACertsDirectory;}

    /**
     * A handle to the inputstream of the connection.
     */
    protected BufferedReader fInStream = null;
    /**
     * A handle to the inputstream of the connection
     * @return fInStream
     */
    public BufferedReader in() {
        if (this.fInStream == null) {this.invalidate();}
        return (this.fInStream);
    }

    /**
     * A handle to the outputstream of the connection
     */
    protected PrintWriter fOutStream = null;
    /**
     * A handle to the outputstream of the connection
     * @return fOutStream
     */
    public PrintWriter out() {
        if (this.fOutStream == null) {this.invalidate();}
        return (this.fOutStream);
    }

    /**
     * A method to get the input and outputstreams for the connection; important when
     * the actual socket has changed (new connection or when switching from normal sockets
     * to secure (SSL/TLS) sockets).
     */
    protected void invalidate() {
        try {
            this.fInStream = new BufferedReader(new InputStreamReader(this.conn().mySocket().getInputStream()));
            this.fOutStream = new PrintWriter(new OutputStreamWriter(this.conn().mySocket().getOutputStream()));
        } catch (Exception e) {
            try {this.conn().disconnect();} catch (Exception e2) {}
        }
    }

    /**
     * override this method in a subclass to allow for X509 certificate revocation status checking; if this
     * is omitted, this function will return true to indicate that it is "not" revoked.
     * @param anX509 the X509 of which the revocation status is to be checked
     * @return true if the certificate is still valid, false if it is revoked
     */
    public boolean onCheckRevocationStatus(X509Certificate anX509) {
        return true;
    }


    /**
     * method to test for certificate validity
     * @param trustedCACertsDirectory the folder that contains trusted certificate authority certificates in PEM format;
     * note ALL are assumed to be valid
     * @param clientX509Certificate the certificate of wich validity has to be tested
     * @return true if the certificate is signed by a trusted certificate authority, false if it is not valid
     */
    public boolean isValid(String trustedCACertsDirectory, X509Certificate clientX509Certificate) {
        if (
                    (trustedCACertsDirectory != null)
                &&
                    (clientX509Certificate != null)
                &&
                    ((new File(trustedCACertsDirectory)).exists())
                &&
                    ((new File(trustedCACertsDirectory)).list() != null)
                &&
                    ((new File(trustedCACertsDirectory)).list().length > 0)

            ) {

                String[] trustedCAsList = (new File(trustedCACertsDirectory)).list();
                int i = 1-1; int n = trustedCAsList.length;
                while (i <= (n-1)) {
                    try {
                        X509Certificate trustedCA = KeyStoreHelper.loadX509(trustedCAsList[i]);
                        if (KeyStoreHelper.isValid(clientX509Certificate, trustedCA)) {
                            //cert is valid: check for revocation status here
                            return onCheckRevocationStatus(clientX509Certificate);
                        }
                    } catch (Exception e) {}
                    i++;
                }
        }
        return false;
    }

    protected SSLSocketFactory fSSLSocketFactory = null;
    public synchronized SSLSocketFactory getSSLSocketFactory() {return this.fSSLSocketFactory;}
    public synchronized void setSSLSocketFactory(SSLSocketFactory sslSocketFactory) {this.fSSLSocketFactory = sslSocketFactory;}

    protected void establishTLS() throws IOException {
        if (this.fSSLSocketFactory == null) {
            try{this.fSSLSocketFactory = TTLSProvider.createSSLSocketFactory(KeyStoreHelper.createKeyManagers(this.getPEMEncodedServerCertAndKeyFileName(), this.getKeyPassphrase().toCharArray()), null, null);} catch (Exception e) {this.conn().disconnect();}
        }
        if (this.getRequireClientAuthentication()) {

            try{this.conn().makeSecureWithClientCertAuthByServer(this.getSSLSocketFactory());} catch (Exception e) {this.conn().disconnect();}

            java.security.cert.Certificate[] peerCertChain = this.conn().getPeerCertificateChain();
            //check whether we have a peerCertChain
            if (peerCertChain == null) {this.conn().disconnect(); throw new IOException();}
            if (peerCertChain.length == 0) {this.conn().disconnect(); throw new IOException();}
            if (!(peerCertChain[0] instanceof X509Certificate)) {this.conn().disconnect(); throw new IOException();}

            //get peercertificate
            X509Certificate anX509 = ((X509Certificate)(peerCertChain[0]));
           if (!(this.isValid(this.getTrustedCACertsDirectory(), anX509))) {
                this.conn().disconnect(); throw new IOException();
            }
        } else {

            try {this.conn().makeSecureWithServerCertAuthByClient(this.getSSLSocketFactory());} catch (Exception e) {this.conn().disconnect(); throw new IOException();}

        }
        this.invalidate();
    }

    /**
     * A temporary place holder for the request that was made, so that other methods
     * can access the original request if needed
     */
    protected String fRequest = null;

    /**
     * A temporary place holder for the command that was requested, so that other methods
     * can still access the command that was given
     */
    protected String fCommand = null;

    /**
     * A temporary place holder for the argument that was provided together with a command,
     * so that other methods can utilize it
     */
    protected String fArgument = null;

    private boolean fAUTHTLS = false;
    public synchronized boolean getAUTHTLS() {return this.fAUTHTLS;}
    public synchronized void setAUTHTLS(boolean authTLS) {
        this.fAUTHTLS = authTLS;
        if (this.fAUTHTLS) {
            try{this.establishTLS();} catch(Exception e){try{this.conn().disconnect();} catch (Exception e2) {}}
        }
    }

    private boolean fPROTP = false;
    public synchronized boolean getPROTP() {return this.fPROTP;}
    public synchronized void setPROTP(boolean protP) {this.fPROTP = protP;}
}