/* ============================================================================
 * TFTPProtocolStatics(TServerSideProtocol).java
 * ============================================================================
 * THIS SOFTWARE IS RELEASED UNDER THE FOLLOWING BINDING LEGAL AGREEMENT
 * (BSD-LICENSE).
 * ============================================================================
 *
 * Copyright (C) 1999-2001 by Paregos Mediadesign, Erik Eklund,
                 Johan "Spocke" Sorlin, http://sourceforge.net/projects/danoftp
 * Copyright (C) 2009
 * by J.H.H.C. van der Meijs (tcpconns@jhhcvandermeijs.nl)
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    * Neither the name of tcpconns nor the names of its copyright holders or
 *      its contributors may be used to endorse or promote products derived from
 *      this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 * ============================================================================
 * IF YOU DO NOT AGREE TO THE ABOVE TERMS AND CONDITIONS THEN DO *NOT* USE THIS
 * SOFTWARE AND *DO* DESTROY ALL COPIES OF THIS SOFTWARE THAT YOU HAVE.
 * ============================================================================
 *
 * TCPCONNS’ CURRENT CODE IS NOT KNOWN TO VIOLATE ANY COPYRIGHTS OR INFRINGE ON
 * ANY PATENTS. MOREOVER, IT ADHERES TO GENERAL PROGRAMMING RULES AND PRACTICES
 * AND IS OF SUCH A SIMPLISTIC NATURE THAT A SIMILAR PACKAGE COULD BE
 * CONSTRUCTED BY ANYONE. IMPORTANT(!): IF YOU FEEL THAT A PIECE OF CODE IN
 * TCPCONNS VIOLATES A COPYRIGHT OR INFRINGES ON A PATENT, PLEASE DO INFORM ONE
 * OF THE AUTHORS IMMEDIATELY. DO ALWAYS STATE EXACTLY WHAT CODE IS NOT ALLOWED
 * UNDER EXACTLY WHAT RULING, COPYRIGHT, OR PATENT; AND PLEASE DO ELABORATE ON
 * EXACTLY HOW THE CODE VIOLATES THE SPECIFIC COPYRIGHT OR INFRINGES ON
 * THE SPECIFIC PATENT.
 *
 * ============================================================================
 *
 * This software is Open Source Initiative approved Open Source Software.
 * Open Source Initiative Approved is a trademark of the Open Source Initiative.
 * http://www.opensource.org/
 *
 * ============================================================================
 */
package nl.jhhcvandermeijs.ftp.server;

import java.io.File;
import java.net.InetSocketAddress;
import java.net.ServerSocket;
import nl.jhhcvandermeijs.tcpconns.client.TTCPConnection;
import nl.jhhcvandermeijs.tcpconns.server.TServerSideProtocol;
import nl.jhhcvandermeijs.tcpconns.server.TTCPServer;
import nl.jhhcvandermeijs.tcpconns.utils.Utils;

/**
 * TFTPProtocolParentToBase implements some basic features of TFTPProtocol
 */
public class TFTPProtocolStatics extends TServerSideProtocol {

    /**
     * Constructor. Do not make objects of this class directly. Instances are created
     * by the TFTPServer class when a client connects through the method newSeverSideProtocol.
     *
     * @param aTTCPConnection handle to the client; the client can be accessed through this.conn()
     * @param aTTCPServer handle to the server
     */
    public TFTPProtocolStatics(TTCPConnection aTTCPConnection, TTCPServer aTTCPServer) {super(aTTCPConnection, aTTCPServer);}

	/**
     * A FTP Protocol server side reply code
     */
	public static final int FTP_DATA_TRANSFER_PORT_WILL_OPEN  = 150;
	/**
     * A FTP Protocol server side reply code
     */
	public static final int FTP_COMMAND_OK                    = 200;
	/**
     * A FTP Protocol server side reply code
     */
	public static final int FTP_FUNCTIONALITY_NOT_SUPPORTED   = 202;
    public static final int FTP_LIST_OF_FEATURES              = 211;
	/**
     * A FTP Protocol server side reply code
     */
	public static final int FTP_FILE_STATUS                   = 213;
	/**
     * A FTP Protocol server side reply code
     */
	public static final int FTP_SYSTEM_NAME                   = 215;
	/**
     * A FTP Protocol server side reply code
     */
	public static final int FTP_SERVICE_READY                 = 220;
	/**
     * A FTP Protocol server side reply code
     */
	public static final int FTP_CLOSING_DATA_TRANSFER_PORT    = 221;
	/**
     * A FTP Protocol server side reply code
     */
	public static final int FTP_CLOSING_DATA_TRANSFER_PORT_OK = 226;
	/**
     * A FTP Protocol server side reply code
     */
	public static final int FTP_PASSIVE_MODE                  = 227;
	/**
     * A FTP Protocol server side reply code
     */
	public static final int FTP_LOGIN_OK                      = 230;
    /**
     * A FTP Protocol server side reply code
     */
    public static final int FTP_TLS_START_OK                  = 234;
	/**
     * A FTP Protocol server side reply code
     */
	public static final int FTP_FILE_CMD_OK                   = 250;
	/**
     * A FTP Protocol server side reply code
     */
	public static final int FTP_PATH_OK                       = 257;
	/**
     * A FTP Protocol server side reply code
     */
	public static final int FTP_USER_OK                       = 331;
	/**
     * A FTP Protocol server side reply code
     */
	public static final int FTP_MORE_FILE_INFO                = 350;
	/**
     * A FTP Protocol server side reply code
     */
	public static final int FTP_NO_CONNECTION                 = 425;
	/**
     * A FTP Protocol server side reply code
     */
	public static final int FTP_FILE_ABORTED                  = 426;
	/**
     * A FTP Protocol server side reply code
     */
	public static final int FTP_SYNTAX_ERROR                  = 500;
	/**
     * A FTP Protocol server side reply code
     */
	public static final int FTP_BAD_ARGUMENT                  = 501;
	/**
     * A FTP Protocol server side reply code
     */
	public static final int FTP_NOT_IMPLEMENTED               = 504;
	/**
     * A FTP Protocol server side reply code
     */
	public static final int FTP_NOT_LOGGED_IN                 = 530;
	/**
     * A FTP Protocol server side reply code
     */
	public static final int FTP_ACCESS_DENIED                 = 550;

    public static final String EMPTY = "          ";

    public static final String ROOT_ELEMENT_NAME       = "TCPCONNS_FTPSERVER";
    public static final String ROOT_COMMENT            = "THIS IS THE .XML CONFIGURATION FILE FOR THE TCPCONNS FTPSERVER";
    /*===========================================================*/
    public static final String SERVER_SETTINGS = "SERVER_SETTINGS";
    /*===========================================================*/
            /*---------------------------------------------------*/
            public static final String LOG_SETTINGS = "LOG_SETTINGS";
            /*---------------------------------------------------*/
                public static final String LOG_DIRECTORY      = "LOG_DIRECTORY"     ;
                public static final String MAIN_LOG_FILE_NAME = "MAIN_LOG_FILE_NAME";
                public static final String LOG_EVENTS         = "LOG_EVENTS"        ;
                public static final String LOG_APPEND         = "LOG_APPEND"        ;

            public static final String MAX_CLIENTS                = "MAX_CLIENTS";
            public static final String BIND_IP_ADDRESS            = "BIND_IP_ADDRESS";
            public static final String SERVER_PORT                = "SERVER_PORT";
            public static final String EXTERNAL_IP_OF_THIS_SERVER = "EXTERNAL_IP_OF_THIS_SERVER";
            public static final String TRACK_CHANGES              = "TRACK_CHANGES";

            /*---------------------------------------------------*/
            public static final String TLS_SETTINGS = "TLS_SETTINGS";
            /*---------------------------------------------------*/
                public static final String ALLOW_TLS                                 = "ALLOW_TLS"                                ;
                public static final String IMPLICIT_TLS                              = "IMPLICIT_TLS"                             ;
                public static final String ENFORCE_EXPLICIT_TLS_WHEN_NOT_IMPLICIT    = "ENFORCE_EXPLICIT_TLS_WHEN_NOT_IMPLICIT"   ;
                public static final String ENFORCE_PROTP_WHEN_EXPLICIT               = "ENFORCE_PROTP_WHEN_EXPLICIT"              ;
                public static final String REQUIRE_CLIENT_AUTHENTICATION             = "REQUIRE_CLIENT_AUTHENTICATION"            ;
                public static final String PEM_ENCODED_SERVER_CERT_AND_KEY_FILE_NAME = "PEM_ENCODED_SERVER_CERT_AND_KEY_FILE_NAME";
                public static final String KEY_PASSPHRASE_ATTRIBUTE                  = "KEY_PASSPHRASE"                           ;
                public static final String TRUSTEDCA_CERTS_DIRECTORY                 = "TRUSTEDCA_CERTS_DIRECTORY"                ;
    /*=========================================*/
    public static final String GROUPS = "GROUPS";
    /*=========================================*/
        public static final String GROUP = "GROUP";
            public static final String GROUP_NAME_ATTRIBUTE    = "GROUP_NAME"   ;
            public static final String GROUP_OPENINGDIR_ATTRIBUTE = "GROUP_OPENING_DIRECTORY";
            public static final String FOLDERS                 = "FOLDERS"      ;
                /*-----------------------------------------*/
                public static final String FOLDER = "FOLDER";
                /*-----------------------------------------*/
    public static final String FOLDER_NAME_ATTRIBUTE = "FOLDER_NAME";

    public static final String IS_ALLOWED_TO_SEE_FILES = "IS_ALLOWED_TO_SEE_FILES";
    public static final String IS_ALLOWED_TO_READ_FILES = "IS_ALLOWED_TO_READ_FILES";
    public static final String IS_ALLOWED_TO_RENAME_FILES = "IS_ALLOWED_TO_RENAME_FILES";
    public static final String IS_ALLOWED_TO_APPEND_TO_FILES = "IS_ALLOWED_TO_APPEND_TO_FILES";
    public static final String IS_ALLOWED_TO_CREATE_NEW_FILES = "IS_ALLOWED_TO_CREATE_NEW_FILES";
    public static final String IS_ALLOWED_TO_OVERWRITE_FILES = "IS_ALLOWED_TO_OVERWRITE_FILES";
    public static final String IS_ALLOWED_TO_DELETE_FILES = "IS_ALLOWED_TO_DELETE_FILES";

    public static final String IS_ALLOWED_TO_SEE_SUB_DIRECTORIES = "IS_ALLOWED_TO_SEE_SUB_DIRECTORIES";
    public static final String IS_ALLOWED_TO_TRAVERSE_SUB_DIRECTORIES = "IS_ALLOWED_TO_TRAVERSE_SUB_DIRECTORIES";
    public static final String IS_ALLOWED_TO_RENAME_SUB_DIRECTORIES = "IS_ALLOWED_TO_RENAME_SUB_DIRECTORIES";
    public static final String IS_ALLOWED_TO_MAKE_SUB_DIRECTORIES = "IS_ALLOWED_TO_MAKE_SUB_DIRECTORIES";
    public static final String IS_ALLOWED_TO_REMOVE_SUB_DIRECTORIES = "IS_ALLOWED_TO_REMOVE_SUB_DIRECTORIES";

    public static final String IS_ALLOWED_TO_SEE_HIDDEN_FILES = "IS_ALLOWED_TO_SEE_HIDDEN_FILES";
    public static final String IS_ALLOWED_TO_READ_HIDDEN_FILES = "IS_ALLOWED_TO_READ_HIDDEN_FILES";
    public static final String IS_ALLOWED_TO_RENAME_HIDDEN_FILES = "IS_ALLOWED_TO_RENAME_HIDDEN_FILES";
    public static final String IS_ALLOWED_TO_APPEND_TO_HIDDEN_FILES = "IS_ALLOWED_TO_APPEND_TO_HIDDEN_FILES";
    public static final String IS_ALLOWED_TO_CREATE_NEW_HIDDEN_FILES = "IS_ALLOWED_TO_CREATE_NEW_HIDDEN_FILES";
    public static final String IS_ALLOWED_TO_OVERWRITE_HIDDEN_FILES = "IS_ALLOWED_TO_OVERWRITE_HIDDEN_FILES";
    public static final String IS_ALLOWED_TO_DELETE_HIDDEN_FILES = "IS_ALLOWED_TO_DELETE_HIDDEN_FILES";

    public static final String IS_ALLOWED_TO_SEE_HIDDEN_SUB_DIRECTORIES = "IS_ALLOWED_TO_SEE_HIDDEN_SUB_DIRECTORIES";
    public static final String IS_ALLOWED_TO_TRAVERSE_HIDDEN_SUB_DIRECTORIES = "IS_ALLOWED_TO_TRAVERSE_HIDDEN_SUB_DIRECTORIES";
    public static final String IS_ALLOWED_TO_RENAME_HIDDEN_SUB_DIRECTORIES = "IS_ALLOWED_TO_RENAME_HIDDEN_SUB_DIRECTORIES";
    public static final String IS_ALLOWED_TO_MAKE_HIDDEN_SUB_DIRECTORIES = "IS_ALLOWED_TO_MAKE_HIDDEN_SUB_DIRECTORIES";
    public static final String IS_ALLOWED_TO_REMOVE_HIDDEN_SUB_DIRECTORIES = "IS_ALLOWED_TO_REMOVE_HIDDEN_SUB_DIRECTORIES";

    /*================================================*/
    public static final String USERS = "USERS";
    /*================================================*/
        public static final String USER = "USER";
            public static final String USER_NAME_ATTRIBUTE                   = "USER_NAME"                       ;
            public static final String USER_PASSPHRASE_ATTRIBUTE             = "PASSPHRASE"                      ;
            public static final String USER_REQUIRE_PASSPHRASE_TO_BE_CORRECT = "REQUIRE_PASSPHRASE_TO_BE_CORRECT";
            public static final String USER_ACCOUNT_IS_ACTIVE                = "ACCOUNT_IS_ACTIVE"               ;
            public static final String USER_GROUP_MEMBERSHIP                 = "GROUP"                           ;

    public static final String ANONYMOUS = "anonymous";

    /**
     * A method that is used to transform relative paths to true paths. The path that is constructed
     * must be a subpath to the top level basePath. If it is not access is denied.
     *
     * @param path a path to be validated
     * @param currentPath if the parameter path is relative, it is relative to currentPath
     * @return a valid path, or null if the path is not valid
     */
    public static String getAbsolutePath(String path, String currentPath) {
        if ((path == null) || (currentPath == null)) {return null;}
        path = path.trim(); if (path.equals("")) {return null;}

        /*
         * convert currentPath into its canonical form
         *
         */
        currentPath = Utils.validateDirectory(currentPath);

        /*
         * introduce a variable named absolutePath:
         *      if path is NOT relative   THEN    absolutePath =               path;
         *      if path IS relative       THEN    absolutePath = currentPath + path;
         */
        String absolutePath = null;
        if (Utils.isAbsolutePath(path)) {

            absolutePath =               path;

        } else {

            absolutePath = currentPath + path;

        }

        /*
         * convert the absolutepath into a canonicalpath: ie get rid of /../../..
         *
         */
        absolutePath = Utils.validateDirectory(absolutePath);
        if (absolutePath == null) {return null;}

        //done, all is OK
        return absolutePath;
    }

    /**
     * A method to convert a server IP address and port number into a FTP formatted
     * string; necessary for transfer to the client. Only needed when in passive mode
     * (which is like always).
     * @param IP an IP string
     * @param port a port number
     * @return a string like: xxx,xxx,xxx,xxx,xxx,xxx (the last two xxx denote the port number)
     */
    public static String getFTPIPPortString(String IP, int port) {
        int i,posb, posa=0;
		int[] numbers = new int[6];

		for (i=0;i<3;i++) {
			posb = IP.indexOf(".", posa);
			numbers[i]= Integer.parseInt(IP.substring(posa,posb));
			posa=posb+1;
		}
		numbers[i++]=Integer.parseInt(IP.substring(posa,IP.length()));

		numbers[i++]=(port>>8);
		numbers[i]=port & 0xFF;

		return ""+numbers[0]+","+numbers[1]+","+numbers[2]+","+numbers[3]+","+numbers[4]+","+numbers[5];
	}

    /*
     * apparently windows requires a sort of "rotating" port number, so that there is more
     * time to "cool down" the last one used (at least on my windows vista test machine)
     */
    private static int fPort = 1025-2;

    /**
     * method to obtain a "rotating" server socket, necessary for windows
     * @param IP e.g. 0.0.0.0
     * @return a ServerSocket; use ServerSocket.getLocalPort() to obtain the port number
     */
    public synchronized static ServerSocket getServerSocket(String IP) {
            ServerSocket aServerSocket = null;

            int i = 1025-2; int n = 65535;
            while (i <= (n-1)) {
                i++;
                fPort++;
                if (fPort >= 65534) {fPort = 1025-2;}

                try {
                    aServerSocket = new ServerSocket();
                    aServerSocket.setSoTimeout(10*60*1000);
                    aServerSocket.setReuseAddress(false);
                    aServerSocket.bind(new InetSocketAddress(IP, fPort));
                    break; //no bind exception so escape the while loop
                } catch (Exception e) {
                    continue; //port in use, try next
                }
            }
      		if (i >= 65534) {return null;}

            return aServerSocket;
    }
}