/* ============================================================================
 * TFTPProtocolUserManagement(TFTPProtocolSettingsAndState).java
 * ============================================================================
 * THIS SOFTWARE IS RELEASED UNDER THE FOLLOWING BINDING LEGAL AGREEMENT
 * (BSD-LICENSE).
 * ============================================================================
 *
 * Copyright (C) 1999-2001 by Paregos Mediadesign, Erik Eklund,
                 Johan "Spocke" Sorlin, http://sourceforge.net/projects/danoftp
 * Copyright (C) 2009
 * by J.H.H.C. van der Meijs (tcpconns@jhhcvandermeijs.nl)
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    * Neither the name of tcpconns nor the names of its copyright holders or
 *      its contributors may be used to endorse or promote products derived from
 *      this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 * ============================================================================
 * IF YOU DO NOT AGREE TO THE ABOVE TERMS AND CONDITIONS THEN DO *NOT* USE THIS
 * SOFTWARE AND *DO* DESTROY ALL COPIES OF THIS SOFTWARE THAT YOU HAVE.
 * ============================================================================
 *
 * TCPCONNS’ CURRENT CODE IS NOT KNOWN TO VIOLATE ANY COPYRIGHTS OR INFRINGE ON
 * ANY PATENTS. MOREOVER, IT ADHERES TO GENERAL PROGRAMMING RULES AND PRACTICES
 * AND IS OF SUCH A SIMPLISTIC NATURE THAT A SIMILAR PACKAGE COULD BE
 * CONSTRUCTED BY ANYONE. IMPORTANT(!): IF YOU FEEL THAT A PIECE OF CODE IN
 * TCPCONNS VIOLATES A COPYRIGHT OR INFRINGES ON A PATENT, PLEASE DO INFORM ONE
 * OF THE AUTHORS IMMEDIATELY. DO ALWAYS STATE EXACTLY WHAT CODE IS NOT ALLOWED
 * UNDER EXACTLY WHAT RULING, COPYRIGHT, OR PATENT; AND PLEASE DO ELABORATE ON
 * EXACTLY HOW THE CODE VIOLATES THE SPECIFIC COPYRIGHT OR INFRINGES ON
 * THE SPECIFIC PATENT.
 *
 * ============================================================================
 *
 * This software is Open Source Initiative approved Open Source Software.
 * Open Source Initiative Approved is a trademark of the Open Source Initiative.
 * http://www.opensource.org/
 *
 * ============================================================================
 */
package nl.jhhcvandermeijs.ftp.server;

import java.io.File;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import nl.jhhcvandermeijs.tcpconns.client.TTCPConnection;
import nl.jhhcvandermeijs.tcpconns.server.TTCPServer;
import nl.jhhcvandermeijs.tcpconns.utils.Utils;


/**
 * TFTPProtocolParentToBase implements some basic features of TFTPProtocol
 */
public class TFTPProtocolUserManagement extends TFTPProtocolSettingsAndState {

    /**
     * Constructor. Do not make objects of this class directly. Instances are created
     * by the TFTPServer class when a client connects through the method newSeverSideProtocol.
     *
     * @param aTTCPConnection handle to the client; the client can be accessed through this.conn()
     * @param aTTCPServer handle to the server
     */
    public TFTPProtocolUserManagement(TTCPConnection aTTCPConnection, TTCPServer aTTCPServer) {super(aTTCPConnection, aTTCPServer);}

    private String fUserName = EMPTY;
    public synchronized String getUserName() {return this.fUserName;}
    public synchronized void setUserName(String userName) {
        this.fLoggedIn = false;
        this.setUser(null);
        this.setGroup(null);
        this.setFolders(null);

        this.fPassphrase = null;

        this.fUserName = userName;
    }

    private String fPassphrase = EMPTY;
    public synchronized String getPassphrase() {return this.fPassphrase;}
    public synchronized void setPassphrase(String passphrase) {
        this.fLoggedIn = false;
        this.setUser(null);
        this.setGroup(null);
        this.setFolders(null);

        this.fPassphrase = passphrase;
    }

    private String fCurrentDir = EMPTY;
    public synchronized String getCurrentDir() {return this.fCurrentDir;}
    public synchronized void setCurrentDir(String currentDir) {this.fCurrentDir = currentDir;}

    private boolean fLoggedIn = false;
    public synchronized boolean isLoggedIn() {return this.fLoggedIn;}
    public synchronized boolean logIn() {
        this.fLoggedIn = false;
        this.setUser(this.getServer().data().getUser(this.getUserName()));
        this.setGroup(this.getServer().data().getGroup(this.getUser().getGroupName()));
        if ((this.getUser() != null) || (this.getGroup() != null)) {
            if (
                    (this.getUser().getRequirePassphraseToBeCorrect() == false)
                ||
                    ((this.getUser().getRequirePassphraseToBeCorrect() == true) && (this.getPassphrase().equals(this.getUser().getPassphrase())))

                ) {

                    if (this.getUser().getAccountIsActive()) {
                        this.fLoggedIn = true;
                        this.setCurrentDir(this.getGroup().getOpeningDirectory());
                        this.setFolders(this.getGroup().getFoldersList());
                        return this.fLoggedIn;
                    }

            }
        }
        this.setUser(null);
        this.setGroup(null);
        this.setFolders(null);
        return this.fLoggedIn;
    }

    public synchronized TFTPServer getServer() {return ((TFTPServer)(this.fTTCPServer));}

    private TFTPProtocol.FTPProtocolData.User fUser = null;
    public synchronized TFTPProtocol.FTPProtocolData.User getUser() {return this.fUser;}
    public synchronized void setUser(TFTPProtocol.FTPProtocolData.User user) {this.fUser = user;}

    private TFTPProtocol.FTPProtocolData.Group fGroup = null;
    public synchronized TFTPProtocol.FTPProtocolData.Group getGroup() {return this.fGroup;}
    public synchronized void setGroup(TFTPProtocol.FTPProtocolData.Group group) {this.fGroup = group;}

    private List<FTPProtocolData.Folder> fFolders = Collections.synchronizedList(new LinkedList<FTPProtocolData.Folder>());
    public synchronized List<FTPProtocolData.Folder> getFolders() {return this.fFolders;}
    public synchronized void setFolders(List<FTPProtocolData.Folder> folders) {this.fFolders = folders;}

    public boolean isHidden(String path, FTPProtocolData.Folder folder) {
        path = getAbsolutePath(path, this.fCurrentDir);       
        if (folder != null) {
            try {
                byte separator = ((new String(File.separator)).getBytes("UTF-8"))[0];
                byte dot = (new String(".").getBytes("UTF-8"))[0];

                byte[] searchIn = path.substring(Utils.validateDirectory(folder.getFolderName()).length()).getBytes("UTF-8");

                if (searchIn[0] == dot) {return true;}

                int i = 1-1; int n = searchIn.length;
                while (i <= (n-1)) {
                    if (searchIn[i] == separator) {
                        if (i < (n-1)) {
                            if (searchIn[i+1] == dot) {
                                return true;
                            } else {
                                i++;
                            }
                        }
                    }
                    i++;
                }
            } catch (Exception e) {}
        }
        return false;
    }

    public FTPProtocolData.Folder getFolder(String folderName) {
        try {
            //if no absolute path can be constructed, then there is no point in testing further
            if (folderName == null) {return null;}

            //get absolute path of folderName
            folderName = getAbsolutePath(folderName, this.fCurrentDir);

            //if no absolute path can be constructed, then there is no point in testing further
            if (folderName == null) {return null;}

            //initialization
            FTPProtocolData.Folder folder = null;
            List<FTPProtocolData.Folder> folders = new LinkedList<FTPProtocolData.Folder>();

            //if there are folders, then examine them!
            if (this.getFolders().size() > 0) {
                FTPProtocolData.Folder[] folderArray = new FTPProtocolData.Folder[this.getFolders().size()];
                folderArray = this.getFolders().toArray(folderArray);
                                                          
                int i = 1-1; int n = folderArray.length;
                while (i <= (n-1)) {
                    if (folderName.startsWith(folderArray[i].getFolderName())) {
                        folders.add(folderArray[i]); //select folders that are parent folder to folderName
                    }
                    i++;
                }
                if (folders.size() > 0) {
                    //select the most appropriate one
                    folderArray = new FTPProtocolData.Folder[folders.size()];
                    folderArray = folders.toArray(folderArray);
                    folder = folderArray[0]; //found at least one entry
                    i = 1-1; n = folderArray.length;
                    while (i <= (n-1)) {
                        if (folderArray[i].getFolderName().length() > folder.getFolderName().length()) {
                            folder = folderArray[i]; //folder now contains the most near parent folder to folderName
                        }
                        i++;
                    }
               }
            }

            /*
             * check if traversing subdirectories is allowed
             */
            if ((new File(folderName)).isFile()) {
                folderName = Utils.validateDirectory(((new File(folderName)).getParent()));
                if (folderName == null) {return null;}
            }
            if (Utils.validateDirectory(folder.getFolderName()).length() != folderName.length()) {
                if (this.isHidden(folderName, folder) == true) {
                    if (folder.getIsAllowedToTraverseHiddenSubDirectories()) {
                        return folder;
                    } else {
                        return null;
                    }
                } else if (this.isHidden(folderName, folder) == false) {
                    if (folder.getIsAllowedToTraverseSubDirectories()) {
                        return folder;
                    } else {
                        return null;
                    }
                }
            }
            return folder;
        } catch (Exception e) {return null;}
    }

}