/* ============================================================================
 * FTPServer.java
 * ============================================================================
 * THIS SOFTWARE IS RELEASED UNDER THE FOLLOWING BINDING LEGAL AGREEMENT
 * (BSD-LICENSE).
 * ============================================================================
 *
 * Copyright (C) 2009
 * by J.H.H.C. van der Meijs (tcpconns@jhhcvandermeijs.nl)
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    * Neither the name of tcpconns nor the names of its copyright holders or
 *      its contributors may be used to endorse or promote products derived from
 *      this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 * ============================================================================
 * IF YOU DO NOT AGREE TO THE ABOVE TERMS AND CONDITIONS THEN DO *NOT* USE THIS
 * SOFTWARE AND *DO* DESTROY ALL COPIES OF THIS SOFTWARE THAT YOU HAVE.
 * ============================================================================
 *
 * TCPCONNS’ CURRENT CODE IS NOT KNOWN TO VIOLATE ANY COPYRIGHTS OR INFRINGE ON
 * ANY PATENTS. MOREOVER, IT ADHERES TO GENERAL PROGRAMMING RULES AND PRACTICES
 * AND IS OF SUCH A SIMPLISTIC NATURE THAT A SIMILAR PACKAGE COULD BE
 * CONSTRUCTED BY ANYONE. IMPORTANT(!): IF YOU FEEL THAT A PIECE OF CODE IN
 * TCPCONNS VIOLATES A COPYRIGHT OR INFRINGES ON A PATENT, PLEASE DO INFORM ONE
 * OF THE AUTHORS IMMEDIATELY. DO ALWAYS STATE EXACTLY WHAT CODE IS NOT ALLOWED
 * UNDER EXACTLY WHAT RULING, COPYRIGHT, OR PATENT; AND PLEASE DO ELABORATE ON
 * EXACTLY HOW THE CODE VIOLATES THE SPECIFIC COPYRIGHT OR INFRINGES ON
 * THE SPECIFIC PATENT.
 *
 * ============================================================================
 *
 * This software is Open Source Initiative approved Open Source Software.
 * Open Source Initiative Approved is a trademark of the Open Source Initiative.
 * http://www.opensource.org/
 *
 * ============================================================================
 */
package nl.jhhcvandermeijs.ftp.ftpserver;

import java.io.File;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.UIManager;
import nl.jhhcvandermeijs.ftp.server.StartFTPServer;
import nl.jhhcvandermeijs.ftp.server.TFTPProtocol;
import nl.jhhcvandermeijs.ftp.server.TFTPProtocolSettingsAndState.FTPProtocolData;
import nl.jhhcvandermeijs.ftp.server.TFTPServer;
import nl.jhhcvandermeijs.tcpconns.utils.Utils;

/**
 *
 * @author username
 */
public class FTPServer extends javax.swing.JFrame {
    private boolean fUpdating = false;
    public void startUpdate() {
        this.fUpdating = true;
    }
    public void endUpdate() {
        this.fUpdating = false;
    }

    public void thereAreChanges() {
        if (this.fUpdating == false) {
            this.btnSaveChanges.setEnabled(true);
            this.btnReload.setEnabled(true);
        }
    }

    public FTPProtocolData data = new FTPProtocolData();

    public void clearUserRights() {
        this.cbIsAllowedToSeeFiles.setSelected(false);
        this.cbIsAllowedToReadFiles.setSelected(false);
        this.cbIsAllowedToRenameFiles.setSelected(false);
        this.cbIsAllowedToAppendToFiles.setSelected(false);
        this.cbIsAllowedToCreateNewFiles.setSelected(false);
        this.cbIsAllowedToOverwriteFiles.setSelected(false);
        this.cbIsAllowedToDeleteFiles.setSelected(false);

        this.cbIsAllowedToSeeSubDirectories.setSelected(false);
        this.cbIsAllowedToTraverseSubDirectories.setSelected(false);
        this.cbIsAllowedToRenameSubDirectories.setSelected(false);
        this.cbIsAllowedToMakeSubDirectories.setSelected(false);
        this.cbIsAllowedToRemoveSubDirectories.setSelected(false);

        this.cbIsAllowedToSeeHiddenFiles.setSelected(false);
        this.cbIsAllowedToReadHiddenFiles.setSelected(false);
        this.cbIsAllowedToRenameHiddenFiles.setSelected(false);
        this.cbIsAllowedToAppendToHiddenFiles.setSelected(false);
        this.cbIsAllowedToCreateNewHiddenFiles.setSelected(false);
        this.cbIsAllowedToOverwriteHiddenFiles.setSelected(false);
        this.cbIsAllowedToDeleteHiddenFiles.setSelected(false);

        this.cbIsAllowedToSeeHiddenSubDirectories.setSelected(false);
        this.cbIsAllowedToTraverseHiddenSubDirectories.setSelected(false);
        this.cbIsAllowedToRenameHiddenSubDirectories.setSelected(false);
        this.cbIsAllowedToMakeHiddenSubDirectories.setSelected(false);
        this.cbIsAllowedToRemoveHiddenSubDirectories.setSelected(false);
    }

    public void clearGroups() {        
        this.comboGroups.removeAllItems();
        this.comboGroups.addItem(TFTPProtocol.EMPTY);
        this.comboGroups.setSelectedItem(TFTPProtocol.EMPTY);
        
        this.comboGroupMembership.removeAllItems();
        this.comboGroupMembership.addItem(TFTPProtocol.EMPTY);
        this.comboGroupMembership.setSelectedItem(TFTPProtocol.EMPTY);
    }

    public void populateGroups() {
        this.clearGroups();

        if (this.data.getGroupsList().size() > 0) {
            int i = 1-1; int n = this.data.getGroupsList().size();
            while (i <= (n-1)) {
                this.comboGroups.addItem(this.data.getGroupsList().get(i).getGroupName());
                this.comboGroupMembership.addItem(this.data.getGroupsList().get(i).getGroupName());
                i++;
            }
        }
    }

    public void onGroupSelected() {
        this.startUpdate();
        String groupName = ((String)(this.comboGroups.getSelectedItem()));
        if (data.groupExists(groupName)) {
            this.populateFolders(groupName);
            FTPProtocolData.Group group = data.getGroup(groupName);
            int i = 1-1; int n = this.comboOpeningFolder.getItemCount();
            while (i <= (n-1)) {
                if (group.getOpeningDirectory().equals((String)(this.comboOpeningFolder.getItemAt(i)))) {
                    this.comboOpeningFolder.setSelectedIndex(i);
                    int j = 1-1; int m = this.comboFolders.getItemCount();
                    while (j <= (m-1)) {
                        if (group.getOpeningDirectory().equals((String)(this.comboFolders.getItemAt(j)))) {
                            this.comboFolders.setSelectedIndex(j);
                            if (this.data.folderExists(groupName, group.getOpeningDirectory())) {
                                this.setUserRights(data.getFolder(groupName, group.getOpeningDirectory()));
                            }
                            break;
                        }
                        j++;
                    }
                    break;
                }
                i++;
            }
        } else {
            this.clearFolders();
        }
        this.endUpdate();
    }

    public void onFolderSelected() {
        this.startUpdate();
        String groupName = ((String)(this.comboGroups.getSelectedItem()));
        String folderName = ((String)(this.comboFolders.getSelectedItem()));
        if (data.folderExists(groupName, folderName)) {
            FTPProtocolData.Folder folder = this.data.getFolder(groupName, folderName);
            this.setUserRights(folder);
        } else {
            this.clearUserRights();
        }
        this.endUpdate();
    }

    public void clearUsers() {
        this.comboUsers.removeAllItems();
        this.comboUsers.addItem(TFTPProtocol.EMPTY);
        this.comboUsers.setSelectedItem(TFTPProtocol.EMPTY);
    }
   
    public void populateUsers() {
        this.clearUsers();

        if (this.data.getUsersList().size() > 0) {
            int i = 1-1; int n = this.data.getUsersList().size();
            while (i <= (n-1)) {
                this.comboUsers.addItem(this.data.getUsersList().get(i).getUserName());
                i++;
            }
        }
    }

    public void clearFolders() {
        this.comboFolders.removeAllItems();
        this.comboFolders.addItem(TFTPProtocol.EMPTY);
        this.comboFolders.setSelectedItem(TFTPProtocol.EMPTY);
        
        this.comboOpeningFolder.removeAllItems();
        this.comboOpeningFolder.addItem(TFTPProtocol.EMPTY);
        this.comboOpeningFolder.setSelectedItem(TFTPProtocol.EMPTY);

        this.clearUserRights();
    }

    public void populateFolders(String groupName) {
        this.clearFolders();

        if (this.data.groupExists(groupName)) {
            FTPProtocolData.Group group = this.data.getGroup(groupName);
            if (group.getFoldersList().size() > 0) {
                int j = 1-1; int m = group.getFoldersList().size();
                while (j <= (m-1)) {
                    this.comboFolders.addItem(group.getFoldersList().get(j).getFolderName());
                    this.comboOpeningFolder.addItem(group.getFoldersList().get(j).getFolderName());
                j++;
                }
            }
        }
    }

    public void clearAndPopulateTLSLevels() {
        this.comboTLSLevel.removeAllItems();
        this.comboTLSLevel.removeAllItems();
        this.comboTLSLevel.addItem("Disallow the use of SSL/TLS"); //0
        this.comboTLSLevel.addItem("Explicit (connection OPTIONAL TLS, data channel OPTIONAL TLS)"); //1
        this.comboTLSLevel.addItem("Explicit (connection ENFORCE TLS, data channel OPTIONAL TLS)"); //2
        this.comboTLSLevel.addItem("Explicit (connection ENFORCE TLS, data channel ENFORCE TLS"); //3
        this.comboTLSLevel.addItem("Implicit (connection MANDATORY TLS, data channel MANDATORY TLS"); //4
    }

    public void setAccountIsActive() {
        String userName = ((String)(this.comboUsers.getSelectedItem()));
        if (userName != null) {
            if (this.data.userExists(userName)) {
                this.cbAccountIsActive.setSelected(this.data.getUser(userName).getAccountIsActive());
            } else {
                this.cbAccountIsActive.setSelected(false);
            }
        }
    }

    public void setRequirePassphraseToBeCorrect() {
        String userName = ((String)(this.comboUsers.getSelectedItem()));
        if (userName != null) {
            if (this.data.userExists(userName)) {
                this.cbRequirePassphraseToBeCorrect.setSelected(this.data.getUser(userName).getRequirePassphraseToBeCorrect());
            } else {
                this.cbRequirePassphraseToBeCorrect.setSelected(false);
            }

        }
    }
    
    public void setUserRights(FTPProtocolData.Folder folder) {
        if (folder == null) {this.clearUserRights(); return;}

        this.cbIsAllowedToSeeFiles.setSelected(folder.getIsAllowedToSeeFiles());
        this.cbIsAllowedToReadFiles.setSelected(folder.getIsAllowedToReadFiles());
        this.cbIsAllowedToRenameFiles.setSelected(folder.getIsAllowedToRenameFiles());
        this.cbIsAllowedToAppendToFiles.setSelected(folder.getIsAllowedToAppendToFiles());
        this.cbIsAllowedToCreateNewFiles.setSelected(folder.getIsAllowedToCreateNewFiles());
        this.cbIsAllowedToOverwriteFiles.setSelected(folder.getIsAllowedToOverwriteFiles());
        this.cbIsAllowedToDeleteFiles.setSelected(folder.getIsAllowedToDeleteFiles());

        this.cbIsAllowedToSeeSubDirectories.setSelected(folder.getIsAllowedToSeeSubDirectories());
        this.cbIsAllowedToTraverseSubDirectories.setSelected(folder.getIsAllowedToTraverseSubDirectories());
        this.cbIsAllowedToRenameSubDirectories.setSelected(folder.getIsAllowedToRenameSubDirectories());
        this.cbIsAllowedToMakeSubDirectories.setSelected(folder.getIsAllowedToMakeSubDirectories());
        this.cbIsAllowedToRemoveSubDirectories.setSelected(folder.getIsAllowedToRemoveSubDirectories());

        this.cbIsAllowedToSeeHiddenFiles.setSelected(folder.getIsAllowedToSeeHiddenFiles());
        this.cbIsAllowedToReadHiddenFiles.setSelected(folder.getIsAllowedToReadHiddenFiles());
        this.cbIsAllowedToRenameHiddenFiles.setSelected(folder.getIsAllowedToRenameHiddenFiles());
        this.cbIsAllowedToAppendToHiddenFiles.setSelected(folder.getIsAllowedToAppendToHiddenFiles());
        this.cbIsAllowedToCreateNewHiddenFiles.setSelected(folder.getIsAllowedToCreateNewHiddenFiles());
        this.cbIsAllowedToOverwriteHiddenFiles.setSelected(folder.getIsAllowedToOverwriteHiddenFiles());
        this.cbIsAllowedToDeleteHiddenFiles.setSelected(folder.getIsAllowedToDeleteHiddenFiles());

        this.cbIsAllowedToSeeHiddenSubDirectories.setSelected(folder.getIsAllowedToSeeHiddenSubDirectories());
        this.cbIsAllowedToTraverseHiddenSubDirectories.setSelected(folder.getIsAllowedToTraverseHiddenSubDirectories());
        this.cbIsAllowedToRenameHiddenSubDirectories.setSelected(folder.getIsAllowedToRenameHiddenSubDirectories());
        this.cbIsAllowedToMakeHiddenSubDirectories.setSelected(folder.getIsAllowedToMakeHiddenSubDirectories());
        this.cbIsAllowedToRemoveHiddenSubDirectories.setSelected(folder.getIsAllowedToRemoveHiddenSubDirectories());
    }  

    public void loadSettings() {
        if ((new File(TFTPServer.getConfigurationFileName())).exists() == false) {
            StartFTPServer.writeSampleCerts();
            StartFTPServer.createEmptySettingsFile();
        }
        
        data.loadFromXML(TFTPServer.getConfigurationFileName());

        //NOW GET THE SETTINGS INTO THE APPLICATION
        this.startUpdate();
        this.cbLogEvents.setSelected(data.getLogEvents());
        this.cbAppendLogs.setSelected(data.getLogAppend());
        this.edtExternalIP.setText(data.getExternalIPOfThisServer());
        this.edtPort.setValue(data.getServerPort());
        this.cbTrackChanges.setSelected(data.getTrackChanges());

        this.clearAndPopulateTLSLevels();
        if (data.getAllowTLS()) {
            if (data.getImplicitTLS()) {
                this.comboTLSLevel.setSelectedIndex(4); //implicit TLS
            } else if (data.getEnforceExplicitTLSWhenNotImplicit()) {
                if (data.getEnforcePROTPWhenExplicit()) {
                    this.comboTLSLevel.setSelectedIndex(3); //enforce auth and enforce protp
                } else {
                    this.comboTLSLevel.setSelectedIndex(2);
                }
            } else {
                this.comboTLSLevel.setSelectedIndex(1); //do not enforce anything
            }
        } else {
            this.comboTLSLevel.setSelectedIndex(0);
        }

        this.edtServerCertAndKey.setText(data.getPEMEncodedServerCertAndKeyFileName());
        this.edtKeyPassphrase.setText(data.getKeyPassphrase());
        this.cbRequireClientAuthentication.setSelected(data.getRequireClientAuthentication());
        this.edtTrustedCAsFolder.setText(data.getTrustedCACertsDirectory());

        this.populateGroups();
        this.populateUsers();
        this.clearFolders();

        this.endUpdate();

        if (this.comboUsers.getItemCount() > 1) {
            this.comboUsers.setSelectedIndex(1);
        }
        this.onUserSelected();
    }

    public void saveSettings() {
        data.saveToXML(TFTPServer.getConfigurationFileName());
    }

    /** Creates new form FTPServer */
    public FTPServer() {
        initComponents();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        TabbedPane = new javax.swing.JTabbedPane();
        tabGeneral = new javax.swing.JPanel();
        panelGeneralSettings = new javax.swing.JPanel();
        lblExternalIP = new javax.swing.JLabel();
        lblPort = new javax.swing.JLabel();
        cbLogEvents = new javax.swing.JCheckBox();
        cbAppendLogs = new javax.swing.JCheckBox();
        edtExternalIP = new javax.swing.JTextField();
        edtPort = new javax.swing.JSpinner();
        cbTrackChanges = new javax.swing.JCheckBox();
        tabGroupsUsersAndUserRights = new javax.swing.JPanel();
        panelUsersAndGroupMembership = new javax.swing.JPanel();
        lblUser = new javax.swing.JLabel();
        comboUsers = new javax.swing.JComboBox();
        lblMemberOfGroup = new javax.swing.JLabel();
        btnAddUser = new javax.swing.JButton();
        btnRenameUser = new javax.swing.JButton();
        btnDeleteUser = new javax.swing.JButton();
        btnResetPassphrase = new javax.swing.JButton();
        cbAccountIsActive = new javax.swing.JCheckBox();
        comboGroupMembership = new javax.swing.JComboBox();
        cbRequirePassphraseToBeCorrect = new javax.swing.JCheckBox();
        panelGroupsAndUserRights = new javax.swing.JPanel();
        lblGroups = new javax.swing.JLabel();
        comboGroups = new javax.swing.JComboBox();
        btnAddGroup = new javax.swing.JButton();
        btnRenameGroup = new javax.swing.JButton();
        btnDeleteGroup = new javax.swing.JButton();
        lblFolders = new javax.swing.JLabel();
        comboFolders = new javax.swing.JComboBox();
        btnAddFolder = new javax.swing.JButton();
        btnDeleteFolder = new javax.swing.JButton();
        lblOpeningFolder = new javax.swing.JLabel();
        comboOpeningFolder = new javax.swing.JComboBox();
        lblFiles = new javax.swing.JLabel();
        lblSubDirectories = new javax.swing.JLabel();
        lblHiddenFiles = new javax.swing.JLabel();
        lblHiddenSubDirectories = new javax.swing.JLabel();
        cbIsAllowedToSeeFiles = new javax.swing.JCheckBox();
        cbIsAllowedToReadFiles = new javax.swing.JCheckBox();
        cbIsAllowedToRenameFiles = new javax.swing.JCheckBox();
        cbIsAllowedToAppendToFiles = new javax.swing.JCheckBox();
        cbIsAllowedToCreateNewFiles = new javax.swing.JCheckBox();
        cbIsAllowedToOverwriteFiles = new javax.swing.JCheckBox();
        cbIsAllowedToDeleteFiles = new javax.swing.JCheckBox();
        cbIsAllowedToSeeSubDirectories = new javax.swing.JCheckBox();
        cbIsAllowedToRenameSubDirectories = new javax.swing.JCheckBox();
        cbIsAllowedToSeeHiddenFiles = new javax.swing.JCheckBox();
        cbIsAllowedToSeeHiddenSubDirectories = new javax.swing.JCheckBox();
        cbIsAllowedToReadHiddenFiles = new javax.swing.JCheckBox();
        cbIsAllowedToRenameHiddenSubDirectories = new javax.swing.JCheckBox();
        cbIsAllowedToRenameHiddenFiles = new javax.swing.JCheckBox();
        cbIsAllowedToAppendToHiddenFiles = new javax.swing.JCheckBox();
        cbIsAllowedToMakeSubDirectories = new javax.swing.JCheckBox();
        cbIsAllowedToCreateNewHiddenFiles = new javax.swing.JCheckBox();
        cbIsAllowedToMakeHiddenSubDirectories = new javax.swing.JCheckBox();
        cbIsAllowedToOverwriteHiddenFiles = new javax.swing.JCheckBox();
        cbIsAllowedToRemoveSubDirectories = new javax.swing.JCheckBox();
        cbIsAllowedToDeleteHiddenFiles = new javax.swing.JCheckBox();
        cbIsAllowedToRemoveHiddenSubDirectories = new javax.swing.JCheckBox();
        cbIsAllowedToTraverseSubDirectories = new javax.swing.JCheckBox();
        cbIsAllowedToTraverseHiddenSubDirectories = new javax.swing.JCheckBox();
        cbUserRightsSubDirectoriesSelectAll = new javax.swing.JCheckBox();
        cbUserRightsSubDirectoriesDeselectAll = new javax.swing.JCheckBox();
        cbUserRightsFilesSelectAll = new javax.swing.JCheckBox();
        cbUserRightsFilesDeselectAll = new javax.swing.JCheckBox();
        cbUserRightsHiddenFilesSelectAll = new javax.swing.JCheckBox();
        cbUserRightsHiddenFilesDeselectAll = new javax.swing.JCheckBox();
        cbUserRightsHiddenSubDirectoriesSelectAll = new javax.swing.JCheckBox();
        cbUserRightsHiddenSubDirectoriesDeselectAll = new javax.swing.JCheckBox();
        btnCopyGroup = new javax.swing.JButton();
        tabTLS = new javax.swing.JPanel();
        panelTLSLevel = new javax.swing.JPanel();
        comboTLSLevel = new javax.swing.JComboBox();
        panelServerCertificateAndKey = new javax.swing.JPanel();
        ldlServerCertAndKey = new javax.swing.JLabel();
        lblKeyPassphrase = new javax.swing.JLabel();
        btnBrowseForServerCertAndKey = new javax.swing.JButton();
        edtServerCertAndKey = new javax.swing.JTextField();
        edtKeyPassphrase = new javax.swing.JPasswordField();
        panelClientAuthentication = new javax.swing.JPanel();
        cbRequireClientAuthentication = new javax.swing.JCheckBox();
        lblTrustedCAsFolder = new javax.swing.JLabel();
        btnBrowseForTrustedCAsFolder = new javax.swing.JButton();
        edtTrustedCAsFolder = new javax.swing.JTextField();
        btnSaveChanges = new javax.swing.JButton();
        btnStartServer = new javax.swing.JButton();
        btnStopServer = new javax.swing.JButton();
        btnDaemonServer = new javax.swing.JButton();
        btnReload = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("Tcpconns FTPServer");
        setLocationByPlatform(true);

        panelGeneralSettings.setBorder(javax.swing.BorderFactory.createTitledBorder("General Settings "));

        lblExternalIP.setText("External IP Address (if behind router)");

        lblPort.setText("Bind Port (e.g. 21)");

        cbLogEvents.setSelected(true);
        cbLogEvents.setText("log events");
        cbLogEvents.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbLogEventsActionPerformed(evt);
            }
        });

        cbAppendLogs.setSelected(true);
        cbAppendLogs.setText("append log files");
        cbAppendLogs.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbAppendLogsActionPerformed(evt);
            }
        });

        edtExternalIP.setText("jTextField3");
        edtExternalIP.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                edtExternalIPKeyReleased(evt);
            }
        });

        edtPort.setValue(2100);
        edtPort.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                edtPortStateChanged(evt);
            }
        });
        edtPort.addInputMethodListener(new java.awt.event.InputMethodListener() {
            public void caretPositionChanged(java.awt.event.InputMethodEvent evt) {
            }
            public void inputMethodTextChanged(java.awt.event.InputMethodEvent evt) {
                edtPortInputMethodTextChanged(evt);
            }
        });

        cbTrackChanges.setText("\"track changes\"");
        cbTrackChanges.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbTrackChangesActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout panelGeneralSettingsLayout = new org.jdesktop.layout.GroupLayout(panelGeneralSettings);
        panelGeneralSettings.setLayout(panelGeneralSettingsLayout);
        panelGeneralSettingsLayout.setHorizontalGroup(
            panelGeneralSettingsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(panelGeneralSettingsLayout.createSequentialGroup()
                .addContainerGap()
                .add(panelGeneralSettingsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(cbTrackChanges)
                    .add(cbAppendLogs)
                    .add(panelGeneralSettingsLayout.createSequentialGroup()
                        .add(panelGeneralSettingsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(lblExternalIP)
                            .add(lblPort)
                            .add(cbLogEvents))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(panelGeneralSettingsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(edtPort, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 592, Short.MAX_VALUE)
                            .add(edtExternalIP, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 592, Short.MAX_VALUE))))
                .addContainerGap())
        );
        panelGeneralSettingsLayout.setVerticalGroup(
            panelGeneralSettingsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(panelGeneralSettingsLayout.createSequentialGroup()
                .addContainerGap()
                .add(panelGeneralSettingsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(lblExternalIP)
                    .add(edtExternalIP, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(panelGeneralSettingsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(lblPort)
                    .add(edtPort, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(cbLogEvents)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(cbAppendLogs)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(cbTrackChanges)
                .addContainerGap(67, Short.MAX_VALUE))
        );

        org.jdesktop.layout.GroupLayout tabGeneralLayout = new org.jdesktop.layout.GroupLayout(tabGeneral);
        tabGeneral.setLayout(tabGeneralLayout);
        tabGeneralLayout.setHorizontalGroup(
            tabGeneralLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(tabGeneralLayout.createSequentialGroup()
                .addContainerGap()
                .add(panelGeneralSettings, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addContainerGap())
        );
        tabGeneralLayout.setVerticalGroup(
            tabGeneralLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(tabGeneralLayout.createSequentialGroup()
                .addContainerGap()
                .add(panelGeneralSettings, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(288, Short.MAX_VALUE))
        );

        TabbedPane.addTab("General", tabGeneral);

        panelUsersAndGroupMembership.setBorder(javax.swing.BorderFactory.createTitledBorder("Users and group membership "));

        lblUser.setText("user");

        comboUsers.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Item 1", "Item 2", "Item 3", "Item 4" }));
        comboUsers.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                comboUsersActionPerformed(evt);
            }
        });

        lblMemberOfGroup.setText("member of");

        btnAddUser.setText("add user");
        btnAddUser.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnAddUserActionPerformed(evt);
            }
        });

        btnRenameUser.setText("rename user");
        btnRenameUser.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnRenameUserActionPerformed(evt);
            }
        });

        btnDeleteUser.setText("delete user");
        btnDeleteUser.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnDeleteUserActionPerformed(evt);
            }
        });

        btnResetPassphrase.setText("reset passphrase");
        btnResetPassphrase.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnResetPassphraseActionPerformed(evt);
            }
        });

        cbAccountIsActive.setSelected(true);
        cbAccountIsActive.setText("account is active");
        cbAccountIsActive.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbAccountIsActiveActionPerformed(evt);
            }
        });

        comboGroupMembership.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Item 1", "Item 2", "Item 3", "Item 4" }));
        comboGroupMembership.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                comboGroupMembershipActionPerformed(evt);
            }
        });

        cbRequirePassphraseToBeCorrect.setSelected(true);
        cbRequirePassphraseToBeCorrect.setText("require password to be correct");
        cbRequirePassphraseToBeCorrect.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbRequirePassphraseToBeCorrectActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout panelUsersAndGroupMembershipLayout = new org.jdesktop.layout.GroupLayout(panelUsersAndGroupMembership);
        panelUsersAndGroupMembership.setLayout(panelUsersAndGroupMembershipLayout);
        panelUsersAndGroupMembershipLayout.setHorizontalGroup(
            panelUsersAndGroupMembershipLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(panelUsersAndGroupMembershipLayout.createSequentialGroup()
                .addContainerGap()
                .add(panelUsersAndGroupMembershipLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, lblUser)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, lblMemberOfGroup))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(panelUsersAndGroupMembershipLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(panelUsersAndGroupMembershipLayout.createSequentialGroup()
                        .add(cbAccountIsActive)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(cbRequirePassphraseToBeCorrect))
                    .add(comboUsers, 0, 754, Short.MAX_VALUE)
                    .add(comboGroupMembership, 0, 754, Short.MAX_VALUE)
                    .add(panelUsersAndGroupMembershipLayout.createSequentialGroup()
                        .add(btnAddUser)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(btnRenameUser)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(btnDeleteUser)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(btnResetPassphrase)))
                .addContainerGap())
        );
        panelUsersAndGroupMembershipLayout.setVerticalGroup(
            panelUsersAndGroupMembershipLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(panelUsersAndGroupMembershipLayout.createSequentialGroup()
                .add(panelUsersAndGroupMembershipLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(lblUser)
                    .add(comboUsers, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(panelUsersAndGroupMembershipLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(btnAddUser)
                    .add(btnRenameUser)
                    .add(btnDeleteUser)
                    .add(btnResetPassphrase))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(panelUsersAndGroupMembershipLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(cbAccountIsActive)
                    .add(cbRequirePassphraseToBeCorrect))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(panelUsersAndGroupMembershipLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(lblMemberOfGroup)
                    .add(comboGroupMembership, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );

        panelGroupsAndUserRights.setBorder(javax.swing.BorderFactory.createTitledBorder("Groups and user rights "));

        lblGroups.setText("group");

        comboGroups.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Item 1", "Item 2", "Item 3", "Item 4" }));
        comboGroups.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                comboGroupsActionPerformed(evt);
            }
        });

        btnAddGroup.setText("add group");
        btnAddGroup.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnAddGroupActionPerformed(evt);
            }
        });

        btnRenameGroup.setText("rename group");
        btnRenameGroup.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnRenameGroupActionPerformed(evt);
            }
        });

        btnDeleteGroup.setText("delete group");
        btnDeleteGroup.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnDeleteGroupActionPerformed(evt);
            }
        });

        lblFolders.setText("folders");

        comboFolders.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Item 1", "Item 2", "Item 3", "Item 4" }));
        comboFolders.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                comboFoldersActionPerformed(evt);
            }
        });

        btnAddFolder.setText("add folder");
        btnAddFolder.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnAddFolderActionPerformed(evt);
            }
        });

        btnDeleteFolder.setText("delete folder");
        btnDeleteFolder.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnDeleteFolderActionPerformed(evt);
            }
        });

        lblOpeningFolder.setText("opening folder");

        comboOpeningFolder.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Item 1", "Item 2", "Item 3", "Item 4" }));
        comboOpeningFolder.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                comboOpeningFolderActionPerformed(evt);
            }
        });

        lblFiles.setText("files");

        lblSubDirectories.setText("subdirectories");

        lblHiddenFiles.setText("hidden files");

        lblHiddenSubDirectories.setText("hidden subdirectories");

        cbIsAllowedToSeeFiles.setText("see");
        cbIsAllowedToSeeFiles.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbIsAllowedToSeeFilesActionPerformed(evt);
            }
        });

        cbIsAllowedToReadFiles.setText("read");
        cbIsAllowedToReadFiles.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbIsAllowedToReadFilesActionPerformed(evt);
            }
        });

        cbIsAllowedToRenameFiles.setText("rename");
        cbIsAllowedToRenameFiles.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbIsAllowedToRenameFilesActionPerformed(evt);
            }
        });

        cbIsAllowedToAppendToFiles.setText("append to");
        cbIsAllowedToAppendToFiles.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbIsAllowedToAppendToFilesActionPerformed(evt);
            }
        });

        cbIsAllowedToCreateNewFiles.setText("create new");
        cbIsAllowedToCreateNewFiles.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbIsAllowedToCreateNewFilesActionPerformed(evt);
            }
        });

        cbIsAllowedToOverwriteFiles.setText("overwrite");
        cbIsAllowedToOverwriteFiles.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbIsAllowedToOverwriteFilesActionPerformed(evt);
            }
        });

        cbIsAllowedToDeleteFiles.setText("delete");
        cbIsAllowedToDeleteFiles.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbIsAllowedToDeleteFilesActionPerformed(evt);
            }
        });

        cbIsAllowedToSeeSubDirectories.setText("see");
        cbIsAllowedToSeeSubDirectories.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbIsAllowedToSeeSubDirectoriesActionPerformed(evt);
            }
        });

        cbIsAllowedToRenameSubDirectories.setText("rename");
        cbIsAllowedToRenameSubDirectories.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbIsAllowedToRenameSubDirectoriesActionPerformed(evt);
            }
        });

        cbIsAllowedToSeeHiddenFiles.setText("see");
        cbIsAllowedToSeeHiddenFiles.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbIsAllowedToSeeHiddenFilesActionPerformed(evt);
            }
        });

        cbIsAllowedToSeeHiddenSubDirectories.setText("see");
        cbIsAllowedToSeeHiddenSubDirectories.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbIsAllowedToSeeHiddenSubDirectoriesActionPerformed(evt);
            }
        });

        cbIsAllowedToReadHiddenFiles.setText("read");
        cbIsAllowedToReadHiddenFiles.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbIsAllowedToReadHiddenFilesActionPerformed(evt);
            }
        });

        cbIsAllowedToRenameHiddenSubDirectories.setText("rename");
        cbIsAllowedToRenameHiddenSubDirectories.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbIsAllowedToRenameHiddenSubDirectoriesActionPerformed(evt);
            }
        });

        cbIsAllowedToRenameHiddenFiles.setText("rename");
        cbIsAllowedToRenameHiddenFiles.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbIsAllowedToRenameHiddenFilesActionPerformed(evt);
            }
        });

        cbIsAllowedToAppendToHiddenFiles.setText("append to");
        cbIsAllowedToAppendToHiddenFiles.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbIsAllowedToAppendToHiddenFilesActionPerformed(evt);
            }
        });

        cbIsAllowedToMakeSubDirectories.setText("make");
        cbIsAllowedToMakeSubDirectories.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbIsAllowedToMakeSubDirectoriesActionPerformed(evt);
            }
        });

        cbIsAllowedToCreateNewHiddenFiles.setText("create new");
        cbIsAllowedToCreateNewHiddenFiles.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbIsAllowedToCreateNewHiddenFilesActionPerformed(evt);
            }
        });

        cbIsAllowedToMakeHiddenSubDirectories.setText("make");
        cbIsAllowedToMakeHiddenSubDirectories.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbIsAllowedToMakeHiddenSubDirectoriesActionPerformed(evt);
            }
        });

        cbIsAllowedToOverwriteHiddenFiles.setText("overwrite");
        cbIsAllowedToOverwriteHiddenFiles.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbIsAllowedToOverwriteHiddenFilesActionPerformed(evt);
            }
        });

        cbIsAllowedToRemoveSubDirectories.setText("remove");
        cbIsAllowedToRemoveSubDirectories.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbIsAllowedToRemoveSubDirectoriesActionPerformed(evt);
            }
        });

        cbIsAllowedToDeleteHiddenFiles.setText("delete");
        cbIsAllowedToDeleteHiddenFiles.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbIsAllowedToDeleteHiddenFilesActionPerformed(evt);
            }
        });

        cbIsAllowedToRemoveHiddenSubDirectories.setText("remove");
        cbIsAllowedToRemoveHiddenSubDirectories.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbIsAllowedToRemoveHiddenSubDirectoriesActionPerformed(evt);
            }
        });

        cbIsAllowedToTraverseSubDirectories.setText("traverse");
        cbIsAllowedToTraverseSubDirectories.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbIsAllowedToTraverseSubDirectoriesActionPerformed(evt);
            }
        });

        cbIsAllowedToTraverseHiddenSubDirectories.setText("traverse");
        cbIsAllowedToTraverseHiddenSubDirectories.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbIsAllowedToTraverseHiddenSubDirectoriesActionPerformed(evt);
            }
        });

        cbUserRightsSubDirectoriesSelectAll.setText("all");
        cbUserRightsSubDirectoriesSelectAll.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbUserRightsSubDirectoriesSelectAllActionPerformed(evt);
            }
        });

        cbUserRightsSubDirectoriesDeselectAll.setText("none");
        cbUserRightsSubDirectoriesDeselectAll.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbUserRightsSubDirectoriesDeselectAllActionPerformed(evt);
            }
        });

        cbUserRightsFilesSelectAll.setText("all");
        cbUserRightsFilesSelectAll.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbUserRightsFilesSelectAllActionPerformed(evt);
            }
        });

        cbUserRightsFilesDeselectAll.setText("none");
        cbUserRightsFilesDeselectAll.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbUserRightsFilesDeselectAllActionPerformed(evt);
            }
        });

        cbUserRightsHiddenFilesSelectAll.setText("all");
        cbUserRightsHiddenFilesSelectAll.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbUserRightsHiddenFilesSelectAllActionPerformed(evt);
            }
        });

        cbUserRightsHiddenFilesDeselectAll.setText("none");
        cbUserRightsHiddenFilesDeselectAll.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbUserRightsHiddenFilesDeselectAllActionPerformed(evt);
            }
        });

        cbUserRightsHiddenSubDirectoriesSelectAll.setText("all");
        cbUserRightsHiddenSubDirectoriesSelectAll.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbUserRightsHiddenSubDirectoriesSelectAllActionPerformed(evt);
            }
        });

        cbUserRightsHiddenSubDirectoriesDeselectAll.setText("none");
        cbUserRightsHiddenSubDirectoriesDeselectAll.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbUserRightsHiddenSubDirectoriesDeselectAllActionPerformed(evt);
            }
        });

        btnCopyGroup.setText("copy group");
        btnCopyGroup.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnCopyGroupActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout panelGroupsAndUserRightsLayout = new org.jdesktop.layout.GroupLayout(panelGroupsAndUserRights);
        panelGroupsAndUserRights.setLayout(panelGroupsAndUserRightsLayout);
        panelGroupsAndUserRightsLayout.setHorizontalGroup(
            panelGroupsAndUserRightsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(panelGroupsAndUserRightsLayout.createSequentialGroup()
                .add(45, 45, 45)
                .add(panelGroupsAndUserRightsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, lblOpeningFolder)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, lblHiddenSubDirectories)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, lblHiddenFiles)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, lblSubDirectories)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, lblFiles)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, lblFolders)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, lblGroups))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(panelGroupsAndUserRightsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(panelGroupsAndUserRightsLayout.createSequentialGroup()
                        .add(btnAddFolder)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(btnDeleteFolder))
                    .add(panelGroupsAndUserRightsLayout.createSequentialGroup()
                        .add(btnAddGroup)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(btnRenameGroup)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(btnDeleteGroup)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(btnCopyGroup))
                    .add(comboGroups, 0, 653, Short.MAX_VALUE)
                    .add(comboFolders, 0, 653, Short.MAX_VALUE)
                    .add(panelGroupsAndUserRightsLayout.createSequentialGroup()
                        .add(panelGroupsAndUserRightsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(panelGroupsAndUserRightsLayout.createSequentialGroup()
                                .add(panelGroupsAndUserRightsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                    .add(cbIsAllowedToSeeFiles)
                                    .add(cbIsAllowedToSeeHiddenFiles))
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(panelGroupsAndUserRightsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                    .add(cbIsAllowedToReadFiles)
                                    .add(cbIsAllowedToReadHiddenFiles)))
                            .add(panelGroupsAndUserRightsLayout.createSequentialGroup()
                                .add(cbIsAllowedToSeeHiddenSubDirectories)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(cbIsAllowedToTraverseHiddenSubDirectories))
                            .add(panelGroupsAndUserRightsLayout.createSequentialGroup()
                                .add(cbIsAllowedToSeeSubDirectories)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(cbIsAllowedToTraverseSubDirectories)))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(panelGroupsAndUserRightsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(panelGroupsAndUserRightsLayout.createSequentialGroup()
                                .add(panelGroupsAndUserRightsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                    .add(panelGroupsAndUserRightsLayout.createSequentialGroup()
                                        .add(cbIsAllowedToRenameFiles)
                                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                        .add(cbIsAllowedToAppendToFiles))
                                    .add(cbIsAllowedToRenameSubDirectories))
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(panelGroupsAndUserRightsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                    .add(panelGroupsAndUserRightsLayout.createSequentialGroup()
                                        .add(cbIsAllowedToCreateNewFiles)
                                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                        .add(cbIsAllowedToOverwriteFiles))
                                    .add(cbIsAllowedToMakeSubDirectories))
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(panelGroupsAndUserRightsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                    .add(cbIsAllowedToRemoveSubDirectories)
                                    .add(cbIsAllowedToDeleteFiles)))
                            .add(panelGroupsAndUserRightsLayout.createSequentialGroup()
                                .add(panelGroupsAndUserRightsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                    .add(panelGroupsAndUserRightsLayout.createSequentialGroup()
                                        .add(cbIsAllowedToRenameHiddenFiles)
                                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                        .add(cbIsAllowedToAppendToHiddenFiles))
                                    .add(cbIsAllowedToRenameHiddenSubDirectories))
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(panelGroupsAndUserRightsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                    .add(panelGroupsAndUserRightsLayout.createSequentialGroup()
                                        .add(cbIsAllowedToCreateNewHiddenFiles)
                                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                        .add(cbIsAllowedToOverwriteHiddenFiles))
                                    .add(cbIsAllowedToMakeHiddenSubDirectories))
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(panelGroupsAndUserRightsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                    .add(cbIsAllowedToRemoveHiddenSubDirectories)
                                    .add(cbIsAllowedToDeleteHiddenFiles))))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(panelGroupsAndUserRightsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(panelGroupsAndUserRightsLayout.createSequentialGroup()
                                .add(cbUserRightsHiddenSubDirectoriesSelectAll)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(cbUserRightsHiddenSubDirectoriesDeselectAll))
                            .add(panelGroupsAndUserRightsLayout.createSequentialGroup()
                                .add(cbUserRightsHiddenFilesSelectAll)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(cbUserRightsHiddenFilesDeselectAll))
                            .add(panelGroupsAndUserRightsLayout.createSequentialGroup()
                                .add(cbUserRightsSubDirectoriesSelectAll)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(cbUserRightsSubDirectoriesDeselectAll))
                            .add(panelGroupsAndUserRightsLayout.createSequentialGroup()
                                .add(cbUserRightsFilesSelectAll)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(cbUserRightsFilesDeselectAll))))
                    .add(comboOpeningFolder, 0, 653, Short.MAX_VALUE))
                .addContainerGap())
        );
        panelGroupsAndUserRightsLayout.setVerticalGroup(
            panelGroupsAndUserRightsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(panelGroupsAndUserRightsLayout.createSequentialGroup()
                .add(panelGroupsAndUserRightsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(lblGroups)
                    .add(comboGroups, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(panelGroupsAndUserRightsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(btnAddGroup)
                    .add(btnRenameGroup)
                    .add(btnDeleteGroup)
                    .add(btnCopyGroup))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(panelGroupsAndUserRightsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(lblFolders)
                    .add(comboFolders, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(panelGroupsAndUserRightsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(btnAddFolder)
                    .add(btnDeleteFolder))
                .add(8, 8, 8)
                .add(panelGroupsAndUserRightsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(panelGroupsAndUserRightsLayout.createSequentialGroup()
                        .add(panelGroupsAndUserRightsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(cbIsAllowedToAppendToFiles)
                            .add(cbIsAllowedToCreateNewFiles)
                            .add(cbIsAllowedToOverwriteFiles)
                            .add(cbIsAllowedToDeleteFiles)
                            .add(cbIsAllowedToRenameFiles)
                            .add(cbUserRightsFilesSelectAll)
                            .add(cbUserRightsFilesDeselectAll))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(panelGroupsAndUserRightsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(cbIsAllowedToRenameSubDirectories)
                            .add(cbIsAllowedToMakeSubDirectories)
                            .add(cbIsAllowedToRemoveSubDirectories)
                            .add(cbUserRightsSubDirectoriesSelectAll)
                            .add(cbUserRightsSubDirectoriesDeselectAll))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(panelGroupsAndUserRightsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(cbIsAllowedToRenameHiddenFiles)
                            .add(cbIsAllowedToAppendToHiddenFiles)
                            .add(cbIsAllowedToCreateNewHiddenFiles)
                            .add(cbIsAllowedToOverwriteHiddenFiles)
                            .add(cbIsAllowedToDeleteHiddenFiles)
                            .add(cbUserRightsHiddenFilesSelectAll)
                            .add(cbUserRightsHiddenFilesDeselectAll))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(panelGroupsAndUserRightsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(cbIsAllowedToRenameHiddenSubDirectories)
                            .add(cbIsAllowedToMakeHiddenSubDirectories)
                            .add(cbIsAllowedToRemoveHiddenSubDirectories)
                            .add(cbUserRightsHiddenSubDirectoriesSelectAll)
                            .add(cbUserRightsHiddenSubDirectoriesDeselectAll)))
                    .add(panelGroupsAndUserRightsLayout.createSequentialGroup()
                        .add(panelGroupsAndUserRightsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(cbIsAllowedToSeeFiles)
                            .add(cbIsAllowedToReadFiles)
                            .add(lblFiles))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(panelGroupsAndUserRightsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(cbIsAllowedToSeeSubDirectories)
                            .add(cbIsAllowedToTraverseSubDirectories)
                            .add(lblSubDirectories))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(panelGroupsAndUserRightsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(cbIsAllowedToSeeHiddenFiles)
                            .add(cbIsAllowedToReadHiddenFiles)
                            .add(lblHiddenFiles))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(panelGroupsAndUserRightsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(cbIsAllowedToSeeHiddenSubDirectories)
                            .add(cbIsAllowedToTraverseHiddenSubDirectories)
                            .add(lblHiddenSubDirectories))))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(panelGroupsAndUserRightsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(lblOpeningFolder)
                    .add(comboOpeningFolder, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(81, Short.MAX_VALUE))
        );

        org.jdesktop.layout.GroupLayout tabGroupsUsersAndUserRightsLayout = new org.jdesktop.layout.GroupLayout(tabGroupsUsersAndUserRights);
        tabGroupsUsersAndUserRights.setLayout(tabGroupsUsersAndUserRightsLayout);
        tabGroupsUsersAndUserRightsLayout.setHorizontalGroup(
            tabGroupsUsersAndUserRightsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, tabGroupsUsersAndUserRightsLayout.createSequentialGroup()
                .addContainerGap()
                .add(tabGroupsUsersAndUserRightsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, panelGroupsAndUserRights, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, panelUsersAndGroupMembership, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap())
        );
        tabGroupsUsersAndUserRightsLayout.setVerticalGroup(
            tabGroupsUsersAndUserRightsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(tabGroupsUsersAndUserRightsLayout.createSequentialGroup()
                .addContainerGap()
                .add(panelUsersAndGroupMembership, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(panelGroupsAndUserRights, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addContainerGap())
        );

        TabbedPane.addTab("Groups/Users", tabGroupsUsersAndUserRights);

        panelTLSLevel.setBorder(javax.swing.BorderFactory.createTitledBorder("TLS Level "));

        comboTLSLevel.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Item 1", "Item 2", "Item 3", "Item 4" }));
        comboTLSLevel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                comboTLSLevelActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout panelTLSLevelLayout = new org.jdesktop.layout.GroupLayout(panelTLSLevel);
        panelTLSLevel.setLayout(panelTLSLevelLayout);
        panelTLSLevelLayout.setHorizontalGroup(
            panelTLSLevelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(panelTLSLevelLayout.createSequentialGroup()
                .addContainerGap()
                .add(comboTLSLevel, 0, 834, Short.MAX_VALUE)
                .addContainerGap())
        );
        panelTLSLevelLayout.setVerticalGroup(
            panelTLSLevelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(panelTLSLevelLayout.createSequentialGroup()
                .add(comboTLSLevel, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(19, Short.MAX_VALUE))
        );

        panelServerCertificateAndKey.setBorder(javax.swing.BorderFactory.createTitledBorder("Server Certificate and Key "));

        ldlServerCertAndKey.setText("Server Certificate and Key File (.pem)");

        lblKeyPassphrase.setText("Key Passphrase");

        btnBrowseForServerCertAndKey.setText("...");
        btnBrowseForServerCertAndKey.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnBrowseForServerCertAndKeyActionPerformed(evt);
            }
        });

        edtServerCertAndKey.setText("jTextField2");
        edtServerCertAndKey.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                edtServerCertAndKeyKeyReleased(evt);
            }
        });

        edtKeyPassphrase.setText("jPasswordField1");
        edtKeyPassphrase.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                edtKeyPassphraseKeyReleased(evt);
            }
        });

        org.jdesktop.layout.GroupLayout panelServerCertificateAndKeyLayout = new org.jdesktop.layout.GroupLayout(panelServerCertificateAndKey);
        panelServerCertificateAndKey.setLayout(panelServerCertificateAndKeyLayout);
        panelServerCertificateAndKeyLayout.setHorizontalGroup(
            panelServerCertificateAndKeyLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(panelServerCertificateAndKeyLayout.createSequentialGroup()
                .addContainerGap()
                .add(panelServerCertificateAndKeyLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(ldlServerCertAndKey)
                    .add(lblKeyPassphrase))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(panelServerCertificateAndKeyLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(edtKeyPassphrase, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 539, Short.MAX_VALUE)
                    .add(edtServerCertAndKey, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 539, Short.MAX_VALUE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                .add(btnBrowseForServerCertAndKey)
                .addContainerGap())
        );
        panelServerCertificateAndKeyLayout.setVerticalGroup(
            panelServerCertificateAndKeyLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(panelServerCertificateAndKeyLayout.createSequentialGroup()
                .add(panelServerCertificateAndKeyLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(ldlServerCertAndKey)
                    .add(btnBrowseForServerCertAndKey)
                    .add(edtServerCertAndKey, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(panelServerCertificateAndKeyLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(lblKeyPassphrase)
                    .add(edtKeyPassphrase, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(82, Short.MAX_VALUE))
        );

        panelClientAuthentication.setBorder(javax.swing.BorderFactory.createTitledBorder("Client authentication "));

        cbRequireClientAuthentication.setText("Require Client Authentication");
        cbRequireClientAuthentication.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbRequireClientAuthenticationActionPerformed(evt);
            }
        });

        lblTrustedCAsFolder.setText("Trusted Certificate Authorities Certificates Folder");

        btnBrowseForTrustedCAsFolder.setText("...");
        btnBrowseForTrustedCAsFolder.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnBrowseForTrustedCAsFolderActionPerformed(evt);
            }
        });

        edtTrustedCAsFolder.setText("jTextField1");
        edtTrustedCAsFolder.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                edtTrustedCAsFolderKeyReleased(evt);
            }
        });

        org.jdesktop.layout.GroupLayout panelClientAuthenticationLayout = new org.jdesktop.layout.GroupLayout(panelClientAuthentication);
        panelClientAuthentication.setLayout(panelClientAuthenticationLayout);
        panelClientAuthenticationLayout.setHorizontalGroup(
            panelClientAuthenticationLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(panelClientAuthenticationLayout.createSequentialGroup()
                .addContainerGap()
                .add(panelClientAuthenticationLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(cbRequireClientAuthentication)
                    .add(panelClientAuthenticationLayout.createSequentialGroup()
                        .add(lblTrustedCAsFolder)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(edtTrustedCAsFolder, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 459, Short.MAX_VALUE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                        .add(btnBrowseForTrustedCAsFolder)))
                .addContainerGap())
        );
        panelClientAuthenticationLayout.setVerticalGroup(
            panelClientAuthenticationLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(panelClientAuthenticationLayout.createSequentialGroup()
                .add(cbRequireClientAuthentication)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(panelClientAuthenticationLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(lblTrustedCAsFolder)
                    .add(btnBrowseForTrustedCAsFolder)
                    .add(edtTrustedCAsFolder, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(81, Short.MAX_VALUE))
        );

        org.jdesktop.layout.GroupLayout tabTLSLayout = new org.jdesktop.layout.GroupLayout(tabTLS);
        tabTLS.setLayout(tabTLSLayout);
        tabTLSLayout.setHorizontalGroup(
            tabTLSLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(tabTLSLayout.createSequentialGroup()
                .addContainerGap()
                .add(tabTLSLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(panelTLSLevel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, panelServerCertificateAndKey, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(panelClientAuthentication, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap())
        );
        tabTLSLayout.setVerticalGroup(
            tabTLSLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(tabTLSLayout.createSequentialGroup()
                .addContainerGap()
                .add(panelTLSLevel, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                .add(panelServerCertificateAndKey, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                .add(panelClientAuthentication, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(105, Short.MAX_VALUE))
        );

        TabbedPane.addTab("SSL/TLS", tabTLS);

        btnSaveChanges.setText("Save changes");
        btnSaveChanges.setEnabled(false);
        btnSaveChanges.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnSaveChangesActionPerformed(evt);
            }
        });

        btnStartServer.setText("Start Server");
        btnStartServer.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnStartServerActionPerformed(evt);
            }
        });

        btnStopServer.setText("Stop Server");
        btnStopServer.setEnabled(false);
        btnStopServer.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnStopServerActionPerformed(evt);
            }
        });

        btnDaemonServer.setText("Daemon Server");
        btnDaemonServer.setEnabled(false);

        btnReload.setText("Reload");
        btnReload.setEnabled(false);
        btnReload.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnReloadActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, TabbedPane, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 897, Short.MAX_VALUE)
                    .add(layout.createSequentialGroup()
                        .add(btnStartServer)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(btnStopServer)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(btnDaemonServer)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 349, Short.MAX_VALUE)
                        .add(btnReload)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(btnSaveChanges)))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .add(TabbedPane, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 546, Short.MAX_VALUE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(btnSaveChanges)
                    .add(btnReload)
                    .add(btnDaemonServer)
                    .add(btnStopServer)
                    .add(btnStartServer))
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void btnBrowseForServerCertAndKeyActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnBrowseForServerCertAndKeyActionPerformed
        if (this.fUpdating) {return;}
        Dialogs.openDialog(this.edtServerCertAndKey, "Please select the .pem encoded server cert and key");
        if (data.getPEMEncodedServerCertAndKeyFileName().equals(this.edtServerCertAndKey.getText()) == false) {
            data.setPEMEncodedServerCertAndKeyFileName(this.edtServerCertAndKey.getText());
            this.thereAreChanges();
        }
}//GEN-LAST:event_btnBrowseForServerCertAndKeyActionPerformed

    private void btnBrowseForTrustedCAsFolderActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnBrowseForTrustedCAsFolderActionPerformed
        if (this.fUpdating) {return;}
        Dialogs.openDialog(this.edtTrustedCAsFolder, true, "Please select the trusted CAs certs directory");
        if (data.getTrustedCACertsDirectory().equals(this.edtTrustedCAsFolder.getText()) == false) {
            data.setTrustedCACertsDirectory(this.edtTrustedCAsFolder.getText());
            this.thereAreChanges();
        }
}//GEN-LAST:event_btnBrowseForTrustedCAsFolderActionPerformed

    private void comboUsersActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_comboUsersActionPerformed
        if (this.fUpdating) {return;}
        this.onUserSelected();
}//GEN-LAST:event_comboUsersActionPerformed

    private void comboGroupMembershipActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_comboGroupMembershipActionPerformed
        if (this.fUpdating) {return;}
        if (
                (this.comboGroupMembership.getItemCount() > 0)
            &&
                (this.comboGroups.getItemCount() > 0)
            ) {

            int i = 1-1; int n = this.comboGroups.getItemCount();
            while (i <= (n-1)) {
                if (((String)(this.comboGroupMembership.getSelectedItem())).equals(((String)(this.comboGroups.getItemAt(i))))) {
                    this.comboGroups.setSelectedIndex(i);
                    break;
                }
                i++;
            }

            if (data.getUsersList().size() > 0) {
                i = 1-1; n = data.getUsersList().size();
                while (i <= (n-1)) {
                    if (((String)(this.comboUsers.getSelectedItem())).equals(data.getUsersList().get(i).getUserName())) {
                        if (data.getUsersList().get(i).getGroupName().equals(((String)(this.comboGroupMembership.getSelectedItem()))) == false) {
                            data.getUsersList().get(i).setGroupName(((String)(this.comboGroupMembership.getSelectedItem())));
                            this.thereAreChanges();
                        }
                        break;
                    }
                    i++;
                }                
            }
        }
}//GEN-LAST:event_comboGroupMembershipActionPerformed

    private void comboGroupsActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_comboGroupsActionPerformed
        if (this.fUpdating) {return;}
        this.onGroupSelected();
}//GEN-LAST:event_comboGroupsActionPerformed

    private void comboFoldersActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_comboFoldersActionPerformed
        if (this.fUpdating) {return;}
        this.onFolderSelected();
}//GEN-LAST:event_comboFoldersActionPerformed

    private void edtKeyPassphraseKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_edtKeyPassphraseKeyReleased
        if (this.fUpdating) {return;}
        if (data.getKeyPassphrase().equals(new String(this.edtKeyPassphrase.getPassword())) == false) {
            data.setKeyPassphrase(new String(this.edtKeyPassphrase.getPassword()));
            this.thereAreChanges();
        }
}//GEN-LAST:event_edtKeyPassphraseKeyReleased

    private void edtServerCertAndKeyKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_edtServerCertAndKeyKeyReleased
        if (this.fUpdating) {return;}
        if (data.getPEMEncodedServerCertAndKeyFileName().equals(this.edtServerCertAndKey.getText()) == false) {
            data.setPEMEncodedServerCertAndKeyFileName(this.edtServerCertAndKey.getText());
            this.thereAreChanges();
        }
}//GEN-LAST:event_edtServerCertAndKeyKeyReleased

    private void edtTrustedCAsFolderKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_edtTrustedCAsFolderKeyReleased
        if (this.fUpdating) {return;}
        if (data.getTrustedCACertsDirectory().equals(this.edtTrustedCAsFolder.getText()) == false) {
            data.setTrustedCACertsDirectory(this.edtTrustedCAsFolder.getText());
            this.thereAreChanges();
        }
}//GEN-LAST:event_edtTrustedCAsFolderKeyReleased

    private void cbRequireClientAuthenticationActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbRequireClientAuthenticationActionPerformed
        if (this.fUpdating) {return;}
        if (data.getRequireClientAuthentication() != this.cbRequireClientAuthentication.isSelected()) {
            data.setRequireClientAuthentication(this.cbRequireClientAuthentication.isSelected());
            this.thereAreChanges();
        }
}//GEN-LAST:event_cbRequireClientAuthenticationActionPerformed

    private void comboTLSLevelActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_comboTLSLevelActionPerformed
        if (this.fUpdating) {return;}
        switch (this.comboTLSLevel.getSelectedIndex()) {
            case 0: {//disallow the use of TLS
                        if (data.getAllowTLS() == true) {
                            data.setAllowTLS(false);
                            this.thereAreChanges();
                        }
                        if (data.getImplicitTLS() == true) {
                            data.setImplicitTLS(false);
                            this.thereAreChanges();
                        }
                        if (data.getEnforceExplicitTLSWhenNotImplicit() == true) {
                            data.setEnforceExplicitTLSWhenNotImplicit(false);
                            this.thereAreChanges();
                        }
                        if (data.getEnforcePROTPWhenExplicit() == true) {
                            data.setEnforcePROTPWhenExplicit(false);
                            this.thereAreChanges();
                        }
                        break;
                    }
            case 1: {//explicit (connection OPTIONAL, data channel OPTIONAL)
                        if (data.getAllowTLS() == false) {
                            data.setAllowTLS(true);
                            this.thereAreChanges();
                        }
                        if (data.getImplicitTLS() == true) {
                            data.setImplicitTLS(false);
                            this.thereAreChanges();
                        }
                        if (data.getEnforceExplicitTLSWhenNotImplicit() == true) {
                            data.setEnforceExplicitTLSWhenNotImplicit(false);                            
                            this.thereAreChanges();
                        }
                        if (data.getEnforcePROTPWhenExplicit() == true) {
                            data.setEnforcePROTPWhenExplicit(false);
                            this.thereAreChanges();
                        }
                        break;
                    }
            case 2: {//explicit (connection TLS, data channel OPTIONAL)
                        if (data.getAllowTLS() == false) {
                            data.setAllowTLS(true);
                            this.thereAreChanges();
                        }
                        if (data.getImplicitTLS() == true) {
                            data.setImplicitTLS(false);
                            this.thereAreChanges();
                        }
                        if (data.getEnforceExplicitTLSWhenNotImplicit() == false) {
                            data.setEnforceExplicitTLSWhenNotImplicit(true);
                            this.thereAreChanges();
                        }
                        if (data.getEnforcePROTPWhenExplicit() == true) {
                            data.setEnforcePROTPWhenExplicit(false);
                            this.thereAreChanges();
                        }
                        break;
                    }
            case 3: {//explicit (connection TLS, data channel TLS)
                        if (data.getAllowTLS() == false) {
                            data.setAllowTLS(true);
                            this.thereAreChanges();
                        }
                        if (data.getImplicitTLS() == true) {
                            data.setImplicitTLS(false);
                            this.thereAreChanges();
                        }
                        if (data.getEnforceExplicitTLSWhenNotImplicit() == false) {
                            data.setEnforceExplicitTLSWhenNotImplicit(true);
                            this.thereAreChanges();
                        }
                        if (data.getEnforcePROTPWhenExplicit() == false) {
                            data.setEnforcePROTPWhenExplicit(true);
                            this.thereAreChanges();
                        }
                        break;
                    }
            case 4: {//implicit (connection TLS, data channel TLS)
                        if (data.getAllowTLS() == false) {
                            data.setAllowTLS(true);
                            this.thereAreChanges();
                        }
                        if (data.getImplicitTLS() == false) {
                            data.setImplicitTLS(true);
                            this.thereAreChanges();
                        }
                        if (data.getEnforceExplicitTLSWhenNotImplicit() == true) {
                            data.setEnforceExplicitTLSWhenNotImplicit(false);
                            this.thereAreChanges();
                        }
                        if (data.getEnforcePROTPWhenExplicit() == true) {
                            data.setEnforcePROTPWhenExplicit(false);
                            this.thereAreChanges();
                        }
                        break;
                    }
        }
}//GEN-LAST:event_comboTLSLevelActionPerformed

    private void edtExternalIPKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_edtExternalIPKeyReleased
        if (this.fUpdating) {return;}
        if (data.getExternalIPOfThisServer().equals(this.edtExternalIP.getText()) == false) {
            data.setExternalIPOfThisServer(this.edtExternalIP.getText());
            this.thereAreChanges();
        }
}//GEN-LAST:event_edtExternalIPKeyReleased

    private void edtPortStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_edtPortStateChanged
        if (this.fUpdating) {return;}
        if (data.getServerPort() != ((Integer)(this.edtPort.getValue())).intValue()) {
            data.setServerPort(((Integer)(this.edtPort.getValue())).intValue());
            this.thereAreChanges();
        }
}//GEN-LAST:event_edtPortStateChanged

    private void edtPortInputMethodTextChanged(java.awt.event.InputMethodEvent evt) {//GEN-FIRST:event_edtPortInputMethodTextChanged
        if (this.fUpdating) {return;}
        if (data.getServerPort() != ((Integer)(this.edtPort.getValue())).intValue()) {
            data.setServerPort(((Integer)(this.edtPort.getValue())).intValue());
            this.thereAreChanges();
        }
}//GEN-LAST:event_edtPortInputMethodTextChanged

    private void cbLogEventsActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbLogEventsActionPerformed
        if (this.fUpdating) {return;}
        if (data.getLogEvents() != this.cbLogEvents.isSelected()) {
            data.setLogEvents(this.cbLogEvents.isSelected());
            this.thereAreChanges();
        }
}//GEN-LAST:event_cbLogEventsActionPerformed

    private void cbAppendLogsActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbAppendLogsActionPerformed
        if (this.fUpdating) {return;}
        if (data.getLogAppend() != this.cbAppendLogs.isSelected()) {
            data.setLogAppend(this.cbAppendLogs.isSelected());
            this.thereAreChanges();
        }
}//GEN-LAST:event_cbAppendLogsActionPerformed

    private void cbAccountIsActiveActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbAccountIsActiveActionPerformed
        if (this.fUpdating) {return;}
        String userName = ((String)(this.comboUsers.getSelectedItem()));
        if (this.data.userExists(userName)) {
            FTPProtocolData.User user = this.data.getUser(userName);
            if (user.getAccountIsActive() != this.cbAccountIsActive.isSelected()) {
                user.setAccountIsActive(this.cbAccountIsActive.isSelected());
                this.thereAreChanges();
            }
        }
}//GEN-LAST:event_cbAccountIsActiveActionPerformed

    private void cbRequirePassphraseToBeCorrectActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbRequirePassphraseToBeCorrectActionPerformed
        if (this.fUpdating) {return;}
        String userName = ((String)(this.comboUsers.getSelectedItem()));
        if (this.data.userExists(userName)) {
            FTPProtocolData.User user = this.data.getUser(userName);
            if (user.getRequirePassphraseToBeCorrect() != this.cbRequirePassphraseToBeCorrect.isSelected()) {
                user.setRequirePassphraseToBeCorrect(this.cbRequirePassphraseToBeCorrect.isSelected());
                this.thereAreChanges();
            }
        }
}//GEN-LAST:event_cbRequirePassphraseToBeCorrectActionPerformed

    private void comboOpeningFolderActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_comboOpeningFolderActionPerformed
        if (this.fUpdating) {return;}
        if (data.getGroupsList().size() > 0) {
            int i = 1-1; int n = data.getGroupsList().size();
            while (i <= (n-1)) {
                if (((String)(this.comboGroups.getSelectedItem())).equals(data.getGroupsList().get(i).getGroupName())) {
                    if (((String)(this.comboOpeningFolder.getSelectedItem())).equals(data.getGroupsList().get(i).getOpeningDirectory()) == false) {
                        data.getGroupsList().get(i).setOpeningDirectory(((String)(this.comboOpeningFolder.getSelectedItem())));
                        this.thereAreChanges();
                    }
                    break;
                }
                i++;
            }
        }
}//GEN-LAST:event_comboOpeningFolderActionPerformed

    private void btnAddUserActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnAddUserActionPerformed
        String newUserName = JOptionPane.showInputDialog(this, "Enter new username", "Add new user", JOptionPane.QUESTION_MESSAGE);
        if (newUserName != null) {
            if (this.data.userExists(newUserName)) {
                JOptionPane.showMessageDialog(this, "Username \"" + newUserName + "\"" + " already exists!", "Duplicate name error", JOptionPane.ERROR_MESSAGE);
            } else {
                String passphrase = JOptionPane.showInputDialog(this, "Enter passphrase for \"" + newUserName + "\"", "Please enter passphrase for new user", JOptionPane.QUESTION_MESSAGE);
                if (passphrase != null) {
                    this.startUpdate();
                        FTPProtocolData.User newUser = new FTPProtocolData.User();
                                             newUser.setUserName(newUserName);
                                             newUser.setPassphrase(passphrase);
                                             newUser.setAccountIsActive(true); this.cbAccountIsActive.setSelected(true);
                                             newUser.setRequirePassphraseToBeCorrect(true); this.cbRequirePassphraseToBeCorrect.setSelected(true);
                                             newUser.setGroupName(TFTPProtocol.EMPTY);
                        this.data.getUsersList().add(newUser);
                        this.comboUsers.addItem(newUser.getUserName());
                        this.comboUsers.setSelectedItem(newUser.getUserName());
                    this.endUpdate();

                    this.thereAreChanges();
                    this.onUserSelected();

                    JOptionPane.showMessageDialog(this, "User \"" + newUser.getUserName() + "\"" + " was added. Don't forget to select group membership.", "New user added successfully", JOptionPane.INFORMATION_MESSAGE);
                }
            }
        }
}//GEN-LAST:event_btnAddUserActionPerformed

    private void btnRenameUserActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnRenameUserActionPerformed
        String oldUserName = ((String)(this.comboUsers.getSelectedItem()));

        if (oldUserName.equals(TFTPProtocol.EMPTY) || (this.data.userExists(oldUserName) == false)) {
            JOptionPane.showMessageDialog(this, "No user selected! Cannot rename unnamed user.", "Cannot rename unnamed user", JOptionPane.ERROR_MESSAGE);        
            return;
        }

        FTPProtocolData.User user = this.data.getUser(oldUserName);
        oldUserName = user.getUserName();

        String newUserName = JOptionPane.showInputDialog(this, "Enter new username for \"" + ((String)(this.comboUsers.getSelectedItem())) + "\"", "Rename user", JOptionPane.QUESTION_MESSAGE);
        if (newUserName == null) {return;}

        if (newUserName.equals(oldUserName)) {
            JOptionPane.showMessageDialog(this, "An identical username was given! User \"" + newUserName + "\" was left unchanged.", "New username identical to old username", JOptionPane.ERROR_MESSAGE);
            return;
        }

        if (data.userExists(newUserName)) {
            JOptionPane.showMessageDialog(this, "Username \"" + newUserName + "\"" + " already exists!", "Duplicate name error", JOptionPane.ERROR_MESSAGE);
            return;
        }
        
        this.startUpdate();        
            this.comboUsers.setSelectedItem(TFTPProtocol.EMPTY);
            this.comboUsers.removeItem(oldUserName);
            user.setUserName(newUserName);
            this.comboUsers.addItem(user.getUserName());
            this.comboUsers.setSelectedItem(user.getUserName());
        this.endUpdate();
        this.thereAreChanges();

        JOptionPane.showMessageDialog(this, "Name of user \"" + oldUserName + "\"" + " was changed to \"" + newUserName + "\".", "Username was changed successfully", JOptionPane.INFORMATION_MESSAGE);
}//GEN-LAST:event_btnRenameUserActionPerformed

    private void btnDeleteUserActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnDeleteUserActionPerformed
        String userName = ((String)(this.comboUsers.getSelectedItem()));

        if (userName.equals(TFTPProtocol.EMPTY) || (this.data.userExists(userName) == false)) {
            JOptionPane.showMessageDialog(this, "No user selected! Cannot delete unnamed user.", "Cannot delete unnamed user", JOptionPane.ERROR_MESSAGE);
            return;
        }

        if (JOptionPane.showConfirmDialog(this, "Are you absolutely sure you want to delete user \"" + userName + "\"??" + "\r\nWarning: if you press yes, user \"" + userName + "\" will be deleted permanently!", "Confirm user delete", JOptionPane.YES_NO_CANCEL_OPTION) == JOptionPane.YES_OPTION) {

            this.startUpdate();
                FTPProtocolData.User oldUser = this.data.getUser(userName);
                this.comboUsers.setSelectedItem(TFTPProtocol.EMPTY);
                this.comboUsers.removeItem(oldUser.getUserName());
                this.data.getUsersList().remove(oldUser);
                this.onUserSelected();
            this.endUpdate();
            this.thereAreChanges();

            JOptionPane.showMessageDialog(this, "User \"" + userName + "\"" + " was deleted!", "Username was deleted successfully", JOptionPane.INFORMATION_MESSAGE);             
        }

}//GEN-LAST:event_btnDeleteUserActionPerformed

    private void btnResetPassphraseActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnResetPassphraseActionPerformed
        String userName = ((String)(this.comboUsers.getSelectedItem()));
        String oldPassphrase = null;
        if (userName.equals(TFTPProtocol.EMPTY)) {
            JOptionPane.showMessageDialog(this, "No user selected! Cannot change passphrase of an unnamed user.", "Cannot change passphrase of an unnamed user", JOptionPane.ERROR_MESSAGE);
        } else {
            if (data.getUsersList().size() > 0) {
                int i = 1-1; int n = data.getUsersList().size();
                while (i <= (n-1)) {
                    if (userName.equals(data.getUsersList().get(i).getUserName())) {
                        oldPassphrase = data.getUsersList().get(i).getPassphrase();
                        break;
                    }
                    i++;
                }
            }
            String passphrase = (String)JOptionPane.showInputDialog(this, "Enter new passphrase for \"" + userName + "\"", "Please enter passphrase", JOptionPane.QUESTION_MESSAGE, null, null, oldPassphrase);
            if (passphrase != null) {
                if (data.getUsersList().size() > 0) {
                    int i = 1-1; int n = data.getUsersList().size();
                    while (i <= (n-1)) {
                        if (userName.equals(data.getUsersList().get(i).getUserName())) {
                            if (data.getUsersList().get(i).getPassphrase().equals(passphrase)) {
                                JOptionPane.showMessageDialog(this, "Passphrase of \"" + userName + "\"" + " was left unchanged.", "Unchanged passphrase for user", JOptionPane.INFORMATION_MESSAGE);
                            } else {
                                data.getUsersList().get(i).setPassphrase(passphrase);
                                this.thereAreChanges();
                                JOptionPane.showMessageDialog(this, "Changed passphrase of \"" + userName + "\"", "Passphrase of user was changed successfully", JOptionPane.INFORMATION_MESSAGE);
                            }
                            break;
                        }
                        i++;
                    }
                }
            } else {
                JOptionPane.showMessageDialog(this, "User abort. Passphrase of \"" + userName + "\"" + " was left unchanged.", "Unchanged passphrase for user", JOptionPane.INFORMATION_MESSAGE);
            }
        }
}//GEN-LAST:event_btnResetPassphraseActionPerformed

    private void btnAddGroupActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnAddGroupActionPerformed
        String newGroupName = "";
        while (newGroupName != null) {
            newGroupName = JOptionPane.showInputDialog(this, "Enter new groupname", "Add new group", JOptionPane.QUESTION_MESSAGE);
            if (newGroupName != null) {
                if (data.getGroupsList().size() >= 0) {
                    int i = 1-1; int n = data.getGroupsList().size();
                    while (i <= (n-1)) {
                        if (data.getGroupsList().get(i).getGroupName().equals(newGroupName)) {
                            JOptionPane.showMessageDialog(this, "Groupname \"" + newGroupName + "\"" + " already exists!", "Duplicate name error", JOptionPane.ERROR_MESSAGE);
                            newGroupName = null;
                            break;
                        }
                        i++;
                    }
                    if (newGroupName != null) {
                                    this.startUpdate();
                                    this.clearFolders();
                                    this.clearUserRights();
                                    FTPProtocolData.Group newGroup = new FTPProtocolData.Group();
                                    newGroup.setGroupName(newGroupName);
                                    newGroup.setOpeningDirectory(TFTPProtocol.EMPTY);
                                    data.getGroupsList().add(newGroup);
                                    this.comboGroups.addItem(newGroup.getGroupName());
                                    this.comboGroupMembership.addItem(newGroup.getGroupName());
                                    this.endUpdate();
                                    this.thereAreChanges();
                                    this.comboGroups.setSelectedItem(newGroup.getGroupName());
                                    JOptionPane.showMessageDialog(this, "Group \"" + newGroup.getGroupName() + "\"" + " was added.", "New group added successfully", JOptionPane.INFORMATION_MESSAGE);
                                    newGroupName = null;
                    }
                }
            }
        }
}//GEN-LAST:event_btnAddGroupActionPerformed

    private void btnRenameGroupActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnRenameGroupActionPerformed
        if (((String)(this.comboGroups.getSelectedItem())).equals(TFTPProtocol.EMPTY)) {
            JOptionPane.showMessageDialog(this, "No group selected! Cannot rename unnamed group.", "Cannot rename unnamed group", JOptionPane.ERROR_MESSAGE);
            return;
        }

        String newGroupName = "";
        while (newGroupName != null) {
            newGroupName = JOptionPane.showInputDialog(this, "Enter new groupname for \"" + ((String)(this.comboGroups.getSelectedItem()))
                    + "\"", "Rename group", JOptionPane.QUESTION_MESSAGE);
            if (newGroupName != null) {
                if (((String)(this.comboGroups.getSelectedItem())).equals(newGroupName)) {
                    JOptionPane.showMessageDialog(this, "An identical groupname was given! Group \"" + newGroupName + "\" was left unchanged.", "New groupname identical to old groupname", JOptionPane.ERROR_MESSAGE);
                    newGroupName = null;
                } else {
                    if (data.getGroupsList().size() > 0) {
                        int i = 1-1; int n = data.getGroupsList().size();
                        while (i <= (n-1)) {
                            if (data.getGroupsList().get(i).getGroupName().equals(newGroupName)) {
                                JOptionPane.showMessageDialog(this, "Groupname \"" + newGroupName + "\"" + " already exists!", "Duplicate name error", JOptionPane.ERROR_MESSAGE);
                                newGroupName = null;
                                break;
                            }
                            i++;
                        }
                        if (newGroupName != null) {
                            i = 1-1; n = data.getGroupsList().size();
                            while (i <= (n-1)) {
                                if (data.getGroupsList().get(i).getGroupName().equals(((String)(this.comboGroups.getSelectedItem())))) {
                                    this.startUpdate();
                                    String oldGroupName = data.getGroupsList().get(i).getGroupName();
                                    this.comboGroups.setSelectedItem(TFTPProtocol.EMPTY);
                                    this.comboGroups.removeItem(data.getGroupsList().get(i).getGroupName());
                                    int j = 1-1; int m = this.comboGroupMembership.getItemCount();
                                    while (j <= (m-1)) {
                                        if (((String)(this.comboGroupMembership.getItemAt(j))).equals(data.getGroupsList().get(i).getGroupName())) {
                                            this.comboGroupMembership.setSelectedItem(TFTPProtocol.EMPTY);
                                            this.comboGroupMembership.removeItem(data.getGroupsList().get(i).getGroupName());
                                            break;
                                        }
                                        j++;
                                    }
                                    data.getGroupsList().get(i).setGroupName(newGroupName);
                                    this.comboGroups.addItem(data.getGroupsList().get(i).getGroupName());
                                    this.comboGroups.setSelectedItem(data.getGroupsList().get(i).getGroupName());
                                    this.comboGroupMembership.addItem(data.getGroupsList().get(i).getGroupName());
                                    this.comboGroupMembership.setSelectedItem(data.getGroupsList().get(i).getGroupName());
                                    j = 1-1; m = data.getUsersList().size();
                                    while (j <= (m-1)) {
                                        if (data.getUsersList().get(j).getGroupName().equals(oldGroupName)) {
                                            data.getUsersList().get(j).setGroupName(newGroupName);
                                        }
                                        j++;
                                    }
                                    this.endUpdate();
                                    this.thereAreChanges();
                                    JOptionPane.showMessageDialog(this, "Name of group \"" + oldGroupName + "\"" + " was changed to \"" + newGroupName + "\".", "Groupname was changed successfully", JOptionPane.INFORMATION_MESSAGE);
                                    newGroupName = null;
                                    break;
                                }
                                i++;
                            }
                        }
                    }
                }
            }
        }
}//GEN-LAST:event_btnRenameGroupActionPerformed

    private void btnDeleteGroupActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnDeleteGroupActionPerformed
        String groupName = ((String)(this.comboGroups.getSelectedItem()));
        if (groupName.equals(TFTPProtocol.EMPTY)) {
            JOptionPane.showMessageDialog(this, "No group selected! Cannot delete unnamed group.", "Cannot delete unnamed group", JOptionPane.ERROR_MESSAGE);
        } else {
            if (data.getUsersList().size() > 0) {
                int i = 1-1; int n = data.getUsersList().size();
                while (i <= (n-1)) {
                    if (data.getUsersList().get(i).getGroupName().equals(groupName)) {
                        JOptionPane.showMessageDialog(this, "One or more users are part of group \"" + groupName + "\"!", "Advance warning", JOptionPane.WARNING_MESSAGE);
                        break;
                    }
                    i++;
                }
            }
            if (groupName != null) {
                if (JOptionPane.showConfirmDialog(this, "Are you absolutely sure you want to delete group \"" + groupName + "\"??"
                        + "\r\nWarning: if you press yes, group \"" + groupName + "\" will be deleted permanently!", "Confirm group delete", JOptionPane.YES_NO_CANCEL_OPTION) == JOptionPane.YES_OPTION) {
                    if (data.getUsersList().size() > 0) {
                        int i = 1-1; int n = data.getUsersList().size();
                        while (i <= (n-1)) {
                            if (groupName.equals(data.getUsersList().get(i).getGroupName())) {
                                data.getUsersList().get(i).setGroupName(TFTPProtocol.EMPTY);
                            }
                            i++;
                        }
                    }
                    if (data.getGroupsList().size() > 0) {
                        int i = 1-1; int n = data.getGroupsList().size();
                        while (i <= (n-1)) {
                            if (groupName.equals(data.getGroupsList().get(i).getGroupName())) {
                                if (((String)(this.comboGroupMembership.getSelectedItem())).equals(groupName)) {
                                    this.comboGroupMembership.setSelectedItem(TFTPProtocol.EMPTY);
                                }
                                this.comboGroupMembership.removeItem(data.getGroupsList().get(i).getGroupName());
                                this.comboGroups.setSelectedItem(TFTPProtocol.EMPTY);
                                this.comboGroups.removeItem(data.getGroupsList().get(i).getGroupName());
                                data.getGroupsList().remove(data.getGroupsList().get(i));
                                this.thereAreChanges();
                                JOptionPane.showMessageDialog(this, "Group \"" + groupName + "\"" + " was deleted!", "Group was deleted successfully", JOptionPane.INFORMATION_MESSAGE);
                                break;
                            }
                            i++;
                        }
                    }
                    }
                }
        }        
}//GEN-LAST:event_btnDeleteGroupActionPerformed

    private void btnAddFolderActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnAddFolderActionPerformed
        JTextField aTextField = new JTextField();
        Dialogs.openDialog(aTextField, true, "Select the folder you want to add for \"" + ((String)(this.comboGroups.getSelectedItem())) + "\"");
        String directory = aTextField.getText();
        if ((new File(directory)).exists() == false) {
            JOptionPane.showMessageDialog(this, "Selected folder \"" + directory + "\" does not exist!!", "Selected folder does not exist", JOptionPane.ERROR_MESSAGE);
        } else {
            directory = Utils.validateDirectory(directory);
            FTPProtocolData.Folder folder = new FTPProtocolData.Folder();
            folder.setFolderName(directory);
            if (data.getGroupsList().size() > 0) {
                int i = 1-1; int n = data.getGroupsList().size();
                while (i <= (n-1)) {
                    if (data.getGroupsList().get(i).getGroupName().equals(((String)(this.comboGroups.getSelectedItem())))) {
                        if (data.getGroupsList().get(i).getFoldersList().size() >= 0) {
                            int j = 1-1; int m = data.getGroupsList().get(i).getFoldersList().size();
                            while (j <= (m-1)) {
                                if (directory.equals(Utils.validateDirectory(data.getGroupsList().get(i).getFoldersList().get(j).getFolderName()))) {
                                    JOptionPane.showMessageDialog(this, "Selected folder \"" + directory + "\" already in list!", "Selected folder already in list", JOptionPane.ERROR_MESSAGE);
                                    int k = 1-1; int o = this.comboFolders.getItemCount();
                                    while (k <= (o-1)) {
                                        if (directory.equals(Utils.validateDirectory(((String)(this.comboFolders.getItemAt(k)))))) {
                                            this.comboFolders.setSelectedIndex(k);
                                            this.onFolderSelected();
                                            break;
                                        }
                                        k++;
                                    }
                                    directory = null;
                                    break;
                                }
                                j++;
                            }
                        }
                        if (directory != null) {
                            data.getGroupsList().get(i).getFoldersList().add(folder);
                            this.comboFolders.addItem(folder.getFolderName());
                            this.comboOpeningFolder.addItem(folder.getFolderName());
                            this.comboFolders.setSelectedItem(folder.getFolderName());
                            this.thereAreChanges();
                            JOptionPane.showMessageDialog(this, "Folder \"" + directory + "\"" + " was added to the list!", "Folder was added successfully", JOptionPane.INFORMATION_MESSAGE);
                        }
                        break;
                    }
                    i++;
                }
            }
        }
}//GEN-LAST:event_btnAddFolderActionPerformed

    private void btnDeleteFolderActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnDeleteFolderActionPerformed
        String folderName = ((String)(this.comboFolders.getSelectedItem()));
        if (folderName.equals(TFTPProtocol.EMPTY)) {
            JOptionPane.showMessageDialog(this, "No folder selected! Cannot delete unnamed folder.", "Cannot delete unnamed folder", JOptionPane.ERROR_MESSAGE);
        } else {
            if (data.getUsersList().size() > 0) {
                int i = 1-1; int n = data.getUsersList().size();
                while (i <= (n-1)) {
                    if (data.getGroupsList().get(i).getGroupName().equals(((String)(this.comboGroups.getSelectedItem())))) {
                        if (data.getGroupsList().get(i).getFoldersList().size() > 0) {
                            int j = 1-1; int m = data.getGroupsList().get(i).getFoldersList().size();
                            while (j <= (m-1)) {
                                if (data.getGroupsList().get(i).getFoldersList().get(j).getFolderName().equals(folderName)) {
                                    if (JOptionPane.showConfirmDialog(this, "Are you absolutely sure you want to remove folder \"" + folderName + "\"??"
                                        + "\r\nWarning: if you press yes, folder \"" + folderName + "\" will be removed from the list!", "Confirm folder removal", JOptionPane.YES_NO_CANCEL_OPTION) == JOptionPane.YES_OPTION) {
                                        this.comboFolders.setSelectedItem(TFTPProtocol.EMPTY);
                                        this.comboFolders.removeItem(data.getGroupsList().get(i).getFoldersList().get(j).getFolderName());
                                        if (((String)(this.comboOpeningFolder.getSelectedItem())).equals(data.getGroupsList().get(i).getFoldersList().get(j).getFolderName())) {
                                            this.comboOpeningFolder.setSelectedItem(TFTPProtocol.EMPTY);
                                        }
                                        this.comboOpeningFolder.removeItem(data.getGroupsList().get(i).getFoldersList().get(j).getFolderName());
                                        if (data.getGroupsList().get(i).getOpeningDirectory().equals(data.getGroupsList().get(i).getFoldersList().get(j).getFolderName())) {
                                            data.getGroupsList().get(i).setOpeningDirectory(TFTPProtocol.EMPTY);
                                        }
                                        data.getGroupsList().get(i).getFoldersList().remove(data.getGroupsList().get(i).getFoldersList().get(j));
                                        this.thereAreChanges();
                                        JOptionPane.showMessageDialog(this, "Folder \"" + folderName + "\"" + " was removed from the list!", "Folder was removed successfully", JOptionPane.INFORMATION_MESSAGE);
                                    }
                                    break;
                                }
                                j++;
                            }
                        }
                        break;
                    }
                    i++;
                }
            }
        }
}//GEN-LAST:event_btnDeleteFolderActionPerformed

    private void btnSaveChangesActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnSaveChangesActionPerformed
        this.saveSettings();
        this.btnSaveChanges.setEnabled(false);
        this.btnReload.setEnabled(false);
}//GEN-LAST:event_btnSaveChangesActionPerformed

    private ServerThread fServerThread = null;

    public static class ServerThread extends Thread {
        public TFTPServer fTFTPServer = new TFTPServer();

        @Override
        public void run() {
            try {
                File configurationFile = new File(TFTPServer.getConfigurationFileName());
                if (configurationFile.exists()) {
                    this.fTFTPServer.loadSettings();
                    if (this.fTFTPServer.data().getLogEvents()) {
                        this.fTFTPServer.startLogging();
                    }
                    this.fTFTPServer.activate("0.0.0.0", this.fTFTPServer.data().getServerPort());
                }
            } catch (Exception e) {
                System.out.println("ERROR " + e);
            }
            while (!this.isInterrupted()) {
                try {
                    Thread.sleep(2000);
                } catch (Exception e) {}
            }
            this.fTFTPServer.stopLogging();
        }
    }

    private void btnStartServerActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnStartServerActionPerformed
        if (this.fServerThread == null) {
            this.fServerThread = new ServerThread();            
        }
        if (this.fServerThread != null) {
            if (this.fServerThread.fTFTPServer != null) {
                this.fServerThread.start();
                this.btnStartServer.setEnabled(false);
                this.btnStopServer.setEnabled(true);
                //this.DaemonServer.setEnabled(true);                
            }
        }
}//GEN-LAST:event_btnStartServerActionPerformed

    private void btnStopServerActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnStopServerActionPerformed
        if (this.fServerThread != null) {
            if (this.fServerThread.fTFTPServer != null) {
                try {this.fServerThread.fTFTPServer.deactivateNow();} catch (Exception e) {}
                try {Thread.sleep(2000);} catch (Exception e) {}
                try {this.fServerThread.fTFTPServer.deactivateNow();} catch (Exception e) {}
                this.fServerThread.interrupt();
                this.fServerThread.fTFTPServer = null;
                this.fServerThread = null;
                this.btnStartServer.setEnabled(true);
                this.btnStopServer.setEnabled(false);
                    //this.DaemonServer.setEnabled(false);

            }
        }
}//GEN-LAST:event_btnStopServerActionPerformed

    private void cbTrackChangesActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbTrackChangesActionPerformed
        if (this.fUpdating) {return;}
        if (data.getTrackChanges() != this.cbTrackChanges.isSelected()) {
            data.setTrackChanges(this.cbTrackChanges.isSelected());
            this.thereAreChanges();
        }
}//GEN-LAST:event_cbTrackChangesActionPerformed

    private void cbIsAllowedToSeeFilesActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbIsAllowedToSeeFilesActionPerformed
        if (this.fUpdating) {return;}
        if ((this.comboGroups.getItemCount() > 0) && (this.comboFolders.getItemCount() > 0)) {
            String groupName = ((String)(this.comboGroups.getSelectedItem()));
            String folderName = ((String)(this.comboFolders.getSelectedItem()));
            if (data.folderExists(groupName, folderName)) {
                TFTPProtocol.FTPProtocolData.Folder folder = data.getFolder(groupName, folderName);
                if (folder.getIsAllowedToSeeFiles() != this.cbIsAllowedToSeeFiles.isSelected()) {
                    folder.setIsAllowedToSeeFiles(this.cbIsAllowedToSeeFiles.isSelected());
                    this.thereAreChanges();
                }
            }
        }
    }//GEN-LAST:event_cbIsAllowedToSeeFilesActionPerformed

    private void cbIsAllowedToReadFilesActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbIsAllowedToReadFilesActionPerformed
        if (this.fUpdating) {return;}
        if ((this.comboGroups.getItemCount() > 0) && (this.comboFolders.getItemCount() > 0)) {
            String groupName = ((String)(this.comboGroups.getSelectedItem()));
            String folderName = ((String)(this.comboFolders.getSelectedItem()));
            if (data.folderExists(groupName, folderName)) {
                TFTPProtocol.FTPProtocolData.Folder folder = data.getFolder(groupName, folderName);
                if (folder.getIsAllowedToReadFiles() != this.cbIsAllowedToReadFiles.isSelected()) {
                    folder.setIsAllowedToReadFiles(this.cbIsAllowedToReadFiles.isSelected());
                    this.thereAreChanges();
                }
            }
        }
    }//GEN-LAST:event_cbIsAllowedToReadFilesActionPerformed

    private void cbIsAllowedToRenameFilesActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbIsAllowedToRenameFilesActionPerformed
        if (this.fUpdating) {return;}
        if ((this.comboGroups.getItemCount() > 0) && (this.comboFolders.getItemCount() > 0)) {
            String groupName = ((String)(this.comboGroups.getSelectedItem()));
            String folderName = ((String)(this.comboFolders.getSelectedItem()));
            if (data.folderExists(groupName, folderName)) {
                TFTPProtocol.FTPProtocolData.Folder folder = data.getFolder(groupName, folderName);
                if (folder.getIsAllowedToRenameFiles() != this.cbIsAllowedToRenameFiles.isSelected()) {
                    folder.setIsAllowedToRenameFiles(this.cbIsAllowedToRenameFiles.isSelected());
                    this.thereAreChanges();
                }
            }
        }
    }//GEN-LAST:event_cbIsAllowedToRenameFilesActionPerformed

    private void cbIsAllowedToAppendToFilesActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbIsAllowedToAppendToFilesActionPerformed
        if (this.fUpdating) {return;}
        if ((this.comboGroups.getItemCount() > 0) && (this.comboFolders.getItemCount() > 0)) {
            String groupName = ((String)(this.comboGroups.getSelectedItem()));
            String folderName = ((String)(this.comboFolders.getSelectedItem()));
            if (data.folderExists(groupName, folderName)) {
                TFTPProtocol.FTPProtocolData.Folder folder = data.getFolder(groupName, folderName);
                if (folder.getIsAllowedToAppendToFiles() != this.cbIsAllowedToAppendToFiles.isSelected()) {
                    folder.setIsAllowedToAppendToFiles(this.cbIsAllowedToAppendToFiles.isSelected());
                    this.thereAreChanges();
                }
            }
        }
    }//GEN-LAST:event_cbIsAllowedToAppendToFilesActionPerformed

    private void cbIsAllowedToCreateNewFilesActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbIsAllowedToCreateNewFilesActionPerformed
        if (this.fUpdating) {return;}
        if ((this.comboGroups.getItemCount() > 0) && (this.comboFolders.getItemCount() > 0)) {
            String groupName = ((String)(this.comboGroups.getSelectedItem()));
            String folderName = ((String)(this.comboFolders.getSelectedItem()));
            if (data.folderExists(groupName, folderName)) {
                TFTPProtocol.FTPProtocolData.Folder folder = data.getFolder(groupName, folderName);
                if (folder.getIsAllowedToCreateNewFiles() != this.cbIsAllowedToCreateNewFiles.isSelected()) {
                    folder.setIsAllowedToCreateNewFiles(this.cbIsAllowedToCreateNewFiles.isSelected());
                    this.thereAreChanges();
                }
            }
        }
    }//GEN-LAST:event_cbIsAllowedToCreateNewFilesActionPerformed

    private void cbIsAllowedToOverwriteFilesActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbIsAllowedToOverwriteFilesActionPerformed
        if (this.fUpdating) {return;}
        if ((this.comboGroups.getItemCount() > 0) && (this.comboFolders.getItemCount() > 0)) {
            String groupName = ((String)(this.comboGroups.getSelectedItem()));
            String folderName = ((String)(this.comboFolders.getSelectedItem()));
            if (data.folderExists(groupName, folderName)) {
                TFTPProtocol.FTPProtocolData.Folder folder = data.getFolder(groupName, folderName);
                if (folder.getIsAllowedToOverwriteFiles() != this.cbIsAllowedToOverwriteFiles.isSelected()) {
                    folder.setIsAllowedToOverwriteFiles(this.cbIsAllowedToOverwriteFiles.isSelected());
                    this.thereAreChanges();
                }
            }
        }
    }//GEN-LAST:event_cbIsAllowedToOverwriteFilesActionPerformed

    private void cbIsAllowedToDeleteFilesActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbIsAllowedToDeleteFilesActionPerformed
        if (this.fUpdating) {return;}
        if ((this.comboGroups.getItemCount() > 0) && (this.comboFolders.getItemCount() > 0)) {
            String groupName = ((String)(this.comboGroups.getSelectedItem()));
            String folderName = ((String)(this.comboFolders.getSelectedItem()));
            if (data.folderExists(groupName, folderName)) {
                TFTPProtocol.FTPProtocolData.Folder folder = data.getFolder(groupName, folderName);
                if (folder.getIsAllowedToDeleteFiles() != this.cbIsAllowedToDeleteFiles.isSelected()) {
                    folder.setIsAllowedToDeleteFiles(this.cbIsAllowedToDeleteFiles.isSelected());
                    this.thereAreChanges();
                }
            }
        }
    }//GEN-LAST:event_cbIsAllowedToDeleteFilesActionPerformed

    private void cbIsAllowedToSeeSubDirectoriesActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbIsAllowedToSeeSubDirectoriesActionPerformed
        if (this.fUpdating) {return;}
        if ((this.comboGroups.getItemCount() > 0) && (this.comboFolders.getItemCount() > 0)) {
            String groupName = ((String)(this.comboGroups.getSelectedItem()));
            String folderName = ((String)(this.comboFolders.getSelectedItem()));
            if (data.folderExists(groupName, folderName)) {
                TFTPProtocol.FTPProtocolData.Folder folder = data.getFolder(groupName, folderName);
                if (folder.getIsAllowedToSeeSubDirectories() != this.cbIsAllowedToSeeSubDirectories.isSelected()) {
                    folder.setIsAllowedToSeeSubDirectories(this.cbIsAllowedToSeeSubDirectories.isSelected());
                    this.thereAreChanges();
                }
            }
        }
    }//GEN-LAST:event_cbIsAllowedToSeeSubDirectoriesActionPerformed

    private void cbIsAllowedToRenameSubDirectoriesActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbIsAllowedToRenameSubDirectoriesActionPerformed
        if (this.fUpdating) {return;}
        if ((this.comboGroups.getItemCount() > 0) && (this.comboFolders.getItemCount() > 0)) {
            String groupName = ((String)(this.comboGroups.getSelectedItem()));
            String folderName = ((String)(this.comboFolders.getSelectedItem()));
            if (data.folderExists(groupName, folderName)) {
                TFTPProtocol.FTPProtocolData.Folder folder = data.getFolder(groupName, folderName);
                if (folder.getIsAllowedToRenameSubDirectories() != this.cbIsAllowedToRenameSubDirectories.isSelected()) {
                    folder.setIsAllowedToRenameSubDirectories(this.cbIsAllowedToRenameSubDirectories.isSelected());
                    this.thereAreChanges();
                }
            }
        }
    }//GEN-LAST:event_cbIsAllowedToRenameSubDirectoriesActionPerformed

    private void cbIsAllowedToMakeSubDirectoriesActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbIsAllowedToMakeSubDirectoriesActionPerformed
        if (this.fUpdating) {return;}
        if ((this.comboGroups.getItemCount() > 0) && (this.comboFolders.getItemCount() > 0)) {
            String groupName = ((String)(this.comboGroups.getSelectedItem()));
            String folderName = ((String)(this.comboFolders.getSelectedItem()));
            if (data.folderExists(groupName, folderName)) {
                TFTPProtocol.FTPProtocolData.Folder folder = data.getFolder(groupName, folderName);
                if (folder.getIsAllowedToMakeSubDirectories() != this.cbIsAllowedToMakeSubDirectories.isSelected()) {
                    folder.setIsAllowedToMakeSubDirectories(this.cbIsAllowedToMakeSubDirectories.isSelected());
                    this.thereAreChanges();
                }
            }
        }
    }//GEN-LAST:event_cbIsAllowedToMakeSubDirectoriesActionPerformed

    private void cbIsAllowedToRemoveSubDirectoriesActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbIsAllowedToRemoveSubDirectoriesActionPerformed
        if (this.fUpdating) {return;}
        if ((this.comboGroups.getItemCount() > 0) && (this.comboFolders.getItemCount() > 0)) {
            String groupName = ((String)(this.comboGroups.getSelectedItem()));
            String folderName = ((String)(this.comboFolders.getSelectedItem()));
            if (data.folderExists(groupName, folderName)) {
                TFTPProtocol.FTPProtocolData.Folder folder = data.getFolder(groupName, folderName);
                if (folder.getIsAllowedToRemoveSubDirectories() != this.cbIsAllowedToRemoveSubDirectories.isSelected()) {
                    folder.setIsAllowedToRemoveSubDirectories(this.cbIsAllowedToRemoveSubDirectories.isSelected());
                    this.thereAreChanges();
                }
            }
        }
    }//GEN-LAST:event_cbIsAllowedToRemoveSubDirectoriesActionPerformed

    private void cbIsAllowedToSeeHiddenFilesActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbIsAllowedToSeeHiddenFilesActionPerformed
        if (this.fUpdating) {return;}
        if ((this.comboGroups.getItemCount() > 0) && (this.comboFolders.getItemCount() > 0)) {
            String groupName = ((String)(this.comboGroups.getSelectedItem()));
            String folderName = ((String)(this.comboFolders.getSelectedItem()));
            if (data.folderExists(groupName, folderName)) {
                TFTPProtocol.FTPProtocolData.Folder folder = data.getFolder(groupName, folderName);
                if (folder.getIsAllowedToSeeHiddenFiles() != this.cbIsAllowedToSeeHiddenFiles.isSelected()) {
                    folder.setIsAllowedToSeeHiddenFiles(this.cbIsAllowedToSeeHiddenFiles.isSelected());
                    this.thereAreChanges();
                }
            }
        }
    }//GEN-LAST:event_cbIsAllowedToSeeHiddenFilesActionPerformed

    private void cbIsAllowedToReadHiddenFilesActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbIsAllowedToReadHiddenFilesActionPerformed
        if (this.fUpdating) {return;}
        if ((this.comboGroups.getItemCount() > 0) && (this.comboFolders.getItemCount() > 0)) {
            String groupName = ((String)(this.comboGroups.getSelectedItem()));
            String folderName = ((String)(this.comboFolders.getSelectedItem()));
            if (data.folderExists(groupName, folderName)) {
                TFTPProtocol.FTPProtocolData.Folder folder = data.getFolder(groupName, folderName);
                if (folder.getIsAllowedToReadHiddenFiles() != this.cbIsAllowedToReadHiddenFiles.isSelected()) {
                    folder.setIsAllowedToReadHiddenFiles(this.cbIsAllowedToReadHiddenFiles.isSelected());
                    this.thereAreChanges();
                }
            }
        }
    }//GEN-LAST:event_cbIsAllowedToReadHiddenFilesActionPerformed

    private void cbIsAllowedToRenameHiddenFilesActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbIsAllowedToRenameHiddenFilesActionPerformed
        if (this.fUpdating) {return;}
        if ((this.comboGroups.getItemCount() > 0) && (this.comboFolders.getItemCount() > 0)) {
            String groupName = ((String)(this.comboGroups.getSelectedItem()));
            String folderName = ((String)(this.comboFolders.getSelectedItem()));
            if (data.folderExists(groupName, folderName)) {
                TFTPProtocol.FTPProtocolData.Folder folder = data.getFolder(groupName, folderName);
                if (folder.getIsAllowedToRenameHiddenFiles() != this.cbIsAllowedToRenameHiddenFiles.isSelected()) {
                    folder.setIsAllowedToRenameHiddenFiles(this.cbIsAllowedToRenameHiddenFiles.isSelected());
                    this.thereAreChanges();
                }
            }
        }
    }//GEN-LAST:event_cbIsAllowedToRenameHiddenFilesActionPerformed

    private void cbIsAllowedToAppendToHiddenFilesActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbIsAllowedToAppendToHiddenFilesActionPerformed
        if (this.fUpdating) {return;}
        if ((this.comboGroups.getItemCount() > 0) && (this.comboFolders.getItemCount() > 0)) {
            String groupName = ((String)(this.comboGroups.getSelectedItem()));
            String folderName = ((String)(this.comboFolders.getSelectedItem()));
            if (data.folderExists(groupName, folderName)) {
                TFTPProtocol.FTPProtocolData.Folder folder = data.getFolder(groupName, folderName);
                if (folder.getIsAllowedToAppendToHiddenFiles() != this.cbIsAllowedToAppendToHiddenFiles.isSelected()) {
                    folder.setIsAllowedToAppendToHiddenFiles(this.cbIsAllowedToAppendToHiddenFiles.isSelected());
                    this.thereAreChanges();
                }
            }
        }
    }//GEN-LAST:event_cbIsAllowedToAppendToHiddenFilesActionPerformed

    private void cbIsAllowedToCreateNewHiddenFilesActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbIsAllowedToCreateNewHiddenFilesActionPerformed
        if (this.fUpdating) {return;}
        if ((this.comboGroups.getItemCount() > 0) && (this.comboFolders.getItemCount() > 0)) {
            String groupName = ((String)(this.comboGroups.getSelectedItem()));
            String folderName = ((String)(this.comboFolders.getSelectedItem()));
            if (data.folderExists(groupName, folderName)) {
                TFTPProtocol.FTPProtocolData.Folder folder = data.getFolder(groupName, folderName);
                if (folder.getIsAllowedToCreateNewHiddenFiles() != this.cbIsAllowedToCreateNewHiddenFiles.isSelected()) {
                    folder.setIsAllowedToCreateNewHiddenFiles(this.cbIsAllowedToCreateNewHiddenFiles.isSelected());
                    this.thereAreChanges();
                }
            }
        }
    }//GEN-LAST:event_cbIsAllowedToCreateNewHiddenFilesActionPerformed

    private void cbIsAllowedToOverwriteHiddenFilesActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbIsAllowedToOverwriteHiddenFilesActionPerformed
        if (this.fUpdating) {return;}
        if ((this.comboGroups.getItemCount() > 0) && (this.comboFolders.getItemCount() > 0)) {
            String groupName = ((String)(this.comboGroups.getSelectedItem()));
            String folderName = ((String)(this.comboFolders.getSelectedItem()));
            if (data.folderExists(groupName, folderName)) {
                TFTPProtocol.FTPProtocolData.Folder folder = data.getFolder(groupName, folderName);
                if (folder.getIsAllowedToOverwriteHiddenFiles() != this.cbIsAllowedToOverwriteHiddenFiles.isSelected()) {
                    folder.setIsAllowedToOverwriteHiddenFiles(this.cbIsAllowedToOverwriteHiddenFiles.isSelected());
                    this.thereAreChanges();
                }
            }
        }
    }//GEN-LAST:event_cbIsAllowedToOverwriteHiddenFilesActionPerformed

    private void cbIsAllowedToDeleteHiddenFilesActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbIsAllowedToDeleteHiddenFilesActionPerformed
        if (this.fUpdating) {return;}
        if ((this.comboGroups.getItemCount() > 0) && (this.comboFolders.getItemCount() > 0)) {
            String groupName = ((String)(this.comboGroups.getSelectedItem()));
            String folderName = ((String)(this.comboFolders.getSelectedItem()));
            if (data.folderExists(groupName, folderName)) {
                TFTPProtocol.FTPProtocolData.Folder folder = data.getFolder(groupName, folderName);
                if (folder.getIsAllowedToDeleteHiddenFiles() != this.cbIsAllowedToDeleteHiddenFiles.isSelected()) {
                    folder.setIsAllowedToDeleteHiddenFiles(this.cbIsAllowedToDeleteHiddenFiles.isSelected());
                    this.thereAreChanges();
                }
            }
        }
    }//GEN-LAST:event_cbIsAllowedToDeleteHiddenFilesActionPerformed

    private void cbIsAllowedToSeeHiddenSubDirectoriesActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbIsAllowedToSeeHiddenSubDirectoriesActionPerformed
        if (this.fUpdating) {return;}
        if ((this.comboGroups.getItemCount() > 0) && (this.comboFolders.getItemCount() > 0)) {
            String groupName = ((String)(this.comboGroups.getSelectedItem()));
            String folderName = ((String)(this.comboFolders.getSelectedItem()));
            if (data.folderExists(groupName, folderName)) {
                TFTPProtocol.FTPProtocolData.Folder folder = data.getFolder(groupName, folderName);
                if (folder.getIsAllowedToSeeHiddenSubDirectories() != this.cbIsAllowedToSeeHiddenSubDirectories.isSelected()) {
                    folder.setIsAllowedToSeeHiddenSubDirectories(this.cbIsAllowedToSeeHiddenSubDirectories.isSelected());
                    this.thereAreChanges();
                }
            }
        }
    }//GEN-LAST:event_cbIsAllowedToSeeHiddenSubDirectoriesActionPerformed

    private void cbIsAllowedToRenameHiddenSubDirectoriesActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbIsAllowedToRenameHiddenSubDirectoriesActionPerformed
        if (this.fUpdating) {return;}
        if ((this.comboGroups.getItemCount() > 0) && (this.comboFolders.getItemCount() > 0)) {
            String groupName = ((String)(this.comboGroups.getSelectedItem()));
            String folderName = ((String)(this.comboFolders.getSelectedItem()));
            if (data.folderExists(groupName, folderName)) {
                TFTPProtocol.FTPProtocolData.Folder folder = data.getFolder(groupName, folderName);
                if (folder.getIsAllowedToRenameHiddenSubDirectories() != this.cbIsAllowedToRenameHiddenSubDirectories.isSelected()) {
                    folder.setIsAllowedToRenameHiddenSubDirectories(this.cbIsAllowedToRenameHiddenSubDirectories.isSelected());
                    this.thereAreChanges();
                }
            }
        }
    }//GEN-LAST:event_cbIsAllowedToRenameHiddenSubDirectoriesActionPerformed

    private void cbIsAllowedToMakeHiddenSubDirectoriesActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbIsAllowedToMakeHiddenSubDirectoriesActionPerformed
        if (this.fUpdating) {return;}
        if ((this.comboGroups.getItemCount() > 0) && (this.comboFolders.getItemCount() > 0)) {
            String groupName = ((String)(this.comboGroups.getSelectedItem()));
            String folderName = ((String)(this.comboFolders.getSelectedItem()));
            if (data.folderExists(groupName, folderName)) {
                TFTPProtocol.FTPProtocolData.Folder folder = data.getFolder(groupName, folderName);
                if (folder.getIsAllowedToMakeHiddenSubDirectories() != this.cbIsAllowedToMakeHiddenSubDirectories.isSelected()) {
                    folder.setIsAllowedToMakeHiddenSubDirectories(this.cbIsAllowedToMakeHiddenSubDirectories.isSelected());
                    this.thereAreChanges();
                }
            }
        }
    }//GEN-LAST:event_cbIsAllowedToMakeHiddenSubDirectoriesActionPerformed

    private void cbIsAllowedToRemoveHiddenSubDirectoriesActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbIsAllowedToRemoveHiddenSubDirectoriesActionPerformed
        if (this.fUpdating) {return;}
        if ((this.comboGroups.getItemCount() > 0) && (this.comboFolders.getItemCount() > 0)) {
            String groupName = ((String)(this.comboGroups.getSelectedItem()));
            String folderName = ((String)(this.comboFolders.getSelectedItem()));
            if (data.folderExists(groupName, folderName)) {
                TFTPProtocol.FTPProtocolData.Folder folder = data.getFolder(groupName, folderName);
                if (folder.getIsAllowedToRemoveHiddenSubDirectories() != this.cbIsAllowedToRemoveHiddenSubDirectories.isSelected()) {
                    folder.setIsAllowedToRemoveHiddenSubDirectories(this.cbIsAllowedToRemoveHiddenSubDirectories.isSelected());
                    this.thereAreChanges();
                }
            }
        }
    }//GEN-LAST:event_cbIsAllowedToRemoveHiddenSubDirectoriesActionPerformed

    private void cbIsAllowedToTraverseSubDirectoriesActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbIsAllowedToTraverseSubDirectoriesActionPerformed
        if (this.fUpdating) {return;}
        if ((this.comboGroups.getItemCount() > 0) && (this.comboFolders.getItemCount() > 0)) {
            String groupName = ((String)(this.comboGroups.getSelectedItem()));
            String folderName = ((String)(this.comboFolders.getSelectedItem()));
            if (data.folderExists(groupName, folderName)) {
                TFTPProtocol.FTPProtocolData.Folder folder = data.getFolder(groupName, folderName);
                if (folder.getIsAllowedToTraverseSubDirectories() != this.cbIsAllowedToTraverseSubDirectories.isSelected()) {
                    folder.setIsAllowedToTraverseSubDirectories(this.cbIsAllowedToTraverseSubDirectories.isSelected());
                    this.thereAreChanges();
                }
            }
        }
    }//GEN-LAST:event_cbIsAllowedToTraverseSubDirectoriesActionPerformed

    private void cbIsAllowedToTraverseHiddenSubDirectoriesActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbIsAllowedToTraverseHiddenSubDirectoriesActionPerformed
        if (this.fUpdating) {return;}
        if ((this.comboGroups.getItemCount() > 0) && (this.comboFolders.getItemCount() > 0)) {
            String groupName = ((String)(this.comboGroups.getSelectedItem()));
            String folderName = ((String)(this.comboFolders.getSelectedItem()));
            if (data.folderExists(groupName, folderName)) {
                TFTPProtocol.FTPProtocolData.Folder folder = data.getFolder(groupName, folderName);
                if (folder.getIsAllowedToTraverseHiddenSubDirectories() != this.cbIsAllowedToTraverseHiddenSubDirectories.isSelected()) {
                    folder.setIsAllowedToTraverseHiddenSubDirectories(this.cbIsAllowedToTraverseHiddenSubDirectories.isSelected());
                    this.thereAreChanges();
                }
            }
        }
    }//GEN-LAST:event_cbIsAllowedToTraverseHiddenSubDirectoriesActionPerformed

    private void cbUserRightsFilesSelectAllActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbUserRightsFilesSelectAllActionPerformed
        if (this.fUpdating) {return;}
        this.startUpdate();
            this.cbUserRightsFilesSelectAll.setSelected(false);
        this.endUpdate();

        this.cbIsAllowedToSeeFiles.setSelected(true);
        this.cbIsAllowedToReadFiles.setSelected(true);
        this.cbIsAllowedToRenameFiles.setSelected(true);
        this.cbIsAllowedToAppendToFiles.setSelected(true);
        this.cbIsAllowedToCreateNewFiles.setSelected(true);
        this.cbIsAllowedToOverwriteFiles.setSelected(true);
        this.cbIsAllowedToDeleteFiles.setSelected(true);

        this.cbIsAllowedToSeeFilesActionPerformed(evt);
        this.cbIsAllowedToReadFilesActionPerformed(evt);
        this.cbIsAllowedToRenameFilesActionPerformed(evt);
        this.cbIsAllowedToAppendToFilesActionPerformed(evt);
        this.cbIsAllowedToCreateNewFilesActionPerformed(evt);
        this.cbIsAllowedToOverwriteFilesActionPerformed(evt);
        this.cbIsAllowedToDeleteFilesActionPerformed(evt);
    }//GEN-LAST:event_cbUserRightsFilesSelectAllActionPerformed

    private void cbUserRightsFilesDeselectAllActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbUserRightsFilesDeselectAllActionPerformed
        if (this.fUpdating) {return;}
        this.startUpdate();
            this.cbUserRightsFilesDeselectAll.setSelected(false);
        this.endUpdate();

        this.cbIsAllowedToSeeFiles.setSelected(false);
        this.cbIsAllowedToReadFiles.setSelected(false);
        this.cbIsAllowedToRenameFiles.setSelected(false);
        this.cbIsAllowedToAppendToFiles.setSelected(false);
        this.cbIsAllowedToCreateNewFiles.setSelected(false);
        this.cbIsAllowedToOverwriteFiles.setSelected(false);
        this.cbIsAllowedToDeleteFiles.setSelected(false);

        this.cbIsAllowedToSeeFilesActionPerformed(evt);
        this.cbIsAllowedToReadFilesActionPerformed(evt);
        this.cbIsAllowedToRenameFilesActionPerformed(evt);
        this.cbIsAllowedToAppendToFilesActionPerformed(evt);
        this.cbIsAllowedToCreateNewFilesActionPerformed(evt);
        this.cbIsAllowedToOverwriteFilesActionPerformed(evt);
        this.cbIsAllowedToDeleteFilesActionPerformed(evt);
    }//GEN-LAST:event_cbUserRightsFilesDeselectAllActionPerformed

    private void cbUserRightsHiddenFilesSelectAllActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbUserRightsHiddenFilesSelectAllActionPerformed
        if (this.fUpdating) {return;}
        this.startUpdate();
            this.cbUserRightsHiddenFilesSelectAll.setSelected(false);
        this.endUpdate();

        this.cbIsAllowedToSeeHiddenFiles.setSelected(true);
        this.cbIsAllowedToReadHiddenFiles.setSelected(true);
        this.cbIsAllowedToRenameHiddenFiles.setSelected(true);
        this.cbIsAllowedToAppendToHiddenFiles.setSelected(true);
        this.cbIsAllowedToCreateNewHiddenFiles.setSelected(true);
        this.cbIsAllowedToOverwriteHiddenFiles.setSelected(true);
        this.cbIsAllowedToDeleteHiddenFiles.setSelected(true);

        this.cbIsAllowedToSeeHiddenFilesActionPerformed(evt);
        this.cbIsAllowedToReadHiddenFilesActionPerformed(evt);
        this.cbIsAllowedToRenameHiddenFilesActionPerformed(evt);
        this.cbIsAllowedToAppendToHiddenFilesActionPerformed(evt);
        this.cbIsAllowedToCreateNewHiddenFilesActionPerformed(evt);
        this.cbIsAllowedToOverwriteHiddenFilesActionPerformed(evt);
        this.cbIsAllowedToDeleteHiddenFilesActionPerformed(evt);
    }//GEN-LAST:event_cbUserRightsHiddenFilesSelectAllActionPerformed

    private void cbUserRightsHiddenFilesDeselectAllActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbUserRightsHiddenFilesDeselectAllActionPerformed
        if (this.fUpdating) {return;}
        this.startUpdate();
            this.cbUserRightsHiddenFilesDeselectAll.setSelected(false);
        this.endUpdate();

        this.cbIsAllowedToSeeHiddenFiles.setSelected(false);
        this.cbIsAllowedToReadHiddenFiles.setSelected(false);
        this.cbIsAllowedToRenameHiddenFiles.setSelected(false);
        this.cbIsAllowedToAppendToHiddenFiles.setSelected(false);
        this.cbIsAllowedToCreateNewHiddenFiles.setSelected(false);
        this.cbIsAllowedToOverwriteHiddenFiles.setSelected(false);
        this.cbIsAllowedToDeleteHiddenFiles.setSelected(false);

        this.cbIsAllowedToSeeHiddenFilesActionPerformed(evt);
        this.cbIsAllowedToReadHiddenFilesActionPerformed(evt);
        this.cbIsAllowedToRenameHiddenFilesActionPerformed(evt);
        this.cbIsAllowedToAppendToHiddenFilesActionPerformed(evt);
        this.cbIsAllowedToCreateNewHiddenFilesActionPerformed(evt);
        this.cbIsAllowedToOverwriteHiddenFilesActionPerformed(evt);
        this.cbIsAllowedToDeleteHiddenFilesActionPerformed(evt);
    }//GEN-LAST:event_cbUserRightsHiddenFilesDeselectAllActionPerformed

    private void cbUserRightsSubDirectoriesSelectAllActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbUserRightsSubDirectoriesSelectAllActionPerformed
        if (this.fUpdating) {return;}
        this.startUpdate();
            this.cbUserRightsSubDirectoriesSelectAll.setSelected(false);
        this.endUpdate();

        this.cbIsAllowedToSeeSubDirectories.setSelected(true);
        this.cbIsAllowedToTraverseSubDirectories.setSelected(true);
        this.cbIsAllowedToRenameSubDirectories.setSelected(true);
        this.cbIsAllowedToMakeSubDirectories.setSelected(true);
        this.cbIsAllowedToRemoveSubDirectories.setSelected(true);

        this.cbIsAllowedToSeeSubDirectoriesActionPerformed(evt);
        this.cbIsAllowedToTraverseSubDirectoriesActionPerformed(evt);
        this.cbIsAllowedToRenameSubDirectoriesActionPerformed(evt);
        this.cbIsAllowedToMakeSubDirectoriesActionPerformed(evt);
        this.cbIsAllowedToRemoveSubDirectoriesActionPerformed(evt);
    }//GEN-LAST:event_cbUserRightsSubDirectoriesSelectAllActionPerformed

    private void cbUserRightsSubDirectoriesDeselectAllActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbUserRightsSubDirectoriesDeselectAllActionPerformed
        if (this.fUpdating) {return;}
        this.startUpdate();
            this.cbUserRightsSubDirectoriesDeselectAll.setSelected(false);
        this.endUpdate();

        this.cbIsAllowedToSeeSubDirectories.setSelected(false);
        this.cbIsAllowedToTraverseSubDirectories.setSelected(false);
        this.cbIsAllowedToRenameSubDirectories.setSelected(false);
        this.cbIsAllowedToMakeSubDirectories.setSelected(false);
        this.cbIsAllowedToRemoveSubDirectories.setSelected(false);

        this.cbIsAllowedToSeeSubDirectoriesActionPerformed(evt);
        this.cbIsAllowedToTraverseSubDirectoriesActionPerformed(evt);
        this.cbIsAllowedToRenameSubDirectoriesActionPerformed(evt);
        this.cbIsAllowedToMakeSubDirectoriesActionPerformed(evt);
        this.cbIsAllowedToRemoveSubDirectoriesActionPerformed(evt);
    }//GEN-LAST:event_cbUserRightsSubDirectoriesDeselectAllActionPerformed

    private void cbUserRightsHiddenSubDirectoriesSelectAllActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbUserRightsHiddenSubDirectoriesSelectAllActionPerformed
        if (this.fUpdating) {return;}
        this.startUpdate();
            this.cbUserRightsHiddenSubDirectoriesSelectAll.setSelected(false);
        this.endUpdate();

        this.cbIsAllowedToSeeHiddenSubDirectories.setSelected(true);
        this.cbIsAllowedToTraverseHiddenSubDirectories.setSelected(true);
        this.cbIsAllowedToRenameHiddenSubDirectories.setSelected(true);
        this.cbIsAllowedToMakeHiddenSubDirectories.setSelected(true);
        this.cbIsAllowedToRemoveHiddenSubDirectories.setSelected(true);

        this.cbIsAllowedToSeeHiddenSubDirectoriesActionPerformed(evt);
        this.cbIsAllowedToTraverseHiddenSubDirectoriesActionPerformed(evt);
        this.cbIsAllowedToRenameHiddenSubDirectoriesActionPerformed(evt);
        this.cbIsAllowedToMakeHiddenSubDirectoriesActionPerformed(evt);
        this.cbIsAllowedToRemoveHiddenSubDirectoriesActionPerformed(evt);
    }//GEN-LAST:event_cbUserRightsHiddenSubDirectoriesSelectAllActionPerformed

    private void cbUserRightsHiddenSubDirectoriesDeselectAllActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbUserRightsHiddenSubDirectoriesDeselectAllActionPerformed
        if (this.fUpdating) {return;}
        this.startUpdate();
            this.cbUserRightsHiddenSubDirectoriesDeselectAll.setSelected(false);
        this.endUpdate();

        this.cbIsAllowedToSeeHiddenSubDirectories.setSelected(false);
        this.cbIsAllowedToTraverseHiddenSubDirectories.setSelected(false);
        this.cbIsAllowedToRenameHiddenSubDirectories.setSelected(false);
        this.cbIsAllowedToMakeHiddenSubDirectories.setSelected(false);
        this.cbIsAllowedToRemoveHiddenSubDirectories.setSelected(false);

        this.cbIsAllowedToSeeHiddenSubDirectoriesActionPerformed(evt);
        this.cbIsAllowedToTraverseHiddenSubDirectoriesActionPerformed(evt);
        this.cbIsAllowedToRenameHiddenSubDirectoriesActionPerformed(evt);
        this.cbIsAllowedToMakeHiddenSubDirectoriesActionPerformed(evt);
        this.cbIsAllowedToRemoveHiddenSubDirectoriesActionPerformed(evt);
    }//GEN-LAST:event_cbUserRightsHiddenSubDirectoriesDeselectAllActionPerformed

    private void btnReloadActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnReloadActionPerformed
        this.loadSettings();
        this.btnReload.setEnabled(false);
        this.btnSaveChanges.setEnabled(false);
        JOptionPane.showMessageDialog(this, "Settings were reread from disk.", "Settings reload", JOptionPane.INFORMATION_MESSAGE);
    }//GEN-LAST:event_btnReloadActionPerformed

    private void btnCopyGroupActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnCopyGroupActionPerformed
        FTPProtocolData.Group oldGroup = this.data.getGroup((String)(this.comboGroups.getSelectedItem()));
        if (oldGroup == null) {JOptionPane.showMessageDialog(this, "Group \"" + ((String)(this.comboGroups.getSelectedItem())) + "\"" + " does not exist!", "Invalid groupname", JOptionPane.ERROR_MESSAGE); return;}
        String oldGroupName = oldGroup.getGroupName();

        String newGroupName = JOptionPane.showInputDialog(this, "Enter new groupname", "Copy existing group", JOptionPane.QUESTION_MESSAGE);
        if (newGroupName == null) {return;} //user cancelled
        if (this.data.groupExists(newGroupName)) {JOptionPane.showMessageDialog(this, "Group \"" + newGroupName + "\"" + " already exists!", "Duplicate name error", JOptionPane.ERROR_MESSAGE); return;}

        FTPProtocolData.Group newGroup = new FTPProtocolData.Group();

        this.startUpdate();

        newGroup.setGroupName(newGroupName);
        newGroup.setOpeningDirectory(oldGroup.getOpeningDirectory());

        int i = 1-1; int n = oldGroup.getFoldersList().size();
        while (i <= (n-1)) {
            FTPProtocolData.Folder oldFolder = oldGroup.getFoldersList().get(i);
            FTPProtocolData.Folder newFolder = new FTPProtocolData.Folder();

            newFolder.setFolderName(oldFolder.getFolderName());

            newFolder.setIsAllowedToSeeFiles(oldFolder.getIsAllowedToSeeFiles());
            newFolder.setIsAllowedToReadFiles(oldFolder.getIsAllowedToReadFiles());
            newFolder.setIsAllowedToRenameFiles(oldFolder.getIsAllowedToRenameFiles());
            newFolder.setIsAllowedToAppendToFiles(oldFolder.getIsAllowedToAppendToFiles());
            newFolder.setIsAllowedToCreateNewFiles(oldFolder.getIsAllowedToCreateNewFiles());
            newFolder.setIsAllowedToOverwriteFiles(oldFolder.getIsAllowedToOverwriteFiles());
            newFolder.setIsAllowedToDeleteFiles(oldFolder.getIsAllowedToDeleteFiles());

            newFolder.setIsAllowedToSeeSubDirectories(oldFolder.getIsAllowedToSeeSubDirectories());
            newFolder.setIsAllowedToTraverseSubDirectories(oldFolder.getIsAllowedToTraverseSubDirectories());
            newFolder.setIsAllowedToRenameSubDirectories(oldFolder.getIsAllowedToRenameSubDirectories());
            newFolder.setIsAllowedToMakeSubDirectories(oldFolder.getIsAllowedToMakeSubDirectories());
            newFolder.setIsAllowedToRemoveSubDirectories(oldFolder.getIsAllowedToRemoveSubDirectories());

            newFolder.setIsAllowedToSeeHiddenFiles(oldFolder.getIsAllowedToSeeHiddenFiles());
            newFolder.setIsAllowedToReadHiddenFiles(oldFolder.getIsAllowedToReadHiddenFiles());
            newFolder.setIsAllowedToRenameHiddenFiles(oldFolder.getIsAllowedToRenameHiddenFiles());
            newFolder.setIsAllowedToAppendToHiddenFiles(oldFolder.getIsAllowedToAppendToHiddenFiles());
            newFolder.setIsAllowedToCreateNewHiddenFiles(oldFolder.getIsAllowedToCreateNewHiddenFiles());
            newFolder.setIsAllowedToOverwriteHiddenFiles(oldFolder.getIsAllowedToOverwriteHiddenFiles());
            newFolder.setIsAllowedToDeleteHiddenFiles(oldFolder.getIsAllowedToDeleteHiddenFiles());

            newFolder.setIsAllowedToSeeHiddenSubDirectories(oldFolder.getIsAllowedToSeeHiddenSubDirectories());
            newFolder.setIsAllowedToTraverseHiddenSubDirectories(oldFolder.getIsAllowedToTraverseHiddenSubDirectories());
            newFolder.setIsAllowedToRenameHiddenSubDirectories(oldFolder.getIsAllowedToRenameHiddenSubDirectories());
            newFolder.setIsAllowedToMakeHiddenSubDirectories(oldFolder.getIsAllowedToMakeHiddenSubDirectories());
            newFolder.setIsAllowedToRemoveHiddenSubDirectories(oldFolder.getIsAllowedToRemoveHiddenSubDirectories());

            newGroup.getFoldersList().add(newFolder);
            i++;
        }

        data.getGroupsList().add(newGroup);
        this.comboGroups.addItem(newGroup.getGroupName());
        this.comboGroupMembership.addItem(newGroup.getGroupName());
        this.endUpdate();
        this.thereAreChanges();
        this.comboGroups.setSelectedItem(newGroup.getGroupName());
        JOptionPane.showMessageDialog(this, "Group \"" + newGroup.getGroupName() + "\"" + " was added (copy of \"" + oldGroupName + "\").", "New group added successfully", JOptionPane.INFORMATION_MESSAGE);
    }//GEN-LAST:event_btnCopyGroupActionPerformed

    /**
    * @param args the command line arguments
    */
    public static void main(String args[]) {
        try {
            UIManager.setLookAndFeel( new com.nilo.plaf.nimrod.NimRODLookAndFeel());
        } catch (Exception e) {}

        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                FTPServer aFTPServer = new FTPServer();
                aFTPServer.startUpdate();
                aFTPServer.loadSettings();
                aFTPServer.endUpdate();
                aFTPServer.setVisible(true);
            }
        });
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTabbedPane TabbedPane;
    private javax.swing.JButton btnAddFolder;
    private javax.swing.JButton btnAddGroup;
    private javax.swing.JButton btnAddUser;
    private javax.swing.JButton btnBrowseForServerCertAndKey;
    private javax.swing.JButton btnBrowseForTrustedCAsFolder;
    private javax.swing.JButton btnCopyGroup;
    private javax.swing.JButton btnDaemonServer;
    private javax.swing.JButton btnDeleteFolder;
    private javax.swing.JButton btnDeleteGroup;
    private javax.swing.JButton btnDeleteUser;
    private javax.swing.JButton btnReload;
    private javax.swing.JButton btnRenameGroup;
    private javax.swing.JButton btnRenameUser;
    private javax.swing.JButton btnResetPassphrase;
    private javax.swing.JButton btnSaveChanges;
    private javax.swing.JButton btnStartServer;
    private javax.swing.JButton btnStopServer;
    private javax.swing.JCheckBox cbAccountIsActive;
    private javax.swing.JCheckBox cbAppendLogs;
    private javax.swing.JCheckBox cbIsAllowedToAppendToFiles;
    private javax.swing.JCheckBox cbIsAllowedToAppendToHiddenFiles;
    private javax.swing.JCheckBox cbIsAllowedToCreateNewFiles;
    private javax.swing.JCheckBox cbIsAllowedToCreateNewHiddenFiles;
    private javax.swing.JCheckBox cbIsAllowedToDeleteFiles;
    private javax.swing.JCheckBox cbIsAllowedToDeleteHiddenFiles;
    private javax.swing.JCheckBox cbIsAllowedToMakeHiddenSubDirectories;
    private javax.swing.JCheckBox cbIsAllowedToMakeSubDirectories;
    private javax.swing.JCheckBox cbIsAllowedToOverwriteFiles;
    private javax.swing.JCheckBox cbIsAllowedToOverwriteHiddenFiles;
    private javax.swing.JCheckBox cbIsAllowedToReadFiles;
    private javax.swing.JCheckBox cbIsAllowedToReadHiddenFiles;
    private javax.swing.JCheckBox cbIsAllowedToRemoveHiddenSubDirectories;
    private javax.swing.JCheckBox cbIsAllowedToRemoveSubDirectories;
    private javax.swing.JCheckBox cbIsAllowedToRenameFiles;
    private javax.swing.JCheckBox cbIsAllowedToRenameHiddenFiles;
    private javax.swing.JCheckBox cbIsAllowedToRenameHiddenSubDirectories;
    private javax.swing.JCheckBox cbIsAllowedToRenameSubDirectories;
    private javax.swing.JCheckBox cbIsAllowedToSeeFiles;
    private javax.swing.JCheckBox cbIsAllowedToSeeHiddenFiles;
    private javax.swing.JCheckBox cbIsAllowedToSeeHiddenSubDirectories;
    private javax.swing.JCheckBox cbIsAllowedToSeeSubDirectories;
    private javax.swing.JCheckBox cbIsAllowedToTraverseHiddenSubDirectories;
    private javax.swing.JCheckBox cbIsAllowedToTraverseSubDirectories;
    private javax.swing.JCheckBox cbLogEvents;
    private javax.swing.JCheckBox cbRequireClientAuthentication;
    private javax.swing.JCheckBox cbRequirePassphraseToBeCorrect;
    private javax.swing.JCheckBox cbTrackChanges;
    private javax.swing.JCheckBox cbUserRightsFilesDeselectAll;
    private javax.swing.JCheckBox cbUserRightsFilesSelectAll;
    private javax.swing.JCheckBox cbUserRightsHiddenFilesDeselectAll;
    private javax.swing.JCheckBox cbUserRightsHiddenFilesSelectAll;
    private javax.swing.JCheckBox cbUserRightsHiddenSubDirectoriesDeselectAll;
    private javax.swing.JCheckBox cbUserRightsHiddenSubDirectoriesSelectAll;
    private javax.swing.JCheckBox cbUserRightsSubDirectoriesDeselectAll;
    private javax.swing.JCheckBox cbUserRightsSubDirectoriesSelectAll;
    private javax.swing.JComboBox comboFolders;
    private javax.swing.JComboBox comboGroupMembership;
    private javax.swing.JComboBox comboGroups;
    private javax.swing.JComboBox comboOpeningFolder;
    private javax.swing.JComboBox comboTLSLevel;
    private javax.swing.JComboBox comboUsers;
    private javax.swing.JTextField edtExternalIP;
    private javax.swing.JPasswordField edtKeyPassphrase;
    private javax.swing.JSpinner edtPort;
    private javax.swing.JTextField edtServerCertAndKey;
    private javax.swing.JTextField edtTrustedCAsFolder;
    private javax.swing.JLabel lblExternalIP;
    private javax.swing.JLabel lblFiles;
    private javax.swing.JLabel lblFolders;
    private javax.swing.JLabel lblGroups;
    private javax.swing.JLabel lblHiddenFiles;
    private javax.swing.JLabel lblHiddenSubDirectories;
    private javax.swing.JLabel lblKeyPassphrase;
    private javax.swing.JLabel lblMemberOfGroup;
    private javax.swing.JLabel lblOpeningFolder;
    private javax.swing.JLabel lblPort;
    private javax.swing.JLabel lblSubDirectories;
    private javax.swing.JLabel lblTrustedCAsFolder;
    private javax.swing.JLabel lblUser;
    private javax.swing.JLabel ldlServerCertAndKey;
    private javax.swing.JPanel panelClientAuthentication;
    private javax.swing.JPanel panelGeneralSettings;
    private javax.swing.JPanel panelGroupsAndUserRights;
    private javax.swing.JPanel panelServerCertificateAndKey;
    private javax.swing.JPanel panelTLSLevel;
    private javax.swing.JPanel panelUsersAndGroupMembership;
    private javax.swing.JPanel tabGeneral;
    private javax.swing.JPanel tabGroupsUsersAndUserRights;
    private javax.swing.JPanel tabTLS;
    // End of variables declaration//GEN-END:variables

    private void onUserSelected() {
        String userName = ((String)(this.comboUsers.getSelectedItem()));
        if (this.data.userExists(userName)) {
            FTPProtocolData.User user = this.data.getUser(userName);
            boolean groupNameSet = false;

            int i = 1-1; int n = this.comboGroupMembership.getItemCount();
            while (i <= (n-1)) {
                if (user.getGroupName().equals((String)(this.comboGroupMembership.getItemAt(i)))) {
                    groupNameSet = true;
                    this.startUpdate();
                    this.comboGroupMembership.setSelectedIndex(i);
                    this.endUpdate();
                    int j = 1-1; int m = this.comboGroups.getItemCount();
                    while (j <= (m-1)) {
                        if (user.getGroupName().equals((String)(this.comboGroups.getItemAt(j)))) {
                            this.startUpdate();
                            this.comboGroups.setSelectedIndex(j);
                            this.endUpdate();
                            this.onGroupSelected();
                            break;
                        }
                        j++;
                    }
                    break;
                }
                i++;
            }
            if (groupNameSet == false) {
                this.startUpdate();
                this.comboGroupMembership.setSelectedItem(TFTPProtocol.EMPTY);
                this.comboGroups.setSelectedItem(TFTPProtocol.EMPTY);
                this.endUpdate();                
                this.onGroupSelected();
            }
            this.cbAccountIsActive.setSelected(user.getAccountIsActive());
            this.cbRequirePassphraseToBeCorrect.setSelected(user.getRequirePassphraseToBeCorrect());
        } else {
            this.startUpdate();
                this.comboGroupMembership.setSelectedItem(TFTPProtocol.EMPTY);
                this.comboGroups.setSelectedItem(TFTPProtocol.EMPTY);
                this.cbAccountIsActive.setSelected(true);
                this.cbRequirePassphraseToBeCorrect.setSelected(true);
            this.endUpdate();
            this.onGroupSelected();
        }
    }

}
