/* ============================================================================
 * TTCPConnection.java
 * ============================================================================
 * THIS SOFTWARE IS RELEASED UNDER THE FOLLOWING BINDING LEGAL AGREEMENT
 * (BSD-LICENSE).
 * ============================================================================
 *
 * Copyright (C) 2007-2009
 * by J.H.H.C. van der Meijs (tcpconns@jhhcvandermeijs.nl)
 * 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright notice, 
 *      this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright 
 *      notice, this list of conditions and the following disclaimer in the 
 *      documentation and/or other materials provided with the distribution.
 *    * Neither the name of tcpconns nor the names of its copyright holders or 
 *      its contributors may be used to endorse or promote products derived from 
 *      this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 * ============================================================================
 * IF YOU DO NOT AGREE TO THE ABOVE TERMS AND CONDITIONS THEN DO *NOT* USE THIS 
 * SOFTWARE AND *DO* DESTROY ALL COPIES OF THIS SOFTWARE THAT YOU HAVE.
 * ============================================================================
 * 
 * TCPCONNS’ CURRENT CODE IS NOT KNOWN TO VIOLATE ANY COPYRIGHTS OR INFRINGE ON 
 * ANY PATENTS. MOREOVER, IT ADHERES TO GENERAL PROGRAMMING RULES AND PRACTICES 
 * AND IS OF SUCH A SIMPLISTIC NATURE THAT A SIMILAR PACKAGE COULD BE 
 * CONSTRUCTED BY ANYONE. IMPORTANT(!): IF YOU FEEL THAT A PIECE OF CODE IN
 * TCPCONNS VIOLATES A COPYRIGHT OR INFRINGES ON A PATENT, PLEASE DO INFORM ONE 
 * OF THE AUTHORS IMMEDIATELY. DO ALWAYS STATE EXACTLY WHAT CODE IS NOT ALLOWED
 * UNDER EXACTLY WHAT RULING, COPYRIGHT, OR PATENT; AND PLEASE DO ELABORATE ON 
 * EXACTLY HOW THE CODE VIOLATES THE SPECIFIC COPYRIGHT OR INFRINGES ON
 * THE SPECIFIC PATENT.
 * 
 * ============================================================================
 * 
 * This software is Open Source Initiative approved Open Source Software.
 * Open Source Initiative Approved is a trademark of the Open Source Initiative.
 * http://www.opensource.org/
 * 
 * ============================================================================
 */
package nl.jhhcvandermeijs.tcpconns.client;

import nl.jhhcvandermeijs.tcpconns.tls.TTLSProvider;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.Certificate;
import javax.net.ssl.KeyManager;
import javax.net.ssl.SSLPeerUnverifiedException;
import javax.net.ssl.SSLSocket;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.TrustManager;

/**
 * use this class to establish a client connection to a server. 
 */
public class TTCPConnection {
        /**
         * boolean to indicate client side or server side
         */
        protected boolean fClientSide = true;
    
        /**
         * constructor which assumes the connection is established on client 
         * side
         */
        public TTCPConnection() {
            super();
        }

        /**
         * constructor
         * @param clientSide must be false when on server side
         */
        public TTCPConnection(boolean clientSide) {
            super();
            this.fClientSide = clientSide;
        }
    
        /**
         * holds the reference to the currently used Socket object
         */
        protected Socket fmySocket = null;
        
        /**
         * holds the reference to a regular Socket
         */
        protected Socket fmyRegularSocket = null;
        public Socket myRegularSocket() {
            return this.fmyRegularSocket;
        }

        /**
         * holds the reference to an SSLSocket
         */
        protected SSLSocket fmySSLSocket = null;
        
        /**
         * use this method to get access to the internally used Socket object
         * 
         * @return the Socket that is used internally
         */
        public Socket mySocket() {
            return this.fmySocket;
        }                   
        
        /**
         * use this method to connect to a server
         * 
         * @param ip String that contains the server ip address, e.g.
         *        "127.0.0.1" for the local (ipv4) loopback address for
         *        a server running on the same machine
         * @param port integer value on which the server serves its clients
         * @throws java.io.IOException
         */        
        public void connect(String ip, int port) throws IOException {
            //terminate any active connection prn
            this.disconnect();

            //connect
            this.fmySocket = new Socket();            
            this.fmyRegularSocket = this.fmySocket;
            this.fmySSLSocket = null;
            
            this.fmySocket.connect(new InetSocketAddress(
                                   InetAddress.getByName(ip), port));
            
            //assert connected
            if (!(this.isConnected())) {
                this.disconnect();
            } else {
                this.fmyDataInputStream = new DataInputStream(
                                            new BufferedInputStream(
                                            this.fmySocket.getInputStream()));            
                this.fmyDataOutputStream = new DataOutputStream(
                                            new BufferedOutputStream(
                                            this.fmySocket.getOutputStream()));
            }
        }

        /**
         * this method is used by TTCPServer to "connect" an incoming client (in
         * reality the "incoming client" is already connected); it is also 
         * utilized by instances of the TTLSProvider class
         *         
         * @param aSocket the Socket that holds an active connection to a server
         * @throws java.io.IOException
         */
        public void connect(Socket aSocket) throws IOException {                        
            if (aSocket instanceof SSLSocket) {
                if (this.fmySocket != null) {
                    if (!(this.fmySocket instanceof SSLSocket)) {
                        this.fmyRegularSocket = this.fmySocket;
                    }
                }
                this.fmySSLSocket = ((SSLSocket)(aSocket));
                this.fmySocket = aSocket;
            } else {
                this.fmySSLSocket = null;
                this.fmySocket = aSocket;
                this.fmyRegularSocket = aSocket;                
            }
                                    
            this.fmyDataInputStream = new DataInputStream(
                                         new BufferedInputStream(
                                         this.fmySocket.getInputStream()));            
            this.fmyDataOutputStream = new DataOutputStream(
                                         new BufferedOutputStream(
                                         this.fmySocket.getOutputStream()));
        }
        
        
        /**
         * use this method to check whether this object is connected or not
         * 
         * @return true if connected; important (!): if peer has "reset" the
         * connection (i.e. when the peer has disconnected) it will not be
         * apparent until actual data is send towards the server
         */
        public boolean isConnected() {
            if (this.fmySocket == null) {return false;}
            return this.fmySocket.isConnected();
        }                           
        /**
         * use this method to disconnect an active connection
         * 
         * @throws java.io.IOException
         */
        public void disconnect() throws IOException {            
            //try to close SSLSocket prn
            if (this.fmySSLSocket != null) {
                try {this.fmySSLSocket.getSession().invalidate();} catch (Exception e) {}
                try {this.fmySSLSocket.close();} catch (Exception e) {}
                     this.fmySSLSocket = null;
            }
            
            //try to close Regular Socket
            if (this.fmyRegularSocket != null) {
                try {this.fmyRegularSocket.close();} catch (Exception e) {}
                     this.fmyRegularSocket = null;
            }
            
            //try to close fmySocket => should be closed already!!!
            if (this.fmySocket != null) {
                 try{this.fmySocket.close();} catch (Exception e) {}
                     this.fmySocket = null;
            }
 
            //free in && output streams
            try {this.fmyDataInputStream.close();} catch (Exception e) {} 
                 this.fmyDataInputStream = null;
            try {this.fmyDataOutputStream.close();} catch (Exception e) {} 
                 this.fmyDataOutputStream = null;
        }
        
        /**
         * holds the reference to the DataInputStream of an active
         * connection
         * 
         */        
        protected DataInputStream fmyDataInputStream = null;
        /**
         * use this method to obtain the reference to the DataInputStream of
         * the active connection
         * 
         * @return DataInputStream of current connection
         * @throws java.io.IOException
         */
        public DataInputStream in() throws IOException {
            if (!(this.isConnected())) {
                throw new IOException();
            }
            return this.fmyDataInputStream;
        }
        
        /**
         * holds the reference to the DataOutputStream of an active
         * connection
         */        
        protected DataOutputStream fmyDataOutputStream = null;
        /**
         * use this method to obtain a handle towards the DataOutputStream of
         * the active connection; do not forget to flush() data!
         * 
         * @return DataOutputStream of current connection
         * @throws java.io.IOException
         */
        public DataOutputStream out() throws IOException {
            if (!(this.isConnected())) {
                throw new IOException();
            }
            return this.fmyDataOutputStream;
        }        
                
        /**
         * use this method to obtain the used protocol when secure
         * @return a string containing the used protocol
         */
        public String getProtocol() {
            if (this.fmySocket instanceof SSLSocket) {
                return ((SSLSocket)(this.fmySocket)).getSession().getProtocol();
            } else {
                return "INSECURE CHANNEL";
            }
        }

        /**
         * use this method to obtain the used ciphersuite when secure
         * @return a string containing the used ciphersuite
         */
        public String getCipherSuite() {
            if (this.fmySocket instanceof SSLSocket) {                
                return ((SSLSocket)(this.fmySocket)).getSession().getCipherSuite();
            } else {
                return "INSECURE CHANNEL";
            }
        }

        /**
         * use this method to determine whether the connection is in secure mode
         * @return true if secure
         */
        public boolean isSecure() {
            boolean result = false;
            if (this.fmySocket != null) {
                if (this.fmySocket instanceof SSLSocket) {
                    result = true;
                }
            }
            return result;
        }        
        
        /**
         * use this method to obtain the peer X509 certificates; note: you
         * need to cast the Certificate to be a X509Certificate
         * @return certificate chain of peer
         * @throws javax.net.ssl.SSLPeerUnverifiedException if there is no
         * certificate chain to present
         */        
        public Certificate[] getPeerCertificateChain() throws SSLPeerUnverifiedException {
            if (this.fmySocket instanceof SSLSocket) {
                return ((SSLSocket)(this.fmySocket)).getSession().getPeerCertificates();                
            } else {
                throw new SSLPeerUnverifiedException("PEER UNVERIFIED DUE TO INSECURE CHANNEL");
            }
        }
        
        /**
         * place holder for a TTLSProvider instance
         */
        protected TTLSProvider fmyTTLSProvider = null;

        /**
         *
         * @throws java.io.IOException
         * @throws java.security.NoSuchAlgorithmException
         * @throws java.security.KeyManagementException
         */
        public void makeSecureWithoutAuthentication(
                String[] protocols,
                String[] ciphersuites,
                boolean  regularSocketAutoClose) 
                throws IOException, NoSuchAlgorithmException, KeyManagementException {
            if (this.fmyTTLSProvider == null) {
                this.fmyTTLSProvider = new TTLSProvider(this, this.fClientSide);
            }
            this.fmyTTLSProvider.makeSecureWithoutAuthentication(protocols, ciphersuites, regularSocketAutoClose);
        }

        public void makeSecureWithoutAuthentication(
                SSLSocketFactory anSSLSocketFactory,
                String[] protocols,
                String[] ciphersuites,
                boolean  regularSocketAutoClose)
                throws IOException, NoSuchAlgorithmException, KeyManagementException {
            if (this.fmyTTLSProvider == null) {
                this.fmyTTLSProvider = new TTLSProvider(this, this.fClientSide);
            }
            this.fmyTTLSProvider.makeSecureWithoutAuthentication(anSSLSocketFactory, protocols, ciphersuites, regularSocketAutoClose);
        }

        public void makeSecureWithoutAuthentication(
                String[] protocols,
                String[] ciphersuites) 
                throws IOException, NoSuchAlgorithmException, KeyManagementException {
            this.makeSecureWithoutAuthentication(protocols, ciphersuites, false);
        }

        public void makeSecureWithoutAuthentication(
                SSLSocketFactory anSSLSocketFactory,
                String[] protocols,
                String[] ciphersuites)
                throws IOException, NoSuchAlgorithmException, KeyManagementException {
            this.makeSecureWithoutAuthentication(anSSLSocketFactory, protocols, ciphersuites, false);
        }

        public void makeSecureWithoutAuthentication() 
                throws IOException, NoSuchAlgorithmException, KeyManagementException {
            this.makeSecureWithoutAuthentication(null, null, false);
        }

        public void makeSecureWithoutAuthentication(SSLSocketFactory anSSLSocketFactory)
                throws IOException, NoSuchAlgorithmException, KeyManagementException {
            this.makeSecureWithoutAuthentication(anSSLSocketFactory, null, null, false);
        }
        
        public void makeInsecure() throws IOException {
            if (this.fmyTTLSProvider == null) {
                this.fmyTTLSProvider = new TTLSProvider(this, this.fClientSide);
            }
            this.fmyTTLSProvider.makeInsecure();
        }
        
        public void makeSecureWithServerCertAuthByClient(
                String[] protocols,
                String[] ciphersuites,
                KeyManager[] keyManagers,
                TrustManager[] trustManagers,                
                boolean  regularSocketAutoClose                
                ) throws IOException, 
                         NoSuchAlgorithmException, 
                         KeyManagementException {
            if (this.fmyTTLSProvider == null) {
                this.fmyTTLSProvider = new TTLSProvider(this, this.fClientSide);
            }
            this.fmyTTLSProvider.makeSecureWithServerCertAuthByClient(protocols, ciphersuites, keyManagers, trustManagers, regularSocketAutoClose);
        }

        public void makeSecureWithServerCertAuthByClient(
                SSLSocketFactory anSSLSocketFactory,
                String[] protocols,
                String[] ciphersuites,
                boolean  regularSocketAutoClose
                ) throws IOException,
                         NoSuchAlgorithmException,
                         KeyManagementException {
            if (this.fmyTTLSProvider == null) {
                this.fmyTTLSProvider = new TTLSProvider(this, this.fClientSide);
            }
            this.fmyTTLSProvider.makeSecureWithServerCertAuthByClient(anSSLSocketFactory, protocols, ciphersuites, regularSocketAutoClose);
        }
        
        public void makeSecureWithServerCertAuthByClient(
                String[] protocols,
                String[] ciphersuites,
                KeyManager[] keyManagers
                ) throws IOException, 
                         NoSuchAlgorithmException, 
                         KeyManagementException {
            if (this.fmyTTLSProvider == null) {
                this.fmyTTLSProvider = new TTLSProvider(this, this.fClientSide);
            }
            this.fmyTTLSProvider.makeSecureWithServerCertAuthByClient(protocols, ciphersuites, keyManagers, null, false);
        }

        public void makeSecureWithServerCertAuthByClient(
                SSLSocketFactory anSSLSocketFactory,
                String[] protocols,
                String[] ciphersuites
                ) throws IOException,
                         NoSuchAlgorithmException,
                         KeyManagementException {
            if (this.fmyTTLSProvider == null) {
                this.fmyTTLSProvider = new TTLSProvider(this, this.fClientSide);
            }
            this.fmyTTLSProvider.makeSecureWithServerCertAuthByClient(anSSLSocketFactory, protocols, ciphersuites, false);
        }

        public void makeSecureWithServerCertAuthByClient(
                KeyManager[] keyManagers
                ) throws IOException, 
                         NoSuchAlgorithmException, 
                         KeyManagementException {
            if (this.fmyTTLSProvider == null) {
                this.fmyTTLSProvider = new TTLSProvider(this, this.fClientSide);
            }
            this.fmyTTLSProvider.makeSecureWithServerCertAuthByClient(null, null, keyManagers, null, false);
        }

        public void makeSecureWithServerCertAuthByClient(
                SSLSocketFactory anSSLSocketFactory
                ) throws IOException,
                         NoSuchAlgorithmException,
                         KeyManagementException {
            if (this.fmyTTLSProvider == null) {
                this.fmyTTLSProvider = new TTLSProvider(this, this.fClientSide);
            }
            this.fmyTTLSProvider.makeSecureWithServerCertAuthByClient(anSSLSocketFactory);
        }

        public void makeSecureWithClientCertAuthByServer(
                String[] protocols,
                String[] ciphersuites,
                KeyManager[] keyManagers,
                TrustManager[] trustManagers,                
                boolean  regularSocketAutoClose                
                ) throws IOException, 
                         NoSuchAlgorithmException, 
                         KeyManagementException {
            if (this.fmyTTLSProvider == null) {
                this.fmyTTLSProvider = new TTLSProvider(this, this.fClientSide);
            }
            this.fmyTTLSProvider.makeSecureWithClientCertAuthByServer(protocols, ciphersuites, keyManagers, trustManagers, regularSocketAutoClose);
        }

        public void makeSecureWithClientCertAuthByServer(
                SSLSocketFactory anSSLSocketFactory,
                String[] protocols,
                String[] ciphersuites,
                boolean  regularSocketAutoClose                
                ) throws IOException, 
                         NoSuchAlgorithmException, 
                         KeyManagementException {
            if (this.fmyTTLSProvider == null) {
                this.fmyTTLSProvider = new TTLSProvider(this, this.fClientSide);
            }
            this.fmyTTLSProvider.makeSecureWithClientCertAuthByServer(anSSLSocketFactory, protocols, ciphersuites, regularSocketAutoClose);
        }

        public void makeSecureWithClientCertAuthByServer(
                String[] protocols,
                String[] ciphersuites,
                KeyManager[] keyManagers
                ) throws IOException, 
                         NoSuchAlgorithmException, 
                         KeyManagementException {
            if (this.fmyTTLSProvider == null) {
                this.fmyTTLSProvider = new TTLSProvider(this, this.fClientSide);
            }
            this.fmyTTLSProvider.makeSecureWithClientCertAuthByServer(protocols, ciphersuites, keyManagers, null, false);
        }

        public void makeSecureWithClientCertAuthByServer(
                SSLSocketFactory anSSLSocketFactory,
                String[] protocols,
                String[] ciphersuites
                ) throws IOException,
                         NoSuchAlgorithmException,
                         KeyManagementException {
            if (this.fmyTTLSProvider == null) {
                this.fmyTTLSProvider = new TTLSProvider(this, this.fClientSide);
            }
            this.fmyTTLSProvider.makeSecureWithClientCertAuthByServer(anSSLSocketFactory, protocols, ciphersuites, false);
        }

        public void makeSecureWithClientCertAuthByServer(
                KeyManager[] keyManagers
                ) throws IOException, 
                         NoSuchAlgorithmException, 
                         KeyManagementException {
            if (this.fmyTTLSProvider == null) {
                this.fmyTTLSProvider = new TTLSProvider(this, this.fClientSide);
            }
            this.fmyTTLSProvider.makeSecureWithClientCertAuthByServer(null, null, keyManagers, null, false);
        }

        public void makeSecureWithClientCertAuthByServer(
                SSLSocketFactory anSSLSocketFactory
                ) throws IOException,
                         NoSuchAlgorithmException,
                         KeyManagementException {
            if (this.fmyTTLSProvider == null) {
                this.fmyTTLSProvider = new TTLSProvider(this, this.fClientSide);
            }
            this.fmyTTLSProvider.makeSecureWithClientCertAuthByServer(anSSLSocketFactory);
        }
}
