/* ============================================================================
 * CertificationRequestGenerator.java
 * ============================================================================
 * THIS SOFTWARE IS RELEASED UNDER THE FOLLOWING BINDING LEGAL AGREEMENT
 * (BSD-LICENSE).
 * ============================================================================
 *
 * Copyright (C) 2009
 * by J.H.H.C. van der Meijs (tcpconns@jhhcvandermeijs.nl)
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    * Neither the name of tcpconns nor the names of its copyright holders or
 *      its contributors may be used to endorse or promote products derived from
 *      this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 * ============================================================================
 * IF YOU DO NOT AGREE TO THE ABOVE TERMS AND CONDITIONS THEN DO *NOT* USE THIS
 * SOFTWARE AND *DO* DESTROY ALL COPIES OF THIS SOFTWARE THAT YOU HAVE.
 * ============================================================================
 *
 * TCPCONNS’ CURRENT CODE IS NOT KNOWN TO VIOLATE ANY COPYRIGHTS OR INFRINGE ON
 * ANY PATENTS. MOREOVER, IT ADHERES TO GENERAL PROGRAMMING RULES AND PRACTICES
 * AND IS OF SUCH A SIMPLISTIC NATURE THAT A SIMILAR PACKAGE COULD BE
 * CONSTRUCTED BY ANYONE. IMPORTANT(!): IF YOU FEEL THAT A PIECE OF CODE IN
 * TCPCONNS VIOLATES A COPYRIGHT OR INFRINGES ON A PATENT, PLEASE DO INFORM ONE
 * OF THE AUTHORS IMMEDIATELY. DO ALWAYS STATE EXACTLY WHAT CODE IS NOT ALLOWED
 * UNDER EXACTLY WHAT RULING, COPYRIGHT, OR PATENT; AND PLEASE DO ELABORATE ON
 * EXACTLY HOW THE CODE VIOLATES THE SPECIFIC COPYRIGHT OR INFRINGES ON
 * THE SPECIFIC PATENT.
 *
 * ============================================================================
 *
 * This software is Open Source Initiative approved Open Source Software.
 * Open Source Initiative Approved is a trademark of the Open Source Initiative.
 * http://www.opensource.org/
 *
 * ============================================================================
 */
package nl.jhhcvandermeijs.tcpconns.keys;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.SecureRandom;
import nl.jhhcvandermeijs.tcpconns.keys.KeyStoreHelper.DNProcessor;
import nl.jhhcvandermeijs.tcpconns.utils.SRNGSeeder;
import org.bouncycastle.asn1.DERSet;
import org.bouncycastle.jce.PKCS10CertificationRequest;
import org.bouncycastle.openssl.PEMReader;
import org.bouncycastle.openssl.PEMWriter;

public class CertificationRequestGenerator extends DNProcessor {
    public static final String SIGNATURE_ALGORITHM_MD2_WITH_RSA = "MD2withRSA";
    public static final String SIGNATURE_ALGORITHM_MD5_WITH_RSA = "MD5withRSA";
    public static final String SIGNATURE_ALGORITHM_SHA1_WITH_RSA = "SHA1withRSA";
    public static final String SIGNATURE_ALGORITHM_RIPEMD128_WITH_RSA = "RIPEMD128withRSA";
    public static final String SIGNATURE_ALGORITHM_RIPEMD160_WITH_RSA = "RIPEMD160withRSA";
    public static final String SIGNATURE_ALGORITHM_RIPEMD256_WITH_RSA = "RIPEMD256withRSA";
    //public static final String SIGNATURE_ALGORITHM_SHA1_WITH_DSA = "SHA1_WITH_DSA";
    public static final String SIGNATURE_ALGORITHM_SHA224_WITH_RSA = "SHA224withRSA";
    public static final String SIGNATURE_ALGORITHM_SHA256_WITH_RSA = "SHA256withRSA";
    public static final String SIGNATURE_ALGORITHM_SHA384_WITH_RSA = "SHA384withRSA";
    public static final String SIGNATURE_ALGORITHM_SHA512_WITH_RSA = "SHA512withRSA";
    public static final String SIGNATURE_ALGORITHM_SHA1_WITH_RSA_AND_MGF1 = "SHA1withRSAandMGF1";
    public static final String SIGNATURE_ALGORITHM_SHA256_WITH_RSA_AND_MGF1 = "SHA256withRSAandMGF1";
    public static final String SIGNATURE_ALGORITHM_SHA384_WITH_RSA_AND_MGF1 = "SHA384withRSAandMGF1";
    public static final String SIGNATURE_ALGORITHM_SHA512_WITH_RSA_AND_MGF1 = "SHA512withRSAandMGF1";    

    protected PKCS10CertificationRequest fRequest = null;
    public PKCS10CertificationRequest getPKCS10CertificationRequest() {return this.fRequest;}

    protected SecureRandom fSecureRandom = null;
    public SecureRandom getSecureRandom() {return this.fSecureRandom;}
    public void setSecureRandom(SecureRandom secureRandom) {this.fSecureRandom = secureRandom;}

    protected KeyPair fKeyPair = null;
    public KeyPair getKeyPair() {return this.fKeyPair;}
    public void setKeyPair(KeyPair keyPair) {this.fKeyPair = keyPair;}

    protected String fProvider = "BC";
    public String getProvider() {return this.fProvider;}
    public void setProvider(String provider) {this.fProvider = provider;}

    protected String fKeyAlgorithm = "RSA";
    public String getKeyAlgorithm() {return this.fKeyAlgorithm;}
    public void setKeyAlgorithm(String keyAlgorithm) {this.fKeyAlgorithm = keyAlgorithm;}

    protected String fSignatureAlgorithm = SIGNATURE_ALGORITHM_SHA512_WITH_RSA_AND_MGF1;
    public String getSignatureAlgorithm() {return this.fSignatureAlgorithm;}
    public void setSignatureAlgorithm(String signatureAlgorithm) {this.fSignatureAlgorithm = signatureAlgorithm;}

    protected int fKeySize = 4096;
    public int getKeySize() {return this.fKeySize;}
    public void setKeySize(int keySize) {this.fKeySize = keySize;}
    
    public void generateNewKeyPair() {
        if (this.fSecureRandom == null) {this.fSecureRandom = (new SRNGSeeder()).newSecureRandom();}
        if (this.fSecureRandom == null) {this.fSecureRandom = new SecureRandom();}
        try {
            KeyStoreHelper.assertProviders();
            KeyPairGenerator keyPairGenerator = KeyPairGenerator.getInstance(this.fKeyAlgorithm, this.fProvider);
            keyPairGenerator.initialize(this.fKeySize, this.fSecureRandom);
            this.fKeyPair = keyPairGenerator.generateKeyPair();
        } catch (Exception e) {
            System.out.println(e);
            this.fKeyPair = null;
        }
    }

    private javax.swing.JFrame fParent = null;

    public PKCS10CertificationRequest generateCertificationRequest(javax.swing.JFrame parent) {
        this.fParent = parent;
        try {
            if (this.fKeyPair == null) {
                this.generateNewKeyPair();
            }
            DERSet derSet = this.getCertificationRequestAlternativeNameExtension();
            this.fRequest = new PKCS10CertificationRequest(this.getSignatureAlgorithm(),
                                                           this.getDNStringAsX500Principal(),
                                                           this.fKeyPair.getPublic(),
                                                           derSet,
                                                           this.fKeyPair.getPrivate());
        } catch (Exception e) {
            System.out.println(e);
        }
        return this.fRequest;
    }

    public void storePrivateKey() {
        String fileName = KeyStoreHelper.saveDialog(this.fParent, null);
        if (fileName != null) {
            char[] passphrase = KeyStoreHelper.enterPassphrase(this.fParent);
            if (passphrase != null) {
                this.storePrivateKey(fileName, passphrase);
            }
        }
    }

    public void storePrivateKey(String fileName, char[] passphrase) {
        if (this.fKeyPair != null) {
            if (this.fKeyPair.getPrivate() != null) {
                KeyStoreHelper.saveAsKEYFile(this.fKeyPair, passphrase, fileName);
            }
        }
    }

    public void storePrivateKey(OutputStream outputStream, char[] passphrase) {
        if (this.fKeyPair != null) {
            if (this.fKeyPair.getPrivate() != null) {
                KeyStoreHelper.saveAsKEYFile(this.fKeyPair, passphrase, outputStream);
            }
        }
    }

    public void storeRequest() {
        String fileName = KeyStoreHelper.saveDialog(this.fParent, null);
        if (fileName != null) {
            CertificationRequestGenerator.storeRequest(this.fRequest, fileName);
        }
    }

    public void storeRequestDEREncoded() {
        String fileName = KeyStoreHelper.saveDialog(this.fParent, null);
        if (fileName != null) {
            CertificationRequestGenerator.storeRequestDEREncoded(this.fRequest, fileName);
        }
    }

    public void storeRequest(String fileName) {
        CertificationRequestGenerator.storeRequest(this.fRequest, fileName);
    }

    public void storeRequestDEREncoded(String fileName) {
        CertificationRequestGenerator.storeRequestDEREncoded(this.fRequest, fileName);
    }

    public static void storeRequest(javax.swing.JFrame parent, PKCS10CertificationRequest request) {
        String fileName = KeyStoreHelper.saveDialog(parent, null);
        if (fileName != null) {
            CertificationRequestGenerator.storeRequest(request, fileName);
        }
    }

    public static void storeRequestDEREncoded(javax.swing.JFrame parent, PKCS10CertificationRequest request) {
        String fileName = KeyStoreHelper.saveDialog(parent, null);
        if (fileName != null) {
            CertificationRequestGenerator.storeRequestDEREncoded(request, fileName);
        }
    }

    public static void storeRequest(PKCS10CertificationRequest request, String fileName) {
        if (request != null) {
            File file = new File(fileName);
            FileOutputStream fileOutputStream = null;        
            try {
                try {
                    fileOutputStream = new FileOutputStream(file);
                    CertificationRequestGenerator.storeRequest(request, fileOutputStream);
                    fileOutputStream.flush();
                } finally {                    
                    try {fileOutputStream.close();} catch (Exception e2) {}
                }
            } catch (Exception e) {}
        }
    }

    public static void storeRequestDEREncoded(PKCS10CertificationRequest request, String fileName) {
        if (request != null) {
            File file = new File(fileName);
            FileOutputStream fileOutputStream = null;
            try {
                try {
                    fileOutputStream = new FileOutputStream(file);
                    CertificationRequestGenerator.storeRequestDEREncoded(request, fileOutputStream);
                    fileOutputStream.flush();
                } finally {
                    try {fileOutputStream.close();} catch (Exception e2) {}
                }
            } catch (Exception e) {}
        }
    }

    public static void storeRequest(PKCS10CertificationRequest request, OutputStream outputStream) {
        if (request != null) {
            PEMWriter pemWriter = null;
            try {
                try {
                    pemWriter = new PEMWriter(new OutputStreamWriter(outputStream));
                    pemWriter.writeObject(request);
                    pemWriter.flush();
                } finally {}
            } catch (Exception e) {}
        }
    }

    public static void storeRequestDEREncoded(PKCS10CertificationRequest request, OutputStream outputStream) {
        if (request != null) {
            ByteArrayInputStream bais = null;
            try {
                try {
                    bais = new ByteArrayInputStream(request.getDEREncoded());
                    byte[] buffer = new byte[4096];
                    int bytesRead = bais.read(buffer);
                    while (bytesRead != -1) {
                        outputStream.write(buffer);
                        bytesRead = bais.read(buffer);
                    }
                    outputStream.flush();
                } finally {}
            } catch (Exception e) {}
        }
    }

    public static PKCS10CertificationRequest loadPKCS10CertificationRequest(String fileName) {
        FileInputStream fileInputStream = null;
        try {
            try {
                fileInputStream = new FileInputStream(new File(fileName));
                return CertificationRequestGenerator.loadPKCS10CertificationRequest(fileInputStream);
            } finally {
                try {fileInputStream.close();} catch (Exception e2) {}
            }
        } catch (Exception e) {
            return null;
        }
    }

    public static PKCS10CertificationRequest loadPKCS10CertificationRequest(InputStream inputStream) {
        PKCS10CertificationRequest request = null;
        ByteArrayOutputStream baos = null;
        try {
            baos = new ByteArrayOutputStream(inputStream.available());
            byte[] buffer = new byte[4096];
            int bytesRead = inputStream.read(buffer);
            while (bytesRead != -1) {
                baos.write(buffer, 0, bytesRead);
                bytesRead = inputStream.read(buffer);
            }
            baos.flush();

            ByteArrayInputStream bais = new ByteArrayInputStream(baos.toByteArray());

            PEMReader pemReader = null;
            try {
                try {
                    Object object = null;
                    pemReader = new PEMReader(new BufferedReader(new InputStreamReader(bais)));
                    object = pemReader.readObject();
                    while (object != null) {
                        if (object instanceof PKCS10CertificationRequest) {
                            request = ((PKCS10CertificationRequest)(object));
                            return request;
                        } else {
                            object = pemReader.readObject();
                        }
                    }
                } finally {}
            } catch (Exception e) {}

            try {
                try {
                    request = new PKCS10CertificationRequest(baos.toByteArray());
                } finally {}
            } catch (Exception e2) {}            
        } catch (Exception e) {}
    return request;
    }
}
