/* ============================================================================
 * KeyManagerGUI.java
 * ============================================================================
 * THIS SOFTWARE IS RELEASED UNDER THE FOLLOWING BINDING LEGAL AGREEMENT
 * (BSD-LICENSE).
 * ============================================================================
 *
 * Copyright (C) 2009
 * by J.H.H.C. van der Meijs (tcpconns@jhhcvandermeijs.nl)
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    * Neither the name of tcpconns nor the names of its copyright holders or
 *      its contributors may be used to endorse or promote products derived from
 *      this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 * ============================================================================
 * IF YOU DO NOT AGREE TO THE ABOVE TERMS AND CONDITIONS THEN DO *NOT* USE THIS
 * SOFTWARE AND *DO* DESTROY ALL COPIES OF THIS SOFTWARE THAT YOU HAVE.
 * ============================================================================
 *
 * TCPCONNS’ CURRENT CODE IS NOT KNOWN TO VIOLATE ANY COPYRIGHTS OR INFRINGE ON
 * ANY PATENTS. MOREOVER, IT ADHERES TO GENERAL PROGRAMMING RULES AND PRACTICES
 * AND IS OF SUCH A SIMPLISTIC NATURE THAT A SIMILAR PACKAGE COULD BE
 * CONSTRUCTED BY ANYONE. IMPORTANT(!): IF YOU FEEL THAT A PIECE OF CODE IN
 * TCPCONNS VIOLATES A COPYRIGHT OR INFRINGES ON A PATENT, PLEASE DO INFORM ONE
 * OF THE AUTHORS IMMEDIATELY. DO ALWAYS STATE EXACTLY WHAT CODE IS NOT ALLOWED
 * UNDER EXACTLY WHAT RULING, COPYRIGHT, OR PATENT; AND PLEASE DO ELABORATE ON
 * EXACTLY HOW THE CODE VIOLATES THE SPECIFIC COPYRIGHT OR INFRINGES ON
 * THE SPECIFIC PATENT.
 *
 * ============================================================================
 *
 * This software is Open Source Initiative approved Open Source Software.
 * Open Source Initiative Approved is a trademark of the Open Source Initiative.
 * http://www.opensource.org/
 *
 * ============================================================================
 */
package nl.jhhcvandermeijs.tcpconns.keys;

import java.math.BigInteger;
import java.security.KeyFactory;
import java.security.KeyPair;
import java.security.KeyStore;
import java.security.KeyStore.PrivateKeyEntry;
import java.security.KeyStore.SecretKeyEntry;
import java.security.KeyStore.TrustedCertificateEntry;
import java.security.MessageDigest;
import java.security.PrivateKey;
import java.security.SecureRandom;
import java.security.cert.CertificateExpiredException;
import java.security.cert.CertificateNotYetValidException;
import java.security.cert.X509Certificate;
import java.security.spec.RSAPublicKeySpec;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;
import nl.jhhcvandermeijs.tcpconns.keys.KeyStoreHelper.DNProcessor;
import nl.jhhcvandermeijs.tcpconns.pgp.PGPHelper;
import nl.jhhcvandermeijs.tcpconns.utils.SRNGSeeder;
import nl.jhhcvandermeijs.tcpconns.utils.Utils;
import org.bouncycastle.openpgp.PGPSignature;

public class KeyManagerGUI extends javax.swing.JFrame {
    public static final long serialVersionUID = 1L;

    public static class Entry {
        String alias = null;
        String displayName = null;

        @Override()
        public String toString() {
            if ((this.displayName == null) || (this.displayName.equals(""))) {
                if ((alias == null) || (this.alias.equals(""))) {
                    return "n/a";
                } else {
                    return alias;
                }
            } else {
                return displayName;
            }
        }
    }

    KeyStore fKeyStore = null;
    char[] fPassphrase = null;
    boolean statusChanged = false;

    public void saveAs(KeyStore oldKeyStore, char[] oldPassphrase) {
        String[] keyStoreTypes = new String[7];
        keyStoreTypes[0] = "PKCS#12 keystore new type (.pfx/.p12)";
        keyStoreTypes[6] = "PKCS#12 keystore old type (.pfx/.p12)";
        keyStoreTypes[1] = "Java Key Store (.jks/.ts)";
        keyStoreTypes[2] = "BouncyCastle.org Key Store (.bks/.bts)";
        keyStoreTypes[3] = "UBER (3DES/TwoFish) formatted keystore (.uber)";
        keyStoreTypes[4] = "JCEKS formatted keystore (.jceks)";
        keyStoreTypes[5] = "OpenSSL PEM formatted (.pem)";

        String[] pemEncryptionTypes = new String[37];
        pemEncryptionTypes[ 0] = "AES-128-CBC";
        pemEncryptionTypes[ 1] = "AES-128-CFB";
        pemEncryptionTypes[ 2] = "AES-128-ECB";
        pemEncryptionTypes[ 3] = "AES-128-OFB";
        pemEncryptionTypes[ 4] = "AES-192-CBC";
        pemEncryptionTypes[ 5] = "AES-192-CFB";
        pemEncryptionTypes[ 6] = "AES-192-ECB";
        pemEncryptionTypes[ 7] = "AES-192-OFB";
        pemEncryptionTypes[ 8] = "AES-256-CBC";
        pemEncryptionTypes[ 9] = "AES-256-CFB";
        pemEncryptionTypes[10] = "AES-256-ECB";
        pemEncryptionTypes[11] = "AES-256-OFB";
        pemEncryptionTypes[12] = "BF-CBC";
        pemEncryptionTypes[13] = "BF-CFB";
        pemEncryptionTypes[14] = "BF-ECB";
        pemEncryptionTypes[15] = "BF-OFB";
        pemEncryptionTypes[16] = "DES-CBC";
        pemEncryptionTypes[17] = "DES-CFB";
        pemEncryptionTypes[18] = "DES-ECB";
        pemEncryptionTypes[19] = "DES-OFB";
        pemEncryptionTypes[20] = "DES-EDE";
        pemEncryptionTypes[21] = "DES-EDE-CBC";
        pemEncryptionTypes[22] = "DES-EDE-CFB";
        pemEncryptionTypes[23] = "DES-EDE-ECB";
        pemEncryptionTypes[24] = "DES-EDE-OFB";
        pemEncryptionTypes[25] = "DES-EDE3";
        pemEncryptionTypes[26] = "DES-EDE3-CBC";
        pemEncryptionTypes[27] = "DES-EDE3-CFB";
        pemEncryptionTypes[28] = "DES-EDE3-ECB";
        pemEncryptionTypes[29] = "DES-EDE3-OFB";
        pemEncryptionTypes[30] = "RC2-CBC";
        pemEncryptionTypes[31] = "RC2-CFB";
        pemEncryptionTypes[32] = "RC2-ECB";
        pemEncryptionTypes[33] = "RC2-OFB";
        pemEncryptionTypes[34] = "RC2-40-CBC";
        pemEncryptionTypes[35] = "RC2-64-CBC";
        pemEncryptionTypes[36] = "NO_ENCRYPTION";

        String keyStoreType = ((String)(JOptionPane.showInputDialog(this, "Please select the desired keystore type", "Select keystore format", JOptionPane.QUESTION_MESSAGE, null, keyStoreTypes, keyStoreTypes[5])));
        if (keyStoreType != null) {
            String pemEncryptionType = null;
            char[] passphrase = null;
            if (keyStoreType.equals(keyStoreTypes[5])) {
                pemEncryptionType = ((String)(JOptionPane.showInputDialog(this, "Please select the desired encryption type (default = AES-256-CFB)", "Select encryption format", JOptionPane.QUESTION_MESSAGE, null, pemEncryptionTypes, pemEncryptionTypes[9])));
                if (pemEncryptionType == null) {return;}
                if (pemEncryptionType.equals(pemEncryptionTypes[36]) == false) {
                    passphrase = KeyStoreHelper.enterPassphrase(this);
                } else {
                    passphrase = "".toCharArray();
                }
            } else {
                passphrase = KeyStoreHelper.enterPassphrase(this);
            }
            if (passphrase == null) {return;}

            String fileName = KeyStoreHelper.saveDialog(this, null);
            if (fileName == null) {return;}

                   if (keyStoreType.equals(keyStoreTypes[0])) {
                KeyStoreHelper.saveAsPFXNewType(oldKeyStore, oldPassphrase, fileName, passphrase);

            } else if (keyStoreType.equals(keyStoreTypes[1])) {
                KeyStoreHelper.saveAsJKS(oldKeyStore, oldPassphrase, fileName, passphrase);

            } else if (keyStoreType.equals(keyStoreTypes[2])) {
                KeyStoreHelper.saveAsBKS(oldKeyStore, oldPassphrase, fileName, passphrase);

            } else if (keyStoreType.equals(keyStoreTypes[3])) {
                KeyStoreHelper.saveAsUBER(oldKeyStore, oldPassphrase, fileName, passphrase);

            } else if (keyStoreType.equals(keyStoreTypes[4])) {
                KeyStoreHelper.saveAsJCEKS(oldKeyStore, oldPassphrase, fileName, passphrase);

            } else if (keyStoreType.equals(keyStoreTypes[5])) {
                KeyStoreHelper.saveAsPEM(pemEncryptionType, oldKeyStore, oldPassphrase, fileName, passphrase, this.fSRNGSeeder.newSecureRandom());

            } else if (keyStoreType.equals(keyStoreTypes[6])) {
                KeyStoreHelper.saveAsPFXOldType(oldKeyStore, oldPassphrase, fileName, passphrase);

            }
        }
    }

    public void importCertificate() {
        String fileName = KeyStoreHelper.openDialog(this, null);
        X509Certificate x509 = null;
        if (fileName != null) {
            x509 = KeyStoreHelper.loadX509(fileName);                        
        }
        if (x509 != null) {
            KeyStoreHelper.importConfirmOverwrite(x509, this.fKeyStore);            
            KeyManagerGUI.updateKeyStoreTree(this.fKeyStore, this.fPassphrase, this.keyStoreTree);
            this.x509PropertiesTree.setModel(null);
        } else {
            if (fileName != null) {
                JOptionPane.showMessageDialog(this, "Could not open/import \"" + fileName + "\"", "There was an error", JOptionPane.ERROR_MESSAGE);
            }
        }
    }

    public void importKeyStore() {
        String fileName = KeyStoreHelper.openDialog(this, null);
        char[] passphrase = null;
        KeyStore keyStore = null;
        if (fileName != null) {
            passphrase = KeyStoreHelper.enterPassphrase(this);
            if (passphrase != null) {
                keyStore = KeyStoreHelper.openKeyStore(fileName, passphrase);
            }
        }
        if (keyStore != null) {
            KeyStoreHelper.importConfirmOverwrite(keyStore, passphrase, this.fKeyStore, this.fPassphrase);
            KeyManagerGUI.updateKeyStoreTree(this.fKeyStore, this.fPassphrase, this.keyStoreTree);
            this.x509PropertiesTree.setModel(null);
        } else {
            if (fileName != null) {
                JOptionPane.showMessageDialog(this, "Could not open/import \"" + fileName + "\"", "There was an error", JOptionPane.ERROR_MESSAGE);
            }
        }
    }

    public void refresh() {
        KeyManagerGUI.updateKeyStoreTree(this.fKeyStore, this.fPassphrase, this.keyStoreTree);
        this.x509PropertiesTree.setModel(null);
    }

    public void openKeyStore() {
        this.newKeyStore();
        this.importKeyStore();
    }
    
    public void newKeyStore() {
        KeyStore keyStore = KeyStoreHelper.newEmptyKeyStore();
        SecureRandom secureRandom = new SecureRandom(); //for temporary passphrase generation
        byte[] buffer = new byte[64];
        secureRandom.nextBytes(buffer);
        String string = new String(buffer);
        char[] passphrase = string.toCharArray();
        this.fKeyStore = keyStore;
        this.fPassphrase = passphrase;
        KeyManagerGUI.updateKeyStoreTree(this.fKeyStore, this.fPassphrase, this.keyStoreTree);
        this.x509PropertiesTree.setModel(null);
    }

    protected SRNGSeeder fSRNGSeeder = new SRNGSeeder();

    public static void showDialog() {
        KeyStore keyStore = KeyStoreHelper.newEmptyKeyStore();
        SecureRandom secureRandom = new SecureRandom();
        byte[] buffer = new byte[64];
        secureRandom.nextBytes(buffer);
        String string = new String(buffer);
        char[] passphrase = string.toCharArray();
        KeyManagerGUI.showDialog(keyStore, passphrase);
    }

    public static void updateKeyStoreTree(KeyStore keyStore, char[] passphrase, JTree keyStoreTree) {
        DefaultMutableTreeNode root = new DefaultMutableTreeNode(); root.setUserObject("Available Certificates/Keypairs");
        try {
            Enumeration<String> aliases = keyStore.aliases();
            String alias = null;
            while (aliases.hasMoreElements()) {
                alias = aliases.nextElement();
                if (keyStore.entryInstanceOf(alias, PrivateKeyEntry.class)) {
                    try {
                        PrivateKeyEntry privateKeyEntry = ((PrivateKeyEntry)(keyStore.getEntry(alias, new KeyStore.PasswordProtection(passphrase))));
                        if (privateKeyEntry.getCertificateChain() instanceof X509Certificate[]) {
                            X509Certificate[] chain = ((X509Certificate[])(privateKeyEntry.getCertificateChain()));
                            Entry entry = new Entry();
                            entry.alias = alias;
                            entry.displayName = (new DNProcessor(chain[0].getSubjectDN().getName())).getCommonName();
                            DefaultMutableTreeNode node = new DefaultMutableTreeNode();
                            node.setUserObject(entry);
                            root.add(node);
                        }
                    } catch (Exception e2) {}

                } else if (keyStore.entryInstanceOf(alias, SecretKeyEntry.class)) {
                    try {
                        //for now do nothing
                    } catch (Exception e2) {}

                } else if (keyStore.entryInstanceOf(alias, TrustedCertificateEntry.class)) {
                    try {
                        TrustedCertificateEntry trustedCertificateEntry = ((TrustedCertificateEntry)(keyStore.getEntry(alias, null)));
                        if (trustedCertificateEntry.getTrustedCertificate() instanceof X509Certificate) {
                            X509Certificate x509 = ((X509Certificate)(trustedCertificateEntry.getTrustedCertificate()));
                            Entry entry = new Entry();
                            entry.alias = alias;
                            entry.displayName = (new DNProcessor(x509.getSubjectDN().getName())).getCommonName();
                            DefaultMutableTreeNode node = new DefaultMutableTreeNode();
                            node.setUserObject(entry);
                            root.add(node);
                        }
                    } catch (Exception e2) {}
                }
            }

        } catch (Exception e) {
            JOptionPane.showMessageDialog(null, "There was an error \"" + e + "\"", "Something did no go right", JOptionPane.ERROR_MESSAGE);
        }
        DefaultTreeModel defaultTreeModel = new DefaultTreeModel(root);
        keyStoreTree.setModel(defaultTreeModel);
    }

    public static void showDialog(KeyStore keyStore, char[] passphrase) {
        KeyManagerGUI keyManagerGUI = new KeyManagerGUI(null, true);
        keyManagerGUI.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
        keyManagerGUI.setLocationByPlatform(true);
        keyManagerGUI.fKeyStore = keyStore;
        keyManagerGUI.fPassphrase = passphrase;
        KeyManagerGUI.updateKeyStoreTree(keyStore, passphrase, keyManagerGUI.keyStoreTree);
        keyManagerGUI.x509PropertiesTree.setModel(null);
        if (keyManagerGUI.fSRNGSeeder == null) {
            try {
                keyManagerGUI.fSRNGSeeder = new SRNGSeeder();
            } catch (Exception e) {}
        }

        keyManagerGUI.mnuCertificationRequest.setEnabled(true);
        keyManagerGUI.mnuSelfSignedRootCACertificate.setEnabled(true);
        keyManagerGUI.mnuSetCA.setEnabled(true);
        keyManagerGUI.mnuInspectCA.setEnabled(false);
        keyManagerGUI.mnuClearCA.setEnabled(false);
        keyManagerGUI.mnuSignCertificationRequest.setEnabled(false);
        keyManagerGUI.mnuCreateCertificate.setEnabled(false);

        keyManagerGUI.setLocationRelativeTo(null);
        keyManagerGUI.setVisible(true);
    }

    public void updateX509PropertiesTree(X509Certificate[] chain, PrivateKey privateKey, JTree jTree) {
        DefaultMutableTreeNode root = new DefaultMutableTreeNode(); root.setUserObject("v" + chain[0].getVersion() + " X.509 " + chain[0].getPublicKey().getAlgorithm() + " (Serial no.: " + Utils.toHex(chain[0].getSerialNumber().toByteArray(), true) + ")");

        if (chain[0].getSigAlgName().equals("1.2.840.113549.1.1.10")) {
            DefaultMutableTreeNode signatureType = new DefaultMutableTreeNode(); signatureType.setUserObject("Signature Algorithm:  RSASSA_PSS");
                root.add(signatureType);
        } else {
            DefaultMutableTreeNode signatureType = new DefaultMutableTreeNode(); signatureType.setUserObject("Signature Algorithm:  " + chain[0].getSigAlgName());
                root.add(signatureType);
        }

        DefaultMutableTreeNode keySize = new DefaultMutableTreeNode();
            if (chain[0].getPublicKey().getAlgorithm().toUpperCase().contains("RSA")) {
                try {
                    KeyFactory keyFactory = KeyFactory.getInstance("RSA");
                    RSAPublicKeySpec rsaPublicKeySpec = ((RSAPublicKeySpec)(keyFactory.getKeySpec(chain[0].getPublicKey(), RSAPublicKeySpec.class)));
                    BigInteger bigInteger = rsaPublicKeySpec.getModulus();
                    keySize.setUserObject("Keysize (in bits): " + bigInteger.toString(2).length());
                } catch (Exception e) {
                    keySize.setUserObject("Keysize: unknown keysize");
                }
            }
            root.add(keySize);

        if (privateKey != null) {
            DefaultMutableTreeNode privkey = new DefaultMutableTreeNode(); privkey.setUserObject("Private Key:  present");
            root.add(privkey);
        } else {
            DefaultMutableTreeNode privkey = new DefaultMutableTreeNode(); privkey.setUserObject("Private Key:  n/a");
            root.add(privkey);
        }

        if (KeyStoreHelper.isSelfSigned(chain[0])) {
            DefaultMutableTreeNode selfsigned = new DefaultMutableTreeNode(); selfsigned.setUserObject("Self-signed:  true");
            root.add(selfsigned);
        } else {
            DefaultMutableTreeNode selfsigned = new DefaultMutableTreeNode(); selfsigned.setUserObject("Self-signed:  false");
            root.add(selfsigned);
        }

        int pathDepth = chain[0].getBasicConstraints();
        if (pathDepth == -1) {
            DefaultMutableTreeNode isCA = new DefaultMutableTreeNode(); isCA.setUserObject("Is or can act as CA:  false");
            root.add(isCA);
        } else {
            if (pathDepth == Integer.MAX_VALUE) {
                DefaultMutableTreeNode isCA = new DefaultMutableTreeNode(); isCA.setUserObject("Is or can act as CA:  true (path depth: unlimited)");
                root.add(isCA);
            } else {
                DefaultMutableTreeNode isCA = new DefaultMutableTreeNode(); isCA.setUserObject("Is or can act as CA:  true (path depth: " + pathDepth + ")");
                root.add(isCA);
            }
        }

        DNProcessor dn = new DNProcessor(chain[0]);
        this.lblX509Properties.setText("Properties of \"" + dn.getCommonName() + "\"");
        DefaultMutableTreeNode subject = new DefaultMutableTreeNode(); subject.setUserObject("Issued To");
            DefaultMutableTreeNode subjectCN = new DefaultMutableTreeNode(); subjectCN.setUserObject("Common Name:  " + dn.getCommonName()); subject.add(subjectCN);
            DefaultMutableTreeNode subjectOU = new DefaultMutableTreeNode(); subjectOU.setUserObject("Organizational Unit:  " + dn.getOrganizationalUnit()); subject.add(subjectOU);
            DefaultMutableTreeNode subjectO  = new DefaultMutableTreeNode(); subjectO.setUserObject("Organization:  " + dn.getOrganization()); subject.add(subjectO);
            DefaultMutableTreeNode subjectL  = new DefaultMutableTreeNode(); subjectL.setUserObject("Locality:  " + dn.getLocality()); subject.add(subjectL);
            DefaultMutableTreeNode subjectST = new DefaultMutableTreeNode(); subjectST.setUserObject("State or Province:  " + dn.getStateOrProvince()); subject.add(subjectST);
            DefaultMutableTreeNode subjectC  = new DefaultMutableTreeNode(); subjectC.setUserObject("Country:  " + dn.getCountry()); subject.add(subjectC);
            DefaultMutableTreeNode subjectUID = new DefaultMutableTreeNode(); subjectUID.setUserObject("Unique ID: " + dn.getUID()); subject.add(subjectUID);
            DefaultMutableTreeNode subjectEmail = new DefaultMutableTreeNode(); subjectEmail.setUserObject("Email: " + dn.getEmailAddress()); subject.add(subjectEmail);
            DefaultMutableTreeNode subjectWebsite = new DefaultMutableTreeNode(); subjectWebsite.setUserObject("Website: " + dn.getURI()); subject.add(subjectWebsite);
            DefaultMutableTreeNode subjectIP = new DefaultMutableTreeNode(); subjectIP.setUserObject("IPAddress: " + dn.getIPAddress()); subject.add(subjectIP);
            root.add(subject);

        dn = new DNProcessor(chain[0].getIssuerDN().getName());
        DefaultMutableTreeNode issuer = new DefaultMutableTreeNode(); issuer.setUserObject("Issued By");
            DefaultMutableTreeNode issuerCN = new DefaultMutableTreeNode(); issuerCN.setUserObject("Common Name:  " + dn.getCommonName()); issuer.add(issuerCN);
            DefaultMutableTreeNode issuerOU = new DefaultMutableTreeNode(); issuerOU.setUserObject("Organizational Unit:  " + dn.getOrganizationalUnit()); issuer.add(issuerOU);
            DefaultMutableTreeNode issuerO  = new DefaultMutableTreeNode(); issuerO.setUserObject("Organization:  " + dn.getOrganization()); issuer.add(issuerO);
            DefaultMutableTreeNode issuerL  = new DefaultMutableTreeNode(); issuerL.setUserObject("Locality:  " + dn.getLocality()); issuer.add(issuerL);
            DefaultMutableTreeNode issuerST = new DefaultMutableTreeNode(); issuerST.setUserObject("State or Province:  " + dn.getStateOrProvince()); issuer.add(issuerST);
            DefaultMutableTreeNode issuerC  = new DefaultMutableTreeNode(); issuerC.setUserObject("Country:  " + dn.getCountry()); issuer.add(issuerC);
            DefaultMutableTreeNode issuerUID = new DefaultMutableTreeNode(); issuerUID.setUserObject("Unique ID: " + dn.getUID()); issuer.add(issuerUID);
            root.add(issuer);

        if (chain.length > 1) {
            int i = 1-1+1; int n = chain.length-1;
            while (i <= (n-1)) {
                if (KeyStoreHelper.isSelfSigned(chain[i]) == false) {
                    dn = new DNProcessor(chain[i].getIssuerDN().getName());
                    issuer = new DefaultMutableTreeNode(); issuer.setUserObject("Issued By");
                        issuerCN = new DefaultMutableTreeNode(); issuerCN.setUserObject("Common Name:  " + dn.getCommonName()); issuer.add(issuerCN);
                        issuerOU = new DefaultMutableTreeNode(); issuerOU.setUserObject("Organizational Unit:  " + dn.getOrganizationalUnit()); issuer.add(issuerOU);
                        issuerO  = new DefaultMutableTreeNode(); issuerO.setUserObject("Organization:  " + dn.getOrganization()); issuer.add(issuerO);
                        issuerL  = new DefaultMutableTreeNode(); issuerL.setUserObject("Locality:  " + dn.getLocality()); issuer.add(issuerL);
                        issuerST = new DefaultMutableTreeNode(); issuerST.setUserObject("State or Province:  " + dn.getStateOrProvince()); issuer.add(issuerST);
                        issuerC  = new DefaultMutableTreeNode(); issuerC.setUserObject("Country:  " + dn.getCountry()); issuer.add(issuerC);
                        issuerUID = new DefaultMutableTreeNode(); issuerUID.setUserObject("Unique ID: " + dn.getUID()); issuer.add(issuerUID);
                        root.add(issuer);
                }
                i = i + 1;
            }
        }

        DefaultMutableTreeNode validityPeriod = new DefaultMutableTreeNode(); validityPeriod.setUserObject("Validity Period");
            DefaultMutableTreeNode notBefore = new DefaultMutableTreeNode(); notBefore.setUserObject("Not Before:  " + chain[0].getNotBefore()); validityPeriod.add(notBefore);
            DefaultMutableTreeNode notAfter = new DefaultMutableTreeNode(); notAfter.setUserObject("Not After:  " + chain[0].getNotAfter()); validityPeriod.add(notAfter);
            DefaultMutableTreeNode currentlyValid = new DefaultMutableTreeNode();
            try {
                chain[0].checkValidity();
                currentlyValid.setUserObject("Currently *within* validity period".toUpperCase());
            } catch (CertificateExpiredException e) {
                currentlyValid.setUserObject("N.B.(!): Certificate/keypair *is no longer valid*".toUpperCase());
                this.lblX509Properties.setText("N.B.(!):  \"" + (new KeyStoreHelper.DNProcessor(chain[0].getSubjectDN().getName())).getCommonName() + "\" HAS EXPIRED!");
                JOptionPane.showMessageDialog(this, "N.B.(!):  \"" + (new KeyStoreHelper.DNProcessor(chain[0].getSubjectDN().getName())).getCommonName() + "\" has EXPIRED!", "Certificate Validity Warning", JOptionPane.WARNING_MESSAGE);
            } catch (CertificateNotYetValidException e2) {
                currentlyValid.setUserObject("N.B.(!): Certificate/keypair *not yet valid*".toUpperCase());
                this.lblX509Properties.setText("N.B.(!):  \"" + (new KeyStoreHelper.DNProcessor(chain[0].getSubjectDN().getName())).getCommonName() + "\" IS NOT(!) YET VALID");
                JOptionPane.showMessageDialog(this, "N.B.(!):  \"" + (new KeyStoreHelper.DNProcessor(chain[0].getSubjectDN().getName())).getCommonName() + "\" is NOT(!) YET VALID", "Certificate Validity Warning", JOptionPane.WARNING_MESSAGE);
            }                                
            validityPeriod.add(currentlyValid);
            root.add(validityPeriod);

        DefaultMutableTreeNode pubkey = new DefaultMutableTreeNode();
        pubkey.setUserObject("Public key:  " + chain[0].getPublicKey().toString());
        root.add(pubkey);

        DefaultMutableTreeNode signature = new DefaultMutableTreeNode();
        signature.setUserObject("Signature: " + Utils.toHex(chain[0].getSignature(), true));
        root.add(signature);

        try {
            boolean[] keyUsage = chain[0].getKeyUsage();
            if (keyUsage != null) {
                DefaultMutableTreeNode keyUsageNode = new DefaultMutableTreeNode();
                //keyUsageNode.setUserObject("Key Usage");
                keyUsageNode.setUserObject("Key Usage (" + Utils.toHex(chain[0].getExtensionValue("2.5.29.15"), true) + ")");
                if (keyUsage[0] == true) {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("digitalSignature: YES");
                    keyUsageNode.add(ku);
                } else {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("digitalSignature: NO");
                    keyUsageNode.add(ku);
                }
                if (keyUsage[1] == true) {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("nonRepudiation: YES");
                    keyUsageNode.add(ku);
                } else {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("nonRepudiation: NO");
                    keyUsageNode.add(ku);
                }
                if (keyUsage[2] == true) {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("keyEncipherment: YES");
                    keyUsageNode.add(ku);
                } else {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("keyEncipherment: NO");
                    keyUsageNode.add(ku);
                }
                if (keyUsage[3] == true) {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("dataEncipherment: YES");
                    keyUsageNode.add(ku);
                } else {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("dataEncipherment: NO");
                    keyUsageNode.add(ku);
                }
                if (keyUsage[4] == true) {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("keyAgreement: YES");
                    keyUsageNode.add(ku);
                } else {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("keyAgreement: NO");
                    keyUsageNode.add(ku);
                }
                if (keyUsage[5] == true) {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("keyCertSign: YES");
                    keyUsageNode.add(ku);
                } else {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("keyCertSign: NO");
                    keyUsageNode.add(ku);
                }
                if (keyUsage[6] == true) {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("cRLSign: YES");
                    keyUsageNode.add(ku);
                } else {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("cRLSign: NO");
                    keyUsageNode.add(ku);
                }
                if (keyUsage[7] == true) {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("encipherOnly: YES");
                    keyUsageNode.add(ku);
                } else {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("encipherOnly: NO");
                    keyUsageNode.add(ku);
                }
                if (keyUsage[8] == true) {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("decipherOnly: YES");
                    keyUsageNode.add(ku);
                } else {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("decipherOnly: NO");
                    keyUsageNode.add(ku);
                }
                root.add(keyUsageNode);
            }
        } catch (Exception e) {}

        DefaultMutableTreeNode fingerprints = new DefaultMutableTreeNode(); fingerprints.setUserObject("Fingerprints");
            KeyStoreHelper.assertProviders();
            try {
                byte[] encodedCert = chain[0].getEncoded();
                try {
                    MessageDigest messageDigest = MessageDigest.getInstance("SHA-512", "BC");
                    messageDigest.update(encodedCert);
                    DefaultMutableTreeNode hash = new DefaultMutableTreeNode("SHA-512:  " + Utils.toHex(messageDigest.digest(), true));
                    fingerprints.add(hash);
                } catch (Exception e) {}
                try {
                    MessageDigest messageDigest = MessageDigest.getInstance("SHA-256", "BC");
                    messageDigest.update(encodedCert);
                    DefaultMutableTreeNode hash = new DefaultMutableTreeNode("SHA-256:  " + Utils.toHex(messageDigest.digest(), true));
                    fingerprints.add(hash);
                } catch (Exception e) {}
                try {
                    MessageDigest messageDigest = MessageDigest.getInstance("SHA1");
                    messageDigest.update(encodedCert);
                    DefaultMutableTreeNode hash = new DefaultMutableTreeNode("SHA1:  " + Utils.toHex(messageDigest.digest(), true));
                    fingerprints.add(hash);
                } catch (Exception e) {}
                try {
                    MessageDigest messageDigest = MessageDigest.getInstance("MD5");
                    messageDigest.update(encodedCert);
                    DefaultMutableTreeNode hash = new DefaultMutableTreeNode("MD5:  " + Utils.toHex(messageDigest.digest(), true));
                    fingerprints.add(hash);
                } catch (Exception e) {}
                try {
                    MessageDigest messageDigest = MessageDigest.getInstance("Whirlpool", "BC");
                    messageDigest.update(encodedCert);
                    DefaultMutableTreeNode hash = new DefaultMutableTreeNode("Whirlpool:  " + Utils.toHex(messageDigest.digest(), true));
                    fingerprints.add(hash);
                } catch (Exception e) {}
                try {
                    MessageDigest messageDigest = MessageDigest.getInstance("GOST3411", "BC");
                    messageDigest.update(encodedCert);
                    DefaultMutableTreeNode hash = new DefaultMutableTreeNode("GOST3411:  " + Utils.toHex(messageDigest.digest(), true));
                    fingerprints.add(hash);
                } catch (Exception e) {}
                try {
                    MessageDigest messageDigest = MessageDigest.getInstance("RipeMD128", "BC");
                    messageDigest.update(encodedCert);
                    DefaultMutableTreeNode hash = new DefaultMutableTreeNode("RipeMD128:  " + Utils.toHex(messageDigest.digest(), true));
                    fingerprints.add(hash);
                } catch (Exception e) {}
                try {
                    MessageDigest messageDigest = MessageDigest.getInstance("RipeMD160", "BC");
                    messageDigest.update(encodedCert);
                    DefaultMutableTreeNode hash = new DefaultMutableTreeNode("RipeMD160:  " + Utils.toHex(messageDigest.digest(), true));
                    fingerprints.add(hash);
                } catch (Exception e) {}
                try {
                    MessageDigest messageDigest = MessageDigest.getInstance("RipeMD256", "BC");
                    messageDigest.update(encodedCert);
                    DefaultMutableTreeNode hash = new DefaultMutableTreeNode("RipeMD256:  " + Utils.toHex(messageDigest.digest(), true));
                    fingerprints.add(hash);
                } catch (Exception e) {}
                try {
                    MessageDigest messageDigest = MessageDigest.getInstance("RipeMD320", "BC");
                    messageDigest.update(encodedCert);
                    DefaultMutableTreeNode hash = new DefaultMutableTreeNode("RipeMD320:  " + Utils.toHex(messageDigest.digest(), true));
                    fingerprints.add(hash);
                } catch (Exception e) {}
                try {
                    MessageDigest messageDigest = MessageDigest.getInstance("SHA-224", "BC");
                    messageDigest.update(encodedCert);
                    DefaultMutableTreeNode hash = new DefaultMutableTreeNode("SHA-224:  " + Utils.toHex(messageDigest.digest(), true));
                    fingerprints.add(hash);
                } catch (Exception e) {}
                try {
                    MessageDigest messageDigest = MessageDigest.getInstance("SHA-384", "BC");
                    messageDigest.update(encodedCert);
                    DefaultMutableTreeNode hash = new DefaultMutableTreeNode("SHA-384:  " + Utils.toHex(messageDigest.digest(), true));
                    fingerprints.add(hash);
                } catch (Exception e) {}
                try {
                    MessageDigest messageDigest = MessageDigest.getInstance("Tiger", "BC");
                    messageDigest.update(encodedCert);
                    DefaultMutableTreeNode hash = new DefaultMutableTreeNode("Tiger:  " + Utils.toHex(messageDigest.digest(), true));
                    fingerprints.add(hash);
                } catch (Exception e) {}
                try {
                    MessageDigest messageDigest = MessageDigest.getInstance("MD4", "BC");
                    messageDigest.update(encodedCert);
                    DefaultMutableTreeNode hash = new DefaultMutableTreeNode("MD4:  " + Utils.toHex(messageDigest.digest(), true));
                    fingerprints.add(hash);
                } catch (Exception e) {}
                try {
                    MessageDigest messageDigest = MessageDigest.getInstance("MD2", "BC");
                    messageDigest.update(encodedCert);
                    DefaultMutableTreeNode hash = new DefaultMutableTreeNode("MD2:  " + Utils.toHex(messageDigest.digest(), true));
                    fingerprints.add(hash);
                } catch (Exception e) {}
            root.add(fingerprints);
            } catch (Exception e2) {}

        try {
        DefaultMutableTreeNode pgpData = new DefaultMutableTreeNode(); pgpData.setUserObject("PGP");            
        DefaultMutableTreeNode pgpKeyID = new DefaultMutableTreeNode(); pgpKeyID.setUserObject("PGP KeyID:  " + PGPHelper.getPGPKeyID(chain[0])); pgpData.add(pgpKeyID);
        DefaultMutableTreeNode pgpFingerprint = new DefaultMutableTreeNode(); pgpFingerprint.setUserObject("PGP Fingerprint:  " + PGPHelper.getPGPFingerprintAsHexEncodedString(chain[0], true)); pgpData.add(pgpFingerprint);
            root.add(pgpData);
        } catch (Exception e) {}

        DefaultTreeModel defaultTreeModel = new DefaultTreeModel(root);
        jTree.setModel(defaultTreeModel);

        int i = 1-1; int n = root.getChildCount();
        while (i <= (n-1)) {
            jTree.expandPath(new TreePath(defaultTreeModel.getPathToRoot(root.getChildAt(i))));
            int j = 1-1; int m = root.getChildAt(i).getChildCount();
            while (j <= (m-1)) {
                jTree.expandPath(new TreePath(defaultTreeModel.getPathToRoot(root.getChildAt(i).getChildAt(j))));
                j = j + 1;
            }
            i = i + 1;
        }
    }

    ///** Creates new form X509PropertiesDialog */
    public KeyManagerGUI(java.awt.Frame parent, boolean modal) {
//        super(parent, modal);
        initComponents();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        keyStoreTreePopUpMenu = new javax.swing.JPopupMenu();
        mnuExport = new javax.swing.JMenuItem();
        mnuDelete = new javax.swing.JMenuItem();
        mnuPrivateKey = new javax.swing.JMenu();
        mnuExportPrivateKey = new javax.swing.JMenuItem();
        mnuImportPrivateKey = new javax.swing.JMenuItem();
        mnuDetachPrivateKey = new javax.swing.JMenuItem();
        mnuRemovePrivateKey = new javax.swing.JMenuItem();
        jScrollPane1 = new javax.swing.JScrollPane();
        x509PropertiesTree = new javax.swing.JTree();
        jScrollPane2 = new javax.swing.JScrollPane();
        keyStoreTree = new javax.swing.JTree();
        lblX509Properties = new javax.swing.JLabel();
        jMenuBar1 = new javax.swing.JMenuBar();
        mnuFile = new javax.swing.JMenu();
        mnuNewKeyStore = new javax.swing.JMenuItem();
        mnuOpenKeyStore = new javax.swing.JMenuItem();
        mnuSaveAs = new javax.swing.JMenuItem();
        jSeparator1 = new javax.swing.JSeparator();
        mnuQuit = new javax.swing.JMenuItem();
        mnuEdit = new javax.swing.JMenu();
        mnuImportKeyStore = new javax.swing.JMenuItem();
        mnuImportCertificate = new javax.swing.JMenuItem();
        mnuRefresh = new javax.swing.JMenuItem();
        mnuCertificateAuthority = new javax.swing.JMenu();
        mnuCertificationRequest = new javax.swing.JMenuItem();
        mnuSelfSignedRootCACertificate = new javax.swing.JMenuItem();
        jSeparator2 = new javax.swing.JSeparator();
        mnuSetCA = new javax.swing.JMenuItem();
        mnuInspectCA = new javax.swing.JMenuItem();
        mnuClearCA = new javax.swing.JMenuItem();
        jSeparator3 = new javax.swing.JSeparator();
        mnuSignCertificationRequest = new javax.swing.JMenuItem();
        mnuCreateCertificate = new javax.swing.JMenuItem();
        mnuDigestsAndSignatures = new javax.swing.JMenu();
        mnuMessageDigest = new javax.swing.JMenuItem();
        mnuCreateOpenPGPDetachedSignature = new javax.swing.JMenuItem();
        mnuVerifyOpenPGPDetachedSignature = new javax.swing.JMenuItem();

        keyStoreTreePopUpMenu.setComponentPopupMenu(keyStoreTreePopUpMenu);

        mnuExport.setText("Export");
        mnuExport.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mnuExportActionPerformed(evt);
            }
        });
        keyStoreTreePopUpMenu.add(mnuExport);

        mnuDelete.setText("Delete");
        mnuDelete.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mnuDeleteActionPerformed(evt);
            }
        });
        keyStoreTreePopUpMenu.add(mnuDelete);

        mnuPrivateKey.setText("Private Key");

        mnuExportPrivateKey.setText("Export Private Key");
        mnuExportPrivateKey.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mnuExportPrivateKeyActionPerformed(evt);
            }
        });
        mnuPrivateKey.add(mnuExportPrivateKey);

        mnuImportPrivateKey.setText("Import Private Key");
        mnuImportPrivateKey.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mnuImportPrivateKeyActionPerformed(evt);
            }
        });
        mnuPrivateKey.add(mnuImportPrivateKey);

        mnuDetachPrivateKey.setText("Detach Private Key");
        mnuDetachPrivateKey.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mnuDetachPrivateKeyActionPerformed(evt);
            }
        });
        mnuPrivateKey.add(mnuDetachPrivateKey);

        mnuRemovePrivateKey.setText("Remove Private Key");
        mnuRemovePrivateKey.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mnuRemovePrivateKeyActionPerformed(evt);
            }
        });
        mnuPrivateKey.add(mnuRemovePrivateKey);

        keyStoreTreePopUpMenu.add(mnuPrivateKey);

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Keystore Manager (C)2009 by J.H.H.C. van der Meijs");

        jScrollPane1.setViewportView(x509PropertiesTree);

        keyStoreTree.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                keyStoreTreeMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                keyStoreTreeMouseReleased(evt);
            }
        });
        keyStoreTree.addTreeSelectionListener(new javax.swing.event.TreeSelectionListener() {
            public void valueChanged(javax.swing.event.TreeSelectionEvent evt) {
                keyStoreTreeValueChanged(evt);
            }
        });
        jScrollPane2.setViewportView(keyStoreTree);

        lblX509Properties.setText("Certificate/key Properties:");

        mnuFile.setText("File");

        mnuNewKeyStore.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_N, java.awt.event.InputEvent.CTRL_MASK));
        mnuNewKeyStore.setText("New KeyStore");
        mnuNewKeyStore.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mnuNewKeyStoreActionPerformed(evt);
            }
        });
        mnuFile.add(mnuNewKeyStore);

        mnuOpenKeyStore.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_O, java.awt.event.InputEvent.CTRL_MASK));
        mnuOpenKeyStore.setText("Open KeyStore");
        mnuOpenKeyStore.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mnuOpenKeyStoreActionPerformed(evt);
            }
        });
        mnuFile.add(mnuOpenKeyStore);

        mnuSaveAs.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_S, java.awt.event.InputEvent.CTRL_MASK));
        mnuSaveAs.setText("Save As...");
        mnuSaveAs.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mnuSaveAsActionPerformed(evt);
            }
        });
        mnuFile.add(mnuSaveAs);
        mnuFile.add(jSeparator1);

        mnuQuit.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_X, java.awt.event.InputEvent.CTRL_MASK));
        mnuQuit.setText("Quit");
        mnuQuit.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mnuQuitActionPerformed(evt);
            }
        });
        mnuFile.add(mnuQuit);

        jMenuBar1.add(mnuFile);

        mnuEdit.setText("Edit");

        mnuImportKeyStore.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_I, java.awt.event.InputEvent.CTRL_MASK));
        mnuImportKeyStore.setText("Import KeyStore");
        mnuImportKeyStore.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mnuImportKeyStoreActionPerformed(evt);
            }
        });
        mnuEdit.add(mnuImportKeyStore);

        mnuImportCertificate.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_I, java.awt.event.InputEvent.ALT_MASK));
        mnuImportCertificate.setText("Import Certificate");
        mnuImportCertificate.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mnuImportCertificateActionPerformed(evt);
            }
        });
        mnuEdit.add(mnuImportCertificate);

        mnuRefresh.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_F5, 0));
        mnuRefresh.setText("Refresh");
        mnuRefresh.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mnuRefreshActionPerformed(evt);
            }
        });
        mnuEdit.add(mnuRefresh);

        jMenuBar1.add(mnuEdit);

        mnuCertificateAuthority.setText("Certificate Authority");

        mnuCertificationRequest.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_R, java.awt.event.InputEvent.CTRL_MASK));
        mnuCertificationRequest.setText("New Certification Request");
        mnuCertificationRequest.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mnuCertificationRequestActionPerformed(evt);
            }
        });
        mnuCertificateAuthority.add(mnuCertificationRequest);

        mnuSelfSignedRootCACertificate.setText("New Self-signed Root CA Certificate");
        mnuSelfSignedRootCACertificate.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mnuSelfSignedRootCACertificateActionPerformed(evt);
            }
        });
        mnuCertificateAuthority.add(mnuSelfSignedRootCACertificate);
        mnuCertificateAuthority.add(jSeparator2);

        mnuSetCA.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_F2, 0));
        mnuSetCA.setText("Set Certificate Authority");
        mnuSetCA.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mnuSetCAActionPerformed(evt);
            }
        });
        mnuCertificateAuthority.add(mnuSetCA);

        mnuInspectCA.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_F3, 0));
        mnuInspectCA.setText("Inspect Certificate Authority");
        mnuInspectCA.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mnuInspectCAActionPerformed(evt);
            }
        });
        mnuCertificateAuthority.add(mnuInspectCA);

        mnuClearCA.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_F4, 0));
        mnuClearCA.setText("Clear Certificate Authority");
        mnuClearCA.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mnuClearCAActionPerformed(evt);
            }
        });
        mnuCertificateAuthority.add(mnuClearCA);
        mnuCertificateAuthority.add(jSeparator3);

        mnuSignCertificationRequest.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_F9, 0));
        mnuSignCertificationRequest.setText("Sign certification request");
        mnuSignCertificationRequest.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mnuSignCertificationRequestActionPerformed(evt);
            }
        });
        mnuCertificateAuthority.add(mnuSignCertificationRequest);

        mnuCreateCertificate.setText("Create new certificate");
        mnuCreateCertificate.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mnuCreateCertificateActionPerformed(evt);
            }
        });
        mnuCertificateAuthority.add(mnuCreateCertificate);

        jMenuBar1.add(mnuCertificateAuthority);

        mnuDigestsAndSignatures.setText("Digests and signatures");

        mnuMessageDigest.setText("Calculate message digest (hash)");
        mnuMessageDigest.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mnuMessageDigestActionPerformed(evt);
            }
        });
        mnuDigestsAndSignatures.add(mnuMessageDigest);

        mnuCreateOpenPGPDetachedSignature.setText("Create OpenPGP Detached Signature");
        mnuCreateOpenPGPDetachedSignature.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mnuCreateOpenPGPDetachedSignatureActionPerformed(evt);
            }
        });
        mnuDigestsAndSignatures.add(mnuCreateOpenPGPDetachedSignature);

        mnuVerifyOpenPGPDetachedSignature.setText("Verify OpenPGP Detached Signature");
        mnuVerifyOpenPGPDetachedSignature.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mnuVerifyOpenPGPDetachedSignatureActionPerformed(evt);
            }
        });
        mnuDigestsAndSignatures.add(mnuVerifyOpenPGPDetachedSignature);

        jMenuBar1.add(mnuDigestsAndSignatures);

        setJMenuBar(jMenuBar1);

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(jScrollPane2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 169, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .add(6, 6, 6)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(lblX509Properties)
                    .add(jScrollPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 400, Short.MAX_VALUE))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(lblX509Properties)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, jScrollPane2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 362, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, jScrollPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 362, Short.MAX_VALUE))
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void keyStoreTreeValueChanged(javax.swing.event.TreeSelectionEvent evt) {//GEN-FIRST:event_keyStoreTreeValueChanged
        TreePath treePath = this.keyStoreTree.getSelectionPath();
        if (treePath == null) {
            this.x509PropertiesTree.setModel(null);
            this.lblX509Properties.setText("No certificate/keypair selected. ");
        } else {
            if ((this.keyStoreTree.getSelectionCount() == 0) || (this.keyStoreTree.getSelectionCount() > 1)) {
                this.x509PropertiesTree.setModel(null);
                if (this.keyStoreTree.getSelectionCount() > 1) {
                    this.lblX509Properties.setText("Multiple certificates/keypairs selected. ");
                } else {
                    this.lblX509Properties.setText("No certificate/keypair selected. ");
                }
            } else {
                Object[] objects = treePath.getPath();
                Object object = objects[objects.length-1];
                if ((object instanceof DefaultMutableTreeNode) == false) { 
                    this.x509PropertiesTree.setModel(null);
                    this.lblX509Properties.setText("No certificate/keypair selected. ");
                } else {
                    DefaultMutableTreeNode node = ((DefaultMutableTreeNode)(object));
                    if (node.getUserObject() instanceof Entry) {
                        Entry entry = ((Entry)(node.getUserObject()));
                        String alias = entry.alias;
                        try {
                            if (this.fKeyStore.entryInstanceOf(alias, PrivateKeyEntry.class)) {
                                try {
                                    PrivateKeyEntry privateKeyEntry = ((PrivateKeyEntry)(this.fKeyStore.getEntry(alias, new KeyStore.PasswordProtection(this.fPassphrase))));
                                    if (privateKeyEntry.getCertificateChain() instanceof X509Certificate[]) {
                                        X509Certificate[] chain = ((X509Certificate[])(privateKeyEntry.getCertificateChain()));
                                        PrivateKey privateKey = privateKeyEntry.getPrivateKey();
                                        this.updateX509PropertiesTree(chain, privateKey, this.x509PropertiesTree);
                                    }
                                } catch (Exception e2) {}

                            } else if (this.fKeyStore.entryInstanceOf(alias, SecretKeyEntry.class)) {
                                try {
                                    //for now do nothing
                                } catch (Exception e2) {}

                            } else if (this.fKeyStore.entryInstanceOf(alias, TrustedCertificateEntry.class)) {
                                try {
                                    TrustedCertificateEntry trustedCertificateEntry = ((TrustedCertificateEntry)(this.fKeyStore.getEntry(alias, null)));
                                    if (trustedCertificateEntry.getTrustedCertificate() instanceof X509Certificate) {
                                        X509Certificate x509 = ((X509Certificate)(trustedCertificateEntry.getTrustedCertificate()));
                                        X509Certificate[] chain = new X509Certificate[1];
                                        chain[0] = x509;
                                        this.updateX509PropertiesTree(chain, null, this.x509PropertiesTree);
                                    }
                                } catch (Exception e2) {}
                            }
                        } catch (Exception e) {
                            JOptionPane.showMessageDialog(this, "There was an error \"" + e + "\"", "Something did no go right", JOptionPane.ERROR_MESSAGE);
                        }
                    } else {
                        this.x509PropertiesTree.setModel(null);
                        this.lblX509Properties.setText("No certificate/keypair selected. ");
                    }
                }
            }
        }
    }//GEN-LAST:event_keyStoreTreeValueChanged

    private void mnuQuitActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mnuQuitActionPerformed
        this.dispose();
    }//GEN-LAST:event_mnuQuitActionPerformed

    private void mnuOpenKeyStoreActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mnuOpenKeyStoreActionPerformed
        this.openKeyStore();
    }//GEN-LAST:event_mnuOpenKeyStoreActionPerformed

    private void mnuImportKeyStoreActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mnuImportKeyStoreActionPerformed
        this.importKeyStore();
    }//GEN-LAST:event_mnuImportKeyStoreActionPerformed

    private void mnuNewKeyStoreActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mnuNewKeyStoreActionPerformed
        this.newKeyStore();
    }//GEN-LAST:event_mnuNewKeyStoreActionPerformed

    private void mnuRefreshActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mnuRefreshActionPerformed
        this.refresh();
    }//GEN-LAST:event_mnuRefreshActionPerformed

    private void mnuSaveAsActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mnuSaveAsActionPerformed
        this.saveAs(this.fKeyStore, this.fPassphrase);
    }//GEN-LAST:event_mnuSaveAsActionPerformed

    private void mnuImportCertificateActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mnuImportCertificateActionPerformed
        this.importCertificate();
    }//GEN-LAST:event_mnuImportCertificateActionPerformed

    public List<String> getSelectedAliases() {
        TreePath[] treePaths = this.keyStoreTree.getSelectionPaths();
        List<String> aliases = new LinkedList<String>();
        int i = 1-1; int n = treePaths.length;
        while (i <= (n-1)) {
            TreePath treePath = treePaths[i];
            Object[] objects = treePath.getPath();
            if (objects.length >= 1) {
                Object object = objects[objects.length-1];
                DefaultMutableTreeNode node = null;
                if (object instanceof DefaultMutableTreeNode) {
                    node = ((DefaultMutableTreeNode)(object));
                    if (node.getUserObject() instanceof Entry) {
                        aliases.add(((Entry)(node.getUserObject())).alias);
                    }
                }
            }
            i = i + 1;
        }
        return aliases;
    }


    private void keyStoreTreeMousePressed(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_keyStoreTreeMousePressed
        if (evt.isPopupTrigger()) {
            try {
                if (this.keyStoreTree.getSelectionCount() != 0) {
                    List<String> aliases = this.getSelectedAliases();
                    if (aliases.size() == 0) {
                        //no actual selection => so do nothing
                    } else if (aliases.size() == 1) {
                        this.mnuDelete.setEnabled(true);
                        this.mnuExport.setEnabled(true);
                        this.mnuPrivateKey.setEnabled(true);
                        if (this.fKeyStore.entryInstanceOf(aliases.get(0), PrivateKeyEntry.class)) {                            
                            this.mnuExportPrivateKey.setEnabled(true);
                            this.mnuImportPrivateKey.setEnabled(false);
                            this.mnuDetachPrivateKey.setEnabled(true);
                            this.mnuRemovePrivateKey.setEnabled(true);
                        } else {                            
                            this.mnuExportPrivateKey.setEnabled(false);
                            this.mnuImportPrivateKey.setEnabled(true);
                            this.mnuDetachPrivateKey.setEnabled(false);
                            this.mnuRemovePrivateKey.setEnabled(false);
                        }
                        this.keyStoreTreePopUpMenu.show(this.keyStoreTree, evt.getX(), evt.getY());

                    } else if (aliases.size() > 1) {
                        boolean containsPrivateKeyMaterial = false;
                        Iterator<String> i = aliases.iterator();
                        while (i.hasNext()) {
                            String alias = i.next();
                            if (this.fKeyStore.entryInstanceOf(alias, PrivateKeyEntry.class)) {
                                containsPrivateKeyMaterial = true;
                                break;
                            }
                        }

                        this.mnuDelete.setEnabled(true);
                        this.mnuExport.setEnabled(true);
                        this.mnuPrivateKey.setEnabled(true);
                        if (containsPrivateKeyMaterial) {                            
                            this.mnuExportPrivateKey.setEnabled(false);
                            this.mnuImportPrivateKey.setEnabled(false);
                            this.mnuDetachPrivateKey.setEnabled(false);
                            this.mnuRemovePrivateKey.setEnabled(true);
                        } else {
                            this.mnuPrivateKey.setEnabled(false);                            
                            this.mnuExportPrivateKey.setEnabled(false);
                            this.mnuImportPrivateKey.setEnabled(false);
                            this.mnuDetachPrivateKey.setEnabled(false);
                            this.mnuRemovePrivateKey.setEnabled(false);
                        }

                        this.keyStoreTreePopUpMenu.show(this.keyStoreTree, evt.getX(), evt.getY());
                    }
                }
            } catch (Exception e) {
                JOptionPane.showMessageDialog(this, "There was an error \"" + e + "\"", "Something did no go right", JOptionPane.ERROR_MESSAGE);
            }
        }
    }//GEN-LAST:event_keyStoreTreeMousePressed

    private void keyStoreTreeMouseReleased(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_keyStoreTreeMouseReleased
        this.keyStoreTreeMousePressed(evt);
    }//GEN-LAST:event_keyStoreTreeMouseReleased

    public void delete() {
        List<String> aliases = this.getSelectedAliases();
        if (aliases.size() == 0) {
            //then there is nothing to delete
        } else {
            String all = "";
            Iterator<String> i = aliases.iterator();
            while (i.hasNext()) {
                all = all + i.next() + "\n\r";
            }
            int choice = JOptionPane.showConfirmDialog(this, "Are you sure you wish to permanently delete the following item(s) from the keystore?\n\r\n\r" + all + "\n\rPressing [YES] will delete this/these item(s), N.B.(!): there is no undo for this operation.", "Confirm delete", JOptionPane.YES_NO_CANCEL_OPTION);
            if (choice == JOptionPane.YES_OPTION) {
                i = aliases.iterator();
                while (i.hasNext()) {
                    try {
                        this.fKeyStore.deleteEntry(i.next());
                    } catch (Exception e) {
                        JOptionPane.showMessageDialog(this, "There was an error \"" + e + "\"", "Something did no go right", JOptionPane.ERROR_MESSAGE);
                    }
                }
                this.refresh();
            }
        }
    }

    private void mnuDeleteActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mnuDeleteActionPerformed
        this.delete();
    }//GEN-LAST:event_mnuDeleteActionPerformed

    public void export() {
        KeyStore keyStore = KeyStoreHelper.newEmptyKeyStore();
        SecureRandom secureRandom = new SecureRandom();
        byte[] buffer = new byte[64];
        secureRandom.nextBytes(buffer);
        String string = new String(buffer);
        char[] passphrase = string.toCharArray();

        List<String> aliases = this.getSelectedAliases();

        boolean containsPrivateKeyMaterial = false;
        Iterator<String> i = aliases.iterator();
        while (i.hasNext()) {
            String alias = i.next();
            try {
                if (this.fKeyStore.entryInstanceOf(alias, PrivateKeyEntry.class)) {
                    containsPrivateKeyMaterial = true;
                    break;
                }
            } catch (Exception e) {}
        }

        boolean exportPrivateKeyMaterial = false;
        if (containsPrivateKeyMaterial) {
            int choice = JOptionPane.showConfirmDialog(this, "Do you wish to export the private key(s) as well?", "Export Private Key(s)??", JOptionPane.YES_NO_OPTION);
            if (choice == JOptionPane.YES_OPTION) {
                exportPrivateKeyMaterial = true;
            }
        }

        if ((aliases.size() == 1) && (exportPrivateKeyMaterial == false)) {
            String[] certificateTypes = new String[3];
            certificateTypes[0] = "DER encoded (.cer/.der)";
            certificateTypes[1] = "PEM encoded (.crt/.pem)";
            certificateTypes[2] = "As key store....";
            String certificateType = ((String)(JOptionPane.showInputDialog(this, "Please select the desired keystore type", "Select keystore format", JOptionPane.QUESTION_MESSAGE, null, certificateTypes, certificateTypes[1])));
            if (certificateType != null) {
                String alias = aliases.get(0);
                X509Certificate x509 = null;
                try {
                    if (this.fKeyStore.entryInstanceOf(alias, PrivateKeyEntry.class)) {
                        PrivateKeyEntry privateKeyEntry = ((PrivateKeyEntry)(this.fKeyStore.getEntry(alias, new KeyStore.PasswordProtection(this.fPassphrase))));
                        if (privateKeyEntry.getCertificateChain() instanceof X509Certificate[]) {
                            X509Certificate[] chain = ((X509Certificate[])(privateKeyEntry.getCertificateChain()));
                            x509 = chain[0];
                        }
                    } else if (this.fKeyStore.entryInstanceOf(alias, TrustedCertificateEntry.class)) {
                        TrustedCertificateEntry trustedCertificateEntry = ((TrustedCertificateEntry)(this.fKeyStore.getEntry(alias, null)));
                        if (trustedCertificateEntry.getTrustedCertificate() instanceof X509Certificate) {
                            x509 = ((X509Certificate)(trustedCertificateEntry.getTrustedCertificate()));
                        }
                    }
                } catch (Exception e) {}
                if (x509 != null) {
                    if (certificateType.equals(certificateTypes[2])) {
                        try {
                            keyStore.setCertificateEntry(alias, x509);
                        } catch (Exception e) {}
                        this.saveAs(keyStore, passphrase);
                    } else {
                        String fileName = KeyStoreHelper.saveDialog(this, null);
                        if (fileName != null) {
                            if (certificateType.equals(certificateTypes[0])) {
                                KeyStoreHelper.saveAsCER(x509, fileName);
                            } else if (certificateType.equals(certificateTypes[1])) {
                                KeyStoreHelper.saveAsCRT(x509, fileName);
                            }
                        }
                    }
                }
            }
        } else if (aliases.size() >= 1) {
            i = aliases.iterator();
            while (i.hasNext()) {
                String alias = i.next();
                try {
                    if (this.fKeyStore.entryInstanceOf(alias, PrivateKeyEntry.class)) {
                        try {
                            PrivateKeyEntry privateKeyEntry = ((PrivateKeyEntry)(this.fKeyStore.getEntry(alias, new KeyStore.PasswordProtection(this.fPassphrase))));
                            if (privateKeyEntry.getCertificateChain() instanceof X509Certificate[]) {
                                X509Certificate[] chain = ((X509Certificate[])(privateKeyEntry.getCertificateChain()));
                                PrivateKey privateKey = privateKeyEntry.getPrivateKey();
                                if (exportPrivateKeyMaterial) {
                                    keyStore.setKeyEntry(alias, privateKey, passphrase, chain);
                                } else {
                                    keyStore.setCertificateEntry(alias, chain[0]);
                                }
                            }
                        } catch (Exception e2) {}

                    } else if (this.fKeyStore.entryInstanceOf(alias, SecretKeyEntry.class)) {
                        try {
                            //for now do nothing
                        } catch (Exception e2) {}

                    } else if (this.fKeyStore.entryInstanceOf(alias, TrustedCertificateEntry.class)) {
                        try {
                            TrustedCertificateEntry trustedCertificateEntry = ((TrustedCertificateEntry)(this.fKeyStore.getEntry(alias, null)));
                            if (trustedCertificateEntry.getTrustedCertificate() instanceof X509Certificate) {
                                X509Certificate x509 = ((X509Certificate)(trustedCertificateEntry.getTrustedCertificate()));
                                keyStore.setCertificateEntry(alias, x509);
                            }
                        } catch (Exception e2) {}
                    }
                } catch (Exception e) {
                    JOptionPane.showMessageDialog(this, "There was an error \"" + e + "\"", "Something did no go right", JOptionPane.ERROR_MESSAGE);
                }
            }
            this.saveAs(keyStore, passphrase);
        }
        
    }

    private void mnuExportActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mnuExportActionPerformed
        this.export();
    }//GEN-LAST:event_mnuExportActionPerformed

    public void removePrivateKeys() {
        List<String> aliases = this.getSelectedAliases();
        if (aliases.size() == 0) {
            //then there is nothing to remove
        } else {
            String all = "";
            Iterator<String> i = aliases.iterator();
            while (i.hasNext()) {
                all = all + i.next() + "\n\r";
            }
            int choice = JOptionPane.showConfirmDialog(this, "Are you sure you wish to permanently delete the private key(s) of the following item(s)?\n\r\n\r" + all + "\n\rPressing [YES] will remove the private key(s), N.B.(!): there is no undo for this operation.", "Confirm removal of private key(s)", JOptionPane.YES_NO_CANCEL_OPTION);
            if (choice == JOptionPane.YES_OPTION) {
                i = aliases.iterator();
                while (i.hasNext()) {
                    String alias = i.next();
                    try {
                        if (this.fKeyStore.entryInstanceOf(alias, PrivateKeyEntry.class)) {
                            try {
                                PrivateKeyEntry privateKeyEntry = ((PrivateKeyEntry)(this.fKeyStore.getEntry(alias, new KeyStore.PasswordProtection(this.fPassphrase))));
                                if (privateKeyEntry.getCertificateChain() instanceof X509Certificate[]) {
                                    X509Certificate[] chain = ((X509Certificate[])(privateKeyEntry.getCertificateChain()));
                                    this.fKeyStore.deleteEntry(alias);
                                    this.fKeyStore.setCertificateEntry(alias, chain[0]);
                                }
                            } catch (Exception e2) {}
                        }
                    } catch (Exception e) {
                        JOptionPane.showMessageDialog(this, "There was an error \"" + e + "\"", "Something did no go right", JOptionPane.ERROR_MESSAGE);
                    }
                }
                this.refresh();
            }
        }
    }

    private void mnuRemovePrivateKeyActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mnuRemovePrivateKeyActionPerformed
        this.removePrivateKeys();
    }//GEN-LAST:event_mnuRemovePrivateKeyActionPerformed


    public boolean storePrivateKey() {
        List<String> aliases = this.getSelectedAliases();
        if (aliases.size() == 1) {
            String alias = aliases.get(0);
            try {
                if (this.fKeyStore.entryInstanceOf(alias, PrivateKeyEntry.class)) {
                    PrivateKeyEntry privateKeyEntry = ((PrivateKeyEntry)(this.fKeyStore.getEntry(alias, new KeyStore.PasswordProtection(this.fPassphrase))));
                    String[] pemEncryptionTypes = new String[37];
                    pemEncryptionTypes[ 0] = "AES-128-CBC";
                    pemEncryptionTypes[ 1] = "AES-128-CFB";
                    pemEncryptionTypes[ 2] = "AES-128-ECB";
                    pemEncryptionTypes[ 3] = "AES-128-OFB";
                    pemEncryptionTypes[ 4] = "AES-192-CBC";
                    pemEncryptionTypes[ 5] = "AES-192-CFB";
                    pemEncryptionTypes[ 6] = "AES-192-ECB";
                    pemEncryptionTypes[ 7] = "AES-192-OFB";
                    pemEncryptionTypes[ 8] = "AES-256-CBC";
                    pemEncryptionTypes[ 9] = "AES-256-CFB";
                    pemEncryptionTypes[10] = "AES-256-ECB";
                    pemEncryptionTypes[11] = "AES-256-OFB";
                    pemEncryptionTypes[12] = "BF-CBC";
                    pemEncryptionTypes[13] = "BF-CFB";
                    pemEncryptionTypes[14] = "BF-ECB";
                    pemEncryptionTypes[15] = "BF-OFB";
                    pemEncryptionTypes[16] = "DES-CBC";
                    pemEncryptionTypes[17] = "DES-CFB";
                    pemEncryptionTypes[18] = "DES-ECB";
                    pemEncryptionTypes[19] = "DES-OFB";
                    pemEncryptionTypes[20] = "DES-EDE";
                    pemEncryptionTypes[21] = "DES-EDE-CBC";
                    pemEncryptionTypes[22] = "DES-EDE-CFB";
                    pemEncryptionTypes[23] = "DES-EDE-ECB";
                    pemEncryptionTypes[24] = "DES-EDE-OFB";
                    pemEncryptionTypes[25] = "DES-EDE3";
                    pemEncryptionTypes[26] = "DES-EDE3-CBC";
                    pemEncryptionTypes[27] = "DES-EDE3-CFB";
                    pemEncryptionTypes[28] = "DES-EDE3-ECB";
                    pemEncryptionTypes[29] = "DES-EDE3-OFB";
                    pemEncryptionTypes[30] = "RC2-CBC";
                    pemEncryptionTypes[31] = "RC2-CFB";
                    pemEncryptionTypes[32] = "RC2-ECB";
                    pemEncryptionTypes[33] = "RC2-OFB";
                    pemEncryptionTypes[34] = "RC2-40-CBC";
                    pemEncryptionTypes[35] = "RC2-64-CBC";
                    pemEncryptionTypes[36] = "NO_ENCRYPTION";
                    String pemEncryptionType = ((String)(JOptionPane.showInputDialog(this, "Please select the desired encryption type (default = AES-256-CFB)", "Select encryption format", JOptionPane.QUESTION_MESSAGE, null, pemEncryptionTypes, pemEncryptionTypes[9])));
                    if (pemEncryptionType == null) {return false;}
                    char[] passphrase = null;
                    if (pemEncryptionType.equals(pemEncryptionTypes[36]) == false) {
                        passphrase = KeyStoreHelper.enterPassphrase(this);
                    } else {
                        passphrase = "".toCharArray();
                    }
                    if (passphrase == null) {return false;}
                    String fileName = KeyStoreHelper.saveDialog(this, null);
                    if (fileName == null) {return false;}
                    if (KeyStoreHelper.saveAsKEYFile(pemEncryptionType, privateKeyEntry.getPrivateKey(), passphrase, fileName) != null) {
                        return true;
                    }
                }
            } catch (Exception e) {}
        }
        return false;
    }

    private void mnuExportPrivateKeyActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mnuExportPrivateKeyActionPerformed
        this.storePrivateKey();
    }//GEN-LAST:event_mnuExportPrivateKeyActionPerformed

    public void importPrivateKey() {
        List<String> aliases = this.getSelectedAliases();
        if (aliases.size() == 1) {
            String alias = aliases.get(0);
            try {
                if (this.fKeyStore.entryInstanceOf(alias, PrivateKeyEntry.class)) {
                    return;
                }
            } catch (Exception e) {}
            X509Certificate x509 = null;
            try {
                if (this.fKeyStore.entryInstanceOf(alias, TrustedCertificateEntry.class)) {
                    TrustedCertificateEntry trustedCertificateEntry = ((TrustedCertificateEntry)(this.fKeyStore.getEntry(alias, null)));
                    if (trustedCertificateEntry.getTrustedCertificate() instanceof X509Certificate) {
                        x509 = ((X509Certificate)(trustedCertificateEntry.getTrustedCertificate()));
                    }
                }
            } catch (Exception e) {}
            if (x509 != null) {
                X509Certificate[] chain = new X509Certificate[1];
                chain[0] = x509;

                String fileName = KeyStoreHelper.openDialog(this, null);
                if (fileName != null) {
                    char[] passphrase = KeyStoreHelper.enterPassphrase(this);
                    if (passphrase != null) {
                        KeyPair keyPair = KeyStoreHelper.loadKEYFile(fileName, passphrase);
                        if (keyPair != null) {
                            try {
                                if (KeyStoreHelper.samePublicKey(x509.getPublicKey(), keyPair.getPublic())) {
                                    this.fKeyStore.deleteEntry(alias);
                                    this.fKeyStore.setKeyEntry(alias, keyPair.getPrivate(), this.fPassphrase, chain);
                                    this.refresh();
                                    JOptionPane.showMessageDialog(this, "Private key successfully imported!\n\r(" + alias + ")", "Private key import", JOptionPane.INFORMATION_MESSAGE);
                                } else {
                                    JOptionPane.showMessageDialog(this, "Private key belongs to different certificate!", "Could not import private key", JOptionPane.ERROR_MESSAGE);
                                }
                            } catch (Exception e) {
                                JOptionPane.showMessageDialog(this, "Could not import private key due to error \"" + e + "\"", "Could not import private key", JOptionPane.ERROR_MESSAGE);
                            }
                        } else {
                            JOptionPane.showMessageDialog(this, "Could not import private key", "Could not import private key", JOptionPane.ERROR_MESSAGE);
                        }
                    }
                }
            }
        }
    }


    private void mnuImportPrivateKeyActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mnuImportPrivateKeyActionPerformed
        this.importPrivateKey();
    }//GEN-LAST:event_mnuImportPrivateKeyActionPerformed

    private void mnuDetachPrivateKeyActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mnuDetachPrivateKeyActionPerformed
        if (this.storePrivateKey()) {
            this.removePrivateKeys();
        }
    }//GEN-LAST:event_mnuDetachPrivateKeyActionPerformed

    private void mnuCertificationRequestActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mnuCertificationRequestActionPerformed
        CreateCertificationRequestDialog.showDialog(this);
    }//GEN-LAST:event_mnuCertificationRequestActionPerformed

    private void mnuSetCAActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mnuSetCAActionPerformed
        this.selectCA();
        if ((this.fCACertificate != null) && (this.fCAPrivateKey != null)) {
            this.mnuCertificationRequest.setEnabled(false);
            this.mnuSelfSignedRootCACertificate.setEnabled(false);
            this.mnuSetCA.setEnabled(false);
            this.mnuInspectCA.setEnabled(true);
            this.mnuClearCA.setEnabled(true);
            this.mnuSignCertificationRequest.setEnabled(true);
            this.mnuCreateCertificate.setEnabled(true);
        } else {
            this.mnuCertificationRequest.setEnabled(true);
            this.mnuSelfSignedRootCACertificate.setEnabled(true);
            this.mnuSetCA.setEnabled(true);
            this.mnuInspectCA.setEnabled(false);
            this.mnuClearCA.setEnabled(false);
            this.mnuSignCertificationRequest.setEnabled(false);
            this.mnuCreateCertificate.setEnabled(false);
        }
    }//GEN-LAST:event_mnuSetCAActionPerformed

    private void mnuInspectCAActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mnuInspectCAActionPerformed
        this.inspectCA();
    }//GEN-LAST:event_mnuInspectCAActionPerformed

    private void mnuClearCAActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mnuClearCAActionPerformed
        this.clearCA();
        this.mnuCertificationRequest.setEnabled(true);
        this.mnuSelfSignedRootCACertificate.setEnabled(true);
        this.mnuSetCA.setEnabled(true);
        this.mnuInspectCA.setEnabled(false);
        this.mnuClearCA.setEnabled(false);
        this.mnuSignCertificationRequest.setEnabled(false);
        this.mnuCreateCertificate.setEnabled(false);
    }//GEN-LAST:event_mnuClearCAActionPerformed

    private void mnuSignCertificationRequestActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mnuSignCertificationRequestActionPerformed
        this.signCertRequest();
    }//GEN-LAST:event_mnuSignCertificationRequestActionPerformed

    private void mnuCreateCertificateActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mnuCreateCertificateActionPerformed
        this.createNewCertificate();
    }//GEN-LAST:event_mnuCreateCertificateActionPerformed

    private void mnuSelfSignedRootCACertificateActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mnuSelfSignedRootCACertificateActionPerformed
        this.createNewSelfSignedRootCACertificate();
    }//GEN-LAST:event_mnuSelfSignedRootCACertificateActionPerformed

    private void mnuCreateOpenPGPDetachedSignatureActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mnuCreateOpenPGPDetachedSignatureActionPerformed
        PrivateKeyEntry[] privateKeyEntries = KeySelectionDialog.showDialog(this, this.fKeyStore, this.fPassphrase);
        if (privateKeyEntries == null) {return;}
        if (privateKeyEntries.length == 0) {return;}
        X509Certificate[] chain = null;

        if (privateKeyEntries[0].getCertificateChain() instanceof X509Certificate[]) {
            chain = ((X509Certificate[])(privateKeyEntries[0].getCertificateChain()));

            String fileName = KeyStoreHelper.openDialog(this, null);
            if (fileName != null) {
                String signatureFileName = KeyStoreHelper.saveDialog(this, (fileName + ".sig"));
                if (signatureFileName != null) {
                    if (PGPHelper.createPGPDetachedSignature(chain[0], privateKeyEntries[0].getPrivateKey(), fileName, signatureFileName) == true) {
                        JOptionPane.showMessageDialog(this, "Signature created successfully", "PGP Detached Signature Creation", JOptionPane.INFORMATION_MESSAGE);
                    } else {
                        JOptionPane.showMessageDialog(this, "There was an error", "PGP Detached Signature Creation", JOptionPane.ERROR_MESSAGE);
                    }
                }
            }

        }
    }//GEN-LAST:event_mnuCreateOpenPGPDetachedSignatureActionPerformed

    private void mnuVerifyOpenPGPDetachedSignatureActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mnuVerifyOpenPGPDetachedSignatureActionPerformed
        try {
            Enumeration e = this.fKeyStore.aliases();
            if (e.hasMoreElements() == false) {
                JOptionPane.showMessageDialog(this, "Empty keystore! Cannot verify detached signature" , "Signature Verification Failure", JOptionPane.ERROR_MESSAGE);
                return;
            }
        } catch (Exception e) {
            JOptionPane.showMessageDialog(this, "Keystore error! Cannot verify detached signature" , "Signature Verification Failure", JOptionPane.ERROR_MESSAGE);
            return;
        }
        String fileName = KeyStoreHelper.openDialog(this, null);
        if (fileName != null) {
            String signatureFileName = KeyStoreHelper.openDialog(this, (fileName + ".sig"));
            if (signatureFileName != null) {
                String[] keyIDs = PGPHelper.getPGPDetachedSignatureKeyIDs(signatureFileName);
                if (keyIDs == null) {
                    JOptionPane.showMessageDialog(this, "There was an error, could not obtain keyIDs from signature(?) file", "PGP Detached Signature Verification", JOptionPane.ERROR_MESSAGE);
                    return;
                }
                if (keyIDs.length == 0) {
                    JOptionPane.showMessageDialog(this, "There was an error, no keyIDs in signature(?) file", "PGP Detached Signature Verification", JOptionPane.ERROR_MESSAGE);
                    return;
                }

                int i = 1-1; int n = keyIDs.length;
                while (i <= (n-1)) {
                    String keyID = keyIDs[i];
                    X509Certificate x509 = PGPHelper.fetchKey(this.fKeyStore, this.fPassphrase, keyID);
                    if (x509 == null) {
                        JOptionPane.showMessageDialog(this, "Unknown key (" + keyID + ")! Cannot verify detached signature" , "Signature Verification Failure", JOptionPane.ERROR_MESSAGE);
                        return;
                        //try to find a match (in case the keyid is wrong)
                    }
                    if (PGPHelper.verifyPGPDetachedSignature(x509, fileName, signatureFileName) == true) {
                        PGPSignature pgpSignature = PGPHelper.getPGPSignature(signatureFileName, keyID);
                        String message = "VALID signature from \"" + (new DNProcessor(x509)).getCommonName() + "\" (" + keyID + ")" + "\n\r" + "Signature version: " + pgpSignature.getVersion() + "\n\r";
                        if (pgpSignature.getHashAlgorithm() == PGPHelper.HASH_ALGORITHM_HAVAL_5_160) {
                               message = message + "Hash algorithm: HAVAL_5_160" + "\n\r";

                        } else if (pgpSignature.getHashAlgorithm() == PGPHelper.HASH_ALGORITHM_RIPEMD160) {
                               message = message + "Hash algorithm: RIPEMD160" + "\n\r";

                        } else if (pgpSignature.getHashAlgorithm() == PGPHelper.HASH_ALGORITHM_SHA1) {
                               message = message + "Hash algorithm: SHA1" + "\n\r";

                        } else if (pgpSignature.getHashAlgorithm() == PGPHelper.HASH_ALGORITHM_SHA224) {
                               message = message + "Hash algorithm: SHA224" + "\n\r";

                        } else if (pgpSignature.getHashAlgorithm() == PGPHelper.HASH_ALGORITHM_SHA256) {
                               message = message + "Hash algorithm: SHA256" + "\n\r";

                        } else if (pgpSignature.getHashAlgorithm() == PGPHelper.HASH_ALGORITHM_SHA384) {
                               message = message + "Hash algorithm: SHA384" + "\n\r";

                        } else if (pgpSignature.getHashAlgorithm() == PGPHelper.HASH_ALGORITHM_SHA512) {
                               message = message + "Hash algorithm: SHA512" + "\n\r";

                        } else if (pgpSignature.getHashAlgorithm() == PGPHelper.HASH_ALGORITHM_TIGER_192) {
                               message = message + "Hash algorithm: TIGER_192" + "\n\r";

                        } else {
                               message = message + "Hash algorithm: OTHER (" + pgpSignature.getHashAlgorithm() + ")" + "\n\r";

                        }
                        message = message + "Date/time-group: " + pgpSignature.getCreationTime().toString();
                        
                        JOptionPane.showMessageDialog(this, message , "Signature Verified Successfully", JOptionPane.INFORMATION_MESSAGE);
                        X509PropertiesDialog.showDialog(this, x509);

                    } else {
                        JOptionPane.showMessageDialog(this, "INVALID SIGNATURE! Bad signature from \"" + (new DNProcessor(x509)).getCommonName() + "\" (" + keyID + ")" , "Signature Verification Failure", JOptionPane.ERROR_MESSAGE);

                    }
                    i = i + 1;
                }
            }
        }
    }//GEN-LAST:event_mnuVerifyOpenPGPDetachedSignatureActionPerformed

    private void mnuMessageDigestActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mnuMessageDigestActionPerformed
        MessageDigestDialog.showDialog(this);
    }//GEN-LAST:event_mnuMessageDigestActionPerformed

    protected PrivateKeyEntry fCAPrivateKeyEntry = null;
    protected X509Certificate[] fCAChain = null;
    protected X509Certificate fCACertificate = null;
    protected PrivateKey fCAPrivateKey = null;

    public void clearCA() {
        this.fCAPrivateKeyEntry = null;
        this.fCAChain = null;
        this.fCACertificate = null;
        this.fCAPrivateKey = null;
    }

    public void selectCA() {
        this.clearCA();
        String fileName = KeyStoreHelper.openDialog(this, null);
        if (fileName != null) {
            char[] passphrase = KeyStoreHelper.enterPassphrase(this);
            if (passphrase != null) {
                KeyStore keyStore = KeyStoreHelper.openKeyStore(fileName, passphrase);
                if (keyStore != null) {
                    PrivateKeyEntry[] CAKeys = KeySelectionDialog.showDialog(this, keyStore, passphrase);
                    if (CAKeys != null) {
                        if (CAKeys.length != 1) {
                            JOptionPane.showMessageDialog(this, "Please select a (single) X509 entry with a private key attached", "No certificate authority was selected", JOptionPane.ERROR_MESSAGE);
                        } else {
                            this.fCAPrivateKeyEntry = CAKeys[0];
                            if (this.fCAPrivateKeyEntry.getCertificateChain() instanceof X509Certificate[]) {
                                this.fCAChain = ((X509Certificate[])(this.fCAPrivateKeyEntry.getCertificateChain()));
                                if (this.fCAChain.length > 0) {
                                    this.fCACertificate = this.fCAChain[0];
                                    this.fCAPrivateKey = this.fCAPrivateKeyEntry.getPrivateKey();
                                    DNProcessor dnProcessor = new DNProcessor(this.fCACertificate.getSubjectX500Principal());
                                    JOptionPane.showMessageDialog(this, "The following CA was selected: \"" + dnProcessor.getCommonName() + "\"", "Certificate authority is set", JOptionPane.INFORMATION_MESSAGE);
                                }
                            }
                        }
                    }
                }
            }
        }
        if ((this.fCAPrivateKeyEntry == null) || (this.fCAChain == null) || (this.fCACertificate == null) || (this.fCAPrivateKey == null)) {
            this.clearCA();
        }
    }

    public void inspectCA() {
        if (this.fCAPrivateKeyEntry != null) {
            X509PropertiesDialog.showDialog(this, this.fCACertificate, this.fCAPrivateKey);
        } else {
            JOptionPane.showMessageDialog(this, "No CA was set", "No CA present", JOptionPane.ERROR_MESSAGE);
        }
    }

    public void signCertRequest() {
        if ((this.fCAChain != null) && (this.fCAPrivateKey != null)) {
            SignCertificationRequestDialog.showDialog(this, this.fCAChain, this.fCAPrivateKey);
        } else {
            JOptionPane.showMessageDialog(this, "No CA was set", "No CA present", JOptionPane.ERROR_MESSAGE);
        }
    }

    public void createNewCertificate() {
        if ((this.fCAChain != null) && (this.fCAPrivateKey != null)) {
            CreateCertificateDialog.showDialog(this, this.fCAChain, this.fCAPrivateKey);
        } else {
            JOptionPane.showMessageDialog(this, "No CA was set", "No CA present", JOptionPane.ERROR_MESSAGE);
        }
    }

    public void createNewSelfSignedRootCACertificate() {
        CreateSelfSignedRootCACertificateDialog.showDialog(this);
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JMenuBar jMenuBar1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JSeparator jSeparator2;
    private javax.swing.JSeparator jSeparator3;
    private javax.swing.JTree keyStoreTree;
    private javax.swing.JPopupMenu keyStoreTreePopUpMenu;
    private javax.swing.JLabel lblX509Properties;
    private javax.swing.JMenu mnuCertificateAuthority;
    private javax.swing.JMenuItem mnuCertificationRequest;
    private javax.swing.JMenuItem mnuClearCA;
    private javax.swing.JMenuItem mnuCreateCertificate;
    private javax.swing.JMenuItem mnuCreateOpenPGPDetachedSignature;
    private javax.swing.JMenuItem mnuDelete;
    private javax.swing.JMenuItem mnuDetachPrivateKey;
    private javax.swing.JMenu mnuDigestsAndSignatures;
    private javax.swing.JMenu mnuEdit;
    private javax.swing.JMenuItem mnuExport;
    private javax.swing.JMenuItem mnuExportPrivateKey;
    private javax.swing.JMenu mnuFile;
    private javax.swing.JMenuItem mnuImportCertificate;
    private javax.swing.JMenuItem mnuImportKeyStore;
    private javax.swing.JMenuItem mnuImportPrivateKey;
    private javax.swing.JMenuItem mnuInspectCA;
    private javax.swing.JMenuItem mnuMessageDigest;
    private javax.swing.JMenuItem mnuNewKeyStore;
    private javax.swing.JMenuItem mnuOpenKeyStore;
    private javax.swing.JMenu mnuPrivateKey;
    private javax.swing.JMenuItem mnuQuit;
    private javax.swing.JMenuItem mnuRefresh;
    private javax.swing.JMenuItem mnuRemovePrivateKey;
    private javax.swing.JMenuItem mnuSaveAs;
    private javax.swing.JMenuItem mnuSelfSignedRootCACertificate;
    private javax.swing.JMenuItem mnuSetCA;
    private javax.swing.JMenuItem mnuSignCertificationRequest;
    private javax.swing.JMenuItem mnuVerifyOpenPGPDetachedSignature;
    private javax.swing.JTree x509PropertiesTree;
    // End of variables declaration//GEN-END:variables

}
