/* ============================================================================
 * SignCertificationRequestDialog.java
 * ============================================================================
 * THIS SOFTWARE IS RELEASED UNDER THE FOLLOWING BINDING LEGAL AGREEMENT
 * (BSD-LICENSE).
 * ============================================================================
 *
 * Copyright (C) 2009
 * by J.H.H.C. van der Meijs (tcpconns@jhhcvandermeijs.nl)
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    * Neither the name of tcpconns nor the names of its copyright holders or
 *      its contributors may be used to endorse or promote products derived from
 *      this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 * ============================================================================
 * IF YOU DO NOT AGREE TO THE ABOVE TERMS AND CONDITIONS THEN DO *NOT* USE THIS
 * SOFTWARE AND *DO* DESTROY ALL COPIES OF THIS SOFTWARE THAT YOU HAVE.
 * ============================================================================
 *
 * TCPCONNS’ CURRENT CODE IS NOT KNOWN TO VIOLATE ANY COPYRIGHTS OR INFRINGE ON
 * ANY PATENTS. MOREOVER, IT ADHERES TO GENERAL PROGRAMMING RULES AND PRACTICES
 * AND IS OF SUCH A SIMPLISTIC NATURE THAT A SIMILAR PACKAGE COULD BE
 * CONSTRUCTED BY ANYONE. IMPORTANT(!): IF YOU FEEL THAT A PIECE OF CODE IN
 * TCPCONNS VIOLATES A COPYRIGHT OR INFRINGES ON A PATENT, PLEASE DO INFORM ONE
 * OF THE AUTHORS IMMEDIATELY. DO ALWAYS STATE EXACTLY WHAT CODE IS NOT ALLOWED
 * UNDER EXACTLY WHAT RULING, COPYRIGHT, OR PATENT; AND PLEASE DO ELABORATE ON
 * EXACTLY HOW THE CODE VIOLATES THE SPECIFIC COPYRIGHT OR INFRINGES ON
 * THE SPECIFIC PATENT.
 *
 * ============================================================================
 *
 * This software is Open Source Initiative approved Open Source Software.
 * Open Source Initiative Approved is a trademark of the Open Source Initiative.
 * http://www.opensource.org/
 *
 * ============================================================================
 */
package nl.jhhcvandermeijs.tcpconns.keys;

import java.math.BigInteger;
import java.security.KeyFactory;
import java.security.PrivateKey;
import java.security.cert.X509Certificate;
import java.security.spec.RSAPublicKeySpec;
import java.util.Date;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.SpinnerDateModel;
import nl.jhhcvandermeijs.tcpconns.keys.KeyStoreHelper.DNProcessor;
import nl.jhhcvandermeijs.tcpconns.pgp.PGPHelper;
import org.bouncycastle.asn1.x509.BasicConstraints;
import org.bouncycastle.asn1.x509.GeneralNames;
import org.bouncycastle.asn1.x509.KeyUsage;
import org.bouncycastle.asn1.x509.X509Extensions;
import org.bouncycastle.jce.PKCS10CertificationRequest;
import org.bouncycastle.openpgp.PGPPublicKey;
import org.bouncycastle.x509.X509V3CertificateGenerator;

/**
 *
 * @author username
 */
public class SignCertificationRequestDialog extends javax.swing.JDialog {
    public static final long serialVersionUID = 1L;

    /** Creates new form CreateCertificationRequestDialog */
    public SignCertificationRequestDialog(java.awt.Frame parent, boolean modal) {
        super(parent, modal);
        initComponents();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        lblKeyAlgorithm = new javax.swing.JLabel();
        lblKeySize = new javax.swing.JLabel();
        comboKeySize = new javax.swing.JComboBox();
        lblSignatureAlgorithm = new javax.swing.JLabel();
        comboSignatureAlgorithm = new javax.swing.JComboBox();
        lblCommonName = new javax.swing.JLabel();
        edtCommonName = new javax.swing.JTextField();
        lblOrganizationalUnit = new javax.swing.JLabel();
        lblOrganization = new javax.swing.JLabel();
        edtOrganizationalUnit = new javax.swing.JTextField();
        edtOrganization = new javax.swing.JTextField();
        lblLocality = new javax.swing.JLabel();
        edtLocality = new javax.swing.JTextField();
        lblStateOrProvince = new javax.swing.JLabel();
        edtStateOrProvince = new javax.swing.JTextField();
        lblCountry = new javax.swing.JLabel();
        comboCountry = new javax.swing.JComboBox();
        lblUniqueID = new javax.swing.JLabel();
        edtUniqueID = new javax.swing.JTextField();
        lblEmailAddress = new javax.swing.JLabel();
        edtEmailAddress = new javax.swing.JTextField();
        lblWebSite = new javax.swing.JLabel();
        edtWebSite = new javax.swing.JTextField();
        lblIPAddress = new javax.swing.JLabel();
        edtIPAddress = new javax.swing.JTextField();
        btnLoadRequest = new javax.swing.JButton();
        btnInspectCA = new javax.swing.JButton();
        btnSignRequest = new javax.swing.JButton();
        comboKeyAlgorithm = new javax.swing.JComboBox();
        lblValidityPeriod = new javax.swing.JLabel();
        spinValidFrom = new javax.swing.JSpinner();
        spinValidTo = new javax.swing.JSpinner();
        lblSerialNumber = new javax.swing.JLabel();
        edtSerialNumber = new javax.swing.JTextField();
        btnLoadPGPPublicKey = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Sign Certification Request");

        lblKeyAlgorithm.setText("Key algorithm");

        lblKeySize.setText("Key size");

        comboKeySize.setEnabled(false);

        lblSignatureAlgorithm.setText("Signature algorithm");

        lblCommonName.setText("Common name");

        lblOrganizationalUnit.setText("Organizational unit");

        lblOrganization.setText("Organization");

        lblLocality.setText("Locality");

        lblStateOrProvince.setText("State or province");

        lblCountry.setText("Country");

        comboCountry.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "PLEASE SELECT COUNTRY", "AFGHANISTAN (AF)", "ÅLAND ISLANDS (AX)", "ALBANIA (AL)", "ALGERIA (DZ)", "AMERICAN SAMOA (AS)", "ANDORRA (AD)", "ANGOLA (AO)", "ANGUILLA (AI)", "ANTARCTICA (AQ)", "ANTIGUA AND BARBUDA (AG)", "ARGENTINA (AR)", "ARMENIA (AM)", "ARUBA (AW)", "AUSTRALIA (AU)", "AUSTRIA (AT)", "AZERBAIJAN (AZ)", "BAHAMAS (BS)", "BAHRAIN (BH)", "BANGLADESH (BD)", "BARBADOS (BB)", "BELARUS (BY)", "BELGIUM (BE)", "BELIZE (BZ)", "BENIN (BJ)", "BERMUDA (BM)", "BHUTAN (BT)", "BOLIVIA, PLURINATIONAL STATE OF (BO)", "BOSNIA AND HERZEGOVINA (BA)", "BOTSWANA (BW)", "BOUVET ISLAND (BV)", "BRAZIL (BR)", "BRITISH INDIAN OCEAN TERRITORY (IO)", "BRUNEI DARUSSALAM (BN)", "BULGARIA (BG)", "BURKINA FASO (BF)", "BURUNDI (BI)", "CAMBODIA (KH)", "CAMEROON (CM)", "CANADA (CA)", "CAPE VERDE (CV)", "CAYMAN ISLANDS (KY)", "CENTRAL AFRICAN REPUBLIC (CF)", "CHAD (TD)", "CHILE (CL)", "CHINA (CN)", "CHRISTMAS ISLAND (CX)", "COCOS (KEELING) ISLANDS (CC)", "COLOMBIA (CO)", "COMOROS (KM)", "CONGO (CG)", "CONGO, THE DEMOCRATIC REPUBLIC OF THE (CD)", "COOK ISLANDS (CK)", "COSTA RICA (CR)", "CÔTE D'IVOIRE (CI)", "CROATIA (HR)", "CUBA (CU)", "CYPRUS (CY)", "CZECH REPUBLIC (CZ)", "DENMARK (DK)", "DJIBOUTI (DJ)", "DOMINICA (DM)", "DOMINICAN REPUBLIC (DO)", "ECUADOR (EC)", "EGYPT (EG)", "EL SALVADOR (SV)", "EQUATORIAL GUINEA (GQ)", "ERITREA (ER)", "ESTONIA (EE)", "ETHIOPIA (ET)", "FALKLAND ISLANDS (MALVINAS) (FK)", "FAROE ISLANDS (FO)", "FIJI (FJ)", "FINLAND (FI)", "FRANCE (FR)", "FRENCH GUIANA (GF)", "FRENCH POLYNESIA (PF)", "FRENCH SOUTHERN TERRITORIES (TF)", "GABON (GA)", "GAMBIA (GM)", "GEORGIA (GE)", "GERMANY (DE)", "GHANA (GH)", "GIBRALTAR (GI)", "GREECE (GR)", "GREENLAND (GL)", "GRENADA (GD)", "GUADELOUPE (GP)", "GUAM (GU)", "GUATEMALA (GT)", "GUERNSEY (GG)", "GUINEA (GN)", "GUINEA-BISSAU (GW)", "GUYANA (GY)", "HAITI (HT)", "HEARD ISLAND AND MCDONALD ISLANDS (HM)", "HONDURAS (HN)", "HONG KONG (HK)", "HUNGARY (HU)", "ICELAND (IS)", "INDIA (IN)", "INDONESIA (ID)", "IRAN, ISLAMIC REPUBLIC OF (IR)", "IRAQ (IQ)", "IRELAND (IE)", "ISLE OF MAN (IM)", "ISRAEL (IL)", "ITALY (IT)", "JAMAICA (JM)", "JAPAN (JP)", "JERSEY (JE)", "JORDAN (JO)", "KAZAKHSTAN (KZ)", "KENYA (KE)", "KIRIBATI (KI)", "KOREA, DEMOCRATIC PEOPLE'S REPUBLIC OF (KP)", "KOREA, REPUBLIC OF (KR)", "KUWAIT (KW)", "KYRGYZSTAN (KG)", "LAO PEOPLE'S DEMOCRATIC REPUBLIC (LA)", "LATVIA (LV)", "LEBANON (LB)", "LESOTHO (LS)", "LIBERIA (LR)", "LIBYAN ARAB JAMAHIRIYA (LY)", "LIECHTENSTEIN (LI)", "LITHUANIA (LT)", "LUXEMBOURG (LU)", "MACAO (MO)", "MACEDONIA, THE FORMER YUGOSLAV REPUBLIC OF (MK)", "MADAGASCAR (MG)", "MALAWI (MW)", "MALAYSIA (MY)", "MALDIVES (MV)", "MALI (ML)", "MALTA (MT)", "MARSHALL ISLANDS (MH)", "MARTINIQUE (MQ)", "MAURITANIA (MR)", "MAURITIUS (MU)", "MAYOTTE (YT)", "MEXICO (MX)", "MICRONESIA, FEDERATED STATES OF (FM)", "MOLDOVA, REPUBLIC OF (MD)", "MONACO (MC)", "MONGOLIA (MN)", "MONTENEGRO (ME)", "MONTSERRAT (MS)", "MOROCCO (MA)", "MOZAMBIQUE (MZ)", "MYANMAR (MM)", "NAMIBIA (NA)", "NAURU (NR)", "NEPAL (NP)", "NETHERLANDS (NL)", "NETHERLANDS ANTILLES (AN)", "NEW CALEDONIA (NC)", "NEW ZEALAND (NZ)", "NICARAGUA (NI)", "NIGER (NE)", "NIGERIA (NG)", "NIUE (NU)", "NORFOLK ISLAND (NF)", "NORTHERN MARIANA ISLANDS (MP)", "NORWAY (NO)", "OMAN (OM)", "PAKISTAN (PK)", "PALAU (PW)", "PALESTINIAN TERRITORY, OCCUPIED (PS)", "PANAMA (PA)", "PAPUA NEW GUINEA (PG)", "PARAGUAY (PY)", "PERU (PE)", "PHILIPPINES (PH)", "PITCAIRN (PN)", "POLAND (PL)", "PORTUGAL (PT)", "PUERTO RICO (PR)", "QATAR (QA)", "RÉUNION (RE)", "ROMANIA (RO)", "RUSSIAN FEDERATION (RU)", "RWANDA (RW)", "SAINT BARTHÉLEMY (BL)", "SAINT HELENA (SH)", "SAINT KITTS AND NEVIS (KN)", "SAINT LUCIA (LC)", "SAINT MARTIN (MF)", "SAINT PIERRE AND MIQUELON (PM)", "SAINT VINCENT AND THE GRENADINES (VC)", "SAMOA (WS)", "SAN MARINO (SM)", "SAO TOME AND PRINCIPE (ST)", "SAUDI ARABIA (SA)", "SENEGAL (SN)", "SERBIA (RS)", "SEYCHELLES (SC)", "SIERRA LEONE (SL)", "SINGAPORE (SG)", "SLOVAKIA (SK)", "SLOVENIA (SI)", "SOLOMON ISLANDS (SB)", "SOMALIA (SO)", "SOUTH AFRICA (ZA)", "SOUTH GEORGIA AND THE SOUTH SANDWICH ISLANDS (GS)", "SPAIN (ES)", "SRI LANKA (LK)", "SUDAN (SD)", "SURINAME (SR)", "SVALBARD AND JAN MAYEN (SJ)", "SWAZILAND (SZ)", "SWEDEN (SE)", "SWITZERLAND (CH)", "SYRIAN ARAB REPUBLIC (SY)", "TAIWAN, PROVINCE OF CHINA (TW)", "TAJIKISTAN (TJ)", "TANZANIA, UNITED REPUBLIC OF (TZ)", "THAILAND (TH)", "TIMOR-LESTE (TL)", "TOGO (TG)", "TOKELAU (TK)", "TONGA (TO)", "TRINIDAD AND TOBAGO (TT)", "TUNISIA (TN)", "TURKEY (TR)", "TURKMENISTAN (TM)", "TURKS AND CAICOS ISLANDS (TC)", "TUVALU (TV)", "UGANDA (UG)", "UKRAINE (UA)", "UNITED ARAB EMIRATES (AE)", "UNITED KINGDOM (GB)", "UNITED STATES (US)", "UNITED STATES MINOR OUTLYING ISLANDS (UM)", "URUGUAY (UY)", "UZBEKISTAN (UZ)", "VANUATU (VU)", "VATICAN CITY STATE (VA)", "VENEZUELA, BOLIVARIAN REPUBLIC OF (VE)", "VIETNAM (VN)", "VIRGIN ISLANDS, BRITISH (VG)", "VIRGIN ISLANDS, U.S. (VI)", "WALLIS AND FUTUNA (WF)", "WESTERN SAHARA (EH)", "YEMEN (YE)", "ZAMBIA (ZM)", "ZIMBABWE (ZW)" }));

        lblUniqueID.setText("Unique ID");

        lblEmailAddress.setText("Email address");

        lblWebSite.setText("Website");

        lblIPAddress.setText("IP address");

        btnLoadRequest.setText("Load Request");
        btnLoadRequest.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnLoadRequestActionPerformed(evt);
            }
        });

        btnInspectCA.setText("Inspect CA");
        btnInspectCA.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnInspectCAActionPerformed(evt);
            }
        });

        btnSignRequest.setText("Approve (Sign)");
        btnSignRequest.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnSignRequestActionPerformed(evt);
            }
        });

        comboKeyAlgorithm.setEnabled(false);

        lblValidityPeriod.setText("Validity period");

        spinValidFrom.setModel(new javax.swing.SpinnerDateModel(new java.util.Date(1254061700719L), null, null, java.util.Calendar.DAY_OF_MONTH));
        spinValidFrom.setMinimumSize(new java.awt.Dimension(140, 26));

        spinValidTo.setModel(new javax.swing.SpinnerDateModel(new java.util.Date(1254061700719L), null, null, java.util.Calendar.DAY_OF_MONTH));
        spinValidTo.setMinimumSize(new java.awt.Dimension(140, 26));

        lblSerialNumber.setText("Serial number");

        btnLoadPGPPublicKey.setText("load PGP public key");
        btnLoadPGPPublicKey.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnLoadPGPPublicKeyActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                            .add(lblCommonName)
                            .add(lblKeyAlgorithm)
                            .add(lblKeySize)
                            .add(lblSignatureAlgorithm)
                            .add(lblOrganizationalUnit)
                            .add(lblOrganization)
                            .add(lblLocality)
                            .add(lblStateOrProvince)
                            .add(lblCountry)
                            .add(lblUniqueID)
                            .add(lblEmailAddress)
                            .add(lblWebSite)
                            .add(lblIPAddress)
                            .add(lblValidityPeriod)
                            .add(lblSerialNumber))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(edtCommonName, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 301, Short.MAX_VALUE)
                            .add(edtOrganization, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 301, Short.MAX_VALUE)
                            .add(edtLocality, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 301, Short.MAX_VALUE)
                            .add(edtStateOrProvince, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 301, Short.MAX_VALUE)
                            .add(comboCountry, 0, 301, Short.MAX_VALUE)
                            .add(edtUniqueID, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 301, Short.MAX_VALUE)
                            .add(edtEmailAddress, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 301, Short.MAX_VALUE)
                            .add(edtWebSite, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 301, Short.MAX_VALUE)
                            .add(edtIPAddress, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 301, Short.MAX_VALUE)
                            .add(edtOrganizationalUnit, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 301, Short.MAX_VALUE)
                            .add(comboSignatureAlgorithm, 0, 301, Short.MAX_VALUE)
                            .add(comboKeySize, 0, 301, Short.MAX_VALUE)
                            .add(comboKeyAlgorithm, 0, 301, Short.MAX_VALUE)
                            .add(layout.createSequentialGroup()
                                .add(spinValidFrom, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 147, Short.MAX_VALUE)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(spinValidTo, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 148, Short.MAX_VALUE))
                            .add(edtSerialNumber, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 301, Short.MAX_VALUE)))
                    .add(layout.createSequentialGroup()
                        .add(btnLoadRequest)
                        .add(6, 6, 6)
                        .add(btnLoadPGPPublicKey)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(btnInspectCA)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .add(btnSignRequest)))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(lblKeyAlgorithm)
                    .add(comboKeyAlgorithm, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(lblKeySize)
                    .add(comboKeySize, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(lblSignatureAlgorithm)
                    .add(comboSignatureAlgorithm, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(edtCommonName, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(lblCommonName))
                .add(10, 10, 10)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(lblOrganizationalUnit)
                    .add(edtOrganizationalUnit, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(lblOrganization)
                    .add(edtOrganization, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(lblLocality)
                    .add(edtLocality, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(lblStateOrProvince)
                    .add(edtStateOrProvince, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(lblCountry)
                    .add(comboCountry, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(lblUniqueID)
                    .add(edtUniqueID, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(lblEmailAddress)
                    .add(edtEmailAddress, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(lblWebSite)
                    .add(edtWebSite, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(lblIPAddress)
                    .add(edtIPAddress, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(lblValidityPeriod)
                    .add(spinValidFrom, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(spinValidTo, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(lblSerialNumber)
                    .add(edtSerialNumber, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(btnLoadRequest)
                    .add(btnInspectCA)
                    .add(btnSignRequest)
                    .add(btnLoadPGPPublicKey))
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void btnLoadRequestActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnLoadRequestActionPerformed
        this.loadRequest();
    }//GEN-LAST:event_btnLoadRequestActionPerformed

    protected X509Certificate fX509 = null;

    public void signRequest() {
        this.fX509 = null;
        this.fRequestGenerator.setSecureRandom(null);

        this.fRequestGenerator.setKeyAlgorithm((String)(this.comboKeyAlgorithm.getSelectedItem()));

        try {
            X509V3CertificateGenerator acg = new X509V3CertificateGenerator();
            acg.setSerialNumber(BigInteger.valueOf(System.currentTimeMillis()));

            DNProcessor dnProcessor = new DNProcessor();

            if (this.edtCommonName.getText().trim().equals("") != true) {
                dnProcessor.setCommonName(this.edtCommonName.getText());
            } else {
                dnProcessor.setCommonName("n/a");
            }

            String countryName = ((String)(this.comboCountry.getSelectedItem()));
            if (countryName.contains("(") != false) {
                int position = countryName.lastIndexOf("(");
                String countryCode = countryName.substring(position + 1, position + 1 + 2);
                dnProcessor.setCountry(countryCode);
            }

            if (this.edtCommonName.getText().trim().equals("") != true) {
                dnProcessor.setEmailAddress(this.edtEmailAddress.getText());

            }
            if (this.edtIPAddress.getText().trim().equals("") != true) {
                dnProcessor.setIPAddress(this.edtIPAddress.getText());

            }
            if (this.edtLocality.getText().trim().equals("") != true) {
                dnProcessor.setLocality(this.edtLocality.getText());

            }
            if (this.edtOrganization.getText().trim().equals("") != true) {
                dnProcessor.setOrganization(this.edtOrganization.getText());

            }
            if (this.edtOrganizationalUnit.getText().trim().equals("") != true) {
                dnProcessor.setOrganizationalUnit(this.edtOrganizationalUnit.getText());

            }
            if (this.edtStateOrProvince.getText().trim().equals("") != true) {
                dnProcessor.setStateOrProvince(this.edtStateOrProvince.getText());

            }
            if (this.edtUniqueID.getText().trim().equals("") != true) {
                dnProcessor.setUID(this.edtUniqueID.getText());

            }
            if (this.edtWebSite.getText().trim().equals("") != true) {
                dnProcessor.setURI(this.edtWebSite.getText());

            }

            acg.setIssuerDN(this.fCACertificateChain[0].getSubjectX500Principal());
            acg.setNotBefore(((Date)(this.spinValidFrom.getModel().getValue())));
            acg.setNotAfter(((Date)(this.spinValidTo.getModel().getValue())));
            acg.setSubjectDN(dnProcessor.getDNStringAsX500Principal());
            acg.setPublicKey(this.fRequest.getPublicKey());
            acg.setSignatureAlgorithm(((String)(this.comboSignatureAlgorithm.getSelectedItem())));

            acg.addExtension(X509Extensions.BasicConstraints, true, new BasicConstraints(true));
            acg.addExtension(X509Extensions.KeyUsage, true, new KeyUsage(
                                            KeyUsage.cRLSign
                                          | KeyUsage.dataEncipherment
                                          | KeyUsage.digitalSignature
                                          | KeyUsage.keyAgreement
                                          | KeyUsage.keyCertSign
                                          | KeyUsage.keyEncipherment
                                          | KeyUsage.nonRepudiation));

//            acg.addExtension(X509Extensions.ExtendedKeyUsage, false, new ExtendedKeyUsage(KeyPurposeId.anyExtendedKeyUsage));

            GeneralNames generalNames = dnProcessor.getAlternativeNamesAsGeneralNames();
            if (generalNames != null) {
                acg.addExtension(X509Extensions.SubjectAlternativeName, false, generalNames);
            }

            this.fX509 = acg.generate(this.fCAPrivateKey, "BC");

        } catch (Exception e) {System.out.println(e);}
        if (this.fX509 != null) {
            X509Certificate[] chain = new X509Certificate[this.fCACertificateChain.length + 1];
            chain[0] = this.fX509;
            int i = 1-1; int n = this.fCACertificateChain.length;
            while (i <= (n-1)) {
                chain[i+1] = this.fCACertificateChain[i];
                i = i + 1;
            }
            X509PropertiesDialog.showDialog(((JFrame)(this.getParent())), chain);
        }
        this.edtSerialNumber.setText("" + System.currentTimeMillis());
    }
    
    private void btnSignRequestActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnSignRequestActionPerformed
        this.signRequest();
    }//GEN-LAST:event_btnSignRequestActionPerformed

    protected X509Certificate[] fCACertificateChain = null;
    public void setCACertificateChain(X509Certificate[] chain) {this.fCACertificateChain = chain;}

    protected PrivateKey fCAPrivateKey = null;
    public void setCAKey(PrivateKey privateKey) {this.fCAPrivateKey = privateKey;}

    private void btnInspectCAActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnInspectCAActionPerformed
        X509PropertiesDialog.showDialog(((JFrame)(this.getParent())), this.fCACertificateChain[0], this.fCAPrivateKey);
    }//GEN-LAST:event_btnInspectCAActionPerformed

    private void btnLoadPGPPublicKeyActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnLoadPGPPublicKeyActionPerformed
        String publicKeyFileName = KeyStoreHelper.openDialog(((JFrame)(this.getParent())), null);
        if (publicKeyFileName != null) {
            PGPPublicKey pgpPublicKey = PGPHelper.loadPGPPublicKey(publicKeyFileName);
            if (pgpPublicKey != null) {
                Date validFrom = PGPHelper.getValidFrom(pgpPublicKey);
                Date validTo = PGPHelper.getValidTo(pgpPublicKey);

                SpinnerDateModel spinnerDateModel = new SpinnerDateModel();
                spinnerDateModel.setValue(validFrom);
                this.spinValidFrom.setModel(spinnerDateModel);
                this.spinValidFrom.setEnabled(false);

                spinnerDateModel = new SpinnerDateModel();
                spinnerDateModel.setValue(validTo);
                this.spinValidTo.setModel(spinnerDateModel);
                this.spinValidTo.setEnabled(false);

            }
        }
    }//GEN-LAST:event_btnLoadPGPPublicKeyActionPerformed

    public static final String KEY_SIZE_2048 = "2048 bits";
    public static final String KEY_SIZE_3072 = "3072 bits";
    public static final String KEY_SIZE_4096 = "4096 bits";

    protected CertificationRequestGenerator fRequestGenerator = new CertificationRequestGenerator();

    protected PKCS10CertificationRequest fRequest = null;

    public void reset() {
        this.fRequestGenerator = new CertificationRequestGenerator();

        DefaultComboBoxModel model = new DefaultComboBoxModel();
        model.addElement("RSA");
        this.comboKeyAlgorithm.setModel(model);

        model = new DefaultComboBoxModel();
        model.addElement(KEY_SIZE_2048); //0
        model.addElement(KEY_SIZE_3072); //1
        model.addElement(KEY_SIZE_4096); //2
        this.comboKeySize.setModel(model);
        this.comboKeySize.setSelectedIndex(2); //KEY_SIZE_4096

        model = new DefaultComboBoxModel();
        model.addElement(CertificationRequestGenerator.SIGNATURE_ALGORITHM_RIPEMD256_WITH_RSA); //0
        model.addElement(CertificationRequestGenerator.SIGNATURE_ALGORITHM_SHA1_WITH_RSA); //1
        model.addElement(CertificationRequestGenerator.SIGNATURE_ALGORITHM_SHA1_WITH_RSA_AND_MGF1); //2
        model.addElement(CertificationRequestGenerator.SIGNATURE_ALGORITHM_SHA224_WITH_RSA); //3
        model.addElement(CertificationRequestGenerator.SIGNATURE_ALGORITHM_SHA256_WITH_RSA); //4
        model.addElement(CertificationRequestGenerator.SIGNATURE_ALGORITHM_SHA256_WITH_RSA_AND_MGF1); //5
        model.addElement(CertificationRequestGenerator.SIGNATURE_ALGORITHM_SHA384_WITH_RSA); //6
        model.addElement(CertificationRequestGenerator.SIGNATURE_ALGORITHM_SHA384_WITH_RSA_AND_MGF1); //7
        model.addElement(CertificationRequestGenerator.SIGNATURE_ALGORITHM_SHA512_WITH_RSA); //8
        model.addElement(CertificationRequestGenerator.SIGNATURE_ALGORITHM_SHA512_WITH_RSA_AND_MGF1); //9
        this.comboSignatureAlgorithm.setModel(model);
        this.comboSignatureAlgorithm.setSelectedIndex(8); //SHA512withRSA

        this.comboCountry.setSelectedIndex(0);

        this.edtCommonName.setText("");
        this.edtEmailAddress.setText("");
        this.edtIPAddress.setText("");
        this.edtLocality.setText("");
        this.edtOrganization.setText("");
        this.edtOrganizationalUnit.setText("");
        this.edtStateOrProvince.setText("");
        this.edtUniqueID.setText("");
        this.edtWebSite.setText("");

        this.spinValidFrom.setEnabled(true);
        SpinnerDateModel spinnerDateModel = new SpinnerDateModel();
        spinnerDateModel.setValue(new Date(System.currentTimeMillis() -     86400000L));
        this.spinValidFrom.setModel(spinnerDateModel);
        
        this.spinValidTo.setEnabled(true);
        spinnerDateModel = new SpinnerDateModel();
        spinnerDateModel.setValue(new Date(System.currentTimeMillis() + 158112000000L));
        this.spinValidTo.setModel(spinnerDateModel);

        this.btnSignRequest.setEnabled(false);
    }

    public void loadRequest() {
        KeyStoreHelper.assertProviders();
        this.reset();
        String fileName = KeyStoreHelper.openDialog(((JFrame)(this.getParent())), null);
        if (fileName != null) {
            this.fRequest = CertificationRequestGenerator.loadPKCS10CertificationRequest(fileName);
            if (this.fRequest == null) {
                JOptionPane.showMessageDialog(null, "Could not load \"" + fileName + "\"", "Failure to open certification request", JOptionPane.ERROR_MESSAGE);
                return;
            }
            try {
                this.fRequest.verify(this.fRequest.getPublicKey(), "BC");
            } catch (java.security.NoSuchAlgorithmException nsae) {
                JOptionPane.showMessageDialog(null, "An error occurred: " + nsae, "ERROR", JOptionPane.ERROR_MESSAGE);

            } catch (java.security.NoSuchProviderException nspe) {
                JOptionPane.showMessageDialog(null, "An error occurred: " + nspe, "ERROR", JOptionPane.ERROR_MESSAGE);

            } catch (java.security.InvalidKeyException ike) {
                JOptionPane.showMessageDialog(null, "Invalid key exception: " + ike, "ERROR", JOptionPane.ERROR_MESSAGE);

            } catch (java.security.SignatureException se) {
                JOptionPane.showMessageDialog(null, "Signature exception: " + se, "ERROR", JOptionPane.ERROR_MESSAGE);

            }

            DNProcessor dnProcessor = new DNProcessor(this.fRequest);
            
            this.fRequestGenerator = new CertificationRequestGenerator();

            DefaultComboBoxModel model = new DefaultComboBoxModel();
            try {
                model.addElement(this.fRequest.getPublicKey().getAlgorithm());
            } catch (Exception e) {
                model.addElement("unknown key type");
            }
            this.comboKeyAlgorithm.setModel(model);

            model = new DefaultComboBoxModel();
            try {
                if (this.fRequest.getPublicKey().getAlgorithm().toUpperCase().contains("RSA")) {
                    try {
                        KeyFactory keyFactory = KeyFactory.getInstance("RSA");
                        RSAPublicKeySpec rsaPublicKeySpec = ((RSAPublicKeySpec)(keyFactory.getKeySpec(this.fRequest.getPublicKey(), RSAPublicKeySpec.class)));
                        BigInteger bigInteger = rsaPublicKeySpec.getModulus();
                        model.addElement("" + bigInteger.toString(2).length());
                        if (bigInteger.toString(2).length() < 2000) {
                            JOptionPane.showMessageDialog(null, "INADEQUATE KEY SIZE WARNING", "Inadequate keysize", JOptionPane.ERROR_MESSAGE);
                        }
                    } catch (Exception e) {model.addElement("unknown key size");}
                } else {model.addElement("unknown key size");}
            } catch (Exception e) {model.addElement("unknown key size");}
            this.comboKeySize.setModel(model);
            this.comboKeySize.setSelectedIndex(0);

            model = new DefaultComboBoxModel();
            model.addElement(CertificationRequestGenerator.SIGNATURE_ALGORITHM_RIPEMD256_WITH_RSA); //0
            model.addElement(CertificationRequestGenerator.SIGNATURE_ALGORITHM_SHA1_WITH_RSA); //1
            model.addElement(CertificationRequestGenerator.SIGNATURE_ALGORITHM_SHA1_WITH_RSA_AND_MGF1); //2
            model.addElement(CertificationRequestGenerator.SIGNATURE_ALGORITHM_SHA224_WITH_RSA); //3
            model.addElement(CertificationRequestGenerator.SIGNATURE_ALGORITHM_SHA256_WITH_RSA); //4
            model.addElement(CertificationRequestGenerator.SIGNATURE_ALGORITHM_SHA256_WITH_RSA_AND_MGF1); //5
            model.addElement(CertificationRequestGenerator.SIGNATURE_ALGORITHM_SHA384_WITH_RSA); //6
            model.addElement(CertificationRequestGenerator.SIGNATURE_ALGORITHM_SHA384_WITH_RSA_AND_MGF1); //7
            model.addElement(CertificationRequestGenerator.SIGNATURE_ALGORITHM_SHA512_WITH_RSA); //8
            model.addElement(CertificationRequestGenerator.SIGNATURE_ALGORITHM_SHA512_WITH_RSA_AND_MGF1); //9
            this.comboSignatureAlgorithm.setModel(model);
            this.comboSignatureAlgorithm.setSelectedIndex(8); //SHA512withRSA

            this.comboCountry.setSelectedIndex(0);
            String item = null;
            int i = 1-1; int n = this.comboCountry.getItemCount();
            while (i <= (n-1)) {
                item = ((String)(this.comboCountry.getItemAt(i)));
                if (item.contains("(" + dnProcessor.getCountry() + ")")) {
                    this.comboCountry.setSelectedIndex(i);
                    break;
                }
                i = i + 1;
            }

            this.edtCommonName.setText(dnProcessor.getCommonName());
            this.edtEmailAddress.setText(dnProcessor.getEmailAddress());
            this.edtIPAddress.setText(dnProcessor.getIPAddress());
            this.edtLocality.setText(dnProcessor.getLocality());
            this.edtOrganization.setText(dnProcessor.getOrganization());
            this.edtOrganizationalUnit.setText(dnProcessor.getOrganizationalUnit());
            this.edtStateOrProvince.setText(dnProcessor.getStateOrProvince());
            this.edtUniqueID.setText(dnProcessor.getUID());
            this.edtWebSite.setText(dnProcessor.getURI());

            SpinnerDateModel spinnerDateModel = new SpinnerDateModel();
            spinnerDateModel.setValue(new Date(System.currentTimeMillis() -     86400000L));
            this.spinValidFrom.setModel(spinnerDateModel);

            spinnerDateModel = new SpinnerDateModel();
            spinnerDateModel.setValue(new Date(System.currentTimeMillis() + 158112000000L));
            this.spinValidTo.setModel(spinnerDateModel);
           
            this.edtSerialNumber.setText("" + System.currentTimeMillis());

            this.btnSignRequest.setEnabled(true);
        }
    }

    public static void showDialog(java.awt.Frame parent, X509Certificate[] CACertificateChain, PrivateKey CAPrivateKey) {
        SignCertificationRequestDialog dialog = new SignCertificationRequestDialog(parent, true);
        dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
        dialog.setLocationRelativeTo(parent);
        dialog.reset();
        dialog.setCACertificateChain(CACertificateChain);
        dialog.setCAKey(CAPrivateKey);
        dialog.setVisible(true);
    }


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnInspectCA;
    private javax.swing.JButton btnLoadPGPPublicKey;
    private javax.swing.JButton btnLoadRequest;
    private javax.swing.JButton btnSignRequest;
    private javax.swing.JComboBox comboCountry;
    private javax.swing.JComboBox comboKeyAlgorithm;
    private javax.swing.JComboBox comboKeySize;
    private javax.swing.JComboBox comboSignatureAlgorithm;
    private javax.swing.JTextField edtCommonName;
    private javax.swing.JTextField edtEmailAddress;
    private javax.swing.JTextField edtIPAddress;
    private javax.swing.JTextField edtLocality;
    private javax.swing.JTextField edtOrganization;
    private javax.swing.JTextField edtOrganizationalUnit;
    private javax.swing.JTextField edtSerialNumber;
    private javax.swing.JTextField edtStateOrProvince;
    private javax.swing.JTextField edtUniqueID;
    private javax.swing.JTextField edtWebSite;
    private javax.swing.JLabel lblCommonName;
    private javax.swing.JLabel lblCountry;
    private javax.swing.JLabel lblEmailAddress;
    private javax.swing.JLabel lblIPAddress;
    private javax.swing.JLabel lblKeyAlgorithm;
    private javax.swing.JLabel lblKeySize;
    private javax.swing.JLabel lblLocality;
    private javax.swing.JLabel lblOrganization;
    private javax.swing.JLabel lblOrganizationalUnit;
    private javax.swing.JLabel lblSerialNumber;
    private javax.swing.JLabel lblSignatureAlgorithm;
    private javax.swing.JLabel lblStateOrProvince;
    private javax.swing.JLabel lblUniqueID;
    private javax.swing.JLabel lblValidityPeriod;
    private javax.swing.JLabel lblWebSite;
    private javax.swing.JSpinner spinValidFrom;
    private javax.swing.JSpinner spinValidTo;
    // End of variables declaration//GEN-END:variables

}
