/* ============================================================================
 * PGPHelper.java
 * ============================================================================
 * THIS SOFTWARE IS RELEASED UNDER THE FOLLOWING BINDING LEGAL AGREEMENT
 * (BSD-LICENSE).
 * ============================================================================
 *
 * Copyright (C) 2009
 * by J.H.H.C. van der Meijs (tcpconns@jhhcvandermeijs.nl)
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    * Neither the name of tcpconns nor the names of its copyright holders or
 *      its contributors may be used to endorse or promote products derived from
 *      this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 * ============================================================================
 * IF YOU DO NOT AGREE TO THE ABOVE TERMS AND CONDITIONS THEN DO *NOT* USE THIS
 * SOFTWARE AND *DO* DESTROY ALL COPIES OF THIS SOFTWARE THAT YOU HAVE.
 * ============================================================================
 *
 * TCPCONNS’ CURRENT CODE IS NOT KNOWN TO VIOLATE ANY COPYRIGHTS OR INFRINGE ON
 * ANY PATENTS. MOREOVER, IT ADHERES TO GENERAL PROGRAMMING RULES AND PRACTICES
 * AND IS OF SUCH A SIMPLISTIC NATURE THAT A SIMILAR PACKAGE COULD BE
 * CONSTRUCTED BY ANYONE. IMPORTANT(!): IF YOU FEEL THAT A PIECE OF CODE IN
 * TCPCONNS VIOLATES A COPYRIGHT OR INFRINGES ON A PATENT, PLEASE DO INFORM ONE
 * OF THE AUTHORS IMMEDIATELY. DO ALWAYS STATE EXACTLY WHAT CODE IS NOT ALLOWED
 * UNDER EXACTLY WHAT RULING, COPYRIGHT, OR PATENT; AND PLEASE DO ELABORATE ON
 * EXACTLY HOW THE CODE VIOLATES THE SPECIFIC COPYRIGHT OR INFRINGES ON
 * THE SPECIFIC PATENT.
 *
 * ============================================================================
 *
 * This software is Open Source Initiative approved Open Source Software.
 * Open Source Initiative Approved is a trademark of the Open Source Initiative.
 * http://www.opensource.org/
 *
 * ============================================================================
 */
package nl.jhhcvandermeijs.tcpconns.pgp;

import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.security.KeyPair;
import java.security.KeyStore;
import java.security.KeyStore.PrivateKeyEntry;
import java.security.KeyStore.TrustedCertificateEntry;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.SecureRandom;
import java.security.cert.X509Certificate;
import java.util.Collection;
import java.util.Date;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import nl.jhhcvandermeijs.tcpconns.keys.KeyStoreHelper;
import nl.jhhcvandermeijs.tcpconns.utils.Utils;
import org.bouncycastle.bcpg.ArmoredOutputStream;
import org.bouncycastle.bcpg.BCPGOutputStream;
import org.bouncycastle.bcpg.PublicKeyAlgorithmTags;
import org.bouncycastle.openpgp.PGPCompressedData;
import org.bouncycastle.openpgp.PGPKeyPair;
import org.bouncycastle.openpgp.PGPObjectFactory;
import org.bouncycastle.openpgp.PGPPrivateKey;
import org.bouncycastle.openpgp.PGPPublicKey;
import org.bouncycastle.openpgp.PGPPublicKeyRing;
import org.bouncycastle.openpgp.PGPPublicKeyRingCollection;
import org.bouncycastle.openpgp.PGPSecretKey;
import org.bouncycastle.openpgp.PGPSecretKeyRingCollection;
import org.bouncycastle.openpgp.PGPSignature;
import org.bouncycastle.openpgp.PGPSignatureGenerator;
import org.bouncycastle.openpgp.PGPSignatureList;
import org.bouncycastle.openpgp.PGPUtil;

/**
 *
 * @author username
 */
public class PGPHelper {

    public static PGPKeyPair getKeyPair(X509Certificate x509, PrivateKey privateKey) {
        KeyStoreHelper.assertProviders();
        PGPKeyPair pgpKeyPair = null;
        try {
            pgpKeyPair = new PGPKeyPair(PublicKeyAlgorithmTags.RSA_GENERAL, x509.getPublicKey(), privateKey, x509.getNotBefore());
            return pgpKeyPair;
        } catch (Exception e) {
            System.out.println(e);
        }
        return null;
    }

    public static PGPPublicKey getPGPPublicKey(X509Certificate x509) {
        KeyStoreHelper.assertProviders();
        PGPPublicKey pgpPublicKey = null;
        try {
            pgpPublicKey = new PGPPublicKey(PublicKeyAlgorithmTags.RSA_GENERAL, x509.getPublicKey(), x509.getNotBefore());
            return pgpPublicKey;
        } catch (Exception e) {
            System.out.println();
        }
        return null;
    }

    public static String getCStyleHexID(long keyID) {
        try {
            ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
            DataOutputStream dataOutputStream = new DataOutputStream(byteArrayOutputStream);
            dataOutputStream.writeLong(keyID);
            dataOutputStream.flush();
            byteArrayOutputStream.flush();
            String fullHexID = "0x" + Utils.toHex(byteArrayOutputStream.toByteArray(), false);
            return fullHexID;
        } catch (Exception e) {
            return new String("ERROR \"" + e + "\" (KeyID: " + keyID + ")");
        }
    }

    /**
     * method to determine the PGP fullHexID of a key
     * @param x509
     * @return C-style (i.e. "0x.....") hex-encoded full (64 bit) key id
     */
    public static String getPGPKeyID(X509Certificate x509) {
        KeyStoreHelper.assertProviders();
        PGPPublicKey pgpPublicKey = PGPHelper.getPGPPublicKey(x509);
        if (pgpPublicKey == null) {return null;}
        return PGPHelper.getCStyleHexID(pgpPublicKey.getKeyID());
    }

    public static byte[] getPGPFingerprint(X509Certificate x509) {
        KeyStoreHelper.assertProviders();
        PGPPublicKey pgpPublicKey = PGPHelper.getPGPPublicKey(x509);
        if (pgpPublicKey == null) {return null;}
        return pgpPublicKey.getFingerprint();
    }

    public static String getPGPFingerprintAsHexEncodedString(X509Certificate x509, boolean useColonSeparator) {
        byte[] fingerprint = PGPHelper.getPGPFingerprint(x509);
        if (fingerprint == null) {return null;}
        return Utils.toHex(fingerprint, useColonSeparator);
    }

    public static final int HASH_ALGORITHM_SHA1 = PGPUtil.SHA1;
    public static final int HASH_ALGORITHM_SHA224 = PGPUtil.SHA224;
    public static final int HASH_ALGORITHM_SHA256 = PGPUtil.SHA256;
    public static final int HASH_ALGORITHM_SHA384 = PGPUtil.SHA384;
    public static final int HASH_ALGORITHM_SHA512 = PGPUtil.SHA512;
    public static final int HASH_ALGORITHM_TIGER_192 = PGPUtil.TIGER_192;
    public static final int HASH_ALGORITHM_RIPEMD160 = PGPUtil.RIPEMD160;
    public static final int HASH_ALGORITHM_HAVAL_5_160 = PGPUtil.HAVAL_5_160;

    public static boolean createPGPDetachedSignature(X509Certificate x509, PrivateKey privateKey, InputStream sourceInputStream, OutputStream signatureOutputStream) {
        return PGPHelper.createPGPDetachedSignature(x509, privateKey, sourceInputStream, signatureOutputStream, PGPHelper.HASH_ALGORITHM_SHA512);
    }

    public static boolean createPGPDetachedSignature(X509Certificate x509, PrivateKey privateKey, InputStream sourceInputStream, OutputStream signatureOutputStream, SecureRandom secureRandom) {
        return PGPHelper.createPGPDetachedSignature(x509, privateKey, sourceInputStream, signatureOutputStream, PGPHelper.HASH_ALGORITHM_SHA512, secureRandom);
    }

    public static boolean createPGPDetachedSignature(X509Certificate x509, PrivateKey privateKey, InputStream sourceInputStream, OutputStream signatureOutputStream, int hashAlgorithm) {
        return PGPHelper.createPGPDetachedSignature(x509, privateKey, sourceInputStream, signatureOutputStream, hashAlgorithm, new SecureRandom());
    }

    public static boolean createPGPDetachedSignature(X509Certificate x509, PrivateKey privateKey, InputStream sourceInputStream, OutputStream signatureOutputStream, int hashAlgorithm, SecureRandom secureRandom) {
        KeyStoreHelper.assertProviders();
        PGPKeyPair pgpKeyPair = PGPHelper.getKeyPair(x509, privateKey);
        if (pgpKeyPair == null) {return false;}
        try {
            PGPSignatureGenerator pgpSignatureGenerator = new PGPSignatureGenerator(pgpKeyPair.getPublicKey().getAlgorithm(), hashAlgorithm, "BC");
            pgpSignatureGenerator.initSign(PGPSignature.BINARY_DOCUMENT, pgpKeyPair.getPrivateKey(), secureRandom);

            byte[] buffer = new byte[4096];
            int bytesRead = sourceInputStream.read(buffer);
            while (bytesRead > 0) {
                pgpSignatureGenerator.update(buffer, 0, bytesRead);
                bytesRead = sourceInputStream.read(buffer);
            }

            PGPSignature pgpSignature = pgpSignatureGenerator.generate();
            ArmoredOutputStream armoredOutputStream = new ArmoredOutputStream(signatureOutputStream);
                armoredOutputStream.setHeader("Signature type", "X509 OpenPGP/" + pgpSignature.getVersion() + " tcpconns");
                if (pgpSignature.getHashAlgorithm() == PGPHelper.HASH_ALGORITHM_HAVAL_5_160) {
                    armoredOutputStream.setHeader("Hash Algorithm", "HAVAL_5_160");

                } else if (pgpSignature.getHashAlgorithm() == PGPHelper.HASH_ALGORITHM_RIPEMD160) {
                    armoredOutputStream.setHeader("Hash Algorithm", "RIPEMD160");

                } else if (pgpSignature.getHashAlgorithm() == PGPHelper.HASH_ALGORITHM_SHA1) {
                    armoredOutputStream.setHeader("Hash Algorithm", "SHA1");

                } else if (pgpSignature.getHashAlgorithm() == PGPHelper.HASH_ALGORITHM_SHA224) {
                    armoredOutputStream.setHeader("Hash Algorithm", "SHA224");

                } else if (pgpSignature.getHashAlgorithm() == PGPHelper.HASH_ALGORITHM_SHA256) {
                    armoredOutputStream.setHeader("Hash Algorithm", "SHA256");

                } else if (pgpSignature.getHashAlgorithm() == PGPHelper.HASH_ALGORITHM_SHA384) {
                    armoredOutputStream.setHeader("Hash Algorithm", "SHA384");

                } else if (pgpSignature.getHashAlgorithm() == PGPHelper.HASH_ALGORITHM_SHA512) {
                    armoredOutputStream.setHeader("Hash Algorithm", "SHA512");

                } else if (pgpSignature.getHashAlgorithm() == PGPHelper.HASH_ALGORITHM_TIGER_192) {
                    armoredOutputStream.setHeader("Hash Algorithm", "TIGER192");

                } else {
                    armoredOutputStream.setHeader("Hash Algorithm", "Other (" + pgpSignature.getHashAlgorithm() + ")");
                }

                armoredOutputStream.setHeader("PGP KeyID", PGPHelper.getPGPKeyID(x509));

            BCPGOutputStream bcpgOutputStream = new BCPGOutputStream(armoredOutputStream);
                pgpSignature.encode(bcpgOutputStream);
                bcpgOutputStream.finish();
            bcpgOutputStream.flush();
            armoredOutputStream.flush();
            signatureOutputStream.flush();
            try {armoredOutputStream.close();} catch (Exception e) {}

        } catch (Exception e) {return false;}
        return true;
    }
    
    public static boolean createPGPDetachedSignature(X509Certificate x509, PrivateKey privateKey, String sourceFileName, String signatureFileName) {
        InputStream sourceInputStream = null;
        OutputStream signatureOutputStream = null;
        try {
            try {
                sourceInputStream = new FileInputStream(sourceFileName);
                signatureOutputStream = new FileOutputStream(signatureFileName);
                
                return PGPHelper.createPGPDetachedSignature(x509, privateKey, sourceInputStream, signatureOutputStream);
            } finally {
                try {sourceInputStream.close();} catch (Exception e2) {}
                try {signatureOutputStream.close();} catch (Exception e2) {}
            }        
        } catch (Exception e) {}                
        return false;
    }

    public static PGPSignature getPGPSignature(String signatureFileName, String keyID) {
        InputStream signatureInputStream = null;
        try {
            try {
                signatureInputStream = new FileInputStream(signatureFileName);

                return PGPHelper.getPGPSignature(signatureInputStream, keyID);
            } finally {
                try {signatureInputStream.close();} catch (Exception e2) {}
            }
        } catch (Exception e) {}
        return null;
    }

    public static PGPSignature getPGPSignature(InputStream signatureInputStream, String keyID) {
        KeyStoreHelper.assertProviders();
        try {
            PGPObjectFactory pgpObjectFactory = new PGPObjectFactory(PGPUtil.getDecoderStream(signatureInputStream));
            PGPSignatureList pgpSignatureList = null;

            Object object = pgpObjectFactory.nextObject();
            while (object != null) {
                if (object instanceof PGPCompressedData) {
                    PGPCompressedData pgpCompressedData = ((PGPCompressedData)(object));
                    PGPObjectFactory pgpCompressedDataObjectFactory = new PGPObjectFactory(pgpCompressedData.getDataStream());
                    pgpSignatureList = ((PGPSignatureList)(pgpCompressedDataObjectFactory.nextObject()));
                } else {
                    pgpSignatureList = ((PGPSignatureList)(object));
                }

                int i = 1-1; int n = pgpSignatureList.size();
                while (i <= (n-1)) {
                    try {
                        PGPSignature pgpSignature = pgpSignatureList.get(i);
                        String fullHexID = PGPHelper.getCStyleHexID(pgpSignature.getKeyID());
                        if (fullHexID.equals(keyID)) {
                            return pgpSignature;
                        }
                    } catch (Exception e2) {}
                    i = i + 1;
                }
                object = pgpObjectFactory.nextObject();
            }

        } catch (Exception e) {}
        return null;
    }

    public static String[] getPGPDetachedSignatureKeyIDs(String signatureFileName) {
        InputStream signatureInputStream = null;
        try {
            try {
                signatureInputStream = new FileInputStream(signatureFileName);

                return PGPHelper.getPGPDetachedSignatureKeyIDs(signatureInputStream);
            } finally {
                try {signatureInputStream.close();} catch (Exception e2) {}
            }
        } catch (Exception e) {}
        return null;
    }

    public static String[] getPGPDetachedSignatureKeyIDs(InputStream signatureInputStream) {
        KeyStoreHelper.assertProviders();
        List<String> list = new LinkedList<String>();
        String[] result = new String[0];
        try {
            PGPObjectFactory pgpObjectFactory = new PGPObjectFactory(PGPUtil.getDecoderStream(signatureInputStream));
            PGPSignatureList pgpSignatureList = null;

            Object object = pgpObjectFactory.nextObject();
            while (object != null) {
                if (object instanceof PGPCompressedData) {
                    PGPCompressedData pgpCompressedData = ((PGPCompressedData)(object));
                    PGPObjectFactory pgpCompressedDataObjectFactory = new PGPObjectFactory(pgpCompressedData.getDataStream());
                    pgpSignatureList = ((PGPSignatureList)(pgpCompressedDataObjectFactory.nextObject()));
                } else {
                    pgpSignatureList = ((PGPSignatureList)(object));
                }

                int i = 1-1; int n = pgpSignatureList.size();
                while (i <= (n-1)) {
                    try {
                        PGPSignature pgpSignature = pgpSignatureList.get(i);
                        String fullHexID = PGPHelper.getCStyleHexID(pgpSignature.getKeyID());
                        list.add(fullHexID);
                    } catch (Exception e2) {}
                    i = i + 1;
                }
                object = pgpObjectFactory.nextObject();
            }

            result = list.toArray(result);
            return result;
        } catch (Exception e) {}
        return null;
    }

    public static boolean verifyPGPDetachedSignature(X509Certificate x509, InputStream sourceInputStream, InputStream signatureInputStream) {
        KeyStoreHelper.assertProviders();
        try {
            PGPPublicKey pgpPublicKey = PGPHelper.getPGPPublicKey(x509);
            if (pgpPublicKey == null) {return false;}

            PGPObjectFactory pgpObjectFactory = new PGPObjectFactory(PGPUtil.getDecoderStream(signatureInputStream));
            PGPSignatureList pgpSignatureList = null;

            Object object = pgpObjectFactory.nextObject();
            while (object != null) {
                if (object instanceof PGPCompressedData) {
                    PGPCompressedData pgpCompressedData = ((PGPCompressedData)(object));
                    PGPObjectFactory pgpCompressedDataObjectFactory = new PGPObjectFactory(pgpCompressedData.getDataStream());
                    pgpSignatureList = ((PGPSignatureList)(pgpCompressedDataObjectFactory.nextObject()));
                } else {
                    pgpSignatureList = ((PGPSignatureList)(object));
                }

                int i = 1-1; int n = pgpSignatureList.size();
                while (i <= (n-1)) {
                    try {
                        PGPSignature pgpSignature = pgpSignatureList.get(i);
                        pgpSignature.initVerify(pgpPublicKey, "BC");

                        byte[] buffer = new byte[4096];
                        int bytesRead = sourceInputStream.read(buffer);
                        while (bytesRead > 0) {
                            pgpSignature.update(buffer, 0, bytesRead);
                            bytesRead = sourceInputStream.read(buffer);
                        }

                        if (pgpSignature.verify()) {
                            return true;
                        }
                    } catch (Exception e2) {}

                    i = i + 1;
                }
                object = pgpObjectFactory.nextObject();
            }
        } catch (Exception e) {}
        return false;
    }

    public static boolean verifyPGPDetachedSignature(X509Certificate x509, String sourceFileName, String signatureFileName) {
        InputStream sourceInputStream = null;
        InputStream signatureInputStream = null;
        try {
            try {
                sourceInputStream = new FileInputStream(sourceFileName);
                signatureInputStream = new FileInputStream(signatureFileName);

                return PGPHelper.verifyPGPDetachedSignature(x509, sourceInputStream, signatureInputStream);
            } finally {
                try {sourceInputStream.close();} catch (Exception e2) {}
                try {signatureInputStream.close();} catch (Exception e2) {}
            }
        } catch (Exception e) {}
        return false;
    }

    public static X509Certificate fetchKey(KeyStore keyStore, char[] keyStorePassphrase, String keyID) {
        try {            
            Enumeration<String> aliases = keyStore.aliases();
            String alias = null;
            while (aliases.hasMoreElements()) {
                alias = aliases.nextElement();

                if (keyStore.entryInstanceOf(alias, PrivateKeyEntry.class)) {
                    try {
                        PrivateKeyEntry privateKeyEntry = ((PrivateKeyEntry)(keyStore.getEntry(alias, new KeyStore.PasswordProtection(keyStorePassphrase))));
                        if (privateKeyEntry.getCertificateChain() instanceof X509Certificate[]) {
                            X509Certificate[] chain = ((X509Certificate[])(privateKeyEntry.getCertificateChain()));
                            if ((PGPHelper.getCStyleHexID( (PGPHelper.getPGPPublicKey(chain[0])).getKeyID() )).equals(keyID)) {
                                return chain[0];
                            }
                        }
                    } catch (Exception e2) {}

                } else if (keyStore.entryInstanceOf(alias, TrustedCertificateEntry.class)) {
                    try {
                        TrustedCertificateEntry trustedCertificateEntry = ((TrustedCertificateEntry)(keyStore.getEntry(alias, null)));
                        if (trustedCertificateEntry.getTrustedCertificate() instanceof X509Certificate) {
                            X509Certificate x509 = ((X509Certificate)(trustedCertificateEntry.getTrustedCertificate()));
                            if ((PGPHelper.getCStyleHexID( (PGPHelper.getPGPPublicKey(x509)).getKeyID() )).equals(keyID)) {
                                return x509;
                            }
                        }
                    } catch (Exception e2) {}

                }
            }
        } catch (Exception e) {}
        return null;
    }

    public static X509Certificate fetchKey(KeyStore keyStore, String keyID) {
        try {
            Enumeration<String> aliases = keyStore.aliases();
            String alias = null;
            while (aliases.hasMoreElements()) {
                alias = aliases.nextElement();

                if (keyStore.entryInstanceOf(alias, TrustedCertificateEntry.class)) {
                    try {
                        TrustedCertificateEntry trustedCertificateEntry = ((TrustedCertificateEntry)(keyStore.getEntry(alias, null)));
                        if (trustedCertificateEntry.getTrustedCertificate() instanceof X509Certificate) {
                            X509Certificate x509 = ((X509Certificate)(trustedCertificateEntry.getTrustedCertificate()));
                            if ((PGPHelper.getCStyleHexID( (PGPHelper.getPGPPublicKey(x509)).getKeyID() )).equals(keyID)) {
                                return x509;
                            }
                        }
                    } catch (Exception e2) {}

                }
            }
        } catch (Exception e) {}
        return null;
    }

    public static X509Certificate fetchKey(X509Certificate[] certificates, String keyID) {
        try {
            KeyStore keyStore = KeyStoreHelper.newEmptyKeyStore();
            int i = 1-1; int n = certificates.length;
            while (i <= (n-1)) {
                keyStore.setCertificateEntry(KeyStoreHelper.getAlias(certificates[i]), certificates[i]);
                i = i + 1;
            }

            return PGPHelper.fetchKey(keyStore, keyID);
        } catch (Exception e) {}
        return null;
    }

    public static X509Certificate fetchKey(Collection<X509Certificate> certificates, String keyID) {
        try {
            X509Certificate[] certificateArray = new X509Certificate[0];
            certificateArray = certificates.toArray(certificateArray);

            return PGPHelper.fetchKey(certificateArray, keyID);
        } catch (Exception e) {}
        return null;
    }

    public static Date getValidTo(PGPPublicKey pgpPublicKey) {
        Date date = PGPHelper.getValidFrom(pgpPublicKey);
        long time = date.getTime() + (pgpPublicKey.getValidDays()*86400000L);
        date.setTime(time);
        return date;
    }

    public static Date getValidTo(PGPKeyPair pgpKeyPair) {
        return PGPHelper.getValidTo(pgpKeyPair.getPublicKey());
    }

    public static Date getValidFrom(PGPPublicKey pgpPublicKey) {
        return pgpPublicKey.getCreationTime();
    }

    public static Date getValidFrom(PGPKeyPair pgpKeyPair) {
        return PGPHelper.getValidFrom(pgpKeyPair.getPublicKey());
    }

    public static KeyPair getKeyPair(PGPKeyPair pgpKeyPair) {
        try {
            PrivateKey privateKey = pgpKeyPair.getPrivateKey().getKey();
            PublicKey publicKey = pgpKeyPair.getPublicKey().getKey("BC");
            if ((privateKey == null) || (publicKey == null)) {return null;}

            KeyPair keyPair = new KeyPair (publicKey, privateKey);
            return keyPair;
        } catch (Exception e) {}
        return null;
    }

    public static PGPKeyPair loadPGPKeyPair(String publicKeyFileName, String secretKeyFileName, char[] passphrase) {
        try {
            PGPPublicKey pgpPublicKey = null;
            PGPPrivateKey pgpPrivateKey = null;

            pgpPublicKey = PGPHelper.loadPGPPublicKey(publicKeyFileName);
            if (pgpPublicKey == null) {return null;}

            pgpPrivateKey = PGPHelper.loadPGPPrivateKey(secretKeyFileName, pgpPublicKey.getKeyID(), passphrase);
            if (pgpPrivateKey == null) {return null;}

            PGPKeyPair pgpKeyPair = new PGPKeyPair(pgpPublicKey, pgpPrivateKey);
            return pgpKeyPair;
        } catch (Exception e) {}
        return null;
    }

    public static PGPPrivateKey loadPGPPrivateKey(String fileName, long keyID, char[] passphrase) {
        InputStream inputStream = null;
        try {
            try {
                inputStream = new FileInputStream(fileName);

                return PGPHelper.loadPGPPrivateKey(inputStream, keyID, passphrase);
            } finally {
                try {inputStream.close();} catch (Exception e) {}
            }
        } catch (Exception e) {}
        return null;
    }

    public static PGPPrivateKey loadPGPPrivateKey(InputStream inputStream, long keyID, char[] passphrase) {
        try {
            InputStream decoderStream = PGPUtil.getDecoderStream(inputStream);
            PGPSecretKeyRingCollection pgpSecretKeyRingCollection = new PGPSecretKeyRingCollection(decoderStream);

            PGPSecretKey pgpSecretKey = null;
            try {
                pgpSecretKey = pgpSecretKeyRingCollection.getSecretKey(keyID);
            } catch (Exception e) {}
            if (pgpSecretKey == null) {return null;}

            PGPPrivateKey pgpPrivateKey = null;
            try {
                pgpPrivateKey = pgpSecretKey.extractPrivateKey(passphrase, "BC");
            } catch (Exception e) {}

            return pgpPrivateKey;

        } catch (Exception e) {}
        return null;
    }

    public static PGPPublicKey loadPGPPublicKey(String fileName) {
        InputStream inputStream = null;
        try {
            try {
                inputStream = new FileInputStream(fileName);

                return PGPHelper.loadPGPPublicKey(inputStream);
            } finally {
                try {inputStream.close();} catch (Exception e) {}
            }
        } catch (Exception e) {}
        return null;
    }

    public static PGPPublicKey loadPGPPublicKey(InputStream inputStream) {
        try {
            InputStream decoderStream = PGPUtil.getDecoderStream(inputStream);
            PGPPublicKeyRingCollection pgpPublicKeyRingCollection = new PGPPublicKeyRingCollection(decoderStream);

            Iterator i = pgpPublicKeyRingCollection.getKeyRings();
            while (i.hasNext()) {
                PGPPublicKeyRing pgpPublicKeyRing = ((PGPPublicKeyRing)(i.next()));
                Iterator j = pgpPublicKeyRing.getPublicKeys();

                while (j.hasNext()) {
                    PGPPublicKey pgpPublicKey = ((PGPPublicKey)(j.next()));
                    if (pgpPublicKey.isMasterKey()) {
                        return pgpPublicKey;
                    }
                }
            }
        } catch (Exception e) {}
        return null;
    }

}
