/* ============================================================================
 * TServerSideProtocol.java
 * ============================================================================
 * THIS SOFTWARE IS RELEASED UNDER THE FOLLOWING BINDING LEGAL AGREEMENT
 * (BSD-LICENSE).
 * ============================================================================
 *
 * Copyright (C) 2007-2009
 * by J.H.H.C. van der Meijs (tcpconns@jhhcvandermeijs.nl)
 * 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright notice, 
 *      this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright 
 *      notice, this list of conditions and the following disclaimer in the 
 *      documentation and/or other materials provided with the distribution.
 *    * Neither the name of tcpconns nor the names of its copyright holders or 
 *      its contributors may be used to endorse or promote products derived from 
 *      this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 * ============================================================================
 * IF YOU DO NOT AGREE TO THE ABOVE TERMS AND CONDITIONS THEN DO *NOT* USE THIS 
 * SOFTWARE AND *DO* DESTROY ALL COPIES OF THIS SOFTWARE THAT YOU HAVE.
 * ============================================================================
 * 
 * TCPCONNS’ CURRENT CODE IS NOT KNOWN TO VIOLATE ANY COPYRIGHTS OR INFRINGE ON 
 * ANY PATENTS. MOREOVER, IT ADHERES TO GENERAL PROGRAMMING RULES AND PRACTICES 
 * AND IS OF SUCH A SIMPLISTIC NATURE THAT A SIMILAR PACKAGE COULD BE 
 * CONSTRUCTED BY ANYONE. IMPORTANT(!): IF YOU FEEL THAT A PIECE OF CODE IN
 * TCPCONNS VIOLATES A COPYRIGHT OR INFRINGES ON A PATENT, PLEASE DO INFORM ONE 
 * OF THE AUTHORS IMMEDIATELY. DO ALWAYS STATE EXACTLY WHAT CODE IS NOT ALLOWED
 * UNDER EXACTLY WHAT RULING, COPYRIGHT, OR PATENT; AND PLEASE DO ELABORATE ON 
 * EXACTLY HOW THE CODE VIOLATES THE SPECIFIC COPYRIGHT OR INFRINGES ON
 * THE SPECIFIC PATENT.
 * 
 * ============================================================================
 * 
 * This software is Open Source Initiative approved Open Source Software.
 * Open Source Initiative Approved is a trademark of the Open Source Initiative.
 * http://www.opensource.org/
 * 
 * ============================================================================
 */
package nl.jhhcvandermeijs.tcpconns.server;

import nl.jhhcvandermeijs.tcpconns.client.*;
import java.io.IOException;

/**
 * this class serves as the parent class of the implementation(s) of your
 * server side protocol(s)
 */
public class TServerSideProtocol implements Runnable {
    
    /**
     * reference to the TTCPConnection object
     */
    protected TTCPConnection fTTCPConnection;
    
    /**
     * reference to the owning TTCPServer object
     */
    protected TTCPServer fTTCPServer;
    
    /**
     * constructor; you should not make instances of this class directly, but 
     * instead override the "synchronized public TServerSideProtocol 
     * newServerSideProtocol(TTCPConnection aTTCPConnection, TTCPServer 
     * aTTCPServer)" method of TTCPServer so that it returns instances of your
     * TServerSideProtocol.
     * 
     * @param aTTCPConnection the TTCPConnection object that holds the active
     *        connection
     * @param myTTCPServer the owning TTCPServer object
     */
    public TServerSideProtocol(TTCPConnection aTTCPConnection, TTCPServer myTTCPServer) {
        this.fTTCPConnection = aTTCPConnection;
        this.fTTCPServer = myTTCPServer;
    }
    
    /**
     * public handle towards the TTCPConnection
     * 
     * @return the TTCPConnection object that is connected to a client
     */
    public TTCPConnection conn() {
        return this.fTTCPConnection;
    }
    
    /**
     * implements a "perpetual" loop that continuously invokes the processRequest()
     * method; do not override this method, override the processRequest() method
     * instead, so that processRequest() contains your protocol interpreting
     * software code
     */
    public void run() {
        if (this.fTTCPConnection == null) {this.fTTCPServer.decClientCount(); return;}
        if (!(this.fTTCPConnection.isConnected())) {this.fTTCPServer.decClientCount(); return;}

        try {
            this.fTTCPConnection.mySocket().setSoTimeout(this.fTTCPServer.getTimeOut());

            while (this.fTTCPConnection.isConnected()
                   &&
                   (!(Thread.currentThread().isInterrupted()))                 
                   &&
                   (this.fTTCPServer.isActive())
                   ) {
                this.processRequest();
            }

        } catch (Exception e) {
        } finally {
            try{this.fTTCPConnection.disconnect();} catch (Exception e2) {}
            this.fTTCPServer.decClientCount();
        }
    }
    
    /**
     * override this method in your class. It is inside a loop process so you
     * do not need (nor want) to create another loop inside this method. Allow
     * exceptions to surface beyond this method. If an exception occurs then the
     * default behaviour should be to rethrow it if caught; the connection is
     * closed when this occurs.
     * 
     * @throws java.io.IOException
     */
    public void processRequest() throws IOException {
        throw new UnsupportedOperationException("Not supported yet.");        
    }    
}
