/* ============================================================================
 * Utils.java
 * ============================================================================
 * THIS SOFTWARE IS RELEASED UNDER THE FOLLOWING BINDING LEGAL AGREEMENT
 * (BSD-LICENSE).
 * ============================================================================
 *
 * Copyright (C) 2008-2009
 * by J.H.H.C. van der Meijs (tcpconns@jhhcvandermeijs.nl)
 * 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright notice, 
 *      this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright 
 *      notice, this list of conditions and the following disclaimer in the 
 *      documentation and/or other materials provided with the distribution.
 *    * Neither the name of tcpconns nor the names of its copyright holders or 
 *      its contributors may be used to endorse or promote products derived from 
 *      this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 * ============================================================================
 * IF YOU DO NOT AGREE TO THE ABOVE TERMS AND CONDITIONS THEN DO *NOT* USE THIS 
 * SOFTWARE AND *DO* DESTROY ALL COPIES OF THIS SOFTWARE THAT YOU HAVE.
 * ============================================================================
 * 
 * TCPCONNS’ CURRENT CODE IS NOT KNOWN TO VIOLATE ANY COPYRIGHTS OR INFRINGE ON 
 * ANY PATENTS. MOREOVER, IT ADHERES TO GENERAL PROGRAMMING RULES AND PRACTICES 
 * AND IS OF SUCH A SIMPLISTIC NATURE THAT A SIMILAR PACKAGE COULD BE 
 * CONSTRUCTED BY ANYONE. IMPORTANT(!): IF YOU FEEL THAT A PIECE OF CODE IN
 * TCPCONNS VIOLATES A COPYRIGHT OR INFRINGES ON A PATENT, PLEASE DO INFORM ONE 
 * OF THE AUTHORS IMMEDIATELY. DO ALWAYS STATE EXACTLY WHAT CODE IS NOT ALLOWED
 * UNDER EXACTLY WHAT RULING, COPYRIGHT, OR PATENT; AND PLEASE DO ELABORATE ON 
 * EXACTLY HOW THE CODE VIOLATES THE SPECIFIC COPYRIGHT OR INFRINGES ON
 * THE SPECIFIC PATENT.
 * 
 * ============================================================================
 * 
 * This software is Open Source Initiative approved Open Source Software.
 * Open Source Initiative Approved is a trademark of the Open Source Initiative.
 * http://www.opensource.org/
 * 
 * ============================================================================
 */
package nl.jhhcvandermeijs.tcpconns.utils;

import java.io.File;
import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.URL;
import java.net.UnknownHostException;
import java.util.Enumeration;

/**
 * The class utils provides various useful methods.
 */
public class Utils {
    /**
     * Converts a byte array into a hexadecimal string representation. This
     * method can be used to display the contents of a byte array in a more
     * or less human readable form.
     *
     * @param buffer a byte array
     * @param from the element from which the byte array should be converted
     * @param count how many bytes from the from element that should be converted
     *        (i.e. number of elements)
     * @param useColonSeparator should be true if output should be formatted like
     *        00:00:00:00:00:00....., should be false if output should be formatted
     *        like 000000000000......
     * @return returns a String
     */
    public static String toHex(byte[] buffer, int from, int count, boolean useColonSeparator) {
        StringBuffer aStringBuffer = new StringBuffer();

        if (   (from < 0)
            || (from > (buffer.length-1))
            || (count > (buffer.length-from))
           ) {throw new IndexOutOfBoundsException();}
        
        int i = from; int n = from + count;
        while (i <= (n-1)) {
            int byteValue;
            if (buffer[i] >= 0) {
                byteValue = buffer[i];                
            } else {
                byteValue = 256 + buffer[i];
            }                        
            
            int a, b;
            a = ((int)(byteValue / 16));
            b = byteValue % 16;
            
            switch (a) {
                case  0: {aStringBuffer.append("0"); break;}
                case  1: {aStringBuffer.append("1"); break;}
                case  2: {aStringBuffer.append("2"); break;}
                case  3: {aStringBuffer.append("3"); break;}
                case  4: {aStringBuffer.append("4"); break;}
                case  5: {aStringBuffer.append("5"); break;}
                case  6: {aStringBuffer.append("6"); break;}
                case  7: {aStringBuffer.append("7"); break;}
                case  8: {aStringBuffer.append("8"); break;}
                case  9: {aStringBuffer.append("9"); break;}
                case 10: {aStringBuffer.append("A"); break;}
                case 11: {aStringBuffer.append("B"); break;}
                case 12: {aStringBuffer.append("C"); break;}
                case 13: {aStringBuffer.append("D"); break;}
                case 14: {aStringBuffer.append("E"); break;}
                case 15: {aStringBuffer.append("F"); break;}                
            }
            
            switch (b) {
                case  0: {aStringBuffer.append("0"); break;}
                case  1: {aStringBuffer.append("1"); break;}
                case  2: {aStringBuffer.append("2"); break;}
                case  3: {aStringBuffer.append("3"); break;}
                case  4: {aStringBuffer.append("4"); break;}
                case  5: {aStringBuffer.append("5"); break;}
                case  6: {aStringBuffer.append("6"); break;}
                case  7: {aStringBuffer.append("7"); break;}
                case  8: {aStringBuffer.append("8"); break;}
                case  9: {aStringBuffer.append("9"); break;}
                case 10: {aStringBuffer.append("A"); break;}
                case 11: {aStringBuffer.append("B"); break;}
                case 12: {aStringBuffer.append("C"); break;}
                case 13: {aStringBuffer.append("D"); break;}
                case 14: {aStringBuffer.append("E"); break;}
                case 15: {aStringBuffer.append("F"); break;}                 
            }                             
            if (useColonSeparator) {
                if (i != (n-1)) {
                    aStringBuffer.append(":");
                }
            }
            i++;
        }
        return aStringBuffer.toString();
    }

    /**
     * Converts a byte array into a hexadecimal string representation. This
     * method can be used to display the contents of a byte array in a more
     * or less human readable form.
     *
     * @param buffer a byte array
     * @param from the element from which the byte array should be converted
     * @param count how many bytes from the from element that should be converted
     *        (i.e. number of elements)
     * @return returns a String
     */
    public static String toHex(byte[] buffer, int from, int count) {
        return Utils.toHex(buffer, from, count, false);
    }

    /**
     * Converts a byte array into a hexadecimal string representation. This
     * method can be used to display the contents of a byte array in a more
     * or less human readable form.
     *
     * @param buffer a byte array
     * @param count the number of bytes from the beginning of the buffer that should be converted
     *        (i.e. number of elements)
     * @param useColonSeparator should be true if output should be formatted like
     *        00:00:00:00:00:00....., should be false if output should be formatted
     *        like 000000000000......
     * @return returns a String
     */
    public static String toHex(byte[] buffer, int count, boolean useColonSeparator) {
        return Utils.toHex(buffer, 0, count, useColonSeparator);
    }

    /**
     * Converts a byte array into a hexadecimal string representation. This
     * method can be used to display the contents of a byte array in a more
     * or less human readable form.
     *
     * @param buffer a byte array
     * @param count the number of bytes from the beginning of the buffer that should be converted
     *        (i.e. number of elements)
     * @return returns a String
     */
    public static String toHex(byte[] buffer, int count) {
        return Utils.toHex(buffer, 0, count);
    }

    /**
     * Converts a byte array into a hexadecimal string representation. This
     * method can be used to display the contents of a byte array in a more
     * or less human readable form.
     *
     * @param buffer a byte array
     * @param useColonSeparator should be true if output should be formatted like
     *        00:00:00:00:00:00....., should be false if output should be formatted
     *        like 000000000000......
     * @return returns a String
     */
    public static String toHex(byte[] buffer, boolean useColonSeparator) {
        return Utils.toHex(buffer, 0, buffer.length, useColonSeparator);
    }

    /**
     * Converts a byte array into a hexadecimal string representation. This
     * method can be used to display the contents of a byte array in a more
     * or less human readable form.
     *
     * @param buffer a byte array
     * @return returns a String
     */
    public static String toHex(byte[] buffer) {
        return Utils.toHex(buffer, 0, buffer.length);
    }

    /**
     * Converts (decodes) a hexadecimal encoded string into a byte array
     * @param value the hex encoded string
     * @return the hex decoded string as byte[]
     */
    public static byte[] decodeHex(String value) {
        int i = 1-1; int n = value.length();
            if ((n % 2) != 0) {return null;}
            byte[] buffer = new byte[((int)(n / 2))];
            int j = 1-1;

            char zero = ((new String("0")).toCharArray())[0];
            char one = ((new String("1")).toCharArray())[0];
            char two = ((new String("2")).toCharArray())[0];
            char three = ((new String("3")).toCharArray())[0];
            char four = ((new String("4")).toCharArray())[0];
            char five = ((new String("5")).toCharArray())[0];
            char six = ((new String("6")).toCharArray())[0];
            char seven = ((new String("7")).toCharArray())[0];
            char eight = ((new String("8")).toCharArray())[0];
            char nine = ((new String("9")).toCharArray())[0];
            char alpha = ((new String("A")).toCharArray())[0];
            char bravo = ((new String("B")).toCharArray())[0];
            char charlie = ((new String("C")).toCharArray())[0];
            char delta = ((new String("D")).toCharArray())[0];
            char echo = ((new String("E")).toCharArray())[0];
            char foxtrot = ((new String("F")).toCharArray())[0];

            int a;

        while (i <= (n-1)) {

            a = 0;

                   if (value.charAt(i) == zero) {
                                                        a =  0*16;
            } else if (value.charAt(i) == one) {
                                                        a =  1*16;
            } else if (value.charAt(i) == two) {
                                                        a =  2*16;
            } else if (value.charAt(i) == three) {
                                                        a =  3*16;
            } else if (value.charAt(i) == four) {
                                                        a =  4*16;
            } else if (value.charAt(i) == five) {
                                                        a =  5*16;
            } else if (value.charAt(i) == six) {
                                                        a =  6*16;
            } else if (value.charAt(i) == seven) {
                                                        a =  7*16;
            } else if (value.charAt(i) == eight) {
                                                        a =  8*16;
            } else if (value.charAt(i) == nine) {
                                                        a =  9*16;
            } else if (value.charAt(i) == alpha) {
                                                        a = 10*16;
            } else if (value.charAt(i) == bravo) {
                                                        a = 11*16;
            } else if (value.charAt(i) == charlie) {
                                                        a = 12*16;
            } else if (value.charAt(i) == delta) {
                                                        a = 13*16;
            } else if (value.charAt(i) == echo) {
                                                        a = 14*16;
            } else if (value.charAt(i) == foxtrot) {
                                                        a = 15*16;
            }


                   if (value.charAt(i + 1) == zero) {
                                                        a = a +  0;
            } else if (value.charAt(i + 1) == one) {
                                                        a = a +  1;
            } else if (value.charAt(i + 1) == two) {
                                                        a = a +  2;
            } else if (value.charAt(i + 1) == three) {
                                                        a = a +  3;
            } else if (value.charAt(i + 1) == four) {
                                                        a = a +  4;
            } else if (value.charAt(i + 1) == five) {
                                                        a = a +  5;
            } else if (value.charAt(i + 1) == six) {
                                                        a = a +  6;
            } else if (value.charAt(i + 1) == seven) {
                                                        a = a +  7;
            } else if (value.charAt(i + 1) == eight) {
                                                        a = a +  8;
            } else if (value.charAt(i + 1) == nine) {
                                                        a = a +  9;
            } else if (value.charAt(i + 1) == alpha) {
                                                        a = a + 10;
            } else if (value.charAt(i + 1) == bravo) {
                                                        a = a + 11;
            } else if (value.charAt(i + 1) == charlie) {
                                                        a = a + 12;
            } else if (value.charAt(i + 1) == delta) {
                                                        a = a + 13;
            } else if (value.charAt(i + 1) == echo) {
                                                        a = a + 14;
            } else if (value.charAt(i + 1) == foxtrot) {
                                                        a = a + 15;
            }

            if (a >= 128) {
                a = a - 256;
            }
            buffer[j] = ((byte)(a));

            i = i + 2; j = j + 1;
        }
        return buffer;
    }

    /**
     * use this method to obtain the user directory of the current user
     * @return the user directory of the current user as string, the user
     * directory will end with a separator (e.g. \ on windows, / on linux,
     * solaris, and bsd)
     */
    public static String getUserHome() {
        String result = null;        

        result = System.getProperty("user.home");
        try {
            result = (new File(result)).getCanonicalPath();

            if (result.endsWith(File.separator)) {
                //do nothing
            } else {
                result = result + File.separator;
            }

            return result;
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * (C) 2009 by Marcello de Sales
     * Assumed to be in public domain
     * http://www.jguru.com/faq/view.jsp?EID=15835
     *
     * @return the current environment's IP address, taking into account the
     *         Internet connection to any of the available machine's Network
     *         interfaces. Examples of the outputs can be in octatos or in IPV6
     *         format.
     */
    public static String getNetworkIp() {
        Enumeration<NetworkInterface> netInterfaces = null;
        try {netInterfaces = NetworkInterface.getNetworkInterfaces();} catch (Exception e) {return null;}

        while (netInterfaces.hasMoreElements()) {
            NetworkInterface ni = netInterfaces.nextElement();
            Enumeration<InetAddress> address = ni.getInetAddresses();
            while (address.hasMoreElements()) {
                InetAddress addr = address.nextElement();
                if (!addr.isLoopbackAddress() && addr.isSiteLocalAddress()
                        && !(addr.getHostAddress().indexOf(":") > -1)) {
                    return addr.getHostAddress();
                }
            }
        }
        //otherwise
        try {
            return InetAddress.getLocalHost().getHostAddress();
        } catch (UnknownHostException e) {
            return "127.0.0.1";
        }
    }

    /**
     * method to obtain a full path description to a running jar name; similar to
     * the get ExeName feature in Delphi
     * adapted from: http://episteme.arstechnica.com/eve/forums/a/tpc/f/6330927813/m/231000361831
     * (C)2006 by "vafarmboy", code assumed to be in public domain
     * @param aClass the class of which you wish to determine the path to its .jar file
     * @return path to the .jar file that contains aClass
     */
    public static String getJarName(Class aClass) {
        if (aClass == null) {return null;}
        URL anURL = aClass.getResource(aClass.getSimpleName() + ".class");
        if (anURL != null) {
            String path = anURL.toString().substring(0, anURL.toString().indexOf("!"));
            path = path.substring(path.lastIndexOf(":")+1, path.length());
            return path;
        }
        return null;
    }

    /**
     * convenience method for getJarName(Class aClass)
     * @param anObject an object of a class of which you wish to determine the path to its .jar file
     * @return path to the .jar file that contains the class of anObject
     */
    public static String getJarName(Object anObject) {
        Class aClass = anObject.getClass();
        //return aClass.getCanonicalName();
        return getJarName(aClass);
    }

    /**
     * this method "validates" a directory; it adds a separator like / or \ when
     * the directory does not end with one
     * @param aDirectory the folder to be "validated"
     * @return a "valid" directory name
     */
    public static String validateDirectory(String aDirectory) {
        if (aDirectory == null) {return null;}
        try {
            aDirectory = (new File(aDirectory)).getCanonicalPath();

            if (aDirectory.endsWith(File.separator)) {
                //do nothing
            } else {
                aDirectory = aDirectory + File.separator;
            }

            return aDirectory;
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * this method "validates" a file name; it removes a separator like / or \ when
     * the file name ends with one
     * @param aFileName the file to be "validated"
     * @return a "valid" file name
     */
    public static String validateFileName(String aFileName) {
        if (aFileName == null) {return null;}
        try {
            aFileName = (new File(aFileName)).getCanonicalPath();

            if (aFileName.endsWith(File.separator)) {
                aFileName = aFileName.substring(0, aFileName.length()-1);
            } else {
                //do nothing
            }
            return aFileName;
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * method to test whether a path is absolute or not
     * @param path a path to be tested
     * @return true if path is absolute, false if it is not
     */
    public static boolean isAbsolutePath(String path) {
        if (path == null) {return false;}

        /*
         * assume relative
         * AND
         * test for absoluteness
         *
         */
                                                       boolean isAbsolutePath = false;

               if (File.separator.equals("/") && (path.startsWith(File.separator))) {

                                                               isAbsolutePath = true;

         } else if (File.separator.equals("\\")) {

                    if (path.startsWith(File.separator)) {

                                                               isAbsolutePath = true;

                    }

                    if (path.length() >= 2) {

                        if (path.substring(1,2).equals(":"  )) {isAbsolutePath = true;}

                    }

         }
         return isAbsolutePath;
    }
}
